package calc;

import java.io.IOException;

import party.PartyInfo;
import state.PokeStateBaton;
import state.PokeStatePlace;
import util.DialogUtil;

/*
*  AfterMoveCalcクラス
*
*  技を繰り出した後の後処理を行うクラスです。
*
*  変更履歴
*  2019/06/22 新規作成
*  2019/07/25 定数ダメージで回復してしまうバグを修正
*  2020/12/19 宿り木で相手のポケモンを倒した時に相手の現HPを超えて回復するバグを修正
*
*/

public class AfterMoveCalc extends DamageCalc {

	/* 参照・反映用Stateオブジェクト */
	public PartyInfo partyInfoMyself = null; //PokeStateUnit配列
	public PartyInfo partyInfoOpponent = null; //PokeStateUnit配列
	public PokeStatePlace pokeStatePlaceMyself = null;
	public PokeStatePlace pokeStatePlaceOpponent = null;
	public PokeStateBaton pokeStateBatonMyself = null;

	/* 出力領域 */
	public int changeNecessityNumber = 0; //0なら交代不要、1なら自分のみ交代

	/* 技繰り出し後処理標準メソッド
	 *   引数　　　：自分のPokeStateUnit配列、相手のPokeStateUnit配列、
	 *   　　　　　：自分のPokeStatePlaceオブジェクト、相手のPokeStatePlaceオブジェクト、
	 *   　　　　　：自分のPokeStateBatonオブジェクト
	 *   戻り値　　：交代要否（int）（0なら交代不要、1なら自分だけ交代）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void afterMoveCalcCommon (PartyInfo partyInfoMyself,PartyInfo partyInfoOpponent,
	                                   PokeStatePlace pokeStatePlaceMyself,PokeStatePlace pokeStatePlaceOpponent,
	                                   PokeStateBaton pokeStateBatonMyself) throws IOException {

		// 初期値セット（各種Stateクラス（PokeStateUnitクラスは配列））
		this.partyInfoMyself = partyInfoMyself;
		this.partyInfoOpponent = partyInfoOpponent;
		this.pokeStatePlaceMyself = pokeStatePlaceMyself;
		this.pokeStatePlaceOpponent = pokeStatePlaceOpponent;
		this.pokeStateBatonMyself = pokeStateBatonMyself;

		// 出力用領域初期化メソッド呼び出し
    	outputAreaInit();

		// 相手の道連れ解除メソッド呼び出し
		downTogetherOff();

		// 毒・火傷・猛毒の実回復・ダメージ計算メソッド呼び出し
		poisonBurnCalc();

		// 宿り木の種計算メソッド呼び出し
		parasiteCalc();

		// 悪夢回復判定・ダメージ計算メソッド呼び出し
		nightmareCalc();

		// 呪い計算メソッド呼び出し
		curceCalc();

		// 戻り値（交代要否）の判定メソッド呼び出し
		changeNecessitySet();
	}

	/* 計算用一時領域・反映用一時領域・出力用領域初期化メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void outputAreaInit() {
    	changeNecessityNumber = 0;
    }

	/* 相手の道連れ解除メソッド（技使用フラグが立っている（交代や混乱自滅や技無しでは立たない）の場合に解除）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void downTogetherOff() throws IOException {
		if (pokeStatePlaceMyself.thisTurnUseMoveFlag) {
			pokeStatePlaceOpponent.downTogetherWaiting(false);
		}
	}

	/* 毒・火傷・猛毒の実回復・ダメージ計算メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void poisonBurnCalc() throws IOException {
		if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("poison") ||
		    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("burn")) {
			if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("poison") &&
			    (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("どくけしのみ") ||
			     partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("きせきのみ"))){
	    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeItem("持ち物無し");
	    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeBadHealth("none");
			} else if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("burn") &&
			           (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("こおったきのみ") ||
						partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("きせきのみ"))){
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeItem("持ち物無し");
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeBadHealth("none");
			} else {
				DialogUtil.dialogMessageOutput("毒・火傷のダメージ発生");
				int damage = 0;
				if (pokeStateBatonMyself.morePoisonContinuousTrun > 0) {
					damage = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP,
							           pokeStateBatonMyself.morePoisonContinuousTrun,
							           16);
				} else {
					damage = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP,
		                               1,
		                               8);
				}
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(damage * (-1));
			}
		}
	}

	/* 宿り木の種計算メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void parasiteCalc() throws IOException {
		if (pokeStateBatonMyself.parasitedFlag) {
			DialogUtil.dialogMessageOutput("宿り木のダメージ・回復発生");
			int damage = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP,
	                       1,
	                       8);
			// 現HPを超える場合は現HPと同じダメージにする
	    	if (damage > partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP) {
	    		damage = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP;
	    	}
			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(damage * (-1));
			partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].HPUpDown(damage);
		}
	}

	/* 悪夢回復判定・ダメージ計算メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void nightmareCalc() throws IOException {
		if (pokeStatePlaceMyself.nightmareFlag) {
			if (!partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("sleep")) {
				pokeStatePlaceMyself.beenIntoNightmare(false);
			} else {
				DialogUtil.dialogMessageOutput("悪夢のダメージ発生");
				int damage = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP,
	                                     1,
	                                     4);
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(damage * (-1));
			}
		}
	}

	/* 呪い計算メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void curceCalc() throws IOException {
		if (pokeStateBatonMyself.curcedFlag) {
			DialogUtil.dialogMessageOutput("呪いのダメージ発生");
			int damage = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP,
                                    1,
                                    4);
			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(damage * (-1));
		}
	}

	/* 戻り値（交代要否）の判定メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeNecessitySet() {
    	changeNecessityNumber = 0;
    	if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP <= 0) {
    		changeNecessityNumber = 1;
    	}
    }

}
