package calc;

import java.io.IOException;

import party.PartyInfo;
import state.PokeStateBaton;
import state.PokeStatePlace;
import state.PokeStateWhole;
import util.DialogUtil;

/*
*  BeforeMoveCalcクラス
*
*  技の繰り出し判定を行うクラスです。
*  アンコールによる選択技上書きや技の繰り出し直前の各種実による状態異常回復も当クラスに実装します。
*
*  変更履歴
*  2019/06/08 新規作成
*  2019/06/16 交代の場合にPokeStatePlaceクラス共通処理が行われないように制御を追加
*  2019/07/15 配列の初期化方法を誤っていたので修正
*  2019/07/27 「技無し」の技コードを「XXc」に統一
*  2019/07/29 悪あがき時に動ける動けない判定が行われないバグを修正
*  2019/09/01 先手でアンコールされた際に選択した技がアンコールされた技で上書きされないバグを修正
*  2020/01/02 混乱が治るターンで、1/2の確率でその後の判定（麻痺等）が行われなくなるバグを修正
*  2020/08/15 バトンタッチ発動失敗時にも追い打ちダメージが２倍になるバグを修正
*  2020/09/12 奇跡の実で眠りから回復した時に悪夢状態を回復しないように修正
*  2020/09/12 スケッチを実装
*  2020/09/12 悪あがきが眠る中にも繰り出せるPBSバグを実装
*  2020/09/12 悪あがきフラグ廃止（実機にはあるらしいが、少なくともPBSにはない）
*  2020/10/15 悪あがきが眠る中にも繰り出せるPBSバグが直っていたのでPBSToolsにも反映
*  2020/10/18 物真似等による技上書きが永続するバグを修正
*  2021/01/23 ルーチンでの参照禁止変数をprivate化したことに伴う修正
*
*/

public class BeforeMoveCalc {

	/* 参照・反映用Stateオブジェクト */
	public PartyInfo partyInfo = null; //PokeStateUnit配列
	public PokeStatePlace pokeStatePlaceMyself = null;
	public PokeStatePlace pokeStatePlaceOpponent = null;
	public PokeStateBaton pokeStateBaton = null;
	public PokeStateWhole pokeStateWhole = null;

	/* 計算用一時領域 */
	public String[] moveName = {"技無し","技無し","技無し","技無し"}; //技の名前（物真似・変身考慮）
	public int[] movePP = {0,0,0,0}; //技のPP（物真似・変身考慮）
	public String[] moveType = {"question","question","question","question"}; //技のタイプ（物真似・変身考慮）
	public int[] movePower = {1,1,1,1}; //技の威力（物真似・変身考慮）
	public double[] moveHitProbability = {100.0,100.0,100.0,100.0}; //技の威力（物真似・変身考慮）
	public int[] movePPPrimitive = {0,0,0,0}; //技の元々のPP（物真似・変身考慮）
	public double[] additionalEffectProbability = {0.0,0.0,0.0,0.0}; //技の威力（物真似・変身考慮）
	public String[] moveEffectCode = {"XXc","XXc","XXc","XXc"}; //技の効果コード（物真似・変身考慮）

	/* 反映用一時領域 */
	public boolean badHealthItemRecoveryFlag = false; //状態異常をアイテムで回復したか
	public boolean confusionItemRecoveryFlag = false; //混乱をアイテムで回復したか
	public boolean confusionSelfDestructionFlag = false; //混乱自滅かどうか
	public boolean sleepCountDownFlag = false; //眠りターンをカウントダウンするか
	public boolean confusionCountDownFlag = false; //混乱ターンをカウントダウンするか
	public boolean moveParalysisCountDownFlag = false; //金縛りターンをカウントダウンするか
	public boolean PPUseFlag = false; //PPを消費するかどうか
	public Boolean wakeupByMiracleBerryFlag = null; //起きてない場合はnull,奇跡の実で起きた場合はtrue,それ以外はfalse

	/* 出力領域 */
	public boolean movePossibleFlag = true; //動けるかどうか

	/* 技の繰り出し判定標準メソッド
	 *   引数　　　：PokeStateUnit配列、
	 *   　　　　　：PokeStatePlaceオブジェクト（自分・相手両方）、
	 *   　　　　　：PokeStateBatonオブジェクト、
	 *   　　　　　：PokeStateWholeオブジェクト
	 *   戻り値　　：boolean（trueなら動ける、falseなら動けない）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void beforeMoveCalcCommon(PartyInfo partyInfo,
    		                          PokeStatePlace pokeStatePlaceMyself,PokeStatePlace pokeStatePlaceOpponent,
    		                          PokeStateBaton pokeStateBaton,
    		                          PokeStateWhole pokeStateWhole) throws IOException {

		// 初期値セット（各種Stateクラス（PokeStateUnitクラスは配列））
    	this.partyInfo = partyInfo;
    	this.pokeStatePlaceMyself = pokeStatePlaceMyself;
    	this.pokeStatePlaceOpponent = pokeStatePlaceOpponent;
    	this.pokeStateBaton = pokeStateBaton;
    	this.pokeStateWhole = pokeStateWhole;

		// 計算用一時領域・反映用一時領域初期化メソッド呼び出し
    	calcTmpOutputAreaInit();

		// スケッチ・物真似・変身・指を振る・オウム返しによるステータス上書きメソッド呼び出し
    	calcAreaSet();

    	// 以下は交代時は実行しない（技選択時、及び悪あがき時に実行）
    	if((pokeStatePlaceMyself.moveIndex != -1 && pokeStatePlaceMyself.changeTarget == -1)||
    	    (pokeStatePlaceMyself.moveIndex == -1 && pokeStatePlaceMyself.changeTarget == -1)) {

    		// アンコールによる選択技上書きメソッド呼び出し
    		encoredMoveChoice();

    		// 動ける動けない判定メソッド呼び出し
    		canMoveJudge();

			// 技繰り出せるか否かの判定の反映メソッド呼び出し
    		canMoveInfluence();

    	}

    }

	/* 計算用一時領域・反映用一時領域・出力領域初期化メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void calcTmpOutputAreaInit() {
    	moveName = null;
    	moveName = new String[4];
    	movePP = null;
    	movePP = new int[4];
    	moveType = null;
    	moveType = new String[4];
    	movePower = null;
    	movePower = new int[4];
    	moveHitProbability = null;
    	moveHitProbability = new double[4];
    	movePPPrimitive = null;
    	movePPPrimitive = new int[4];
    	additionalEffectProbability = null;
    	additionalEffectProbability = new double[4];
    	moveEffectCode = null;
    	moveEffectCode = new String[4];
    	for(int i = 0; i < 4; i++){
    		moveName[i] = "技無し";
    		movePP[i] = 0;
    		moveType[i] = "question";
    		movePower[i] = 1;
    		moveHitProbability[i] = 100.0;
    		movePPPrimitive[i] = 0;
    		additionalEffectProbability[i] = 0.0;
    		moveEffectCode[i] = "XXc";
    	}
    	badHealthItemRecoveryFlag = false;
    	confusionItemRecoveryFlag = false;
    	confusionSelfDestructionFlag = false;
    	sleepCountDownFlag = false;
    	confusionCountDownFlag = false;
    	moveParalysisCountDownFlag = false;
    	PPUseFlag = false;
    	movePossibleFlag = true;
    	wakeupByMiracleBerryFlag = null;
    }

	/* 物真似・変身・指を振る・オウム返しによるステータス上書きメソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void calcAreaSet() {

    	// 物真似・変身を考慮しない場合のステータスをセット
    	for(int i = 0; i < 4; i++){
    		moveName[i] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].moveName[i];
    		movePP[i] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].movePP[i];
    		moveType[i] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].moveType[i];
    		movePower[i] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].movePower[i];
    		moveHitProbability[i] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].moveHitProbability[i];
    		movePPPrimitive[i] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].movePPPrimitive[i];
    		additionalEffectProbability[i] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].additionalEffectProbability[i];
    		moveEffectCode[i] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].moveEffectCode[i];
    	}

    	// 変身を考慮した場合のステータスをセット
    	if (pokeStatePlaceMyself.transformedFlag) {
        	for(int i = 0; i < 4; i++){
        		moveName[i] = pokeStatePlaceMyself.transformedMoveName[i];
        		movePP[i] = pokeStatePlaceMyself.transformedMovePP[i];
        		moveType[i] = pokeStatePlaceMyself.transformedMoveType[i];
        		movePower[i] = pokeStatePlaceMyself.transformedMovePower[i];
        		moveHitProbability[i] = pokeStatePlaceMyself.transformedMoveHitProbability[i];
        		movePPPrimitive[i] = pokeStatePlaceMyself.transformedMovePPPrimitive[i];
        		additionalEffectProbability[i] = pokeStatePlaceMyself.transformedAdditionalEffectProbability[i];
        		moveEffectCode[i] = pokeStatePlaceMyself.transformedMoveEffectCode[i];
        	}
    	}

    	// スケッチを考慮した場合のステータスをセット
    	if (partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveIndex >= 0 && partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveIndex <= 3) {
    		moveName[partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveName;
    		movePP[partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMovePP;
    		moveType[partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveType;
    		movePower[partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMovePower;
    		moveHitProbability[partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveHitProbability;
    		movePPPrimitive[partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMovePPPrimitive;
    		additionalEffectProbability[partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveIndex] =
    				partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchAdditionalEffectProbability;
    		moveEffectCode[partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveEffectCode;
    	}

    	// 物真似を考慮した場合のステータスをセット
    	if (pokeStatePlaceMyself.mimicMoveIndex >= 0 && pokeStatePlaceMyself.mimicMoveIndex <= 3) {
    		moveName[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMoveName;
    		movePP[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMovePP;
    		moveType[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMoveType;
    		movePower[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMovePower;
    		moveHitProbability[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMoveHitProbability;
    		movePPPrimitive[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMovePPPrimitive;
    		additionalEffectProbability[pokeStatePlaceMyself.mimicMoveIndex] =
    				pokeStatePlaceMyself.mimicAdditionalEffectProbability;
    		moveEffectCode[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMoveEffectCode;
    	}

    	// 指を振る・オウム返しを考慮した場合のステータスをセット
    	if (pokeStatePlaceMyself.moveCopyMoveFlag) {
    		moveName[pokeStatePlaceMyself.moveIndex] = pokeStatePlaceMyself.latestUsedMoveNameMoveCopy;
    		moveType[pokeStatePlaceMyself.moveIndex] = pokeStatePlaceMyself.latestUsedMoveTypeMoveCopy;
    		movePower[pokeStatePlaceMyself.moveIndex] = pokeStatePlaceMyself.latestUsedMovePowerMoveCopy;
    		moveHitProbability[pokeStatePlaceMyself.moveIndex] =
    				pokeStatePlaceMyself.latestUsedMoveHitProbabilityMoveCopy;
    		movePPPrimitive[pokeStatePlaceMyself.moveIndex] = pokeStatePlaceMyself.latestUsedMovePPPrimitive;
    		additionalEffectProbability[pokeStatePlaceMyself.moveIndex] =
    				pokeStatePlaceMyself.latestUsedAdditionalEffectProbabilityMoveCopy;
    		moveEffectCode[pokeStatePlaceMyself.moveIndex] = pokeStatePlaceMyself.latestUsedMoveEffectCodeMoveCopy;
    	}

    }

	/* アンコールによる選択技上書きメソッド（交代以外の場合に実行）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void encoredMoveChoice() {
    	if (pokeStatePlaceMyself.isEncored() &&
    	    (pokeStatePlaceMyself.encoredMoveIndex >= 0 &&
    	     pokeStatePlaceMyself.encoredMoveIndex <= 3)) {
    		pokeStatePlaceMyself.moveIndexSet(pokeStatePlaceMyself.encoredMoveIndex);
    	}
    }

	/* 動ける動けない判定メソッド（交代以外の場合に実行）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void canMoveJudge() throws IOException {

		// 眠りの実回復・カウントダウン・動けない判定処理（いびき・寝言なら動ける）　※メッセージはカウントダウン時　※悪あがきは動けない（PBSバグ修正済）
    	if (partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].badHealth.equals("sleep")) {
    		if (partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].itemName.equals("はっかのみ") ||
    			partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) {
    			badHealthItemRecoveryFlag = true;
    		} else {
    			sleepCountDownFlag = true;
    			if (partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].isSleepThisTrun()){
    				if(!moveEffectCode[pokeStatePlaceMyself.moveIndex].equals("5C") &&
    				   !moveEffectCode[pokeStatePlaceMyself.moveIndex].equals("61")) {
    					movePossibleFlag = false;
    					return;
    				}
    			}
    		}
    		// 奇跡の実で起きるかそれ以外の手段で起きるかを管理
    		if (badHealthItemRecoveryFlag || partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].isNotSleepThisTrun()) {
    			if (partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) {
    				wakeupByMiracleBerryFlag = true;
    			} else {
    				wakeupByMiracleBerryFlag = false;
    			}
    		}
    	}

		// 凍りの実回復・動けない処理処理（火炎車・聖なる炎なら動ける）
    	if (partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].badHealth.equals("freeze")) {
    		if (partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].itemName.equals("やけたきのみ") ||
    			partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) {
    			badHealthItemRecoveryFlag = true;
    		} else {
    			if(!moveEffectCode[pokeStatePlaceMyself.moveIndex].equals("6C") &&
    			   !moveEffectCode[pokeStatePlaceMyself.moveIndex].equals("7D")) {
    				movePossibleFlag = false;
    		    	DialogUtil.dialogMessageOutput
    		    		(partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].playerNameDisp+"の"+
    		    		 partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].speciesNameDisp+
    		    		 "は凍っていて動けない");
    		    	return;
    			}
    		}
    	}

		// 怯みにより動けない処理
    	if (pokeStatePlaceMyself.winceFlag) {
    		movePossibleFlag = false;
    		DialogUtil.dialogMessageOutput
    			(pokeStatePlaceMyself.playerNameDisp+"の"+pokeStatePlaceMyself.speciesNameDisp+"はひるんだ");
    		return;
    	}

		// 破壊光線による動けない処理
    	if (pokeStatePlaceMyself.destructionBeemStackRestTrun != 0) {
    		movePossibleFlag = false;
    		DialogUtil.dialogMessageOutput
    			(pokeStatePlaceMyself.playerNameDisp+"の"+pokeStatePlaceMyself.speciesNameDisp+"は反動で動けない");
    		return;
    	}

		// 混乱の実回復・カウントダウン・自滅判定処理
    	if (pokeStateBaton.isConfusion()) {
    		if ((partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].itemName.equals("こおったきのみ") ||
    			partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) &&
    			!badHealthItemRecoveryFlag) {
    			confusionItemRecoveryFlag = true;
    		} else {
    			confusionCountDownFlag = true;
    			if (pokeStateBaton.confusionSelfDestructionJudge() && pokeStateBaton.isNotConfusionRecoveryThisTrun()) {
    				confusionSelfDestructionFlag = true;
    				return;
    			}
    		}
    	}

		// メロメロで動けない判定処理
    	if (pokeStatePlaceMyself.deprivedMindFlag && pokeStatePlaceOpponent.deprivingMindFlag) {
    		if (pokeStatePlaceMyself.deprivingMindJudge()) {
    			movePossibleFlag = false;
    			return;
    		}
    	}

		// 金縛りのカウントダウン・技出せない判定処理
    	if (pokeStatePlaceMyself.isMoveParalysis()) {
    		moveParalysisCountDownFlag = true;
    		if (pokeStatePlaceMyself.moveParalysisJudge()) {
        		movePossibleFlag = false;
        		return;
    		}
    	}

		// 麻痺の実回復・動けない判定処理
    	if (partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].badHealth.equals("paralysis")) {
    		if (partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].itemName.equals("まひなおしのみ") ||
    			partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) {
    			badHealthItemRecoveryFlag = true;
    		} else {
    			if(partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].confusionSelfDestructionJudge()) {
    				movePossibleFlag = false;
    				return;
    			}
    		}
    	}

		// PP消費・技出せない判定処理（コンボ中や悪あがき時は消費・判定しない）
    	if (pokeStatePlaceMyself.isNotBeingCombo() &&
    		(pokeStatePlaceMyself.moveIndex != -1 || pokeStatePlaceMyself.changeTarget != -1) /* &&
    		!partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].struggleFlag*/) {
    		if (movePP[pokeStatePlaceMyself.moveIndex] <= 0) {
				movePossibleFlag = false;
	    		DialogUtil.dialogMessageOutput
    			(partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].playerNameDisp+"の"+
		    	partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].speciesNameDisp+
		    	"はPP不足で技を繰り出せない");
	    		return;
    		} else {
        		PPUseFlag = true;
    		}
    	}

    }

	/* 技繰り出せるか否かの判定の反映メソッド（交代以外の場合に実行）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void canMoveInfluence() throws IOException {

		// 状態異常・混乱を木の実で回復の反映
    	if (badHealthItemRecoveryFlag) {
    		partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].changeItem("持ち物無し");
    		partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].changeBadHealth("none");
    	}
    	if (confusionItemRecoveryFlag) {
    		partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].changeItem("持ち物無し");
    		pokeStateBaton.confusionRecovery();
    	}

		// 混乱自滅の反映
    	if (confusionSelfDestructionFlag) {
    		pokeStateBaton.confusionSelfDestructionSet();
    	}

		// 各種状態のカウントダウンの反映（処理の都合上、コンボターンのカウントダウンは技の繰り出し時に回す）
    	if (sleepCountDownFlag) {
    		partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sleepRecoverCountDown();
    	}
    	if (confusionCountDownFlag) {
    		pokeStateBaton.confusionRestTimeUpDown(-1);
    	}
    	if (moveParalysisCountDownFlag) {
    		pokeStatePlaceMyself.moveParalysisCountDown();
    	}

    	// 眠りを奇跡の実で回復したか否かの状態を反映
    	if (wakeupByMiracleBerryFlag != null) {
    		pokeStatePlaceMyself.wakeupByMiracleBerry(wakeupByMiracleBerryFlag);
    	}

		// PP消費の反映（スケッチ・物真似・変身考慮）
    	if (PPUseFlag) {
    		if (pokeStatePlaceMyself.moveIndex == pokeStatePlaceMyself.mimicMoveIndex) {
    			pokeStatePlaceMyself.mimicPPUpDown(-1);
    		} else if (pokeStatePlaceMyself.moveIndex == partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchMoveIndex) {
        		partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].sketchPPUpDown(-1);
    		} else if (pokeStatePlaceMyself.transformedFlag){
    			pokeStatePlaceMyself.transformedPPUpDown(pokeStatePlaceMyself.moveIndex,-1);
    		} else {
    			partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].PPUpDown(pokeStatePlaceMyself.moveIndex,-1);
    		}
    	}

		// コンボターンリセット、交代待ちフラグ（追い打ち２倍フラグ）下しの反映（繰り出せない場合や混乱自滅の場合のみ）
		// PokeStatePlaceクラス共通処理（繰り出せない場合や混乱自滅の場合を除く）
    	if (!movePossibleFlag || confusionSelfDestructionFlag) {
    		pokeStatePlaceMyself.comboCountReset();
    		pokeStateWhole.changeFailed();
    	} else {
    		// 悪あがきの場合
    		if ((pokeStatePlaceMyself.moveIndex == -1 && pokeStatePlaceMyself.changeTarget == -1) /*||
    		     partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].struggleFlag*/) {
        		pokeStatePlaceMyself.whenMovingCommon(pokeStatePlaceMyself.moveIndex,
                        "わるあがき",
                        "physic",
                        50,
                        100.0,
                        10,
                        0.0,
                        "30b");
        	// 悪あがき以外の技を繰り出した場合（交代含まず）（技無し（実機では空白）は繰り出せないのでここに来れない）
    		} else if (pokeStatePlaceMyself.moveIndex != -1) {
        		pokeStatePlaceMyself.whenMovingCommon(pokeStatePlaceMyself.moveIndex,
                        moveName[pokeStatePlaceMyself.moveIndex],
                        moveType[pokeStatePlaceMyself.moveIndex],
                        movePower[pokeStatePlaceMyself.moveIndex],
                        moveHitProbability[pokeStatePlaceMyself.moveIndex],
                        movePPPrimitive[pokeStatePlaceMyself.moveIndex],
                        additionalEffectProbability[pokeStatePlaceMyself.moveIndex],
                        moveEffectCode[pokeStatePlaceMyself.moveIndex]);
    		}
    	}
    }
}
