package calc;

import java.io.IOException;
import java.util.Random;

import party.PartyInfo;
import state.PokeStateBaton;
import state.PokeStatePlace;
import state.PokeStateWhole;
import util.DialogUtil;

/*
*  ChangeCalcクラス
*
*  交代関連の計算（処理）を行うクラスです。
*  交代直後の処理（撒きびし、破壊の遺伝子）も実装します。
*
*  変更履歴
*  2019/06/30 新規作成
*  2019/07/25 定数ダメージで回復してしまうバグを修正
*  2019/07/25 交代先未設定で呼び出した場合に交代先がランダムで決まらなかったバグを修正
*  2019/07/25 飛行タイプに撒きびしダメージが発生していたバグを修正
*  2019/08/25 対戦開始時に破壊の遺伝子を使用するメソッドを追加
*
*/

public class ChangeCalc extends DamageCalc{

	/* 参照・反映用Stateオブジェクト */
	public PartyInfo partyInfo = null; //PokeStateUnit配列
	public PokeStatePlace pokeStatePlace = null;
	public PokeStateBaton pokeStateBaton = null;
	public PokeStateWhole pokeStateWhole = null;
	public int beforeChangeTarget = -1; //当クラス呼び出し前のコマンド入力前の交代先(changeTarget)
	public int beforeChangeActualHP = 0; //当クラス呼び出し前の現在HP
	public boolean myHandFlag = true; //自分のターンかどうかのフラグ

	/* 計算用一時領域（上書きされる可能性のあるもの） */
	public String type1 = "normal"; //タイプ１
	public String type2 = "none"; //タイプ２

	/* 出力領域 */
	public int changeNecessityNumber = 0; //0なら交代不要、1なら自分のみ交代

	/*
	 * ４通りの交代
	 *
	 * １．通常の交代
	 * 　　PokeStatePlaceクラスのchangeTargetが-1以外　かつ
	 * 　　PartyInfoクラスのactualHPが0より大きい　かつ
	 * 　　自分のターン
	 * 　　　→changeTargetをそのまま交代先にする。
	 * 　　
	 * ２．バトンタッチによる交代
	 * 　　PokeStatePlaceクラスのchangeTargetが-1　かつ
	 * 　　PartyInfoクラスのactualHPが0より大きい　かつ
	 * 　　自分のターン
	 * 　　　→当クラス呼び出し前にコマンド入力させて交代先(changeTarget)をセットする。
	 * 　　　　　※バトンタッチで一部状態を引き継ぐことに注意
	 * 　　
	 * ３．吠える・吹き飛ばしによる強制交代
	 * 　　PokeStatePlaceクラスのchangeTargetは不問　かつ
	 * 　　PartyInfoクラスのactualHPが0より大きい　かつ
	 * 　　相手のターン
	 * 　　　→changeTargetが-1なら当クラスでランダムで交代先(changeTarget)を決める。
	 * 　　　　changeTargetが-1以外ならそれをそのまま交代先にする。
	 * 　　　　　※現在のMoveCalcの実装ではMoveCalc内でchangeTargetをランダムで決定する
	 * 　　
	 * ４．HPが0になることによる交代
	 * 　　PokeStatePlaceクラスのchangeTargetは不問　かつ
	 * 　　PartyInfoクラスのactualHPが0　かつ
	 * 　　自分のターンかどうかは不問
	 * 　　　→当クラス呼び出し前にコマンド入力させて交代先(changeTarget)をセットする。
	 * 　　　　changeTargetが-1以外でもそれを上書きする。
	 *
	 * デシジョンテーブル
	 * 　PokeStatePlaceクラスのchangeTarget =-1  =-1  =-1  =-1  !=-1 !=-1 !=-1 !=-1
	 * 　PartyInfoクラスのactualHP          >0   >0   =0   =0   >0   >0   =0   =0
	 * 　ターン                             自   相   自   相   自   相   自   相
	 * 　--------------------------------------------------------------------------
	 * 　ケース                             ２   ３   ４   ４   １   ３   ４   ４
	 *
	 */

	/* 交代標準メソッド
	 *   引数　　　：自分のPokeStateUnit配列、
	 *   　　　　　：自分のPokeStatePlaceオブジェクト、
	 *   　　　　　：自分のPokeStateBatonオブジェクト、
	 *   　　　　　：自分のPokeStateWholeオブジェクト、
	 *   　　　　　：当クラス呼び出し前のコマンド入力前の交代先(changeTarget)、自分のターンかどうか(boolean)
	 *   戻り値　　：交代要否（int）（0なら交代不要、1なら自分だけ交代）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void changeCalcCommon (PartyInfo partyInfo,
	                               PokeStatePlace pokeStatePlace,
	                               PokeStateBaton pokeStateBaton,
	                               PokeStateWhole pokeStateWhole,
	                               int beforeChangeTarget,boolean myHandFlag) throws IOException {

		// 初期値セット（各種Stateクラス（PokeStateUnitクラスは配列））
		this.partyInfo = partyInfo;
		this.pokeStatePlace = pokeStatePlace;
		this.pokeStateBaton = pokeStateBaton;
		this.pokeStateWhole = pokeStateWhole;
		this.beforeChangeTarget = beforeChangeTarget;
		this.beforeChangeActualHP = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].actualHP;
		this.myHandFlag = myHandFlag;

		// 計算用一時領域・出力領域初期化メソッド呼び出し
		calcOutputAreaInit();

		// 交代先予約メソッド呼び出し
		changeTargetSet();

		if (pokeStatePlace.changeTarget != -1) {

			// PokeStateUnit配列添字変更メソッド呼び出し
			doChange();

			// タイプセットメソッド呼び出し
	    	calcAreaStatusSet();

			// 交代時共通処理メソッド呼び出し
			changeCommonCalc();

		} else {
			DialogUtil.dialogMessageOutput("交代先が指定されていないので交代しません");
		}

		// 交代要否の判定メソッド
		changeNecessitySet();

	}

	/* 対戦開始時用メソッド（破壊の遺伝子使用）
	 *   引数　　　：自分のPokeStateUnit配列、
	 *   　　　　　：自分のPokeStateBatonオブジェクト
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void changeCalcBattleStart(PartyInfo partyInfo,
                                        PokeStateBaton pokeStateBaton) throws IOException {

		// 初期値セット（各種Stateクラス（PokeStateUnitクラスは配列））
		this.partyInfo = partyInfo;
		this.pokeStateBaton = pokeStateBaton;

		// 破壊の遺伝子
		destructionGeneUse();

	}


	/* 計算用一時領域・出力領域初期化メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void calcOutputAreaInit() {
    	type1 = "normal";
    	type2 = "none";
    	changeNecessityNumber = 0;
    }

	/* 交代先予約メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
	public void changeTargetSet() {
		if (pokeStatePlace.changeTarget == -1) {
			int pokeCount = 0;
	    	for(int i = 0; i < 6; i++){
	    		if (partyInfo.pokeStateUnitArray[i] != null) {
	    			if (partyInfo.pokeStateUnitArray[i].actualHP > 0) {
	    				if (i != partyInfo.pokeStateUnitArrayIndex) {
	    					pokeCount++;
	    				}
	    			}
	    		}
	    	}
	    	if (pokeCount > 0) {
    	    	Random random = new Random();
    	    	int forceChangeRandom = random.nextInt(pokeCount);
    	    	int forceChangeBorder = 0;
    	    	for(int i = 0; i < 6; i++){
    	    		if (partyInfo.pokeStateUnitArray[i] != null) {
    	    			if (partyInfo.pokeStateUnitArray[i].actualHP > 0) {
    	    				if (i != partyInfo.pokeStateUnitArrayIndex) {
    	    					if (forceChangeBorder == forceChangeRandom) {
    	    						pokeStatePlace.changeTarget = i;
    	    					}
    	    					forceChangeBorder++;
    	    				}
    	    			}
    	    		}
    	    	}
	    	}
		}
	}

	/* PokeStateUnit配列添字変更メソッド（交代先ポケモンのメッセージ出力も実行）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void doChange() throws IOException {
		DialogUtil.dialogMessageOutput(partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].playerNameDisp +
		                                  "の交代前ポケモン：" +
		                                  partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].speciesNameDisp);
		partyInfo.pokeStateUnitChange(pokeStatePlace.changeTarget);
		DialogUtil.dialogMessageOutput(partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].playerNameDisp +
                                          "の交代後ポケモン：" +
                                          partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].speciesNameDisp);
	}

	/* タイプセットメソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void calcAreaStatusSet() {

    	// タイプをセット（変身・テスクチャーは交代で解け、バトンもできないので、考慮不要）
    	type1 = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].type1;
    	type2 = partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].type2;

    }

	/* 交代時共通処理メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void changeCommonCalc() throws IOException {

		// PokeStatePlaceクラス共通処理
		pokeStatePlace.changeBatonInitCommon(partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].speciesNameDisp);

		// PokeStateBatonクラス共通処理
		boolean batonFlag = false;
		if (beforeChangeTarget == -1 &&
		    beforeChangeActualHP > 0 &&
		    myHandFlag) {
			batonFlag = true;
		}
		pokeStateBaton.changeBatonInitCommon(partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].speciesNameDisp,
		                                     batonFlag);

		// 撒きびしダメージ
		if (pokeStateWhole.spikesFlag) {
			if (!type1.equals("flying") && !type2.equals("flying")) {
				DialogUtil.dialogMessageOutput("撒きびしのダメージ発生");
				int damage = ratioCalc(partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].MAXHP,
                                        1,
                                        8);
				partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].HPUpDown(damage * (-1));
			}
		}

		// 破壊の遺伝子
		destructionGeneUse();

	}

	/* 破壊の遺伝子使用メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void destructionGeneUse() throws IOException {
		if (partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].itemName.equals("はかいのいでんし")) {
			partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].changeItem("持ち物無し");
			pokeStateBaton.attackRankUpDown(2);
			pokeStateBaton.confusionRestTimeUpDown(pokeStateBaton.confusionTimeGenerate());
		}
	}

	/* 交代要否の判定メソッド（倒れたら交代時処理からやり直し）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeNecessitySet() {
    	changeNecessityNumber = 0;
    	if (partyInfo.pokeStateUnitArray[partyInfo.pokeStateUnitArrayIndex].actualHP <= 0) {
    		changeNecessityNumber = 1;
    	}
    }

}
