package calc;

import java.util.Random;

import party.PartyInfo;
import state.PokeStateBaton;
import state.PokeStatePlace;
import state.PokeStateWhole;

/*
*  HandCalcクラス
*
*  先手・後手の判定の計算を行うクラスです。
*
*  変更履歴
*  2019/06/08 新規作成
*  2019/07/15 配列の初期化方法を誤っていたので修正
*  2019/07/22 交代選択時に当クラスを使用すると異常終了するバグを修正
*  2019/07/27 「技無し」の技コードを「XXc」に統一
*  2019/08/01 P2の優先度計算が誤っていたバグを修正
*  2020/09/12 スケッチを実装
*
*/

public class HandCalc extends DamageCalc {

	/* 参照・反映用Stateオブジェクト */
	public PartyInfo partyInfoP1 = null; //PokeStateUnit配列
	public PartyInfo partyInfoP2 = null; //PokeStateUnit配列
	public PokeStatePlace pokeStatePlaceP1 = null;
	public PokeStatePlace pokeStatePlaceP2 = null;
	public PokeStateBaton pokeStateBatonP1 = null;
	public PokeStateBaton pokeStateBatonP2 = null;
	public PokeStateWhole pokeStateWholeP1 = null;
	public PokeStateWhole pokeStateWholeP2 = null;

	/* 計算用一時領域 */
	public String[] moveEffectCodeP1 = {"XXc","XXc","XXc","XXc"}; //技の効果コード（物真似・変身考慮）
	public String[] moveEffectCodeP2 = {"XXc","XXc","XXc","XXc"}; //技の効果コード（物真似・変身考慮）
	public int speedP1 = 1; //素早さ（変身考慮）
	public int speedP2 = 1; //素早さ（変身考慮）
	public int priorityDegreeP1 = 0; //優先度
	public int priorityDegreeP2 = 0; //優先度

	/* 出力領域 */
	public int firstHand = 1; //1ならプレイヤー１が先手、2ならプレイヤー２が先手

	/* 先手・後手判定標準メソッド
	 *   引数　　　：プレイヤー１のPokeStateUnit配列、プレイヤー２のPokeStateUnit配列、
	 *   　　　　　：プレイヤー１のPokeStatePlaceオブジェクト、プレイヤー２のPokeStatePlaceオブジェクト、
	 *   　　　　　：プレイヤー１のPokeStateBatonオブジェクト、プレイヤー２のPokeStateBatonオブジェクト、
	 *   　　　　　：プレイヤー１のPokeStateWholeオブジェクト、プレイヤー２のPokeStateWholeオブジェクト
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void handCalcCommon(PartyInfo partyInfoP1,PartyInfo partyInfoP2,
    		                   PokeStatePlace pokeStatePlaceP1,PokeStatePlace pokeStatePlaceP2,
    		                   PokeStateBaton pokeStateBatonP1,PokeStateBaton pokeStateBatonP2,
    		                   PokeStateWhole pokeStateWholeP1,PokeStateWhole pokeStateWholeP2) {

		// 初期値セット（各種Stateクラス（PokeStateUnitクラスは配列））
    	this.partyInfoP1 = partyInfoP1;
    	this.partyInfoP2 = partyInfoP2;
    	this.pokeStatePlaceP1 = pokeStatePlaceP1;
    	this.pokeStatePlaceP2 = pokeStatePlaceP2;
    	this.pokeStateBatonP1 = pokeStateBatonP1;
    	this.pokeStateBatonP2 = pokeStateBatonP2;
    	this.pokeStateWholeP1 = pokeStateWholeP1;
    	this.pokeStateWholeP2 = pokeStateWholeP2;

		// 計算用一時領域初期化メソッド呼び出し
    	calcOutputAreaInit();

		// スケッチ・物真似・変身・指を振る・オウム返しによるステータス上書きメソッド呼び出し
    	calcAreaSet();

		// 自爆・大爆発による防御半減メソッド呼び出し
    	explosionBlockHalf();

		// 優先度判定メソッド呼び出し
    	priorityDegreeJudge();

		// 素早さ算出メソッド呼び出し
    	speedCalc();

		// 先手後手判定メソッド呼び出し
    	handJudge();

    	// 交代待ちフラグ（追い打ち２倍）セットメソッド呼び出し
    	waitingChangeSet();

    }

	/* 計算用一時領域・出力領域初期化メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void calcOutputAreaInit() {
    	moveEffectCodeP1 = null;
    	moveEffectCodeP1 = new String[4];
    	moveEffectCodeP2 = null;
    	moveEffectCodeP2 = new String[4];
    	for(int i = 0; i < 4; i++){
    		moveEffectCodeP1[i] = "XXc";
    		moveEffectCodeP2[i] = "XXc";
    	}
    	speedP1 = 1;
    	speedP2 = 1;
    	priorityDegreeP1 = 0;
    	priorityDegreeP2 = 0;
    	firstHand = 1;
    }

	/* 物真似・変身・指を振る・オウム返しによるステータス上書きメソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void calcAreaSet() {

    	// 物真似・変身を考慮しない場合のステータスをセット
    	moveEffectCodeP1 = partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].moveEffectCode;
    	moveEffectCodeP2 = partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].moveEffectCode;
    	speedP1 = partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].speed;
    	speedP2 = partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].speed;

    	// 変身を考慮した場合のステータスをセット
    	if (pokeStatePlaceP1.transformedFlag) {
    		moveEffectCodeP1 = pokeStatePlaceP1.transformedMoveEffectCode;
    		speedP1 = pokeStatePlaceP1.transformedSpeed;
    	}
    	if (pokeStatePlaceP2.transformedFlag) {
    		moveEffectCodeP2 = pokeStatePlaceP2.transformedMoveEffectCode;
    		speedP2 = pokeStatePlaceP2.transformedSpeed;
    	}

    	// スケッチを考慮した場合のステータスをセット
    	if (partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].sketchMoveIndex >= 0 && partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].sketchMoveIndex <= 3) {
    		moveEffectCodeP1[partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].sketchMoveEffectCode;
    	}
    	if (partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].sketchMoveIndex >= 0 && partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].sketchMoveIndex <= 3) {
    		moveEffectCodeP2[partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].sketchMoveEffectCode;
    	}

    	// 物真似を考慮した場合のステータスをセット
    	if (pokeStatePlaceP1.mimicMoveIndex >= 0 && pokeStatePlaceP1.mimicMoveIndex <= 3) {
    		moveEffectCodeP1[pokeStatePlaceP1.mimicMoveIndex] = pokeStatePlaceP1.mimicMoveEffectCode;
    	}
    	if (pokeStatePlaceP2.mimicMoveIndex >= 0 && pokeStatePlaceP2.mimicMoveIndex <= 3) {
    		moveEffectCodeP2[pokeStatePlaceP2.mimicMoveIndex] = pokeStatePlaceP2.mimicMoveEffectCode;
    	}

    	// 指を振る・オウム返しを考慮した場合のステータスをセット
    	if (pokeStatePlaceP1.moveCopyMoveFlag) {
    		moveEffectCodeP1[pokeStatePlaceP1.moveIndex] = pokeStatePlaceP1.latestUsedMoveEffectCodeMoveCopy;
    	}
    	if (pokeStatePlaceP2.moveCopyMoveFlag) {
    		moveEffectCodeP2[pokeStatePlaceP2.moveIndex] = pokeStatePlaceP2.latestUsedMoveEffectCodeMoveCopy;
    	}

    }

	/* 自爆・大爆発による防御半減メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void explosionBlockHalf() {
    	if (pokeStatePlaceP1.moveIndex >= 0 && pokeStatePlaceP1.moveIndex <= 3) {
        	if (moveEffectCodeP1[pokeStatePlaceP1.moveIndex].equals("07")) {
        		pokeStateWholeP2.explosionBlockHalf();
        	}
    	}
    	if (pokeStatePlaceP2.moveIndex >= 0 && pokeStatePlaceP2.moveIndex <= 3) {
        	if (moveEffectCodeP2[pokeStatePlaceP2.moveIndex].equals("07")) {
        		pokeStateWholeP1.explosionBlockHalf();
        	}
    	}
    }

	/* 優先度判定・交代待ちフラグON（追い打ち２倍）メソッド（交代は優先度＋３、相手交代時追い打ちは優先度＋４と仮定）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void priorityDegreeJudge() {

    	// 技の優先度判定
    	if (pokeStatePlaceP1.moveIndex >= 0 && pokeStatePlaceP1.moveIndex <= 3) {
    		switch (moveEffectCodeP1[pokeStatePlaceP1.moveIndex]) {
    		case "11b": // 当て身投げ
    		case "59": // カウンター
    		case "90": // ミラーコート
    		case "1Cb": // 吠える
    		case "1Ca": // 吹き飛ばし
    			priorityDegreeP1 = -1;
    			break;
    		case "67": // 先制技
    			priorityDegreeP1 = 1;
    			break;
    		case "6F": // 守る系
    		case "74": // 堪える
    			priorityDegreeP1 = 2;
    			break;
    		default:
    			break;
    		}
    	}
    	if (pokeStatePlaceP2.moveIndex >= 0 && pokeStatePlaceP2.moveIndex <= 3) {
    		switch (moveEffectCodeP2[pokeStatePlaceP2.moveIndex]) {
    		case "11b": // 当て身投げ
    		case "59": // カウンター
    		case "90": // ミラーコート
    		case "1Cb": // 吠える
    		case "1Ca": // 吹き飛ばし
    			priorityDegreeP2 = -1;
    			break;
    		case "67": // 先制技
    			priorityDegreeP2 = 1;
    			break;
    		case "6F": // 守る系
    		case "74": // 堪える
    			priorityDegreeP2 = 2;
    			break;
    		default:
    			break;
    		}
    	}

    	// 交代
    	if(pokeStatePlaceP1.moveIndex == -1 && pokeStatePlaceP1.changeTarget != -1) {
    		priorityDegreeP1 = 3;
    	}
    	if(pokeStatePlaceP2.moveIndex == -1 && pokeStatePlaceP2.changeTarget != -1) {
    		priorityDegreeP2 = 3;
    	}

    	// 交代時追い打ち
    	if (pokeStatePlaceP1.moveIndex >= 0 && pokeStatePlaceP1.moveIndex <= 3) {
    		if (moveEffectCodeP1[pokeStatePlaceP1.moveIndex].equals("80") && priorityDegreeP2 == 3) {
    			priorityDegreeP1 = 4;
    		}
    	}
    	if (pokeStatePlaceP2.moveIndex >= 0 && pokeStatePlaceP2.moveIndex <= 3) {
    		if (moveEffectCodeP2[pokeStatePlaceP2.moveIndex].equals("80") && priorityDegreeP1 == 3) {
    			priorityDegreeP2 = 4;
    		}
    	}

    }

	/* 素早さ算出メソッド（先制の爪発動は素早さ65535と仮定）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void speedCalc() {
    	speedP1 = (int) ABCDSRankCalc(speedP1,pokeStateBatonP1.speedRank);
    	speedP2 = (int) ABCDSRankCalc(speedP2,pokeStateBatonP2.speedRank);
    	if (partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].badHealth.equals("paralysis")) {
    		speedP1 = ratioCalc(speedP1,1,4);
    	}
    	if (partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].badHealth.equals("paralysis")) {
    		speedP2 = ratioCalc(speedP2,1,4);
    	}
    	if (partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].itemName.equals("せんせいのツメ")) {
        	Random randomP1 = new Random();
        	int clawRandom = randomP1.nextInt(10000);
        	int clawBoarder = (int) Math.floor(23.4 * 100.0);
        	if (clawRandom < clawBoarder) {
        		speedP1 = 65535;
        	}
    	}
    	if (partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].itemName.equals("せんせいのツメ")) {
        	Random randomP2 = new Random();
        	int clawRandom = randomP2.nextInt(10000);
        	int clawBoarder = (int) Math.floor(23.4 * 100.0);
        	if (clawRandom < clawBoarder) {
        		speedP2 = 65535;
        	}
    	}
    }

	/* 先手後手判定メソッド（同じ優先度同じ素早さなら半々の確率で決める）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void handJudge() {
    	if (priorityDegreeP1 > priorityDegreeP2) {
    		firstHand = 1;
    	} else if (priorityDegreeP2 > priorityDegreeP1) {
    		firstHand = 2;
    	} else if (speedP1 > speedP2) {
    		firstHand = 1;
    	} else if (speedP2 > speedP1) {
    		firstHand = 2;
    	} else {
    		Random random = new Random();
        	int handRandom = random.nextInt(10000);
        	int handBoarder = (int) Math.floor(50.0 * 100.0);
        	if (handRandom < handBoarder) {
        		firstHand = 1;
        	} else {
        		firstHand = 2;
        	}
    	}
    }

	/* 交代待ちフラグ（追い打ち２倍）セットメソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void waitingChangeSet() {

    	// 交代に対して交代待ちフラグを立てる
    	if (priorityDegreeP1 == 3) {
    		pokeStateWholeP1.waitingChange();
    	}
    	if (priorityDegreeP2 == 3) {
    		pokeStateWholeP2.waitingChange();
    	}

    	// 先制のバトンタッチに対して交代待ちフラグを立てる（PBSバグ再現）
    	if (pokeStatePlaceP1.moveIndex >= 0 && pokeStatePlaceP1.moveIndex <= 4) {
    		if (firstHand == 1 && moveEffectCodeP1[pokeStatePlaceP1.moveIndex].equals("7F")) {
    			pokeStateWholeP1.waitingChange();
    		}
    	}
    	if (pokeStatePlaceP2.moveIndex >= 0 && pokeStatePlaceP2.moveIndex <= 4) {
    		if (firstHand == 2 && moveEffectCodeP2[pokeStatePlaceP2.moveIndex].equals("7F")) {
    			pokeStateWholeP2.waitingChange();
    		}
    	}

    }

}
