package calc;

import java.io.IOException;
import java.util.Random;

import party.PartyInfo;
import state.PlaceState;
import state.PokeStateBaton;
import state.PokeStatePlace;
import state.PokeStateWhole;
import util.DialogUtil;

/*
*  TurnEndCalcクラス
*
*  ターン終了時の計算（処理）を行うクラスです。
*
*  変更履歴
*  2019/06/30 新規作成
*  2019/07/08 技無しに対して不思議な木の実が発動しないように修正
*  2019/07/15 アンコールされていない時はアンコールのカウントダウンをしないように修正
*  2019/07/15 HP満タンの時に食べ残し回復しないように修正（メッセージの問題だが、HP満タンでも食べ残し持ちがバレるので）
*  2019/07/25 定数ダメージで回復してしまうバグを修正
*  2019/07/25 砂嵐のダメージが地面・岩・鋼ポケモンのみに発生するようになっていたのを正しい挙動に修正
*  2019/08/13 トリプルキック倍率リセットを追加（PBSトリプルキックバグ再現）
*  2019/11/07 状態異常や混乱を各種木の実で直す際、状態異常と混乱が重なると状態異常しか治らないバグを修正
*  2020/08/28 親クラスの仕様変更に合わせてソース修正
*  2020/09/12 奇跡の実で眠りから回復した時に悪夢状態を回復しないように修正
*  2020/09/12 スケッチを実装
*  2021/01/23 ルーチンでの参照禁止変数をprivate化したことに伴う修正
*
*/

public class TurnEndCalc extends DamageCalc {

	/* 参照・反映用Stateオブジェクト */
	public PartyInfo partyInfoMyself = null; //PokeStateUnit配列
	public PokeStatePlace pokeStatePlaceMyself = null;
	public PokeStatePlace pokeStatePlaceOpponent = null;
	public PokeStateBaton pokeStateBatonMyself = null;
	public PokeStateWhole pokeStateWholeMyself = null;
	public PlaceState placeState = null;

	/* 計算用一時領域（上書きされる可能性のあるもの） */
	public String type1 = "normal"; //タイプ１
	public String type2 = "none"; //タイプ２

	/* 反映用一時領域 */
	public int predictionDamage = 0; //未来予知のダメージ（気合いのハチマキ考慮）
	public boolean freezeRecoverFlag = false; // 凍りが自然解凍するかどうか

	/* 出力領域 */
	public int changeNecessityNumber = 0; //0なら交代不要、1なら自分のみ交代

	/* ターン終了処理標準メソッド
	 *   引数　　　：自分のPokeStateUnit配列、
	 *   　　　　　：自分のPokeStatePlaceオブジェクト、相手のPokeStatePlaceオブジェクト、
	 *   　　　　　：自分のPokeStateBatonオブジェクト、
	 *   　　　　　：自分のPokeStateWholeオブジェクト、
	 *   　　　　　：PlaceStateオブジェクト
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public void turnEndCalcCommon (PartyInfo partyInfoMyself,
			                         PokeStatePlace pokeStatePlaceMyself,PokeStatePlace pokeStatePlaceOpponent,
			                         PokeStateBaton pokeStateBatonMyself,
			                         PokeStateWhole pokeStateWholeMyself,
			                         PlaceState placeState) throws IOException {

		// 初期値セット（各種Stateクラス（PokeStateUnitクラスは配列））
		this.partyInfoMyself = partyInfoMyself;
		this.pokeStatePlaceMyself = pokeStatePlaceMyself;
		this.pokeStatePlaceOpponent = pokeStatePlaceOpponent;
		this.pokeStateBatonMyself = pokeStateBatonMyself;
		this.pokeStateWholeMyself = pokeStateWholeMyself;
		this.placeState = placeState;

		// 計算用一時領域・反映用一時領域・出力領域初期化メソッド呼び出し
		calcTmpOutputAreaInit();

		// スケッチ・物真似・変身・テクスチャーによるステータス上書きメソッド呼び出し
    	calcAreaStatusSet();

		// 未来予知ダメージ計算メソッド呼び出し（気合いのハチマキ考慮）
		predictionDamageCalc();

		// 凍り自然治癒判定メソッド呼び出し
		freezeRecoverJudge();

		// 未来予知カウントダウン・ダメージ反映・技命中時共通処理メソッド
		predictionCalc();

		// 天候切れ判定メソッド呼び出し
		weatherEndJudge();

		// 砂嵐ダメージ計算メソッド呼び出し
		dustyDamageCalc();

		// 暴れる等解除時の混乱判定メソッド呼び出し
		comboEndConfusionJudge();

		// 滅びの歌カウントダウン・判定メソッド呼び出し
		ruinJudge();

		// 締め付けるダメージ・カウントダウンメソッド
		twindCalc();

		if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP > 0) {
			// 食べ残し回復メソッド呼び出し
			leftoverCalc();

			// 状態異常や混乱を木の実で回復判定メソッド呼び出し
			healthRecoverCalc();

			// 凍り自然治癒反映メソッド呼び出し
			freezeRecoverCalc();

			// 壁カウントダウンメソッド呼び出し
			wallCountDown();

			// 木の実・木の実ジュース・黄金の実・不思議な木の実回復メソッド呼び出し
			nutsRecoverCalc();

			// アンコールカウントダウンメソッド呼び出し
			encoredCountDown();

			// PokeStatePlaceクラス共通処理呼び出し
			pokeStatePlaceCommon();

			// PokeStateBatonクラス共通処理呼び出し
			pokeStateBatonCommon();

			// PokeStateWholeクラス共通処理呼び出し
			pokeStateWholeCommon();

			// PlaceStateクラス共通処理呼び出し
			placeStateCommon();

		}

		// 交代要否の判定メソッド呼び出し
		changeNecessitySet();

	}

	/* 計算用一時領域・反映用一時領域・出力領域初期化メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void calcTmpOutputAreaInit() {
    	type1 = "normal";
    	type2 = "none";
    	predictionDamage = 0;
    	freezeRecoverFlag = false;
    	changeNecessityNumber = 0;
    }

	/* 変身・テクスチャー・物真似によるステータス上書きメソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void calcAreaStatusSet() {

    	// 物真似・変身を考慮しない場合のステータスをセット
    	type1 = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].type1;
    	type2 = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].type2;

    	// 変身を考慮した場合のステータスをセット
    	if (pokeStatePlaceMyself.transformedFlag) {
    		type1 = pokeStatePlaceMyself.transformedType1;
    		type2 = pokeStatePlaceMyself.transformedType2;
    	}

    	// テクスチャーを考慮した場合のステータスをセット
    	if (pokeStatePlaceMyself.pastedFlag) {
    		type1 = pokeStatePlaceMyself.pastedType1;
    		type2 = pokeStatePlaceMyself.pastedType2;
    	}

    }

	/* 未来予知ダメージ計算（気合いのハチマキ考慮）メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void predictionDamageCalc() throws IOException {
    	if (pokeStateWholeMyself.predictionEndTrun == placeState.currentTrun &&
    	    pokeStateWholeMyself.predictionDamage > 0) {
    		predictionDamage = predictionFinalDamageCalc(pokeStateWholeMyself.predictionDamage,
    		                                             partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP,
    		                                             spiritTriggerCalc(pokeStateWholeMyself.predictionDamage,
    		                                            		           partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP,
    						    			                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName));
    	}
    }

	/* 凍り自然治癒判定メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void freezeRecoverJudge() {
    	if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("freeze")) {
        	Random random = new Random();
        	int freezeRecoverRandom = random.nextInt(10);
        	if (freezeRecoverRandom < 1) {
        		freezeRecoverFlag = true;
        	}
    	}
    }

	/* 未来予知カウントダウン・ダメージ反映・技命中時共通処理メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void predictionCalc() throws IOException {
    	int predictionHit = 0;
    	predictionHit = pokeStateWholeMyself.predictionCountDown(placeState.currentTrun);
    	if (predictionHit > 0) {
    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(predictionDamage * (-1));
    		pokeStatePlaceMyself.whenGetPredictionCommon(predictionDamage * (-1));
    	}
    }

	/* 天候切れ判定メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void weatherEndJudge() throws IOException {
    	placeState.weatherCountDown();
    }

	/* 砂嵐ダメージ計算メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void dustyDamageCalc() throws IOException {
    	if (placeState.weather.equals("dusty")) {
    		if (!type1.equals("ground") &&
    		    !type1.equals("rock") &&
    		    !type1.equals("steel") &&
    		    !type2.equals("ground") &&
    		    !type2.equals("rock") &&
    		    !type2.equals("steel")) {
				DialogUtil.dialogMessageOutput("砂嵐のダメージ発生");
    			int damage = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP,
	                                    1,
	                                    8);
    			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(damage * (-1));
    		}
    	}
    }

	/* 暴れる等解除時の混乱判定メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void comboEndConfusionJudge() throws IOException {
    	boolean comboEndConfusionFlag = pokeStatePlaceMyself.comboEndConfusionJudge();
    	if (comboEndConfusionFlag) {
    		pokeStateBatonMyself.confusionRestTimeUpDown(pokeStateBatonMyself.confusionTimeGenerate());
    	}
    }

	/* 滅びの歌カウントダウン・判定メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void ruinJudge() throws IOException {
    	int ruinFlag = pokeStateBatonMyself.ruinCountDown();
    	if (ruinFlag == 1) {
    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP * (-1));
    	}
    }

	/* 締め付けるダメージ・カウントダウンメソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void twindCalc() throws IOException {
    	boolean twindDamageFlag = pokeStatePlaceMyself.twindCountDown(pokeStatePlaceOpponent.getTwiningRestTrun());
    	if (twindDamageFlag) {
			DialogUtil.dialogMessageOutput("締め付けるのダメージ発生");
			int damage = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP,
                                     1,
                                     8);
			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(damage * (-1));
    	}
    }

	/* 食べ残し回復メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void leftoverCalc() throws IOException {
    	if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("たべのこし") &&
    	    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP < partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP) {
    		DialogUtil.dialogMessageOutput("食べ残しによるダメージ回復");
			int recover = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP,
                            1,
                            16);
			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(recover);
    	}
    }

	/* 状態異常や混乱を木の実で回復判定メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void healthRecoverCalc() throws IOException {
    	boolean itemUseFlag = false;
    	switch (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth) {
    	case "burn":
    		if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("こおったきのみ") ||
    		    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) {
    			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeBadHealth("none");
    			itemUseFlag = true;
    		}
    		break;
    	case "freeze":
    		if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("やけたきのみ") ||
    		    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) {
    			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeBadHealth("none");
    			itemUseFlag = true;
    		}
    		break;
    	case "paralysis":
    		if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("まひなおしのみ") ||
    		    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) {
    			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeBadHealth("none");
    			itemUseFlag = true;
    		}
    		break;
    	case "poison":
    		if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("どくけしのみ") ||
    		    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) {
    			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeBadHealth("none");
    			itemUseFlag = true;
    		}
    		break;
    	case "sleep":
    		if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("はっかのみ") ||
    		    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) {
    			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeBadHealth("none");
    			itemUseFlag = true;
    			if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) {
    				pokeStatePlaceMyself.wakeupByMiracleBerry(true);
    			} else {
    				pokeStatePlaceMyself.wakeupByMiracleBerry(false);
    			}
    		}
    		break;
    	}
    	if (pokeStateBatonMyself.isConfusion()) {
    		if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("こおったきのみ") ||
        	    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("きせきのみ")) {
    			pokeStateBatonMyself.confusionRecovery();
    			itemUseFlag = true;
    		}
    	}
    	if (itemUseFlag) {
    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeItem("持ち物無し");
    	}
    }

	/* 凍り自然治癒反映メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void freezeRecoverCalc() throws IOException {
    	if (freezeRecoverFlag) {
    		DialogUtil.dialogMessageOutput("自然解凍");
    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeBadHealth("none");
    	}
    }

	/* 壁カウントダウンメソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void wallCountDown() throws IOException {
    	pokeStateWholeMyself.wallCountDown();
    }

	/* 木の実・木の実ジュース・黄金の実・不思議な木の実回復メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void nutsRecoverCalc() throws IOException {

    	// 木の実・木の実ジュース・黄金の実
		int boarder = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP,
                                  1,
                                  2);
		if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP < boarder) {
			if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("きのみ")) {
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeItem("持ち物無し");
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(10);
			} else if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("きのみジュース")) {
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeItem("持ち物無し");
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(20);
			} else if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうごんのみ")) {
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeItem("持ち物無し");
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(30);
			}
		}

		// 不思議な木の実
		for (int i = 0 ; i < 4 ; i++){
			if (pokeStatePlaceMyself.mimicMoveIndex == i) {
				if (pokeStatePlaceMyself.mimicMovePP == 0 && !pokeStatePlaceMyself.mimicMoveName.equals("技無し")) {
					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("ふしぎなきのみ")) {
						partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeItem("持ち物無し");
						pokeStatePlaceMyself.mimicPPUpDown(5);
					}
				}
			} else if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex == i) {
				if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMovePP == 0 && !partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveName.equals("技無し")) {
					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("ふしぎなきのみ")) {
						partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeItem("持ち物無し");
						partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchPPUpDown(5);
					}
				}
			} else if (pokeStatePlaceMyself.transformedFlag){
				if (pokeStatePlaceMyself.transformedMovePP[i] == 0 && !pokeStatePlaceMyself.transformedMoveName[i].equals("技無し")) {
					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("ふしぎなきのみ")) {
						partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeItem("持ち物無し");
						pokeStatePlaceMyself.transformedPPUpDown(i,5);
					}
				}
			} else {
				if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].movePP[i] == 0 &&
				    !partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].moveName[i].equals("技無し")) {
					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("ふしぎなきのみ")) {
						partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeItem("持ち物無し");
						partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].PPUpDown(i,5);
					}
				}
			}
		}

    }

	/* アンコールカウントダウンメソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void encoredCountDown() throws IOException {
    	if (pokeStatePlaceMyself.isEncored()) {
    		pokeStatePlaceMyself.encoredCountDown(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].movePP[pokeStatePlaceMyself.encoredMoveIndex]);
    	}
    }

	/* PokeStatePlaceクラス共通処理
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void pokeStatePlaceCommon() {
    	pokeStatePlaceMyself.turnEndCommon();
    }

	/* PokeStateBatonクラス共通処理
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void pokeStateBatonCommon() {
    	boolean poisonFlag = false;
    	if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("poison") ||
    	    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("burn")) {
    		poisonFlag = true;
    	}
    	pokeStateBatonMyself.turnEndCommon(poisonFlag);
    }

	/* PokeStateWholeクラス共通処理
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void pokeStateWholeCommon() {
    	pokeStateWholeMyself.turnEndCommon();
    }

	/* PlaceStateクラス共通処理
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void placeStateCommon() {
    	placeState.turnEndCommon();
    }

	/* 戻り値（交代要否）の判定メソッド呼び出し（倒れたら技繰り出し後処理・ターン終了処理をやり直し）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeNecessitySet() {
    	changeNecessityNumber = 0;
    	if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP <= 0) {
    		changeNecessityNumber = 1;
    	}
    }

}
