package calc;

import java.io.IOException;

import state.PlaceState;
import state.PokeStatePlace;

/*
*  TurnStartCalcクラス
*
*  ターン開始時の計算（処理）を行うクラスです。
*
*  変更履歴
*  2019/05/06 新規作成
*  2019/06/22 締め付けターンの補正の計算を追加
*  2021/01/23 ルーチンでの参照禁止変数をprivate化したことに伴う修正
*
*/

public class TurnStartCalc {

	/* 参照・反映用Stateオブジェクト */
	public PokeStatePlace pokeStatePlaceP1 = null;
	public PokeStatePlace pokeStatePlaceP2 = null;
	public PlaceState placeState = null;

	/* ターン開始時標準メソッド
	 *   引数　　　：プレイヤー１のPokeStatePlaceオブジェクト、プレイヤー２のPokeStatePlaceオブジェクト、
	 *   　　　　　：PlaceStateオブジェクト
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void turnStartCalcCommon(PokeStatePlace pokeStatePlaceP1,PokeStatePlace pokeStatePlaceP2,
                                      PlaceState placeState) throws IOException{

    	// 初期値セット（各種Stateクラス）
    	this.pokeStatePlaceP1 = pokeStatePlaceP1;
    	this.pokeStatePlaceP2 = pokeStatePlaceP2;
    	this.placeState = placeState;

    	// ターン数カウントメソッド呼び出し
    	turnCountUp();

    	// 締め付けターンの補正メソッド呼び出し
    	twiningCountAmend();

    }

	/* ターン数カウントメソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void turnCountUp() throws IOException{
    	placeState.turnCountUp();
    }

	/* 締め付けターンの補正メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void twiningCountAmend() {
    	pokeStatePlaceP1.twiningCountDown(pokeStatePlaceP2.getTwindRestTrun());
    	pokeStatePlaceP2.twiningCountDown(pokeStatePlaceP1.getTwindRestTrun());
    }

}

