package calc;

import party.PartyInfo;
import state.PokeStateBaton;

/*
*  WinningJudgeCalcクラス
*
*  勝敗の判定の計算を行うクラスです。
*
*  変更履歴
*  2019/06/30 新規作成
*  2019/07/15 勝敗判定タイミングを判定するメソッドを追加
*  2019/08/05 残りHP割合の計算式に誤りがあったので訂正
*
*/

public class WinningJudgeCalc {

	/* 参照・反映用Stateオブジェクト */
	public PartyInfo partyInfoP1 = null; //PokeStateUnit配列
	public PartyInfo partyInfoP2 = null; //PokeStateUnit配列
	public PokeStateBaton pokeStateBatonP1 = null;
	public PokeStateBaton pokeStateBatonP2 = null;

	/* 計算用一時領域 */
	public int remainPokeP1 = 0;
	public int remainPokeP2 = 0;
	public double actualHPP1 = 0.0;
	public double MAXHPP1 = 0.0;
	public double sumHPRatioP1 = 0.0;
	public double actualHPP2 = 0.0;
	public double MAXHPP2 = 0.0;
	public double sumHPRatioP2 = 0.0;
	public boolean foulFlagP1 = false;
	public boolean foulFlagP2 = false;

	/* 出力領域 */
	public int winner = 0; //0なら引き分け、1ならプレイヤー１の勝利、2ならプレイヤー２の勝利

	/* 勝敗判定タイミング判定メソッド（ターン開始時（ターンカウント直後）、及びHPが0になり得るタイミングで呼び出し）
	 *   引数　　　：プレイヤー１のPokeStateUnit配列、プレイヤー２のPokeStateUnit配列、
	 *   　　　　　：現在ターン(int)、制限ターン数(int)
	 *   戻り値　　：勝敗判定タイミングか否か（boolean）
	 *   例外　　　：無し
	 */
	public boolean winningJudgeTimingJudge (PartyInfo partyInfoP1,PartyInfo partyInfoP2,
                                              int currentTrun,int endTurn) {

		boolean endFlag = false;

		// 全てのポケモンのHPが0なら勝敗判定タイミング
		int pokeCountP1 = 0;
		int pokeCountP2 = 0;
    	for(int i = 0; i < 6; i++){
    		if (partyInfoP1.pokeStateUnitArray[i] != null) {
    			if (partyInfoP1.pokeStateUnitArray[i].actualHP > 0) {
    				pokeCountP1++;
    			}
    		}
    	}
    	for(int i = 0; i < 6; i++){
    		if (partyInfoP2.pokeStateUnitArray[i] != null) {
    			if (partyInfoP2.pokeStateUnitArray[i].actualHP > 0) {
    				pokeCountP2++;
    			}
    		}
    	}
    	if (pokeCountP1 == 0 || pokeCountP2 == 0) {
    		endFlag = true;
    	}

    	// ターン制限を過ぎていたら勝敗判定タイミング
    	if (currentTrun > endTurn) {
    		endFlag = true;
    	}

    	// 処理終了
    	return endFlag;

	}

	/* 勝敗判定標準メソッド
	 *   引数　　　：プレイヤー１のPokeStateUnit配列、プレイヤー２のPokeStateUnit配列、
	 *   　　　　　：プレイヤー１のPokeStateBatonオブジェクト、プレイヤー２のPokeStateBatonオブジェクト
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
	public void winningJudgeCommon (PartyInfo partyInfoP1,PartyInfo partyInfoP2,
	                                  PokeStateBaton pokeStateBatonP1,PokeStateBaton pokeStateBatonP2) {

		// 初期値セット（各種Stateクラス（PokeStateUnitクラスは配列））
		this.partyInfoP1 = partyInfoP1;
		this.partyInfoP2 = partyInfoP2;
		this.pokeStateBatonP1 = pokeStateBatonP1;
		this.pokeStateBatonP2 = pokeStateBatonP2;

		// 計算用一時領域・出力領域初期化メソッド呼び出し
		tmpOutputAreaInit();

		// 残りポケモン数・HP割合判定メソッド呼び出し
		remainPokeHPSet();

		// 反則判定メソッド呼び出し
		faulFlagSet();

		// 勝敗の判定メソッド呼び出し
		winningJudge();

	}

	/* 出力領域初期化メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void tmpOutputAreaInit() {
    	remainPokeP1 = 0;
    	remainPokeP2 = 0;
    	actualHPP1 = 0.0;
    	MAXHPP1 = 0.0;
    	sumHPRatioP1 = 0.0;
    	actualHPP2 = 0.0;
    	MAXHPP2 = 0.0;
    	sumHPRatioP2 = 0.0;
    	foulFlagP1 = false;
    	foulFlagP2 = false;
    	winner = 0;
    }

	/* 残りポケモン数・HP割合判定メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void remainPokeHPSet() {

    	for(int i = 0; i < 6; i++){
    		if (partyInfoP1.pokeStateUnitArray[i] != null) {
    			if (partyInfoP1.pokeStateUnitArray[i].actualHP > 0) {
    				remainPokeP1++;
    				actualHPP1 = (double) partyInfoP1.pokeStateUnitArray[i].actualHP;
    				MAXHPP1 = (double) partyInfoP1.pokeStateUnitArray[i].MAXHP;
    				sumHPRatioP1 = sumHPRatioP1 + (actualHPP1 / MAXHPP1);
    			}
    		}
    	}

    	for(int i = 0; i < 6; i++){
    		if (partyInfoP2.pokeStateUnitArray[i] != null) {
    			if (partyInfoP2.pokeStateUnitArray[i].actualHP > 0) {
    				remainPokeP2++;
    				actualHPP2 = (double) partyInfoP2.pokeStateUnitArray[i].actualHP;
    				MAXHPP2 = (double) partyInfoP2.pokeStateUnitArray[i].MAXHP;
    				sumHPRatioP2 = sumHPRatioP2 + (actualHPP2 / MAXHPP2);
    			}
    		}
    	}

    }

	/* 反則判定メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void faulFlagSet() {
    	foulFlagP1 = pokeStateBatonP1.foulFlag;
    	foulFlagP2 = pokeStateBatonP2.foulFlag;
    }

	/* 勝敗の判定メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void winningJudge() {
    	if (remainPokeP1 > remainPokeP2) {
    		winner = 1;
    	} else if (remainPokeP1 < remainPokeP2) {
    		winner = 2;
    	} else {
    		if (sumHPRatioP1 > sumHPRatioP2) {
    			winner = 1;
    		} else if (sumHPRatioP1 < sumHPRatioP2) {
    			winner = 2;
    		} else {
    			if (!foulFlagP1 && foulFlagP2) {
    				winner = 1;
    			} else if (foulFlagP1 && !foulFlagP2) {
    				winner = 2;
    			} else {
    				winner = 0;
    			}
    		}
    	}
    }

}
