package command;

import java.io.IOException;
import java.util.ArrayList;

import party.PartyInfo;
import util.DialogUtil;

/*
*  ChoiceCommandP2クラス
*
*  ポケモン選出のコマンド入力要求・コマンド受付を行うクラスです。
*  P2用です。
*
*  変更履歴
*  2020/08/22 ChoiceCommandを継承して新規作成
*
*/

public class ChoiceCommandP2 extends ChoiceCommand {

	/* 参照・反映用Stateオブジェクト */
	public PartyInfo partyInfo = null; //PokeMoldオブジェクト配列

	/* プレイヤー名 */
	public String playerName = "名無しさん";

	/* コマンド有効フラグ */
	/*
	 * 0…選出（１匹目）
	 * 1…選出（２匹目）
	 * 2…選出（３匹目）
	 * 3…選出（４匹目）
	 * 4…選出（５匹目）
	 * 5…選出（６匹目）
	 *
	 */
	boolean[] commandEnableFlag = {true,true,true,true,true,true};

	/* コマンド入力要求メソッド
	 *   引数　　　：PokeMoldオブジェクト配列、プレイヤー名
	 *   戻り値　　：0：コマンドの選択肢がある、1：コマンドの選択肢がない（＝異常終了）（int）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public int comanndRequest(PartyInfo partyInfo,String playerName) throws IOException {

		// 初期値セット（各種Stateクラス（PokeMoldクラスは配列））
		this.partyInfo = partyInfo;
		this.playerName = playerName;

		// コマンド有効フラグの初期化
		for(int i = 0; i < 6; i++){
			commandEnableFlag[i] = true;
		}

		// パーティ時点で６匹ではない場合はその欄は入力不可
		for(int i = 0; i < 6; i++){
			if (this.partyInfo.pokeMoldArray[i] == null) {
				commandEnableFlag[i] = false;
			}
		}

		// パーティに１匹もいなければ異常終了
    	boolean abnormalFlag = true;
    	for(int i = 0; i < 6; i++){
    		if (commandEnableFlag[i]) {
    			abnormalFlag = false;
    		}
    	}
    	if (abnormalFlag) {
    		return 1;
    	}

    	// 有効コマンドの表示
    	DialogUtil.dialogMessageOutputP2("■"+
    	                                    this.playerName +
    	                                    "の入力可能コマンド一覧");
    	for(int i = 0; i < 6; i++){
    		if (commandEnableFlag[i]) {
    			String sexDisp;
    			if (this.partyInfo.pokeMoldArray[i].sex.equals("male")) {
    				sexDisp = "♂";
    			} else if (this.partyInfo.pokeMoldArray[i].sex.equals("female")) {
    				sexDisp = "♀";
    			} else {
    				sexDisp = "　";
    			}
    			if (this.partyInfo.pokeMoldArray[i].speciesName.length() == 2) {
    				sexDisp = sexDisp + "　　　";
    			} else if (this.partyInfo.pokeMoldArray[i].speciesName.length() == 3) {
    				sexDisp = sexDisp + "　　";
    			} else if (this.partyInfo.pokeMoldArray[i].speciesName.length() == 4) {
    				sexDisp = sexDisp + "　";
    			}
    			String itemDisp;
    			if (this.partyInfo.pokeMoldArray[i].itemName.equals("持ち物無し")) {
    				itemDisp = "　";
    			} else {
    				itemDisp = "○";
    			}
    			DialogUtil.dialogMessageOutputP2(i+
    		                                        "："+
    		                                        this.partyInfo.pokeMoldArray[i].speciesName+
    		                                        sexDisp+
    		                                        " LV"+
    		                                        String.format("%3s", this.partyInfo.pokeMoldArray[i].level)+
    		                                        itemDisp);
    		}
    	}
    	DialogUtil.dialogMessageOutputP2("　※0-F以外の入力は全て0として扱います");
    	DialogUtil.dialogMessageOutputP2("　※選出順に1-6の長さでコマンドを入力してください（例：012、562）");

    	return 0;

	}

	/* コマンド受付メソッド
	 *   引数　　　：入力されたコマンド(String)
	 *   戻り値　　：0：コマンド受付完了、1：不正コマンド入力により要再受付（int）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 *   その他　　：コマンド入力要求メソッドを予め実行している前提
	 */
	public int comanndAccept(String command) throws IOException {

		// コマンドの長さチェック
		if (command.length() < 1 || command.length() > 6) {
			DialogUtil.dialogMessageOutputP2("コマンドは半角１～６文字で入力してください");
			return 1;
		}

		// コマンドの変換
		ArrayList<Integer> commandArray = new ArrayList<>();
    	for(int i = 0; i < command.length(); i++){
    		String commandTmp = command.substring(i,i+1);
    		commandArray.add(hexToInt(commandTmp));
    	}

		// コマンドの妥当性確認
    	int[] validationCheck = {0,0,0,0,0,0,0};
	    for(int i = 0; i < commandArray.size(); i++){
	    	if (commandArray.get(i) > 5) {
	    		validationCheck[6]++;
	    	} else {
	    		if(!commandEnableFlag[commandArray.get(i)]) {
	    			validationCheck[6]++;
	    		} else {
	    			validationCheck[commandArray.get(i)]++;
	    		}
	    	}
	    }
	    if(validationCheck[6] > 0) {
			DialogUtil.dialogMessageOutputP2("有効なコマンドを入力してください");
			return 1;
	    } else {
	    	boolean duplicateFlag = false;
		    for(int i = 0; i < 6; i++){
		    	if(validationCheck[i] > 1) {
		    		duplicateFlag = true;
		    	}
		    }
		    if (duplicateFlag) {
		    	DialogUtil.dialogMessageOutputP2("同じポケモンを２回以上選出することはできません");
		    	return 1;
		    }
	    }

		// コマンドのセット（PokeStateUnit配列のセット）
	    for(int i = 0; i < commandArray.size(); i++){
	    	partyInfo.pokeStateUnitSet(partyInfo.pokeMoldArray[commandArray.get(i)], playerName);
	    }
	    partyInfo.pokeStateUnitChange(0);

		return 0;

	}

}
