package command;

import java.io.IOException;

import party.PartyInfo;
import state.PlaceState;
import state.PokeStateBaton;
import state.PokeStatePlace;
import state.PokeStateWhole;
import util.CommandScanner;
import util.DialogUtil;

/*
*  CommandManualInputクラス
*
*  思考ルーチン（コマンド受付処理）を記述するクラスです。
*  「Sample4_PseudoBattleSupporter」用です。
*
*  変更履歴
*  2020/11/04 完成
*  2020/11/08 コンストラクタ追加、出力ログ強化
*
*/

public class CommandManualInput extends CommandRTN {

	/* デフォルトコンストラクタ
	 *   引数　　　：無し
	 *   例外　　　：無し
	 */
    public CommandManualInput() {
    	super();
    }

	/* コンストラクタ（オブジェクト設定用）
	 *   引数　　　：PartyInfoオブジェクト（P1）、PartyInfoオブジェクト（P2）、
	 *   　　　　　：PokeStatePlaceオブジェクト（P1）、PokeStatePlaceオブジェクト（P2）、
	 *   　　　　　：PokeStateBatonオブジェクト（P1）、PokeStatebatonオブジェクト（P2）、
	 *   　　　　　：PokeStateWholeオブジェクト（P1）、PokeStateWholeオブジェクト（P2）、
	 *   　　　　　：PlaceStateオブジェクト、
	 *   　　　　　：ChoiceCommandオブジェクト（P1）、ChoiceCommandオブジェクト（P2）、
	 *   　　　　　：MoveChangeCommandオブジェクト（P1）、MoveChangeCommandオブジェクト（P2）、
	 *   　　　　　：プレイヤー名（P1）（String）、プレイヤー名（P2）（String）
	 *   例外　　　：無し
	 */
    public CommandManualInput(PartyInfo partyInfoP1, PartyInfo partyInfoP2,
                               PokeStatePlace pokeStatePlaceP1, PokeStatePlace pokeStatePlaceP2,
                               PokeStateBaton pokeStateBatonP1, PokeStateBaton pokeStateBatonP2,
                               PokeStateWhole pokeStateWholeP1, PokeStateWhole pokeStateWholeP2,
                               PlaceState placeState,
                               ChoiceCommand choiceCommandP1, ChoiceCommand choiceCommandP2,
                               MoveChangeCommand moveChangeCommandP1, MoveChangeCommand moveChangeCommandP2,
                               String playerNameP1, String playerNameP2) {
    	super(partyInfoP1,
    		  partyInfoP2,
    		  pokeStatePlaceP1,
    		  pokeStatePlaceP2,
    		  pokeStateBatonP1,
    		  pokeStateBatonP2,
    		  pokeStateWholeP1,
    		  pokeStateWholeP2,
    		  placeState,
    		  choiceCommandP1,
    		  choiceCommandP2,
    		  moveChangeCommandP1,
    		  moveChangeCommandP2,
    		  playerNameP1,
    		  playerNameP2);
    }

	/* 初期状態指定メソッド（パーティ選択前用）…パーティ選択前から前提条件を指定したい場合はここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void initStateSelectBeforePartySelect() {
    	// choiceOmitFlag = true; // 指定する場合はこのコメントアウトを外してください
    }

	/* パーティ選択メソッド（出力ログ強化のためオーバーライド）
	 *   引数　　　：P1のパーティテキストファイル名（String）、P2のパーティテキストファイル名（String）
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void partySelect(String partyTextFileNameP1, String partyTextFileNameP2) {
    	if (!choiceOmitFlag) {
        	try {
    			partyInfoP1.partyTextFileRead(partyTextFileNameP1);
    			partyInfoP2.partyTextFileRead(partyTextFileNameP2);
    		} catch (NullPointerException e) {
    			System.out.println("パーティテキストファイルが０行でないか確認してください。");
    			System.exit(1);
    		} catch (IOException e) {
    			System.out.println("confフォルダ内にパーティテキストファイルが見つかりません。");
    			System.exit(1);
    		}
        	partyInfoP1.partyPokeMoldSet();
        	partyInfoP2.partyPokeMoldSet();
    	}
    	try {
			DialogUtil.dialogMessageOutput("【パーティ選択】");
			DialogUtil.dialogMessageOutputP1(partyTextFileNameP1);
	    	for (int i = 0;i < 6;i++) {
	    		if (partyInfoP1.partyTextArray[i] != null) {
	    			DialogUtil.dialogMessageOutputP1(partyInfoP1.partyTextArray[i]);
	    		}
	    	}
	    	DialogUtil.dialogMessageOutputP2(partyTextFileNameP2);
	    	for (int i = 0;i < 6;i++) {
	    		if (partyInfoP2.partyTextArray[i] != null) {
	    			DialogUtil.dialogMessageOutputP2(partyInfoP2.partyTextArray[i]);
	    		}
	    	}
		} catch (IOException e1) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}
    }

	/* 選出メソッド（P1用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void choiceP1() {
    	if (!choiceOmitFlag) {
    		try {
    			int commandAcceptFlag = 1; // 0で受付完了、1は受付中
    			String inputText;
    			commandAcceptFlag = 1;
    			while (commandAcceptFlag == 1) {
    				choiceCommandP1.comanndRequest(partyInfoP1,playerNameP1);
    				inputText = CommandScanner.commandScanP1("コマンド：");
    				DialogUtil.dialogMessageOutputP1("コマンド："+inputText);
    				commandAcceptFlag = choiceCommandP1.comanndAccept(inputText);
    			}
    		} catch (IOException e1) {
    			System.out.println("ログ出力に失敗したので終了します。");
        		System.exit(1);
    		}
    	}
    }

	/* 選出メソッド（P2用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void choiceP2() {
    	if (!choiceOmitFlag) {
    		try {
    			int commandAcceptFlag = 1; // 0で受付完了、1は受付中
    			String inputText;
    			commandAcceptFlag = 1;
    			while (commandAcceptFlag == 1) {
    				choiceCommandP2.comanndRequest(partyInfoP2,playerNameP2);
    				inputText = CommandScanner.commandScanP2("コマンド：");
    				DialogUtil.dialogMessageOutputP2("コマンド："+inputText);
    				commandAcceptFlag = choiceCommandP2.comanndAccept(inputText);
    			}
    		} catch (IOException e1) {
    			System.out.println("ログ出力に失敗したので終了します。");
        		System.exit(1);
    		}
    	}
    }

	/* 初期状態指定メソッド（選出後用）（出力ログ強化のためオーバーライド）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void initStateSelectAfterChoice() {
    	try {
			DialogUtil.dialogMessageOutput("");
			DialogUtil.dialogMessageOutput("【対戦開始】");
		} catch (IOException e1) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}
    }

	/* ターン開始時の行動選択メソッド（P1用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void moveChangeP1() {
		try {
			int commandAcceptFlag = 1; // 0で受付完了、1は受付中
			String inputText;
			commandAcceptFlag = 1;
			while (commandAcceptFlag == 1) {
				moveChangeCommandP1.comanndRequest(partyInfoP1,
					                               pokeStatePlaceP1,
			                                       pokeStatePlaceP2,
			                                       pokeStateBatonP1,
			                                       pokeStateBatonP2,
			                                       true);
				inputText = CommandScanner.commandScanP1("コマンド：");
				DialogUtil.dialogMessageOutputP1("コマンド："+inputText);
				commandAcceptFlag = moveChangeCommandP1.comanndAccept(inputText);
			}
		} catch (IOException e1) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}
    }

	/* ターン開始時の行動選択メソッド（P2用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void moveChangeP2() {
		try {
			int commandAcceptFlag = 1; // 0で受付完了、1は受付中
			String inputText;
			commandAcceptFlag = 1;
			while (commandAcceptFlag == 1) {
				moveChangeCommandP2.comanndRequest(partyInfoP2,
				                                   pokeStatePlaceP2,
				                                   pokeStatePlaceP1,
				                                   pokeStateBatonP2,
				                                   pokeStateBatonP1,
				                                   true);
				inputText = CommandScanner.commandScanP2("コマンド：");
				DialogUtil.dialogMessageOutputP2("コマンド："+inputText);
				commandAcceptFlag = moveChangeCommandP2.comanndAccept(inputText);
			}
		} catch (IOException e1) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}
    }

	/* 死に出し交代先指定メソッド（P1用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenFaintedP1() {
		try {
			int commandAcceptFlag = 1; // 0で受付完了、1は受付中
			String inputText;
			commandAcceptFlag = 1;
			while (commandAcceptFlag == 1) {
				moveChangeCommandP1.comanndRequest(partyInfoP1,
					                               pokeStatePlaceP1,
			                                       pokeStatePlaceP2,
			                                       pokeStateBatonP1,
			                                       pokeStateBatonP2,
			                                       false);
				inputText = CommandScanner.commandScanP1("コマンド：");
				DialogUtil.dialogMessageOutputP1("コマンド："+inputText);
				commandAcceptFlag = moveChangeCommandP1.comanndAccept(inputText);
			}
		} catch (IOException e1) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}
    }

	/* 死に出し交代先指定メソッド（P2用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenFaintedP2() {
		try {
			int commandAcceptFlag = 1; // 0で受付完了、1は受付中
			String inputText;
			commandAcceptFlag = 1;
			while (commandAcceptFlag == 1) {
				moveChangeCommandP2.comanndRequest(partyInfoP2,
				                                   pokeStatePlaceP2,
				                                   pokeStatePlaceP1,
				                                   pokeStateBatonP2,
				                                   pokeStateBatonP1,
				                                   false);
				inputText = CommandScanner.commandScanP2("コマンド：");
				DialogUtil.dialogMessageOutputP2("コマンド："+inputText);
				commandAcceptFlag = moveChangeCommandP2.comanndAccept(inputText);
			}
		} catch (IOException e1) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}
    }

	/* バトンタッチ交代先指定メソッド（P1用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenBatonP1() {
		try {
			int commandAcceptFlag = 1; // 0で受付完了、1は受付中
			String inputText;
			commandAcceptFlag = 1;
			while (commandAcceptFlag == 1) {
				moveChangeCommandP1.comanndRequest(partyInfoP1,
					                               pokeStatePlaceP1,
			                                       pokeStatePlaceP2,
			                                       pokeStateBatonP1,
			                                       pokeStateBatonP2,
			                                       false);
				inputText = CommandScanner.commandScanP1("コマンド：");
				DialogUtil.dialogMessageOutputP1("コマンド："+inputText);
				commandAcceptFlag = moveChangeCommandP1.comanndAccept(inputText);
			}
		} catch (IOException e1) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}
    }

	/* バトンタッチ交代先指定メソッド（P2用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenBatonP2() {
		try {
			int commandAcceptFlag = 1; // 0で受付完了、1は受付中
			String inputText;
			commandAcceptFlag = 1;
			while (commandAcceptFlag == 1) {
				moveChangeCommandP2.comanndRequest(partyInfoP2,
				                                   pokeStatePlaceP2,
				                                   pokeStatePlaceP1,
				                                   pokeStateBatonP2,
				                                   pokeStateBatonP1,
				                                   false);
				inputText = CommandScanner.commandScanP2("コマンド：");
				DialogUtil.dialogMessageOutputP2("コマンド："+inputText);
				commandAcceptFlag = moveChangeCommandP2.comanndAccept(inputText);
			}
		} catch (IOException e1) {
			System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
		}
    }

}
