package command;

import java.io.IOException;

import party.PartyInfo;
import state.PlaceState;
import state.PokeStateBaton;
import state.PokeStatePlace;
import state.PokeStateWhole;

/*
*  CommandRTNクラス
*
*  思考ルーチンを記述するクラスです。
*
*  変更履歴
*  2020/11/04 完成
*  2020/11/08 メソッド定義順変更
*
*/

public class CommandRTN {

	/* 引数で受け取るオブジェクト */
	public PartyInfo partyInfoP1 = null;
	public PartyInfo partyInfoP2 = null;
	public PokeStatePlace pokeStatePlaceP1 = null;
	public PokeStatePlace pokeStatePlaceP2 = null;
	public PokeStateBaton pokeStateBatonP1 = null;
	public PokeStateBaton pokeStateBatonP2 = null;
	public PokeStateWhole pokeStateWholeP1 = null;
	public PokeStateWhole pokeStateWholeP2 = null;
	public PlaceState placeState = null;
	public ChoiceCommand choiceCommandP1 = null;
	public ChoiceCommand choiceCommandP2 = null;
	public MoveChangeCommand moveChangeCommandP1 = null;
	public MoveChangeCommand moveChangeCommandP2 = null;
	public String playerNameP1 = "名無しさん";
	public String playerNameP2 = "名無しさん";

	/* 状態保持 */
	public int firstHandPlayer = 0; // 先手のプレイヤー。ターンが終了する度に初期状態(0)に戻す。
	public int nextStep = 0; // 次の処理。ルーチンの記述で使えるように定義。
	                          // 0…初期状態
	                          // 1…次は「ターン開始」
	                          // 2…次は「先手の技・交代繰り出し処理」
	                          // 3…次は「先手の技繰り出し後処理」
	                          // 4…次は「後手の技・交代繰り出し処理」
	                          // 5…次は「後手の技繰り出し後処理」
	                          // 6…次は「ターン終了時処理」
	public boolean choiceOmitFlag = false; // パーティ選択・選出の代わりに初期条件を別途指定する場合のみtrue

	/* デフォルトコンストラクタ
	 *   引数　　　：無し
	 *   例外　　　：無し
	 */
    public CommandRTN() {
    	changeStateInit();
    }

	/* コンストラクタ（オブジェクト設定用）
	 *   引数　　　：PartyInfoオブジェクト（P1）、PartyInfoオブジェクト（P2）、
	 *   　　　　　：PokeStatePlaceオブジェクト（P1）、PokeStatePlaceオブジェクト（P2）、
	 *   　　　　　：PokeStateBatonオブジェクト（P1）、PokeStatebatonオブジェクト（P2）、
	 *   　　　　　：PokeStateWholeオブジェクト（P1）、PokeStateWholeオブジェクト（P2）、
	 *   　　　　　：PlaceStateオブジェクト、
	 *   　　　　　：ChoiceCommandオブジェクト（P1）、ChoiceCommandオブジェクト（P2）、
	 *   　　　　　：MoveChangeCommandオブジェクト（P1）、MoveChangeCommandオブジェクト（P2）、
	 *   　　　　　：プレイヤー名（P1）（String）、プレイヤー名（P2）（String）
	 *   例外　　　：無し
	 */
    public CommandRTN(PartyInfo partyInfoP1, PartyInfo partyInfoP2,
                       PokeStatePlace pokeStatePlaceP1, PokeStatePlace pokeStatePlaceP2,
                       PokeStateBaton pokeStateBatonP1, PokeStateBaton pokeStateBatonP2,
                       PokeStateWhole pokeStateWholeP1, PokeStateWhole pokeStateWholeP2,
                       PlaceState placeState,
                       ChoiceCommand choiceCommandP1, ChoiceCommand choiceCommandP2,
                       MoveChangeCommand moveChangeCommandP1, MoveChangeCommand moveChangeCommandP2,
		               String playerNameP1, String playerNameP2) {
    	changeStateInit();
    	objectSet(partyInfoP1,
    			  partyInfoP2,
    			  pokeStatePlaceP1,
    			  pokeStatePlaceP2,
    			  pokeStateBatonP1,
    			  pokeStateBatonP2,
    			  pokeStateWholeP1,
    			  pokeStateWholeP2,
    			  placeState,
    			  choiceCommandP1,
    			  choiceCommandP2,
    			  moveChangeCommandP1,
    			  moveChangeCommandP2,
    			  playerNameP1,
    			  playerNameP2);
    }

	/* オブジェクト登録メソッド
	 *   引数　　　：PartyInfoオブジェクト（P1）、PartyInfoオブジェクト（P2）、
	 *   　　　　　：PokeStatePlaceオブジェクト（P1）、PokeStatePlaceオブジェクト（P2）、
	 *   　　　　　：PokeStateBatonオブジェクト（P1）、PokeStatebatonオブジェクト（P2）、
	 *   　　　　　：PokeStateWholeオブジェクト（P1）、PokeStateWholeオブジェクト（P2）、
	 *   　　　　　：PlaceStateオブジェクト、
	 *   　　　　　：ChoiceCommandオブジェクト（P1）、ChoiceCommandオブジェクト（P2）、
	 *   　　　　　：MoveChangeCommandオブジェクト（P1）、MoveChangeCommandオブジェクト（P2）、
	 *   　　　　　：プレイヤー名（P1）（String）、プレイヤー名（P2）（String）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void objectSet(PartyInfo partyInfoP1, PartyInfo partyInfoP2,
    		               PokeStatePlace pokeStatePlaceP1, PokeStatePlace pokeStatePlaceP2,
    		               PokeStateBaton pokeStateBatonP1, PokeStateBaton pokeStateBatonP2,
    		               PokeStateWhole pokeStateWholeP1, PokeStateWhole pokeStateWholeP2,
    		               PlaceState placeState,
    		               ChoiceCommand choiceCommandP1, ChoiceCommand choiceCommandP2,
    		               MoveChangeCommand moveChangeCommandP1, MoveChangeCommand moveChangeCommandP2,
    		               String playerNameP1, String playerNameP2) {
    	this.partyInfoP1 = partyInfoP1;
    	this.partyInfoP2 = partyInfoP2;
    	this.pokeStatePlaceP1 = pokeStatePlaceP1;
    	this.pokeStatePlaceP2 = pokeStatePlaceP2;
    	this.pokeStateBatonP1 = pokeStateBatonP1;
    	this.pokeStateBatonP2 = pokeStateBatonP2;
    	this.pokeStateWholeP1 = pokeStateWholeP1;
    	this.pokeStateWholeP2 = pokeStateWholeP2;
    	this.placeState = placeState;
    	this.choiceCommandP1 = choiceCommandP1;
    	this.choiceCommandP2 = choiceCommandP2;
    	this.moveChangeCommandP1 = moveChangeCommandP1;
    	this.moveChangeCommandP2 = moveChangeCommandP2;
    	this.playerNameP1 = playerNameP1;
    	this.playerNameP2 = playerNameP2;
    }

	/* 初期状態指定メソッド（パーティ選択前用）…パーティ選択前から前提条件を指定したい場合はここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void initStateSelectBeforePartySelect() {
    	// choiceOmitFlag = true; // 指定する場合はこのコメントアウトを外してください
    }

	/* パーティ選択メソッド
	 *   引数　　　：P1のパーティテキストファイル名（String）、P2のパーティテキストファイル名（String）
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void partySelect(String partyTextFileNameP1, String partyTextFileNameP2) {
    	if (!choiceOmitFlag) {
        	try {
    			partyInfoP1.partyTextFileRead(partyTextFileNameP1);
    			partyInfoP2.partyTextFileRead(partyTextFileNameP2);
    		} catch (NullPointerException e) {
    			System.out.println("パーティテキストファイルが０行でないか確認してください。");
    			System.exit(1);
    		} catch (IOException e) {
    			System.out.println("confフォルダ内にパーティテキストファイルが見つかりません。");
    			System.exit(1);
    		}
        	partyInfoP1.partyPokeMoldSet();
        	partyInfoP2.partyPokeMoldSet();
    	}
    }

	/* 選出メソッド（全体用メソッド）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void choice() {
    	choiceP1();
    	choiceP2();
    }

	/* 選出メソッド（P1用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void choiceP1() {
    	if (!choiceOmitFlag) {
    		partyInfoP1.pokeStateUnitSet(partyInfoP1.pokeMoldArray[0], playerNameP1);
    		partyInfoP1.pokeStateUnitSet(partyInfoP1.pokeMoldArray[1], playerNameP1);
    		partyInfoP1.pokeStateUnitSet(partyInfoP1.pokeMoldArray[2], playerNameP1);
    		partyInfoP1.pokeStateUnitChange(0);
    	}
    }

	/* 選出メソッド（P2用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void choiceP2() {
    	if (!choiceOmitFlag) {
    		partyInfoP2.pokeStateUnitSet(partyInfoP2.pokeMoldArray[0], playerNameP2);
    		partyInfoP2.pokeStateUnitSet(partyInfoP2.pokeMoldArray[1], playerNameP2);
    		partyInfoP2.pokeStateUnitSet(partyInfoP2.pokeMoldArray[2], playerNameP2);
    		partyInfoP2.pokeStateUnitChange(0);
    	}
    }

	/* 初期状態指定メソッド（選出後用）…選出後に前提条件を指定したい場合はここに記述して下さい。非対話式用です。
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void initStateSelectAfterChoice() {
    }

	/* ターン開始時の行動選択メソッド（全体用メソッド）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void moveChange() {
    	moveChangeP1();
    	moveChangeP2();
    }

	/* ターン開始時の行動選択メソッド（P1用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void moveChangeP1() {
    	if (partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].movePP[0] != 0) {
    		pokeStatePlaceP1.moveIndex = 0;
    	} else if (partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].movePP[1] != 0) {
    		pokeStatePlaceP1.moveIndex = 1;
    	} else if (partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].movePP[2] != 0) {
    		pokeStatePlaceP1.moveIndex = 2;
    	} else if (partyInfoP1.pokeStateUnitArray[partyInfoP1.pokeStateUnitArrayIndex].movePP[3] != 0) {
    		pokeStatePlaceP1.moveIndex = 3;
    	} else {
    		// 悪あがき発動
    	}
    }

	/* ターン開始時の行動選択メソッド（P2用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void moveChangeP2() {
    	if (partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].movePP[0] != 0) {
    		pokeStatePlaceP2.moveIndex = 0;
    	} else if (partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].movePP[1] != 0) {
    		pokeStatePlaceP2.moveIndex = 1;
    	} else if (partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].movePP[2] != 0) {
    		pokeStatePlaceP2.moveIndex = 2;
    	} else if (partyInfoP2.pokeStateUnitArray[partyInfoP2.pokeStateUnitArrayIndex].movePP[3] != 0) {
    		pokeStatePlaceP2.moveIndex = 3;
    	} else {
    		// 悪あがき発動
    	}
    }

	/* 死に出し交代先指定メソッド（先手用）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenFaintedFirstHand() {
    	if (firstHandPlayer == 1) {
    		changeWhenFaintedP1();
    	} else {
    		changeWhenFaintedP2();
    	}
    }

	/* 死に出し交代先指定メソッド（後手用）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenFaintedSecondHand() {
    	if (firstHandPlayer == 1) {
    		changeWhenFaintedP2();
    	} else {
    		changeWhenFaintedP1();
    	}
    }

	/* 死に出し交代先指定メソッド（P1用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenFaintedP1() {
    	pokeStatePlaceP1.changeTarget = 0;
    	for(int i = 0; i < 6; i++){
    		if (partyInfoP1.pokeStateUnitArray[i] != null) {
    			if (partyInfoP1.pokeStateUnitArray[i].actualHP > 0) {
    				if (i != partyInfoP1.pokeStateUnitArrayIndex) {
    					pokeStatePlaceP1.changeTarget = i;
    					break;
    				}
    			}
    		}
    	}
    }

	/* 死に出し交代先指定メソッド（P2用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenFaintedP2() {
    	pokeStatePlaceP2.changeTarget = 0;
    	for(int i = 0; i < 6; i++){
    		if (partyInfoP2.pokeStateUnitArray[i] != null) {
    			if (partyInfoP2.pokeStateUnitArray[i].actualHP > 0) {
    				if (i != partyInfoP2.pokeStateUnitArrayIndex) {
    					pokeStatePlaceP2.changeTarget = i;
    					break;
    				}
    			}
    		}
    	}
    }

	/* バトンタッチ交代先指定メソッド（先手用）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenBatonFirstHand() {
    	if (firstHandPlayer == 1) {
    		changeWhenBatonP1();
    	} else {
    		changeWhenBatonP2();
    	}
    }

	/* バトンタッチ交代先指定メソッド（後手用）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenBatonSecondHand() {
    	if (firstHandPlayer == 1) {
    		changeWhenBatonP2();
    	} else {
    		changeWhenBatonP1();
    	}
    }

	/* バトンタッチ交代先指定メソッド（P1用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenBatonP1() {
    	pokeStatePlaceP1.changeTarget = 0;
    	for(int i = 0; i < 6; i++){
    		if (partyInfoP1.pokeStateUnitArray[i] != null) {
    			if (partyInfoP1.pokeStateUnitArray[i].actualHP > 0) {
    				if (i != partyInfoP1.pokeStateUnitArrayIndex) {
    					pokeStatePlaceP1.changeTarget = i;
    					break;
    				}
    			}
    		}
    	}
    }

	/* バトンタッチ交代先指定メソッド（P2用）…ルーチンはここに記述して下さい
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し（例外発生時は処理終了）
	 */
    public void changeWhenBatonP2() {
    	pokeStatePlaceP2.changeTarget = 0;
    	for(int i = 0; i < 6; i++){
    		if (partyInfoP2.pokeStateUnitArray[i] != null) {
    			if (partyInfoP2.pokeStateUnitArray[i].actualHP > 0) {
    				if (i != partyInfoP2.pokeStateUnitArrayIndex) {
    					pokeStatePlaceP2.changeTarget = i;
    					break;
    				}
    			}
    		}
    	}
    }

	/* 状態初期化メソッド（ポケモン選出前）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeStateInit() {
    	firstHandPlayer = 0;
    	nextStep = 0;
    	choiceOmitFlag = false;
    }

	/* 状態移行メソッド（ターン開始）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeStateTurnStart() {
    	firstHandPlayer = 0;
    	nextStep = 1;
    }

	/* 状態移行メソッド（先手・後手判定時）
	 *   引数　　　：先手のプレイヤー（int）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeStateWhenHandJudge(int firstHandPlayer) {
    	this.firstHandPlayer = firstHandPlayer;
    }

	/* 状態移行メソッド（先手の技・交代繰り出し処理前）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeStateBeforeFirstHandMove() {
    	nextStep = 2;
    }

	/* 状態移行メソッド（先手の技繰り出し後処理前）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeStateAfterFirstHandMove() {
    	nextStep = 3;
    }

	/* 状態移行メソッド（後手の技・交代繰り出し処理前）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeStateBeforeSecondHandMove() {
    	nextStep = 4;
    }

	/* 状態移行メソッド（後手の技繰り出し後処理前）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeStateAfterSecondHandMove() {
    	nextStep = 5;
    }

	/* 状態移行メソッド（ターン終了時処理前）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeStateTurnEnd() {
    	nextStep = 6;
    }

}
