package command;

import java.io.IOException;

import party.PartyInfo;
import state.PokeStateBaton;
import state.PokeStatePlace;
import util.DialogUtil;

/*
*  MoveChangeCommandP2クラス
*
*  交代・技選択のコマンド入力要求・コマンド受付を行うクラスです。
*  P2用です。
*
*  変更履歴
*  2020/08/22 MoveChangeCommandを継承して新規作成
*  2020/09/12 変身を実装
*  2020/09/12 悪あがきフラグ廃止（実機にはあるらしいが、少なくともPBSにはない）
*  2020/11/22 コマンド選択時に場のポケモンの情報を出すように表示を改良
*  2021/01/23 ルーチンでの参照禁止変数をprivate化したことに伴う修正
*
*/

public class MoveChangeCommandP2 extends MoveChangeCommand {

	/* 参照・反映用Stateオブジェクト */
	public PartyInfo partyInfoMyself = null; //PokeStateUnit配列
	public PokeStatePlace pokeStatePlaceMyself = null;
	public PokeStatePlace pokeStatePlaceOpponent = null;
	public PokeStateBaton pokeStateBatonMyself = null;
	public PokeStateBaton pokeStateBatonOpponent = null;

	/* ターン開始時かどうかのフラグ */
	boolean turnStartFlag = true;

	/* コマンド有効フラグ */
	/*
	 * 0…交代（１匹目）
	 * 1…交代（２匹目）
	 * 2…交代（３匹目）
	 * 3…交代（４匹目）
	 * 4…交代（５匹目）
	 * 5…交代（６匹目）
	 * 6…技（１つ目）
	 * 7…技（２つ目）
	 * 8…技（３つ目）
	 * 9…技（４つ目）
	 * A…悪あがき
	 *
	 */
	boolean[] commandEnableFlag = {true,true,true,true,true,true,true,true,true,true,true};

	/* コマンド入力要求メソッド
	 *   引数　　　：PokeStateUnit配列、
	 *   　　　　　：自分のPokeStatePlaceオブジェクト、相手のPokeStatePlaceオブジェクト、
	 *   　　　　　：自分のPokeStateBatonオブジェクト、相手のPokeStateBatonオブジェクト、
	 *   　　　　　：ターン開始時かどうか（boolean）
	 *   戻り値　　：0：コマンドの選択肢がある、1：コマンドの選択肢がない（＝負け）（int）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
	public int comanndRequest(PartyInfo partyInfoMyself,
	                           PokeStatePlace pokeStatePlaceMyself,PokeStatePlace pokeStatePlaceOpponent,
	                           PokeStateBaton pokeStateBatonMyself,PokeStateBaton pokeStateBatonOpponent,
	                           boolean turnStartFlag) throws IOException {

		// 初期値セット（各種Stateクラス（PokeStateUnitクラスは配列））
		this.partyInfoMyself = partyInfoMyself;
		this.pokeStatePlaceMyself = pokeStatePlaceMyself;
		this.pokeStatePlaceOpponent = pokeStatePlaceOpponent;
		this.pokeStateBatonMyself = pokeStateBatonMyself;
		this.pokeStateBatonOpponent = pokeStateBatonOpponent;
		this.turnStartFlag = turnStartFlag;

		// コマンド有効フラグの初期化
		for(int i = 0; i < 11; i++){
			commandEnableFlag[i] = true;
		}

		// 場に出ているポケモンへの交代の無効化
		if (this.partyInfoMyself.pokeStateUnitArrayIndex >= 0 && this.partyInfoMyself.pokeStateUnitArrayIndex <= 5) {
			commandEnableFlag[this.partyInfoMyself.pokeStateUnitArrayIndex] = false;
		}

		// 場に出ているポケモンへの交代の無効化、未選出、HP0のポケモンへの交代の無効化
    	for(int i = 0; i < 6; i++){
    		if (this.partyInfoMyself.pokeStateUnitArray[i] != null) {
    			if (this.partyInfoMyself.pokeStateUnitArray[i].actualHP > 0) {
    				if (i != this.partyInfoMyself.pokeStateUnitArrayIndex) {
    					// このケースは有効のままで良い
    				} else {
    					commandEnableFlag[i] = false;
    				}
    			} else {
    				commandEnableFlag[i] = false;
    			}
    		} else {
    			commandEnableFlag[i] = false;
    		}
    	}

    	// ターン開始時ではない場合は技は全て使用不可
    	if (!turnStartFlag) {
        	for(int i = 6; i < 11; i++){
        		commandEnableFlag[i] = false;
        	}
    	} else {

    		// PP切れの技は使用不可（技無しもPP0固定にすることで使用不可にする）
    		for (int i = 0 ; i < 4 ; i++){
    			if (this.pokeStatePlaceMyself.mimicMoveIndex == i) {
    				if (this.pokeStatePlaceMyself.mimicMovePP == 0) {
    					commandEnableFlag[i+6] = false;
    				}
    			} else if (this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex == i) {
    				if (this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].sketchMovePP == 0) {
    					commandEnableFlag[i+6] = false;
    				}
    			} else if (this.pokeStatePlaceMyself.transformedFlag){
    				if (this.pokeStatePlaceMyself.transformedMovePP[i] == 0) {
    					commandEnableFlag[i+6] = false;
    				}
    			} else {
    				if (this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].movePP[i] == 0) {
    					commandEnableFlag[i+6] = false;
    				}
    			}
    		}

    		// 黒い眼差し時は交代不可
    		if (this.pokeStateBatonMyself.trappedFlag && this.pokeStateBatonOpponent.trappingFlag) {
    	    	for(int i = 0; i < 6; i++){
    	    		commandEnableFlag[i] = false;
    	    	}
    		}

    		// 締め付ける時は交代不可
    		if (this.pokeStatePlaceMyself.isTwind() && this.pokeStatePlaceOpponent.isTwining()) {
    	    	for(int i = 0; i < 6; i++){
    	    		commandEnableFlag[i] = false;
    	    	}
    		}

    		// コンボ中・破壊光線反動・アンコール・金縛りの考慮
    		// ある技についてアンコールと金縛りが同時に発生したら技選択は可能（結局技繰り出し判定で金縛りで繰り出せない）。
    		// コンボ中（破壊光線反動）についても金縛り同時発生で技選択は可能（結局技繰り出し判定で金縛りで繰り出せずコンボ終了）。
    		if (this.pokeStatePlaceMyself.isBeingCombo() ||
    		    this.pokeStatePlaceMyself.destructionBeemStackRestTrun > 0) {
            	for(int i = 0; i < 11; i++){
            		if (this.pokeStatePlaceMyself.latestUsedMoveIndex + 6 != i) {
            			commandEnableFlag[i] = false;
            		}
            	}
    		} else if (this.pokeStatePlaceMyself.isEncored()) {
            	for(int i = 6; i < 10; i++){
            		if (this.pokeStatePlaceMyself.encoredMoveIndex + 6 != i) {
            			commandEnableFlag[i] = false;
            		}
            	}
    		} else if (this.pokeStatePlaceMyself.isMoveParalysis()) {
            	for(int i = 6; i < 10; i++){
            		if (this.pokeStatePlaceMyself.moveParalysisIndex + 6 == i) {
            			commandEnableFlag[i] = false;
            		}
            	}
    		}

    		// 悪あがき状態の考慮
    		// if (this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].struggleFlag) {
            //	for(int i = 6; i < 10; i++){
            //		commandEnableFlag[i] = false;
            //	}
    		// }

    		// 何かしらの技が有効な場合は悪あがき使用不可
    		if (commandEnableFlag[6] || commandEnableFlag[7] || commandEnableFlag[8] || commandEnableFlag[9]) {
    			commandEnableFlag[10] = false;
    		}
    	}

    	// 有効コマンドがなければ1を返して終了
    	boolean loseFlag = true;
    	for(int i = 0; i < 11; i++){
    		if (commandEnableFlag[i]) {
    			loseFlag = false;
    		}
    	}
    	if (loseFlag) {
    		return 1;
    	}

    	// 有効コマンドの表示
    	DialogUtil.dialogMessageOutputP2("■"+
    	                                    this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].playerNameDisp +
    	                                    "の入力可能コマンド一覧");
    	DialogUtil.dialogMessageOutput("場のポケモン（" +
                                          this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].speciesNameDisp+
                                          "（"+
                                          this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].actualHP+
                                          "/"+
                                          this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].MAXHP+
                                          "）、状態異常："+
                                          this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].badHealth+
                                          "）");
    	for(int i = 0; i < 6; i++){
    		if (commandEnableFlag[i]) {
    			DialogUtil.dialogMessageOutputP2(i+
    		                                        "：交代（"+
    		                                        this.partyInfoMyself.pokeStateUnitArray[i].speciesNameDisp+
    		                                        "（"+
    		                                        this.partyInfoMyself.pokeStateUnitArray[i].actualHP+
    		                                        "/"+
    		                                        this.partyInfoMyself.pokeStateUnitArray[i].MAXHP+
    		                                        "）、状態異常："+
    		                                        this.partyInfoMyself.pokeStateUnitArray[i].badHealth+
    		                                        "）");
    		}
    	}
    	for(int i = 6; i < 10; i++){
    		if (commandEnableFlag[i]) {
    			if (this.pokeStatePlaceMyself.mimicMoveIndex == (i-6)) {
        			DialogUtil.dialogMessageOutputP2(i+
                                                         "：技（"+
                                                         this.pokeStatePlaceMyself.mimicMoveName+
                                                         "（"+
                                                         this.pokeStatePlaceMyself.mimicMovePP+
                                                         "））");
    			} else if (this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex == (i-6)) {
        			DialogUtil.dialogMessageOutputP2(i+
                                                         "：技（"+
                                                         this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveName+
                                                         "（"+
                                                         this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].sketchMovePP+
                                                         "））");
    			} else if (this.pokeStatePlaceMyself.transformedFlag){
        			DialogUtil.dialogMessageOutputP2(i+
                                                         "：技（"+
                                                         this.pokeStatePlaceMyself.transformedMoveName[i-6]+
                                                         "（"+
                                                         this.pokeStatePlaceMyself.transformedMovePP[i-6]+
                                                         "））");
    			} else {
        			DialogUtil.dialogMessageOutputP2(i+
                                                         "：技（"+
                                                         this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].moveName[i-6]+
                                                         "（"+
                                                         this.partyInfoMyself.pokeStateUnitArray[this.partyInfoMyself.pokeStateUnitArrayIndex].movePP[i-6]+
                                                         "））");
    			}
    		}
    	}
    	if (commandEnableFlag[10]) {
    		DialogUtil.dialogMessageOutputP2("A：悪あがき");
    	}
    	DialogUtil.dialogMessageOutputP2("　※0-F以外の入力は全て0として扱います");

		return 0;

	}

	/* コマンド受付メソッド
	 *   引数　　　：入力されたコマンド(String)
	 *   戻り値　　：0：コマンド受付完了、1：不正コマンド入力により要再受付（int）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 *   その他　　：コマンド入力要求メソッドを予め実行している前提
	 */
	public int comanndAccept(String command) throws IOException {

		// コマンドの長さチェック
		if (command.length() != 1) {
			DialogUtil.dialogMessageOutputP2("コマンドは半角１文字で入力してください");
			return 1;
		}

		// コマンドの変換
		int commandInt = hexToInt(command);

		// コマンドの妥当性確認
		if (commandInt >= 0 && commandInt <= 10) {
			if(!commandEnableFlag[commandInt]) {
				DialogUtil.dialogMessageOutputP2("有効なコマンドを入力してください");
				return 1;
			}
		} else {
			DialogUtil.dialogMessageOutputP2("有効なコマンドを入力してください");
			return 1;
		}

		// コマンドのセット（PokeStatePlaceオブジェクトのコマンド関連変数のセット）
		if (commandInt >= 0 && commandInt <= 5) {
			pokeStatePlaceMyself.changeTarget = commandInt;
		} else if (commandInt >= 6 && commandInt <= 9) {
			pokeStatePlaceMyself.moveIndex = commandInt - 6;
		} else {
			// 悪あがきの場合は何もセットしない
		}

		return 0;

	}

}
