package main;

import java.io.IOException;

import calc.DamageCalc;
import party.PartyInfo;
import state.PlaceState;
import state.PokeStateBaton;
import state.PokeStatePlace;
import state.PokeStateWhole;
import util.DialogUtil;

/*
*  Sample2_DamageCalc10000クラス
*
*  サンプルのメインクラスです。
*  攻撃側のパーティテキストファイル（１匹目のみ採用）、防御側のパーティテキストファイル（１匹目のみ採用）、
*  技のタイプ、技の威力、ランクや光の粉考慮後の命中率（0.00～1.00）、ランク考慮後の急所率（0.00～1.00）を引数で入力し、
*  何回の攻撃で倒せるのかを10000回試して算出します。
*  急所率（手計算結果を引数指定）・命中率（手計算結果を引数指定）・種族固有アイテム・1.1倍アイテム・ハチマキ等考慮。
*  PP・天候・追加効果・HP回復・ABCDランク変動・火傷・壁・ランダム化無し・プレゼント・変身・テスクチャー等未考慮。
*  何を考慮して何を考慮していないか詳細を知る必要がある場合はソースコードを確認してください。
*  考慮していない要素も、当クラスのソースコードの小改造で考慮可能になります。
*
*  変更履歴
*  2019/05/03 新規作成
*  2019/05/25 DamageCalcクラスの仕様変更に合わせた修正を実施
*  2019/06/09 個人的に気になったのでWarningを解消
*  2020/08/28 DamageCalcクラスの仕様変更に合わせた修正を実施
*
*/

public class Sample2_DamageCalc10000 {

    public static void main(String[] args) {

    	/* 非対話式の処理なので対話式スイッチはOFF */
    	try {
			DialogUtil.dialogSwitchSet(0);
		} catch (IOException e) {
			// TODO 自動生成された catch ブロック
			System.out.println("※非対話式の場合はここに来ることはありません。");
		}

    	/* パーティテキストファイルの読み込み */
    	PartyInfo partyInfo1 = new PartyInfo();
    	PartyInfo partyInfo2 = new PartyInfo();
    	try {
			partyInfo1.partyTextFileRead(args[0]);
			partyInfo2.partyTextFileRead(args[1]);
		} catch (NullPointerException e) {
			System.out.println("パーティテキストファイルが０行でないか確認してください。");
		} catch (IOException e) {
			System.out.println("confフォルダ内にパーティテキストファイル "+args[0]+" が見つかりません。");
		}

    	/* PokeMoldオブジェクトの生成と種族情報の取得 */
    	partyInfo1.partyPokeMoldSet();
    	partyInfo2.partyPokeMoldSet();

    	/* 状態管理オブジェクトの生成と初期状態のセット */
    	partyInfo1.pokeStateUnitSet(partyInfo1.pokeMoldArray[0],args[0]);
    	partyInfo2.pokeStateUnitSet(partyInfo2.pokeMoldArray[0],args[1]);
    	PokeStatePlace pokeStatePlace1 = new PokeStatePlace(args[0]);
    	PokeStatePlace pokeStatePlace2 = new PokeStatePlace(args[1]);
    	PokeStateBaton pokeStateBaton1 = new PokeStateBaton(args[0]);
    	PokeStateBaton pokeStateBaton2 = new PokeStateBaton(args[1]);
    //	PokeStateWhole pokeStateWhole1 = new PokeStateWhole(args[0]);
    	PokeStateWhole pokeStateWhole2 = new PokeStateWhole(args[1]);
    	PlaceState placeState = new PlaceState();

    	/* 引数の取得 */
    	String moveType;
    	switch (args[2]) {
    	case "ノーマル":
    	case "normal":
    		moveType = "normal";
    		break;
    	case "かくとう":
    	case "格闘":
    	case "fighting":
    		moveType = "fighting";
    		break;
    	case "ひこう":
    	case "飛行":
    	case "flying":
    		moveType = "flying";
    		break;
    	case "どく":
    	case "毒":
    	case "poison":
    		moveType = "poison";
    		break;
    	case "じめん":
    	case "地面":
    	case "ground":
    		moveType = "ground";
    		break;
    	case "いわ":
    	case "岩":
    	case "rock":
    		moveType = "rock";
    		break;
    	case "むし":
    	case "虫":
    	case "蟲":
    	case "bug":
    		moveType = "bug";
    		break;
    	case "ゴースト":
    	case "ghost":
    		moveType = "ghost";
    		break;
    	case "はがね":
    	case "鋼":
    	case "steel":
    		moveType = "steel";
    		break;
    	case "ほのお":
    	case "炎":
    	case "fire":
    		moveType = "fire";
    		break;
    	case "みず":
    	case "水":
    	case "water":
    		moveType = "water";
    		break;
    	case "くさ":
    	case "草":
    	case "grass":
    		moveType = "grass";
    		break;
    	case "でんき":
    	case "電気":
    	case "electric":
    		moveType = "electric";
    		break;
    	case "エスパー":
    	case "psychic":
    		moveType = "psychic";
    		break;
    	case "こおり":
    	case "氷":
    	case "ice":
    		moveType = "ice";
    		break;
    	case "ドラゴン":
    	case "dragon":
    		moveType = "dragon";
    		break;
    	case "あく":
    	case "悪":
    	case "dark":
    		moveType = "dark";
    		break;
    	default: // 属性無し物理と判断
    		moveType = "physic";
    		break;
    	}
    	double movePower = Math.floor(Double.parseDouble(args[3]));
    	double hitProbability = Double.parseDouble(args[4]) * 100;
    	double criticalHitProbability = Double.parseDouble(args[5]) * 100;
    	int damage = 0;

    	/* ダメージ計算の実行 */
    	DamageCalc damageCalc = new DamageCalc();
    	int attackTimesMAX = 10; //攻撃打ち切り回数
    	int[] result = new int[attackTimesMAX];
    	int[] resultAmount = new int[attackTimesMAX];
    	for (int i = 0;i < result.length;i++) {
    		result[i] = 0;
    		resultAmount[i] = 0;
    	}
    	int testTimes = 10000; //試行回数
    	for (int i = 0;i < testTimes;i++) {
    	int attackTimes = 0;
    	partyInfo2.pokeStateUnitArray[0].actualHP = partyInfo2.pokeStateUnitArray[0].MAXHP;
    	while (!(partyInfo2.pokeStateUnitArray[0].actualHP <= 0) && attackTimes < attackTimesMAX) {
    	try {
			if (damageCalc.hitJudge
					(hitProbability, pokeStatePlace2.protectSuccessFlag, pokeStatePlace2.latestUsedMoveEffectCode)) {
				double subjectValue = damageCalc.subjectValueCalc(moveType,
			                                                         partyInfo1.pokeStateUnitArray[0].attack,
			                                                         partyInfo1.pokeStateUnitArray[0].contact,
			                                                         pokeStateBaton1.attackRank,
			                                                         pokeStateBaton1.contactRank,
			                                                         partyInfo1.pokeStateUnitArray[0].speciesName,
			                                                         partyInfo1.pokeStateUnitArray[0].itemName,
			                                                         partyInfo1.pokeStateUnitArray[0].badHealth);
				double objectValue = damageCalc.objectValueCalc(moveType,
			                                                       partyInfo2.pokeStateUnitArray[0].block,
			                                                       partyInfo2.pokeStateUnitArray[0].diffence,
			                                                       pokeStateBaton2.blockRank,
			                                                       pokeStateBaton2.diffenceRank,
			                                                       partyInfo2.pokeStateUnitArray[0].speciesName,
			                                                       partyInfo2.pokeStateUnitArray[0].itemName,
			                                                       pokeStateWhole2.blockHalfFlag,
			                                                       pokeStateWhole2.reflectorRestTurn,
			                                                       pokeStateWhole2.lightWallRestTurn);
				damage = damageCalc.damageBodyCalc
						(movePower,subjectValue,objectValue,partyInfo1.pokeStateUnitArray[0].level);
				if(damageCalc.criticalHitJudge(criticalHitProbability)){
					damage = damageCalc.criticalHitDamageCalc(damage,
							                                  moveType,
							                                  pokeStateBaton1.attackRank,
							                                  pokeStateBaton1.contactRank,
							                                  pokeStateBaton2.blockRank,
							                                  pokeStateBaton2.diffenceRank,
							                                  partyInfo1.pokeStateUnitArray[0].badHealth,
							                                  pokeStateWhole2.reflectorRestTurn,
							                                  pokeStateWhole2.lightWallRestTurn);
				}
				damage = damageCalc.typeReinforceItemCalc(damage,moveType,partyInfo1.pokeStateUnitArray[0].itemName);
				damage = damageCalc.constPlus2Calc(damage);
				damage = damageCalc.weatherDamageCalc(damage,placeState.weather,moveType,"00a");
				damage = damageCalc.typeMatchCalc
						(damage,moveType,partyInfo1.pokeStateUnitArray[0].type1,partyInfo1.pokeStateUnitArray[0].type2);
				damage = damageCalc.typeCompatibilityCalc(damage,
				                                          moveType,
				                                          partyInfo2.pokeStateUnitArray[0].type1,
				                                          partyInfo2.pokeStateUnitArray[0].type2,
				                                          pokeStatePlace1.seeingThroughFlag,
				                                          pokeStatePlace2.seenThroughFlag);
				damage = damageCalc.damageRandomCalc(damage);
				damage = damageCalc.finalDamageCalc(damage,
						                            partyInfo2.pokeStateUnitArray[0].actualHP,
						                            pokeStatePlace2.protectSuccessFlag,
						                            pokeStatePlace2.latestUsedMoveEffectCode,
						                            "00a",
						                            damageCalc.spiritTriggerCalc(damage,
						                                                         partyInfo2.pokeStateUnitArray[0].actualHP,
						                                                         partyInfo2.pokeStateUnitArray[0].itemName));
				partyInfo2.pokeStateUnitArray[0].actualHP = partyInfo2.pokeStateUnitArray[0].actualHP - damage;
			}
			attackTimes++;
		} catch (IOException e) {
			System.out.println("※非対話式の場合はここに来ることはありません。");
		}
    	}
		result[attackTimes-1]++;
    	}

    	/* 結果表示 */
    	System.out.println("【攻撃側】");
    	System.out.println(partyInfo1.partyTextArray[0]);
    	System.out.println("【防御側】");
    	System.out.println(partyInfo2.partyTextArray[0]);
    	System.out.println("【攻撃技のタイプ、威力（引数で条件指定）】");
    	System.out.println(moveType+"、"+args[3]);
    	System.out.println("【命中率、急所率（0～1、引数で条件指定）】");
    	System.out.println(args[4]+"、"+args[5]);
    	System.out.println("【攻撃回数毎の撃破確率】");
    	for (int i = 0;i < result.length;i++) {
    		if(i==0) {
    			resultAmount[0] = result[0];
    		} else {
    			resultAmount[i] = result[i] + resultAmount[i-1];
    		}
        	double resultDisp = (double)result[i];
        	double resultAmountDisp = (double)resultAmount[i];
        	if (!(i == result.length - 1)) {
        		System.out.println(i+1+
        				           "発の攻撃で倒せる確率："+
        				           String.format("%.2f",resultDisp*100/testTimes)+
        				           "%（累計"+
        				           String.format("%.2f",resultAmountDisp*100/testTimes)+
        				           "%）");
        	} else {
        		System.out.println(i+1+
				           "発以上の攻撃で倒せる確率："+
				           String.format("%.2f",resultDisp*100/testTimes)+
				           "%（累計"+
				           String.format("%.2f",resultAmountDisp*100/testTimes)+
				           "%）");
        	}
    	}
    }
}
