package main;

import java.io.IOException;
import command.CommandRTN;
import command.MyCommandRTN;
import config.Precondition;
import facade.TurnControlFacade;
import party.PartyInfo;
import state.PlaceState;
import state.PokeStateBaton;
import state.PokeStatePlace;
import state.PokeStateWhole;
import util.DialogUtil;

/*
*  Sample3_TaimanCalc10000クラス
*
*  サンプルのメインクラスです。
*  攻撃側のパーティテキストファイル（１匹目のみ採用）、防御側のパーティテキストファイル（１匹目のみ採用）を引数入力し、
*  １つ目の技を打ち合った場合の勝率を10000回試して算出します。最大20ターン試行し、20ターン経過時点で判定を行います。
*  試行回数や思考ターン数は、当クラスのソースコードを改造することで変更可能です。
*  なお、１つ目の技のPPが切れた際は、２つ目以降の技を使います。
*  MyCommandRTNクラスのソースコードを改造することで、２つ以上の技を使う場合や交代を伴う場合の計算も可能になります。
*  （他人に結果を見せる場合はメッセージも直した方が良いです。メッセージは当クラスで定義しています。）
*
*  変更履歴
*  2019/07/16 新規作成
*  2019/07/22 ターンカットの考慮を追加（ソースを改変し、交代を伴う計算を行うようにする場合を想定）
*  2019/07/26 撒きびしで倒れた時の挙動が誤っていたのを修正
*  2019/07/27 片方のプレイヤーが交代、もう片方のプレイヤーが行動不能の場合、後者のプレイヤーが交代を迫られるバグを修正
*  2019/07/28 バトンタッチが通常の交代になるバグを修正
*  2019/08/06 毒で倒れた時の挙動が誤っていたのを修正
*  2019/08/25 初手の破壊の遺伝子が起動しないバグを修正
*  2019/09/12 最初の試行のログで種族名がけつばんになってしまうバグを修正
*  2020/03/29 相手を倒したターンに毒のダメージが発生するバグを修正
*  2020/08/11 交代のターンカットを実装、毒のダメージで倒れたターンに交代先にも毒のダメージが発生するバグを修正
*  2020/08/15 砂嵐で同時に倒れる場合にDROWにならないバグを修正
*  2020/09/06 １回の試行について、選出からやり直すように変更（拡張性確保のため）
*  2020/10/25 交代時に毒ダメージが発生しないバグを修正
*  2020/11/01 捨て身反動で引き分けの有無の設定を追加（当クラスでは引き分け有で設定）
*  2020/11/08 思考ルーチンクラスを取り込み
*  2020/11/22 ターンコントロール窓口クラスを取り込み
*
*/

public class Sample3_TaimanCalc10000 {

    public static void main(String[] args) {

    	/* 非対話式の処理なので対話式スイッチはOFF */
    	try {
			DialogUtil.dialogSwitchSet(0); // バグ調査の時はコメントアウトしてください
//			DialogUtil.dialogSwitchSet(1); // バグ調査の時はコメントアウトを外してください
    		} catch (IOException e) {
			System.out.println("※非対話式の場合はここに来ることはありません。");
		}

    	/* 状態管理オブジェクトの生成と初期状態のセット */
    	PartyInfo partyInfoP1 = new PartyInfo();
    	PartyInfo partyInfoP2 = new PartyInfo();
    	PokeStatePlace pokeStatePlaceP1 = new PokeStatePlace(args[0]);
    	PokeStatePlace pokeStatePlaceP2 = new PokeStatePlace(args[1]);
    	PokeStateBaton pokeStateBatonP1 = new PokeStateBaton(args[0]);
    	PokeStateBaton pokeStateBatonP2 = new PokeStateBaton(args[1]);
    	PokeStateWhole pokeStateWholeP1 = new PokeStateWhole(args[0]);
    	PokeStateWhole pokeStateWholeP2 = new PokeStateWhole(args[1]);
    	PlaceState placeState = new PlaceState();

    	/* 思考ルーチンクラスの定義 */
    	CommandRTN commandRTN = new MyCommandRTN(partyInfoP1,
    			                                   partyInfoP2,
    			                                   pokeStatePlaceP1,
    			                                   pokeStatePlaceP2,
    			                                   pokeStateBatonP1,
    			                                   pokeStateBatonP2,
    			                                   pokeStateWholeP1,
    			                                   pokeStateWholeP2,
    			                                   placeState,
    			                                   null,
    			                                   null,
    			                                   null,
    			                                   null,
    			                                   args[0],
    			                                   args[1]);

    	/* ターンコントロール窓口クラスの定義 */
    	TurnControlFacade turnControlFacade = new TurnControlFacade();

    	/* 前提条件定義 */
    	Precondition.endTurn = 20; // ターン制限
    	Precondition.recoilWhenEndTurnFlag = true; // 決着時の反動による引き分け有（完全タイマン想定時はfalse（無）にする）

    	/* 結果表示用変数定義 */
    	int winner;
    	int testTimes = 10000; //試行回数。バグ調査の時は少ない数字にしてください。
    	int[] P1Win = new int[Precondition.endTurn+1];
    	int[] P2Win = new int[Precondition.endTurn+1];
    	int[] draw = new int[Precondition.endTurn+1];
    	for (int i = 0;i < (Precondition.endTurn+1);i++) {
    		P1Win[i] = 0;
    		P2Win[i] = 0;
    		draw[i] = 0;
    	}

    	/* 指定された回数分試行 */
    	for (int i = 0;i < testTimes;i++) {

	    	// 勝敗判定
	    	winner = turnControlFacade.battleWhole(partyInfoP1,
                                                   partyInfoP2,
                                                   pokeStatePlaceP1,
                                                   pokeStatePlaceP2,
                                                   pokeStateBatonP1,
                                                   pokeStateBatonP2,
                                                   pokeStateWholeP1,
                                                   pokeStateWholeP2,
                                                   placeState,
                                                   commandRTN,
                                                   args[0],
                                                   args[1],
                                                   true);

	    	// バグ調査の時はコメントアウトを外してください
	    	// System.out.println("勝者：P"+winner);

	    	// 結果保存
	    	if (winner == 1) {
	    		P1Win[turnControlFacade.endTurn-1]++;
	    	} else if (winner == 2) {
	    		P2Win[turnControlFacade.endTurn-1]++;
	    	} else {
	    		draw[turnControlFacade.endTurn-1]++;
	    	}

    	}

    	/* メッセージ出力終了（今回は使用しない） */
    	try {
			DialogUtil.dialogEnd();
		} catch (IOException e) {
			System.out.println("非対話式の場合は通常ここに来ることはありません");
		}

    	/* 結果表示 */
    	int P1WinCount = 0;
    	for (int i = 0;i < (Precondition.endTurn+1);i++) {
    		P1WinCount = P1WinCount + P1Win[i];
    	}
    	int P2WinCount = 0;
    	for (int i = 0;i < (Precondition.endTurn+1);i++) {
    		P2WinCount = P2WinCount + P2Win[i];
    	}
    	int drawCount = 0;
    	for (int i = 0;i < (Precondition.endTurn+1);i++) {
    		drawCount = drawCount + draw[i];
    	}
    	System.out.println("【対戦内容】"); // 改造してルーチンを入れた場合はメッセージも直す
    	System.out.println(args[0]);
    	System.out.println(partyInfoP1.partyTextArray[0]);
    	System.out.println(args[1]);
    	System.out.println(partyInfoP2.partyTextArray[0]);
    	System.out.println("【結果】");
    	System.out.println(args[0]+"の勝率："+String.format("%.2f",(double)P1WinCount*100/testTimes)+"%");
    	System.out.println(args[1]+"の勝率："+String.format("%.2f",(double)P2WinCount*100/testTimes)+"%");
    	System.out.println("引き分けの勝率："+String.format("%.2f",(double)drawCount*100/testTimes)+"%");
    	System.out.println("【内訳】");
    	for (int i = 0;i < Precondition.endTurn;i++) {
    		System.out.println(args[0]+"が"+(i+1)+"ターン目に勝利する確率："+String.format("%.2f",(double)P1Win[i]*100/testTimes)+"%");
    	}
    	System.out.println(args[0]+"が"+Precondition.endTurn+"ターン経過後の判定で勝利する確率："+String.format("%.2f",(double)P1Win[Precondition.endTurn]*100/testTimes)+"%");
    	System.out.println("--------");
    	for (int i = 0;i < Precondition.endTurn;i++) {
    		System.out.println(args[1]+"が"+(i+1)+"ターン目に勝利する確率："+String.format("%.2f",(double)P2Win[i]*100/testTimes)+"%");
    	}
    	System.out.println(args[1]+"が"+Precondition.endTurn+"ターン経過後の判定で勝利する確率："+String.format("%.2f",(double)P2Win[Precondition.endTurn]*100/testTimes)+"%");
    	System.out.println("--------");
    	for (int i = 0;i < Precondition.endTurn;i++) {
    		System.out.println((i+1)+"ターン目に引き分けになる確率："+String.format("%.2f",(double)draw[i]*100/testTimes)+"%");
    	}
    	System.out.println(Precondition.endTurn+"ターン経過後の判定で引き分けになる確率："+String.format("%.2f",(double)draw[Precondition.endTurn]*100/testTimes)+"%");

    }

}
