package main;

import java.io.IOException;

import command.ChoiceCommand;
import command.ChoiceCommandP1;
import command.ChoiceCommandP2;
import command.CommandManualInput;
import command.CommandRTN;
import command.MoveChangeCommand;
import command.MoveChangeCommandP1;
import command.MoveChangeCommandP2;
import config.Precondition;
import facade.TurnControlFacade;
import facade.TurnControlFacadeWithComment;
import party.PartyInfo;
import state.PlaceState;
import state.PokeStateBaton;
import state.PokeStatePlace;
import state.PokeStateWhole;
import util.CommandScanner;
import util.DialogUtil;

/*
*  Sample4_PseudoBattleSupporterクラス
*
*  サンプルのメインクラスです。
*  一人で行う机上での対戦をサポートするためのツールであり、コマンド入力していくだけで机上戦が可能になります。
*  机上での対戦では考慮困難な運要素についても自動的に考慮できます。
*  ログも自動的に生成します。
*
*  変更履歴
*  2019/07/22 新規作成
*  2019/07/25 交代を行っていない状態でポケモンの種族名がメッセージに出力されなくなるバグを修正
*  2019/07/26 撒きびしで倒れた時の挙動が誤っていたのを修正
*  2019/07/27 片方のプレイヤーが交代、もう片方のプレイヤーが行動不能の場合、後者のプレイヤーが交代を迫られるバグを修正
*  2019/07/28 バトンタッチが通常の交代になるバグを修正
*  2019/08/06 毒で倒れた時の挙動が誤っていたのを修正
*  2019/08/25 初手の破壊の遺伝子が起動しないバグを修正
*  2020/03/29 相手を倒したターンに毒のダメージが発生するバグを修正
*  2020/08/11 交代のターンカットを実装、毒のダメージで倒れたターンに交代先にも毒のダメージが発生するバグを修正
*  2020/08/15 砂嵐で同時に倒れる場合にDROWにならないバグを修正
*  2020/08/22 ターン開始時に場に出ているポケモンのHPが表示されるように改善、全体・P1・P2のログを分けるように改善
*  2020/10/25 交代時に毒ダメージが発生しないバグを修正
*  2020/11/01 捨て身反動で引き分けの有無の設定を追加（当クラスでは引き分け無で設定）
*  2020/11/08 思考ルーチンクラスを取り込み
*  2020/11/22 ターンコントロール窓口クラスを取り込み
*
*/

public class Sample4_PseudoBattleSupporter {

    public static void main(String[] args) {

    	/* 対話式スイッチはON */
    	try {
    		DialogUtil.dialogSwitchSet(1);
    	} catch (IOException e) {
    		System.out.println("ログ出力に失敗したので終了します。");
    		System.exit(1);
    	}

    	/* コマンド入力のクラス定義 */
    	ChoiceCommand choiceCommandP1 = new ChoiceCommandP1();
    	ChoiceCommand choiceCommandP2 = new ChoiceCommandP2();
    	MoveChangeCommand moveChangeCommandP1 = new MoveChangeCommandP1();
    	MoveChangeCommand moveChangeCommandP2 = new MoveChangeCommandP2();

    	/* 状態管理オブジェクトの生成と初期状態のセット */
    	PartyInfo partyInfoP1 = new PartyInfo();
    	PartyInfo partyInfoP2 = new PartyInfo();
    	PokeStatePlace pokeStatePlaceP1 = new PokeStatePlace(args[0]);
    	PokeStatePlace pokeStatePlaceP2 = new PokeStatePlace(args[1]);
    	PokeStateBaton pokeStateBatonP1 = new PokeStateBaton(args[0]);
    	PokeStateBaton pokeStateBatonP2 = new PokeStateBaton(args[1]);
    	PokeStateWhole pokeStateWholeP1 = new PokeStateWhole(args[0]);
    	PokeStateWhole pokeStateWholeP2 = new PokeStateWhole(args[1]);
    	PlaceState placeState = new PlaceState();

    	/* 思考ルーチンクラスの定義 */
    	CommandRTN commandRTN = new CommandManualInput(partyInfoP1,
    			                                         partyInfoP2,
    			                                         pokeStatePlaceP1,
    			                                         pokeStatePlaceP2,
    			                                         pokeStateBatonP1,
    			                                         pokeStateBatonP2,
    			                                         pokeStateWholeP1,
    			                                         pokeStateWholeP2,
    			                                         placeState,
    			                                         choiceCommandP1,
    			                                         choiceCommandP2,
    			                                         moveChangeCommandP1,
    			                                         moveChangeCommandP2,
    			                                         args[0],
    			                                         args[1]);

    	/* ターンコントロール窓口クラスの定義 */
    	TurnControlFacade turnControlFacade = new TurnControlFacadeWithComment();

    	/* 前提条件定義 */
    	Precondition.endTurn = 80; // ターン制限
    	Precondition.recoilWhenEndTurnFlag = false; // 決着時の反動による引き分け無

    	/* 対戦 */
    	turnControlFacade.battleWhole(partyInfoP1,
                                      partyInfoP2,
                                      pokeStatePlaceP1,
                                      pokeStatePlaceP2,
                                      pokeStateBatonP1,
                                      pokeStateBatonP2,
                                      pokeStateWholeP1,
                                      pokeStateWholeP2,
                                      placeState,
                                      commandRTN,
                                      args[0],
                                      args[1],
                                      false);

     	/* メッセージ出力終了 */
     	try {
 			DialogUtil.dialogEnd();
 		} catch (IOException e) {
 			// 対戦終了しているので何もしない
 		}
     	CommandScanner.scanEnd();
		System.exit(0);

	}

}
