package main;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Scanner;

import mold.PBSItemIDs;
import mold.PBSMoveIDs;
import mold.PBSSpeciesIDs;

/*
*  Sample5_PBSPartyToPBSToolsPartyクラス
*
*  サンプルのメインクラスです。
*  PBSのパーティ構築ツールで生成したHTMLを、PBSToolsのパーティテキストに変換します。
*  原則として変換対象のファイル名は対話式で与えますが、第一引数で与えることも可能です。
*
*  変更履歴
*  2021/01/11 新規作成
*  2021/01/17 出力ファイルの文字コードが誤っていたのを修正
*
*/

public class Sample5_PBSPartyToPBSToolsParty {

	public static void main(String[] args) {

		/* 変数定義 */
		// スキャナー本体
		Scanner scanner = new Scanner(System.in);

		// パス定義
		String PBSForder = "sub_tools" + "\\" + "002_PBS_to_Conf" + "\\" + "PBS";
		String PBSToolsForder = "sub_tools" + "\\" + "002_PBS_to_Conf" + "\\" + "Conf";
		String fileName = "";
		String PBSFilePath;
		String PBSFilePathCandidate1;
		String PBSFilePathCandidate2;
		String PBSToolsFilePath;

		// ストリーム
		FileReader fr = null;
		BufferedWriter bw = null;

		// 読み取った部品名C
		String CStr = "";
		String[] CStrSplit;
		int CStrOffset = 1;
		int pokeNum;

		// IDクラスのオブジェクト生成
		PBSSpeciesIDs PBSSpeiciesIDsObj = new PBSSpeciesIDs();
		PBSMoveIDs PBSMoveIDsObj = new PBSMoveIDs();
		PBSItemIDs PBSItemIDsObj = new PBSItemIDs();

		// 生成したパーティテキストファイル
		ArrayList<String> partyTextStr = new ArrayList<>();

		/* 処理 */
		// パーティ名入力を求める（第一引数が無い場合）
		if (args.length == 0) {
			System.out.println("\"" + PBSForder + "\\\"にHTMLファイルを格納してください。");
			System.out.println("その後、HTMLファイル名（拡張子無し）をコンソールに入力して下さい。");
			fileName = scanner.nextLine();
		} else {
			fileName = args[0];
		}
		scanner.close();

		// パス存在確認
		PBSFilePathCandidate1 = PBSForder + "\\" + fileName + ".htm";
		PBSFilePathCandidate2 = PBSForder + "\\" + fileName + ".html";
		File filePath = new File(PBSFilePathCandidate1);
        if (filePath.exists()) {
        	PBSFilePath = PBSFilePathCandidate1;
        	System.out.println("【ファイル名：" + fileName + ".htm】");
        } else {
        	filePath = new File(PBSFilePathCandidate2);
            if (filePath.exists()) {
            	PBSFilePath = PBSFilePathCandidate2;
            	System.out.println("【ファイル名：" + fileName + ".html】");
            } else {
            	System.out.println("HTMLファイルが存在しないので処理を終了します。");System.out.println("");
            	return;
            }
        }

        // ファイル読み込み
        try {
			fr = new FileReader(PBSFilePath);
			int ch;
			String chStr;
			int state = 0;
			while ((ch = fr.read()) != -1){
				chStr = String.valueOf((char)ch);
				if (state == 0 && chStr.equals("C")) {
					state = 1;
				} else if (state == 1 && chStr.equals(" ")) {
					state = 2;
				} else if (state == 2 && chStr.equals("V")) {
					state = 3;
				}  else if (state == 3 && chStr.equals("A")) {
					state = 4;
				}  else if (state == 4 && chStr.equals("L")) {
					state = 5;
				}  else if (state == 5 && chStr.equals("U")) {
					state = 6;
				}  else if (state == 6 && chStr.equals("E")) {
					state = 7;
				}  else if (state == 7 && chStr.equals("=")) {
					state = 8;
				}  else if (state == 8 && chStr.equals("\"")) {
					state = 9;
				}  else if (state == 9 && !chStr.equals("\"")) {
					CStr = CStr + chStr;
				}  else {
					state = 0;
				}
			}
		} catch (FileNotFoundException e) {
        	System.out.println("HTMLファイルが存在しないので処理を終了します。");System.out.println("");
        	return;
		} catch (IOException e) {
        	System.out.println("不正なファイルが入力されたため処理を終了します。");System.out.println("");
        	return;
		} finally {
			if (fr != null) {
				try {
					fr.close();
				} catch (IOException e) {
				}
			}
		}

        // 部品名Cを分割
        CStrSplit = CStr.split("_", 0);
        if (CStrSplit.length == 28) {
        	pokeNum = 1;
        } else if (CStrSplit.length == 55) {
        	pokeNum = 2;
        } else if (CStrSplit.length == 82) {
        	pokeNum = 3;
        } else if (CStrSplit.length == 109) {
        	pokeNum = 4;
        } else if (CStrSplit.length == 136) {
        	pokeNum = 5;
        } else if (CStrSplit.length == 163) {
        	pokeNum = 6;
        } else {
        	System.out.println("不正なファイルが入力されたため処理を終了します。");System.out.println("");
        	return;
        }

        // パーティテキストファイル生成
        for (int i = 0; i < pokeNum; i++) {
        	for (int j = 8; j < 13; j++) {
        		if (Integer.parseInt(CStrSplit[j + CStrOffset - 1]) < 10) {
        			CStrSplit[j + CStrOffset - 1] = " " + CStrSplit[j + CStrOffset - 1];
        		}
        	}
            partyTextStr.add("|" +
            		         CStrSplit[3 + CStrOffset - 1] +
            		         "|" +
            		         PBSSpeiciesIDsObj.SpeciesGet(CStrSplit[2 + CStrOffset - 1]) +
            		         "|" +
            		         PBSMoveIDsObj.MoveGet(CStrSplit[13 + CStrOffset - 1]) +
            		         "|" +
            		         CStrSplit[21 + CStrOffset - 1] +
            		         "|" +
            		         PBSMoveIDsObj.MoveGet(CStrSplit[14 + CStrOffset - 1]) +
            		         "|" +
            		         CStrSplit[22 + CStrOffset - 1] +
            		         "|" +
            		         PBSMoveIDsObj.MoveGet(CStrSplit[15 + CStrOffset - 1]) +
            		         "|" +
            		         CStrSplit[23 + CStrOffset - 1] +
            		         "|" +
            		         PBSMoveIDsObj.MoveGet(CStrSplit[16 + CStrOffset - 1]) +
            		         "|" +
            		         CStrSplit[24 + CStrOffset - 1] +
            		         "|" +
            		         PBSItemIDsObj.ItemGet(CStrSplit[27 + CStrOffset - 1]) +
            		         "|" +
            		         CStrSplit[4 + CStrOffset - 1] +
            		         CStrSplit[5 + CStrOffset - 1] +
            		         CStrSplit[6 + CStrOffset - 1] +
            		         CStrSplit[7 + CStrOffset - 1] +
            		         "|" +
            		         CStrSplit[8 + CStrOffset - 1] +
            		         "-" +
            		         CStrSplit[9 + CStrOffset - 1] +
            		         "-" +
            		         CStrSplit[10 + CStrOffset - 1] +
            		         "-" +
            		         CStrSplit[11 + CStrOffset - 1] +
            		         "-" +
            		         CStrSplit[12 + CStrOffset - 1] +
            		         "|");
            CStrOffset = CStrOffset + 27;
        }

        // ファイル書き込み
        try {
        	PBSToolsFilePath = PBSToolsForder + "\\" + fileName + ".txt";
			bw = new BufferedWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(PBSToolsFilePath), "SJIS")));
	        for (int i = 0; i < pokeNum; i++) {
	        	bw.write(partyTextStr.get(i));
	        	bw.newLine();
	        }
		} catch (IOException e) {
        	System.out.println("ファイルを出力できないため処理を終了します。");System.out.println("");
        	return;
		} finally {
			if (bw != null) {
				try {
					bw.close();
				} catch (IOException e) {
				}
			}
		}

        // 処理終了
        System.out.println("\"" + PBSToolsForder + "\\\"にパーティテキストファイルを出力しました。");System.out.println("");
        return;

	}

}
