package main;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.Scanner;

import mold.PBSItemIDs;
import mold.PBSMoveIDs;
import mold.PBSSpeciesIDs;
import party.PartyInfo;

/*
*  Sample6_PBSToolsPartyToPBSPartyクラス
*
*  サンプルのメインクラスです。
*  PBSToolsのパーティテキストを、PBSのパーティ構築ツールのHTMLに変換します。
*  原則として変換対象のファイル名は対話式で与えますが、第一引数で与えることも可能です。
*
*  変更履歴
*  2021/01/17 新規作成
*
*/

public class Sample6_PBSToolsPartyToPBSParty {

	public static void main(String[] args) {

		/* 変数定義 */
		// スキャナー本体
		Scanner scanner = new Scanner(System.in);

		// パス定義
		String PBSForder = "sub_tools" + "\\" + "003_Conf_to_PBS" + "\\" + "PBS";
		String PBSToolsForder = "sub_tools" + "\\" + "003_Conf_to_PBS" + "\\" + "Conf";
		String fileName = "";
		String PBSFilePath;
		String PBSToolsFilePath;

		// パーティテキストファイル処理用オブジェクト
		PartyInfo partyInfo = new PartyInfo();

		// IDクラスのオブジェクト生成
		PBSSpeciesIDs PBSSpeiciesIDsObj = new PBSSpeciesIDs();
		PBSMoveIDs PBSMoveIDsObj = new PBSMoveIDs();
		PBSItemIDs PBSItemIDsObj = new PBSItemIDs();

		// 部品名C
		String CStr;

		// ストリーム
		BufferedWriter bw = null;

		/* 処理 */
		// パーティ名入力を求める（第一引数が無い場合）
		if (args.length == 0) {
			System.out.println("\"" + PBSToolsForder + "\\\"にパーティテキストファイルを格納してください。");
			System.out.println("その後、パーティテキストファイル名（拡張子無し）をコンソールに入力して下さい。");
			fileName = scanner.nextLine();
		} else {
			fileName = args[0];
		}
		scanner.close();
		System.out.println("【ファイル名：" + fileName + ".txt】");

		// パス存在確認
		PBSToolsFilePath = PBSToolsForder + "\\" + fileName + ".txt";
		File file = new File(PBSToolsFilePath);
        if (!file.exists()) {
        	System.out.println("パーティテキストファイルが存在しないので処理を終了します。");System.out.println("");
        	return;
        }

        // ファイル読み込み
        BufferedReader br = null;
        try {
        	br = new BufferedReader(new InputStreamReader(new FileInputStream(file),"SJIS"));
        	String str = br.readLine();
        	int i = 0;
        	if(str == null) {
        		throw new NullPointerException();
        	} else {
        		while(str != null && i < 6){
        			partyInfo.partyTextArray[i] = str;
        			i++;
        			str = br.readLine();
        		}
        	}
        } catch (Exception e) {
        	System.out.println("パーティテキストファイルを読み込めないので処理を終了します。");
        } finally {
        	try {
				br.close();
			} catch (IOException e) {
			}
        }
        partyInfo.partyPokeMoldSet();

        // 部品名C生成
        CStr = "1";
		for (int i = 0; i < 6; i++) {
			if (partyInfo.pokeMoldArray[i] != null) {
				CStr = CStr + "_" + "x";
				CStr = CStr + "_" + PBSSpeiciesIDsObj.IDGet(partyInfo.pokeMoldArray[i].speciesName);
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].level;
				CStr = CStr + "_" + intToHex(partyInfo.pokeMoldArray[i].attackIndividualValue);
				CStr = CStr + "_" + intToHex(partyInfo.pokeMoldArray[i].blockIndividualValue);
				CStr = CStr + "_" + intToHex(partyInfo.pokeMoldArray[i].speedIndividualValue);
				CStr = CStr + "_" + intToHex(partyInfo.pokeMoldArray[i].specialIndividualValue);
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].HPEffortLevel;
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].attackEffortLevel;
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].blockEffortLevel;
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].speedEffortLevel;
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].specialEffortLevel;
				CStr = CStr + "_" + PBSMoveIDsObj.IDGet(partyInfo.pokeMoldArray[i].moveName[0]);
				CStr = CStr + "_" + PBSMoveIDsObj.IDGet(partyInfo.pokeMoldArray[i].moveName[1]);
				CStr = CStr + "_" + PBSMoveIDsObj.IDGet(partyInfo.pokeMoldArray[i].moveName[2]);
				CStr = CStr + "_" + PBSMoveIDsObj.IDGet(partyInfo.pokeMoldArray[i].moveName[3]);
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].movePPMAX1;
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].movePPMAX2;
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].movePPMAX3;
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].movePPMAX4;
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].movePointUpTimes[0];
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].movePointUpTimes[1];
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].movePointUpTimes[2];
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].movePointUpTimes[3];
				CStr = CStr + "_" + partyInfo.pokeMoldArray[i].MAXHP;
				CStr = CStr + "_" + "0";
				CStr = CStr + "_" + PBSItemIDsObj.IDGet(partyInfo.pokeMoldArray[i].itemName);
			}
		}

        // ファイル書き込み
        try {
        	PBSFilePath = PBSForder + "\\" + fileName + ".htm";
			bw = new BufferedWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(PBSFilePath), "EUC-JP")));
			bw.write("<HTML>");bw.newLine();
			bw.write("  <HEAD>");bw.newLine();
			bw.write("    <META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html;charset=EUC-JP\">");bw.newLine();
			bw.write("    <TITLE>" + fileName + "</TITLE>");bw.newLine();
			bw.write("  </HEAD>");bw.newLine();
			bw.write("  <BODY BGCOLOR=\"snow\">");bw.newLine();
			bw.write("    <h1 STYLE=\"COLOR:Goldenrod\">" + fileName + " Generated by PBSTools</h1>");bw.newLine();
			bw.write("    <table border>");bw.newLine();
			bw.write("      <tr>");bw.newLine();
			bw.write("        <th>LV</th>");bw.newLine();
			bw.write("        <th>種族</th>");bw.newLine();
			bw.write("        <th>技１</th>");bw.newLine();
			bw.write("        <th>技２</th>");bw.newLine();
			bw.write("        <th>技３</th>");bw.newLine();
			bw.write("        <th>技４</th>");bw.newLine();
			bw.write("        <th>持ち物</th>");bw.newLine();
			bw.write("        <th>個体値</th>");bw.newLine();
			bw.write("        <th>努力値</th>");bw.newLine();
			bw.write("        <th>PP</th>");bw.newLine();
			bw.write("        <th>性</th>");bw.newLine();
			bw.write("        <th>実HP</th>");bw.newLine();
			bw.write("        <th>実す</th>");bw.newLine();
			bw.write("        <th>めざ色/威</th>");bw.newLine();
			bw.write("      </tr>");bw.newLine();
			for (int i = 0; i < 6; i++) {
				if (partyInfo.pokeMoldArray[i] != null) {
					bw.write("      <tr ALIGN=\"center\">");bw.newLine();
					bw.write("        <td>" + partyInfo.pokeMoldArray[i].level + "</td>");bw.newLine();
					bw.write("        <td>" + partyInfo.pokeMoldArray[i].speciesName + "</td>");bw.newLine();
					bw.write("        <td>" + emptyToHyphen(partyInfo.pokeMoldArray[i].moveName[0]) + "</td>");bw.newLine();
					bw.write("        <td>" + emptyToHyphen(partyInfo.pokeMoldArray[i].moveName[1]) + "</td>");bw.newLine();
					bw.write("        <td>" + emptyToHyphen(partyInfo.pokeMoldArray[i].moveName[2]) + "</td>");bw.newLine();
					bw.write("        <td>" + emptyToHyphen(partyInfo.pokeMoldArray[i].moveName[3]) + "</td>");bw.newLine();
					bw.write("        <td>" + emptyToHyphen(partyInfo.pokeMoldArray[i].itemName) + "</td>");bw.newLine();
					bw.write("        <td>" + intToHex(partyInfo.pokeMoldArray[i].attackIndividualValue) + intToHex(partyInfo.pokeMoldArray[i].blockIndividualValue) + intToHex(partyInfo.pokeMoldArray[i].speedIndividualValue) + intToHex(partyInfo.pokeMoldArray[i].specialIndividualValue) +"</td>");bw.newLine();
					bw.write("        <td>" + intToStrLen2(partyInfo.pokeMoldArray[i].HPEffortLevel) + "-" + intToStrLen2(partyInfo.pokeMoldArray[i].attackEffortLevel) + "-" + intToStrLen2(partyInfo.pokeMoldArray[i].blockEffortLevel) + "-" + intToStrLen2(partyInfo.pokeMoldArray[i].speedEffortLevel) + "-" + intToStrLen2(partyInfo.pokeMoldArray[i].specialEffortLevel) + "</td>");bw.newLine();
					bw.write("        <td>" + intToStrLen2(partyInfo.pokeMoldArray[i].movePPMAX1) + "/" + intToStrLen2(partyInfo.pokeMoldArray[i].movePPMAX2) + "/" + intToStrLen2(partyInfo.pokeMoldArray[i].movePPMAX3) + "/" + intToStrLen2(partyInfo.pokeMoldArray[i].movePPMAX4) + "</td>");bw.newLine();
					bw.write("        <td>" + sex_EngToJpn(partyInfo.pokeMoldArray[i].sex) + "</td>");bw.newLine();
					bw.write("        <td>" + partyInfo.pokeMoldArray[i].MAXHP + "</td>");bw.newLine();
					bw.write("        <td>" + partyInfo.pokeMoldArray[i].speed + "</td>");bw.newLine();
					if (partyInfo.pokeMoldArray[i].moveName[0].equals("めざめるパワー") || partyInfo.pokeMoldArray[i].moveName[1].equals("めざめるパワー") || partyInfo.pokeMoldArray[i].moveName[2].equals("めざめるパワー") || partyInfo.pokeMoldArray[i].moveName[3].equals("めざめるパワー")) {
						bw.write("        <td>めざ" + type_EngToJpn(partyInfo.pokeMoldArray[i].hiddenPowerType) + "/" + partyInfo.pokeMoldArray[i].hiddenPowerPower + "</td>");bw.newLine();
					} else {
						bw.write("        <td>-</td>");bw.newLine();
					}
					bw.write("      </tr>");bw.newLine();
				}
			}
			bw.write("    </table>");bw.newLine();
			bw.write("    <p>");bw.newLine();
			bw.write("      <FORM action=\"http://psense.lib.net/_/PDINPUT.cgi\" method=\"post\">");bw.newLine();
			bw.write("        <INPUT TYPE=HIDDEN NAME=ACTION VALUE=RECEPT>");bw.newLine();
			bw.write("        <INPUT TYPE=HIDDEN NAME=C VALUE=\"" + CStr + "\">");bw.newLine();
			bw.write("        <INPUT TYPE=submit value=\"再現\">");bw.newLine();
			bw.write("      </FORM>");bw.newLine();
			bw.write("    </p>");bw.newLine();
			bw.write("  </body>");bw.newLine();
			bw.write("</html>");
		} catch (IOException e) {
        	System.out.println("ファイルを出力できないため処理を終了します。");System.out.println("");
        	return;
		} finally {
			if (bw != null) {
				try {
					bw.close();
				} catch (IOException e) {
				}
			}
		}

        // 処理終了
        System.out.println("\"" + PBSForder + "\\\"にHTMLファイルを出力しました。");System.out.println("");
        return;

	}

	/* ハイフン化 */
    static public String emptyToHyphen(String str) {
    	if (str.equals("技無し") || str.equals("持ち物無し")) {
    		return "-";
    	} else {
    		return str;
    	}
    }

    /* 10進数→16進数 */
    static public String intToHex(int intVal) {
    	switch (intVal) {
    	case 0:
    	    return "0";
		case 1:
    	    return "1";
		case 2:
    	    return "2";
		case 3:
    	    return "3";
		case 4:
    	    return "4";
		case 5:
    	    return "5";
		case 6:
    	    return "6";
		case 7:
    	    return "7";
		case 8:
    	    return "8";
		case 9:
    	    return "9";
		case 10:
    	    return "A";
		case 11:
    	    return "B";
		case 12:
    	    return "C";
		case 13:
    	    return "D";
		case 14:
    	    return "E";
		case 15:
    	    return "F";
		default:
    		return "0";
    	}
    }

	/* 数値→2桁文字列 */
    static public String intToStrLen2(int intVal) {
    	if (intVal < 10) {
    		return " " + String.valueOf(intVal);
    	} else {
    		return String.valueOf(intVal);
    	}
    }

	/* 性別　英語→日本語 */
    static public String sex_EngToJpn(String eng) {
    	switch (eng) {
    	case "male":
    	    return "♂";
		case "female":
    	    return "♀";
		default:
    		return "無";
    	}
    }

	/* タイプ　英語→日本語 */
    static public String type_EngToJpn(String eng) {
    	switch (eng) {
    	case "dark":
    	    return "悪";
		case "dragon":
    	    return "龍";
		case "ice":
    	    return "氷";
		case "psychic":
    	    return "超";
		case "electric":
    	    return "電";
		case "grass":
    	    return "草";
		case "water":
    	    return "水";
		case "fire":
    	    return "炎";
		case "steel":
    	    return "鋼";
		case "ghost":
    	    return "霊";
		case "bug":
    	    return "虫";
		case "rock":
    	    return "岩";
		case "ground":
    	    return "地";
		case "poison":
    	    return "毒";
		case "flying":
    	    return "飛";
		case "fighting":
    	    return "格";
		default:
    		return "ノ";
    	}
    }

}
