package mold;

import java.util.HashMap;

/*
*  PokeMoldクラス
*
*  ポケモンの型を管理するクラスです。
*
*  変更履歴
*  2019/04/13 新規作成
*  2019/04/13 タイプ「???」は「question」と表記するように変更（?はプログラム上意味を持つ文字のため後々不便になる）
*  2019/04/14 「ダブルニードル」「いびき」の追加効果発動率の誤りを訂正
*  2019/04/21 変数「movePPPrimitive」を追加（「ものまね」で使用する）
*  2019/05/03 技「交代」を追加（実際に技として使うことはないが、実装上の都合）
*  2019/05/03 2019/04/13の修正漏れ対応
*  2019/05/04 技「混乱自滅」を追加（実際に技として使うことはないが、実装上の都合）
*  2019/05/04 ポイントアップ使用後のPPの計算で明示的に切り捨て処理を行う（PPが5の倍数ではないスケッチへの対応）
*  2019/05/06 技コード"10"について、影分身("10a")と小さくなる("10b")に分割（小さくなる状態になるかならないかの違い）
*  2019/05/06 技コード"1C"について、吹き飛ばし("1Ca")と吠える("1Cb")に分割（空中の相手に当たるか否かの違い）
*  2019/05/06 「うずしお」のタイプ誤りを訂正
*  2019/05/06 「おんがえし」「やつあたり」を威力102の普通の攻撃技として登録（PBSの仕様に従う）
*  2019/05/26 「技無し」の技コードを"XXc"として設定
*  2019/07/15 初期化のため、PPの最大値の情報を追加
*  2019/07/22 全ての種族について性別が設定されないバグを修正
*  2019/07/27 「技無し」の技コードを「XXc」に統一
*  2019/09/30 「どくばり」の威力が誤っていたバグを修正
*  2020/06/14 持ち物精査の結果を反映（「おおきなキノコ」「しあわせタマゴ」「ちいさなキノコ」追加、「やめる」削除）
*  2020/07/18 「ころがる」のPPが誤っていたバグを修正
*  2020/08/25 あくむ・きあいだめ・せいちょう・バトンタッチのタイプの誤りを修正
*             いたみわけ・こなゆき・たたきつける・テクスチャーのPPの誤りを修正
*             うずしお・ソニックブームの命中率の誤りを修正
*             スパークの威力の誤りを修正
*  2020/10/29 持ち物の「どくバリ」が「どくばり」になっていたのを修正
*  2021/01/11 持ち物の「くろいめがね」が「くろいメガネ」となっていたのを修正（PBSに合わせる）
*             持ち物の長音記号が統一されていなかったのを訂正（影響は効果無しの持ち物のみなので実質的に影響無し）
*
*/

public class PokeMold {

	/* パーティテキストから取得 */
	public int level = 50; //レベル
	public String speciesName = "けつばん"; //種族名
	public String[] moveName = {"技無し","技無し","技無し","技無し"}; //技名
	public int[] movePointUpTimes = {0,0,0,0}; //ポイントアップ使用回数
	public String itemName = "持ち物無し"; //持ち物
	public int attackIndividualValue = 0; //攻撃個体値
	public int blockIndividualValue = 0; //防御個体値
	public int speedIndividualValue = 0; //素早さ個体値
	public int specialIndividualValue = 0; //特殊個体値
	public int HPEffortLevel = 0; //HP努力レベル
	public int attackEffortLevel = 0; //攻撃努力レベル
	public int blockEffortLevel = 0; //防御努力レベル
	public int speedEffortLevel = 0; //素早さ努力レベル
	public int specialEffortLevel = 0; //特殊努力レベル

	/* 種族から導出する情報 */
	public String type1 = "normal"; //タイプ１
	public String type2 = "none"; //タイプ２
	public int HPSpeciesValue = 0; //HP種族値
	public int attackSpeciesValue = 0; //攻撃種族値
	public int blockSpeciesValue = 0; //防御種族値
	public int contactSpeciesValue = 0; //特攻種族値
	public int diffenceSpeciesValue = 0; //特防種族値
	public int speedSpeciesValue = 0; //素早さ種族値
	public boolean sexExistenceFlag = false; //性別存在フラグ
	public int femaleRatio = 8; //メスの割合（16分率）

	/* ステータス計算により導出する情報 */
	public int HPIndividualValue = 0; //HP個体値
	public int MAXHP = 1; //最大HP
	public int attack = 1; //攻撃実数値
	public int block = 1; //防御実数値
	public int contact = 1; //特攻実数値
	public int diffence = 1; //特防実数値
	public int speed = 1; //素早さ実数値

	/* 性別判定により導出する情報 */
	public String sex = "none"; //性別

	/* めざめるパワー判定により導出する情報 */
	public String hiddenPowerType = "dark"; //めざめるパワーのタイプ
	public int hiddenPowerPower = 31; //めざめるパワーの威力

	/* 技名・ポイントアップ使用回数から導出する情報 */
	public String[] moveType = {"question","question","question","question"}; //技のタイプ
	public int[] movePower = {1,1,1,1}; //技の威力
	public double[] moveHitProbability = {100.0,100.0,100.0,100.0}; //技の命中率(百分率の概算値)
	public int[] movePP = {0,0,0,0}; //PP
	public int movePPMAX1 = 0; //PPMAX(１つ目の技)
	public int movePPMAX2 = 0; //PPMAX(２つ目の技)
	public int movePPMAX3 = 0; //PPMAX(３つ目の技)
	public int movePPMAX4 = 0; //PPMAX(４つ目の技)
	public int[] movePPPrimitive = {0,0,0,0}; //ポイントアップを使用しない場合の最大PP
	public double[] additionalEffectProbability = {0.0,0.0,0.0,0.0}; //追加効果発動率(百分率の概算値)
	public String[] moveEffectCode = {"XXc","XXc","XXc","XXc"}; //技の効果コード
	                                                             //http://pokemon.s20.xrea.com/2nd/movescode.html

	/* 持ち物妥当性確認により導出する情報 */
	/* 特に無し。不正な持ち物を持たされていた場合のみ変数「itemName」を書き換える。 */

	/* デフォルトコンストラクタ：何もしない
	 * 　　　　　　　　　　　　　（ステータス計算等の一部機能のみ使用する場合はこちらのコンストラクタを使用）
	 */
    public PokeMold() {
    }

	/* コンストラクタ：パーティテキストから各項目を取得し、導出可能な各種情報をセットする
	 * 　　　　　　　　（ダメージ計算等で全機能を使用する場合はこちらのコンストラクタを使用）
	 *   引数　：パーティテキスト１行
	 */
    public PokeMold(String partyText) {
    	partyTextSplit(partyText);
    	speciesInfoSet();
    	statusCalc();
    	sexSet();
    	hiddenPowerSet();
    	moveSet();
    	itemCheck();
    }

	/* パーティテキストから各項目を取得するメソッド
	 *   引数　：パーティテキスト１行
	 *   戻り値：無し
	 *   例外　：無し（フォーマット不正があっても例外を出さずに初期値採用で先に進む）
	 */
    public void partyTextSplit(String partyText) {

    	/* パーティテキストをフォーマットに従い分割 */
    	String[] partyTextSplit1 = partyText.split("\\|", 0);
    	String[] partyTextSplit2 = partyTextSplit1[12].split("", 0);
    	String[] partyTextSplit3 = partyTextSplit1[13].split("-", 0);

    	/* 各項目を格納 */
    	if (partyTextSplit1[1] != null){
    		level = Integer.parseInt(partyTextSplit1[1]);
    	}
    	if (partyTextSplit1[2] != null){
    		speciesName = partyTextSplit1[2];
    	}
    	if (partyTextSplit1[3] != null){
    		moveName[0] = partyTextSplit1[3];
    	}
    	if (partyTextSplit1[4] != null){
    		movePointUpTimes[0] = Integer.parseInt(partyTextSplit1[4]);
    	}
    	if (partyTextSplit1[5] != null){
    		moveName[1] = partyTextSplit1[5];
    	}
    	if (partyTextSplit1[6] != null){
    		movePointUpTimes[1] = Integer.parseInt(partyTextSplit1[6]);
    	}
    	if (partyTextSplit1[7] != null){
    		moveName[2] = partyTextSplit1[7];
    	}
    	if (partyTextSplit1[8] != null){
    		movePointUpTimes[2] = Integer.parseInt(partyTextSplit1[8]);
    	}
    	if (partyTextSplit1[9] != null){
    		moveName[3] = partyTextSplit1[9];
    	}
    	if (partyTextSplit1[10] != null){
    		movePointUpTimes[3] = Integer.parseInt(partyTextSplit1[10]);
    	}
    	if (partyTextSplit1[11] != null){
    		itemName = partyTextSplit1[11];
    	}
    	if (partyTextSplit2[0] != null){
    		attackIndividualValue = hexToInt(partyTextSplit2[0]);
    	}
    	if (partyTextSplit2[1] != null){
    		blockIndividualValue = hexToInt(partyTextSplit2[1]);
    	}
    	if (partyTextSplit2[2] != null){
    		speedIndividualValue = hexToInt(partyTextSplit2[2]);
    	}
    	if (partyTextSplit2[3] != null){
    		specialIndividualValue = hexToInt(partyTextSplit2[3]);
    	}
    	if (partyTextSplit3[0] != null){
    		HPEffortLevel = Integer.parseInt(partyTextSplit3[0].trim());
    	}
    	if (partyTextSplit3[1] != null){
    		attackEffortLevel = Integer.parseInt(partyTextSplit3[1].trim());
    	}
    	if (partyTextSplit3[2] != null){
    		blockEffortLevel = Integer.parseInt(partyTextSplit3[2].trim());
    	}
    	if (partyTextSplit3[3] != null){
    		speedEffortLevel = Integer.parseInt(partyTextSplit3[3].trim());
    	}
    	if (partyTextSplit3[4] != null){
    		specialEffortLevel = Integer.parseInt(partyTextSplit3[4].trim());
    	}
    }

    /* 16進数→10進数(partyTextSplitメソッドで内部的に使用する関数) */
    public int hexToInt(String hex) {
    	switch (hex) {
    	case "0":
    	    return 0;
		case "1":
    	    return 1;
		case "2":
    	    return 2;
		case "3":
    	    return 3;
		case "4":
    	    return 4;
		case "5":
    	    return 5;
		case "6":
    	    return 6;
		case "7":
    	    return 7;
		case "8":
    	    return 8;
		case "9":
    	    return 9;
		case "A":
    	    return 10;
		case "B":
    	    return 11;
		case "C":
    	    return 12;
		case "D":
    	    return 13;
		case "E":
    	    return 14;
		case "F":
    	    return 15;
		default:
    		return 0;
    	}
    }

	/* 種族名から各種情報を取得するメソッド
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：無し（種族名に不正があっても例外を出さずに初期値採用で先に進む）
	 */
    public void speciesInfoSet() {
    	/* 種族の各種情報の定義 */
    	HashMap<String, String> speciesMap = new HashMap<String, String>();
    	speciesMap.put("フシギダネ", "grass|poison|45|49|49|65|65|45|1|2");
    	speciesMap.put("フシギソウ", "grass|poison|60|62|63|80|80|60|1|2");
    	speciesMap.put("フシギバナ", "grass|poison|80|82|83|100|100|80|1|2");
    	speciesMap.put("ヒトカゲ", "fire|none|39|52|43|60|50|65|1|2");
    	speciesMap.put("リザード", "fire|none|58|64|58|80|65|80|1|2");
    	speciesMap.put("リザードン", "fire|flying|78|84|78|109|85|100|1|2");
    	speciesMap.put("ゼニガメ", "water|none|44|48|65|50|64|43|1|2");
    	speciesMap.put("カメール", "water|none|59|63|80|65|80|58|1|2");
    	speciesMap.put("カメックス", "water|none|79|83|100|85|105|78|1|2");
    	speciesMap.put("キャタピー", "bug|none|45|30|35|20|20|45|1|8");
    	speciesMap.put("トランセル", "bug|none|50|20|55|25|25|30|1|8");
    	speciesMap.put("バタフリー", "bug|flying|60|45|50|80|80|70|1|8");
    	speciesMap.put("ビードル", "bug|poison|40|35|30|20|20|50|1|8");
    	speciesMap.put("コクーン", "bug|poison|45|25|50|25|25|35|1|8");
    	speciesMap.put("スピアー", "bug|poison|65|80|40|45|80|75|1|8");
    	speciesMap.put("ポッポ", "normal|flying|40|45|40|35|35|56|1|8");
    	speciesMap.put("ピジョン", "normal|flying|63|60|55|50|50|71|1|8");
    	speciesMap.put("ピジョット", "normal|flying|83|80|75|70|70|91|1|8");
    	speciesMap.put("コラッタ", "normal|none|30|56|35|25|35|72|1|8");
    	speciesMap.put("ラッタ", "normal|none|55|81|60|50|70|97|1|8");
    	speciesMap.put("オニスズメ", "normal|flying|40|60|30|31|31|70|1|8");
    	speciesMap.put("オニドリル", "normal|flying|65|90|65|61|61|100|1|8");
    	speciesMap.put("アーボ", "poison|none|35|60|44|40|54|55|1|8");
    	speciesMap.put("アーボック", "poison|none|60|85|69|65|79|80|1|8");
    	speciesMap.put("ピカチュウ", "electric|none|35|55|30|50|40|90|1|8");
    	speciesMap.put("ライチュウ", "electric|none|60|90|55|90|80|100|1|8");
    	speciesMap.put("サンド", "ground|none|50|75|85|20|30|40|1|8");
    	speciesMap.put("サンドパン", "ground|none|75|100|110|45|55|65|1|8");
    	speciesMap.put("ニドラン♀", "poison|none|55|47|52|40|40|41|1|16");
    	speciesMap.put("ニドリーナ", "poison|none|70|62|67|55|55|56|1|16");
    	speciesMap.put("ニドクイン", "poison|ground|90|82|87|75|85|76|1|16");
    	speciesMap.put("ニドラン♂", "poison|none|46|57|40|40|40|50|1|0");
    	speciesMap.put("ニドリーノ", "poison|none|61|72|57|55|55|65|1|0");
    	speciesMap.put("ニドキング", "poison|ground|81|92|77|85|75|85|1|0");
    	speciesMap.put("ピッピ", "normal|none|70|45|48|60|65|35|1|12");
    	speciesMap.put("ピクシー", "normal|none|95|70|73|85|90|60|1|12");
    	speciesMap.put("ロコン", "fire|none|38|41|40|50|65|65|1|12");
    	speciesMap.put("キュウコン", "fire|none|73|76|75|81|100|100|1|12");
    	speciesMap.put("プリン", "normal|none|115|45|20|45|25|20|1|12");
    	speciesMap.put("プクリン", "normal|none|140|70|45|75|50|45|1|12");
    	speciesMap.put("ズバット", "poison|flying|40|45|35|30|40|55|1|8");
    	speciesMap.put("ゴルバット", "poison|flying|75|80|70|65|75|90|1|8");
    	speciesMap.put("ナゾノクサ", "grass|poison|45|50|55|75|65|30|1|8");
    	speciesMap.put("クサイハナ", "grass|poison|60|65|70|85|75|40|1|8");
    	speciesMap.put("ラフレシア", "grass|poison|75|80|85|100|90|50|1|8");
    	speciesMap.put("パラス", "bug|grass|35|70|55|45|55|25|1|8");
    	speciesMap.put("パラセクト", "bug|grass|60|95|80|60|80|30|1|8");
    	speciesMap.put("コンパン", "bug|poison|60|55|50|40|55|45|1|8");
    	speciesMap.put("モルフォン", "bug|poison|70|65|60|90|75|90|1|8");
    	speciesMap.put("ディグダ", "ground|none|10|55|25|35|45|95|1|8");
    	speciesMap.put("ダグトリオ", "ground|none|35|80|50|50|70|120|1|8");
    	speciesMap.put("ニャース", "normal|none|40|45|35|40|40|90|1|8");
    	speciesMap.put("ペルシアン", "normal|none|65|70|60|65|65|115|1|8");
    	speciesMap.put("コダック", "water|none|50|52|48|65|50|55|1|8");
    	speciesMap.put("ゴルダック", "water|none|80|82|78|95|80|85|1|8");
    	speciesMap.put("マンキー", "fighting|none|40|80|35|35|45|70|1|8");
    	speciesMap.put("オコリザル", "fighting|none|65|105|60|60|70|95|1|8");
    	speciesMap.put("ガーディ", "fire|none|55|70|45|70|50|60|1|4");
    	speciesMap.put("ウインディ", "fire|none|90|110|80|100|80|95|1|4");
    	speciesMap.put("ニョロモ", "water|none|40|50|40|40|40|90|1|8");
    	speciesMap.put("ニョロゾ", "water|none|65|65|65|50|50|90|1|8");
    	speciesMap.put("ニョロボン", "water|fighting|90|85|95|70|90|70|1|8");
    	speciesMap.put("ケーシィ", "psychic|none|25|20|15|105|55|90|1|4");
    	speciesMap.put("ユンゲラー", "psychic|none|40|35|30|120|70|105|1|4");
    	speciesMap.put("フーディン", "psychic|none|55|50|45|135|85|120|1|4");
    	speciesMap.put("ワンリキー", "fighting|none|70|80|50|35|35|35|1|4");
    	speciesMap.put("ゴーリキー", "fighting|none|80|100|70|50|60|45|1|4");
    	speciesMap.put("カイリキー", "fighting|none|90|130|80|65|85|55|1|4");
    	speciesMap.put("マダツボミ", "grass|poison|50|75|35|70|30|40|1|8");
    	speciesMap.put("ウツドン", "grass|poison|65|90|50|85|45|55|1|8");
    	speciesMap.put("ウツボット", "grass|poison|80|105|65|100|60|70|1|8");
    	speciesMap.put("メノクラゲ", "water|poison|40|40|35|50|100|70|1|8");
    	speciesMap.put("ドククラゲ", "water|poison|80|70|65|80|120|100|1|8");
    	speciesMap.put("イシツブテ", "rock|ground|40|80|100|30|30|20|1|8");
    	speciesMap.put("ゴローン", "rock|ground|55|95|115|45|45|35|1|8");
    	speciesMap.put("ゴローニャ", "rock|ground|80|110|130|55|65|45|1|8");
    	speciesMap.put("ポニータ", "fire|none|50|85|55|65|65|90|1|8");
    	speciesMap.put("ギャロップ", "fire|none|65|100|70|80|80|105|1|8");
    	speciesMap.put("ヤドン", "water|psychic|90|65|65|40|40|15|1|8");
    	speciesMap.put("ヤドラン", "water|psychic|95|75|110|100|80|30|1|8");
    	speciesMap.put("コイル", "electric|steel|25|35|70|95|55|45|0|8");
    	speciesMap.put("レアコイル", "electric|steel|50|60|95|120|70|70|0|8");
    	speciesMap.put("カモネギ", "normal|flying|52|65|55|58|62|60|1|8");
    	speciesMap.put("ドードー", "normal|flying|35|85|45|35|35|75|1|8");
    	speciesMap.put("ドードリオ", "normal|flying|60|110|70|60|60|100|1|8");
    	speciesMap.put("パウワウ", "water|none|65|45|55|45|70|45|1|8");
    	speciesMap.put("ジュゴン", "water|ice|90|70|80|70|95|70|1|8");
    	speciesMap.put("ベトベター", "poison|none|80|80|50|40|50|25|1|8");
    	speciesMap.put("ベトベトン", "poison|none|105|105|75|65|100|50|1|8");
    	speciesMap.put("シェルダー", "water|none|30|65|100|45|25|40|1|8");
    	speciesMap.put("パルシェン", "water|ice|50|95|180|85|45|70|1|8");
    	speciesMap.put("ゴース", "ghost|poison|30|35|30|100|35|80|1|8");
    	speciesMap.put("ゴースト", "ghost|poison|45|50|45|115|55|95|1|8");
    	speciesMap.put("ゲンガー", "ghost|poison|60|65|60|130|75|110|1|8");
    	speciesMap.put("イワーク", "rock|ground|35|45|160|30|45|70|1|8");
    	speciesMap.put("スリープ", "psychic|none|60|48|45|43|90|42|1|8");
    	speciesMap.put("スリーパー", "psychic|none|85|73|70|73|115|67|1|8");
    	speciesMap.put("クラブ", "water|none|30|105|90|25|25|50|1|8");
    	speciesMap.put("キングラー", "water|none|55|130|115|50|50|75|1|8");
    	speciesMap.put("ビリリダマ", "electric|none|40|30|50|55|55|100|0|8");
    	speciesMap.put("マルマイン", "electric|none|60|50|70|80|80|140|0|8");
    	speciesMap.put("タマタマ", "grass|psychic|60|40|80|60|45|40|1|8");
    	speciesMap.put("ナッシー", "grass|psychic|95|95|85|125|65|55|1|8");
    	speciesMap.put("カラカラ", "ground|none|50|50|95|40|50|35|1|8");
    	speciesMap.put("ガラガラ", "ground|none|60|80|110|50|80|45|1|8");
    	speciesMap.put("サワムラー", "fighting|none|50|120|53|35|110|87|1|0");
    	speciesMap.put("エビワラー", "fighting|none|50|105|79|35|110|76|1|0");
    	speciesMap.put("ベロリンガ", "normal|none|90|55|75|60|75|30|1|8");
    	speciesMap.put("ドガース", "poison|none|40|65|95|60|45|35|1|8");
    	speciesMap.put("マタドガス", "poison|none|65|90|120|85|70|60|1|8");
    	speciesMap.put("サイホーン", "ground|rock|80|85|95|30|30|25|1|8");
    	speciesMap.put("サイドン", "ground|rock|105|130|120|45|45|40|1|8");
    	speciesMap.put("ラッキー", "normal|none|250|5|5|35|105|50|1|16");
    	speciesMap.put("モンジャラ", "grass|none|65|55|115|100|40|60|1|8");
    	speciesMap.put("ガルーラ", "normal|none|105|95|80|40|80|90|1|16");
    	speciesMap.put("タッツー", "water|none|30|40|70|70|25|60|1|8");
    	speciesMap.put("シードラ", "water|none|55|65|95|95|45|85|1|8");
    	speciesMap.put("トサキント", "water|none|45|67|60|35|50|63|1|8");
    	speciesMap.put("アズマオウ", "water|none|80|92|65|65|80|68|1|8");
    	speciesMap.put("ヒトデマン", "water|none|30|45|55|70|55|85|0|8");
    	speciesMap.put("スターミー", "water|psychic|60|75|85|100|85|115|0|8");
    	speciesMap.put("バリヤード", "psychic|none|40|45|65|100|120|90|1|8");
    	speciesMap.put("ストライク", "bug|flying|70|110|80|55|80|105|1|8");
    	speciesMap.put("ルージュラ", "ice|psychic|65|50|35|115|95|95|1|16");
    	speciesMap.put("エレブー", "electric|none|65|83|57|95|85|105|1|4");
    	speciesMap.put("ブーバー", "fire|none|65|95|57|100|85|93|1|4");
    	speciesMap.put("カイロス", "bug|none|65|125|100|55|70|85|1|8");
    	speciesMap.put("ケンタロス", "normal|none|75|100|95|40|70|110|1|0");
    	speciesMap.put("コイキング", "water|none|20|10|55|15|20|80|1|8");
    	speciesMap.put("ギャラドス", "water|flying|95|125|79|60|100|81|1|8");
    	speciesMap.put("ラプラス", "water|ice|130|85|80|85|95|60|1|8");
    	speciesMap.put("メタモン", "normal|none|48|48|48|48|48|48|0|8");
    	speciesMap.put("イーブイ", "normal|none|55|55|50|45|65|55|1|2");
    	speciesMap.put("シャワーズ", "water|none|130|65|60|110|95|65|1|2");
    	speciesMap.put("サンダース", "electric|none|65|65|60|110|95|130|1|2");
    	speciesMap.put("ブースター", "fire|none|65|130|60|95|110|65|1|2");
    	speciesMap.put("ポリゴン", "normal|none|65|60|70|85|75|40|0|8");
    	speciesMap.put("オムナイト", "rock|water|35|40|100|90|55|35|1|2");
    	speciesMap.put("オムスター", "rock|water|70|60|125|115|70|55|1|2");
    	speciesMap.put("カブト", "rock|water|30|80|90|55|45|55|1|2");
    	speciesMap.put("カブトプス", "rock|water|60|115|105|65|70|80|1|2");
    	speciesMap.put("プテラ", "rock|flying|80|105|65|60|75|130|1|2");
    	speciesMap.put("カビゴン", "normal|none|160|110|65|65|110|30|1|2");
    	speciesMap.put("フリーザー", "ice|flying|90|85|100|95|125|85|0|8");
    	speciesMap.put("サンダー", "electric|flying|90|90|85|125|90|100|0|8");
    	speciesMap.put("ファイヤー", "fire|flying|90|100|90|125|85|90|0|8");
    	speciesMap.put("ミニリュウ", "dragon|none|41|64|45|50|50|50|1|8");
    	speciesMap.put("ハクリュー", "dragon|none|61|84|65|70|70|70|1|8");
    	speciesMap.put("カイリュー", "dragon|flying|91|134|95|100|100|80|1|8");
    	speciesMap.put("ミュウツー", "psychic|none|106|110|90|154|90|130|0|8");
    	speciesMap.put("ミュウ", "psychic|none|100|100|100|100|100|100|0|8");
    	speciesMap.put("チコリータ", "grass|none|45|49|65|49|65|45|1|2");
    	speciesMap.put("ベイリーフ", "grass|none|60|62|80|63|80|60|1|2");
    	speciesMap.put("メガニウム", "grass|none|80|82|100|83|100|80|1|2");
    	speciesMap.put("ヒノアラシ", "fire|none|39|52|43|60|50|65|1|2");
    	speciesMap.put("マグマラシ", "fire|none|58|64|58|80|65|80|1|2");
    	speciesMap.put("バクフーン", "fire|none|78|84|78|109|85|100|1|2");
    	speciesMap.put("ワニノコ", "water|none|50|65|64|44|48|43|1|2");
    	speciesMap.put("アリゲイツ", "water|none|65|80|80|59|63|58|1|2");
    	speciesMap.put("オーダイル", "water|none|85|105|100|79|83|78|1|2");
    	speciesMap.put("オタチ", "normal|none|35|46|34|35|45|20|1|8");
    	speciesMap.put("オオタチ", "normal|none|85|76|64|45|55|90|1|8");
    	speciesMap.put("ホーホー", "normal|flying|60|30|30|36|56|50|1|8");
    	speciesMap.put("ヨルノズク", "normal|flying|100|50|50|76|96|70|1|8");
    	speciesMap.put("レディバ", "bug|flying|40|20|30|40|80|55|1|8");
    	speciesMap.put("レディアン", "bug|flying|55|35|50|55|110|85|1|8");
    	speciesMap.put("イトマル", "bug|poison|40|60|40|40|40|30|1|8");
    	speciesMap.put("アリアドス", "bug|poison|70|90|70|60|60|40|1|8");
    	speciesMap.put("クロバット", "poison|flying|85|90|80|70|80|130|1|8");
    	speciesMap.put("チョンチー", "water|electric|75|38|38|56|56|67|1|8");
    	speciesMap.put("ランターン", "water|electric|125|58|58|76|76|67|1|8");
    	speciesMap.put("ピチュー", "electric|none|20|40|15|35|35|60|1|8");
    	speciesMap.put("ピィ", "normal|none|50|25|28|45|55|15|1|12");
    	speciesMap.put("ププリン", "normal|none|90|30|15|40|20|15|1|12");
    	speciesMap.put("トゲピー", "normal|none|35|20|65|40|65|20|1|2");
    	speciesMap.put("トゲチック", "normal|flying|55|40|85|80|105|40|1|2");
    	speciesMap.put("ネイティ", "psychic|flying|40|50|45|70|45|70|1|8");
    	speciesMap.put("ネイティオ", "psychic|flying|65|75|70|95|70|95|1|8");
    	speciesMap.put("メリープ", "electric|none|55|40|40|65|45|35|1|8");
    	speciesMap.put("モココ", "electric|none|70|55|55|80|60|45|1|8");
    	speciesMap.put("デンリュウ", "electric|none|90|75|75|115|90|55|1|8");
    	speciesMap.put("キレイハナ", "grass|none|75|80|85|90|100|50|1|8");
    	speciesMap.put("マリル", "water|none|70|20|50|20|50|40|1|8");
    	speciesMap.put("マリルリ", "water|none|100|50|80|50|80|50|1|8");
    	speciesMap.put("ウソッキー", "rock|none|70|100|115|30|65|30|1|8");
    	speciesMap.put("ニョロトノ", "water|none|90|75|75|90|100|70|1|8");
    	speciesMap.put("ハネッコ", "grass|flying|35|35|40|35|55|50|1|8");
    	speciesMap.put("ポポッコ", "grass|flying|55|45|50|45|65|80|1|8");
    	speciesMap.put("ワタッコ", "grass|flying|75|55|70|55|85|110|1|8");
    	speciesMap.put("エイパム", "normal|none|55|70|55|40|55|85|1|8");
    	speciesMap.put("ヒマナッツ", "grass|none|30|30|30|30|30|30|1|8");
    	speciesMap.put("キマワリ", "grass|none|75|75|55|105|85|30|1|8");
    	speciesMap.put("ヤンヤンマ", "bug|flying|65|65|45|75|45|95|1|8");
    	speciesMap.put("ウパー", "water|ground|55|45|45|25|25|15|1|8");
    	speciesMap.put("ヌオー", "water|ground|95|85|85|65|65|35|1|8");
    	speciesMap.put("エーフィ", "psychic|none|65|65|60|130|95|110|1|2");
    	speciesMap.put("ブラッキー", "dark|none|95|65|110|60|130|65|1|2");
    	speciesMap.put("ヤミカラス", "dark|flying|60|85|42|85|42|91|1|8");
    	speciesMap.put("ヤドキング", "water|psychic|95|75|80|100|110|30|1|8");
    	speciesMap.put("ムウマ", "ghost|none|60|60|60|85|85|85|1|8");
    	speciesMap.put("アンノーン", "psychic|none|48|72|48|72|48|48|0|8");
    	speciesMap.put("ソーナンス", "psychic|none|190|33|58|33|58|33|1|8");
    	speciesMap.put("キリンリキ", "normal|psychic|70|80|65|90|65|85|1|8");
    	speciesMap.put("クヌギダマ", "bug|none|50|65|90|35|35|15|1|8");
    	speciesMap.put("フォレトス", "bug|steel|75|90|140|60|60|40|1|8");
    	speciesMap.put("ノコッチ", "normal|none|100|70|70|65|65|45|1|8");
    	speciesMap.put("グライガー", "ground|flying|65|75|105|35|65|85|1|8");
    	speciesMap.put("ハガネール", "steel|ground|75|85|200|55|65|30|1|8");
    	speciesMap.put("ブルー", "normal|none|60|80|50|40|40|30|1|12");
    	speciesMap.put("グランブル", "normal|none|90|120|75|60|60|45|1|12");
    	speciesMap.put("ハリーセン", "water|poison|65|95|75|55|55|85|1|8");
    	speciesMap.put("ハッサム", "bug|steel|70|130|100|55|80|65|1|8");
    	speciesMap.put("ツボツボ", "bug|rock|20|10|230|10|230|5|1|8");
    	speciesMap.put("ヘラクロス", "bug|fighting|80|125|75|40|95|85|1|8");
    	speciesMap.put("ニューラ", "dark|ice|55|95|55|35|75|115|1|8");
    	speciesMap.put("ヒメグマ", "normal|none|60|80|50|50|50|40|1|8");
    	speciesMap.put("リングマ", "normal|none|90|130|75|75|75|55|1|8");
    	speciesMap.put("マグマッグ", "fire|none|40|40|40|70|40|20|1|8");
    	speciesMap.put("マグカルゴ", "fire|rock|50|50|120|80|80|30|1|8");
    	speciesMap.put("ウリムー", "ice|ground|50|50|40|30|30|50|1|8");
    	speciesMap.put("イノムー", "ice|ground|100|100|80|60|60|50|1|8");
    	speciesMap.put("サニーゴ", "water|rock|55|55|85|65|85|35|1|12");
    	speciesMap.put("テッポウオ", "water|none|35|65|35|65|35|65|1|8");
    	speciesMap.put("オクタン", "water|none|75|105|75|105|75|45|1|8");
    	speciesMap.put("デリバード", "ice|flying|45|55|45|65|45|75|1|8");
    	speciesMap.put("マンタイン", "water|flying|65|40|70|80|140|70|1|8");
    	speciesMap.put("エアームド", "steel|flying|65|80|140|40|70|70|1|8");
    	speciesMap.put("デルビル", "dark|fire|45|60|30|80|50|65|1|8");
    	speciesMap.put("ヘルガー", "dark|fire|75|90|50|110|80|95|1|8");
    	speciesMap.put("キングドラ", "water|dragon|75|95|95|95|95|85|1|8");
    	speciesMap.put("ゴマゾウ", "ground|none|90|60|60|40|40|40|1|8");
    	speciesMap.put("ドンファン", "ground|none|90|120|120|60|60|50|1|8");
    	speciesMap.put("ポリゴン２", "normal|none|85|80|90|105|95|60|0|8");
    	speciesMap.put("オドシシ", "normal|none|73|95|62|85|65|85|1|8");
    	speciesMap.put("ドーブル", "normal|none|55|20|35|20|45|75|1|8");
    	speciesMap.put("バルキー", "fighting|none|35|35|35|35|35|35|1|0");
    	speciesMap.put("カポエラー", "fighting|none|50|95|95|35|110|70|1|0");
    	speciesMap.put("ムチュール", "ice|psychic|45|30|15|85|65|65|1|16");
    	speciesMap.put("エレキッド", "electric|none|45|63|37|65|55|95|1|4");
    	speciesMap.put("ブビィ", "fire|none|45|75|37|70|55|83|1|4");
    	speciesMap.put("ミルタンク", "normal|none|95|80|105|40|70|100|1|16");
    	speciesMap.put("ハピナス", "normal|none|255|10|10|75|135|55|1|16");
    	speciesMap.put("ライコウ", "electric|none|90|85|75|115|100|115|0|8");
    	speciesMap.put("エンテイ", "fire|none|115|115|85|90|75|100|0|8");
    	speciesMap.put("スイクン", "water|none|100|75|115|90|115|85|0|8");
    	speciesMap.put("ヨーギラス", "rock|ground|50|64|50|45|50|41|1|8");
    	speciesMap.put("サナギラス", "rock|ground|70|84|70|65|70|51|1|8");
    	speciesMap.put("バンギラス", "rock|dark|100|134|110|95|100|61|1|8");
    	speciesMap.put("ルギア", "psychic|flying|106|90|130|90|154|110|0|8");
    	speciesMap.put("ホウオウ", "fire|flying|106|130|90|110|154|90|0|8");
    	speciesMap.put("セレビィ", "psychic|grass|100|100|100|100|100|100|0|8");

    	/* 各種情報の取得 */
    	String speciesInfo = speciesMap.get(speciesName);
    	if (speciesInfo == null) {
    		speciesName="けつばん";
    		speciesInfo="normal|none|0|0|0|0|0|0|0|8";
    	}
    	String[] speciesInfoSplit = speciesInfo.split("\\|", 0);
    	type1 = speciesInfoSplit[0];
    	type2 = speciesInfoSplit[1];
    	HPSpeciesValue = Integer.parseInt(speciesInfoSplit[2]);
    	attackSpeciesValue = Integer.parseInt(speciesInfoSplit[3]);
    	blockSpeciesValue = Integer.parseInt(speciesInfoSplit[4]);
    	contactSpeciesValue = Integer.parseInt(speciesInfoSplit[5]);
    	diffenceSpeciesValue = Integer.parseInt(speciesInfoSplit[6]);
    	speedSpeciesValue = Integer.parseInt(speciesInfoSplit[7]);
    	if (speciesInfoSplit[8].equals("1")) {
    		sexExistenceFlag = true;
    	} else {
    		sexExistenceFlag = false;
    	}
    	femaleRatio = Integer.parseInt(speciesInfoSplit[9]);
    }

	/* ステータス計算を行うメソッド
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：無し（計算結果に不正があっても初期値採用で先に進む）
	 */
    public void statusCalc() {

    	/* レベルの妥当性確認 */
    	if(level < 0 || level > 255) {
    		level = 50;
    	}

    	/* 種族値の妥当性確認 */
    	if(HPSpeciesValue < 0 || HPSpeciesValue > 255) {
    		HPSpeciesValue = 0;
    	}
    	if(attackSpeciesValue < 0 || attackSpeciesValue > 255) {
    		attackSpeciesValue = 0;
    	}
		if(blockSpeciesValue < 0 || blockSpeciesValue > 255) {
			blockSpeciesValue = 0;
		}
		if(contactSpeciesValue < 0 || contactSpeciesValue > 255) {
			contactSpeciesValue = 0;
		}
		if(diffenceSpeciesValue < 0 || diffenceSpeciesValue > 255) {
			diffenceSpeciesValue = 0;
		}
		if(speedSpeciesValue < 0 || speedSpeciesValue > 255) {
			speedSpeciesValue = 0;
		}

		/* 個体値の妥当性確認 */
    	if(attackIndividualValue < 0 || attackIndividualValue > 15) {
    		attackIndividualValue = 0;
    	}
    	if(blockIndividualValue < 0 || blockIndividualValue > 15) {
    		blockIndividualValue = 0;
    	}
    	if(speedIndividualValue < 0 || speedIndividualValue > 15) {
    		speedIndividualValue = 0;
    	}
    	if(specialIndividualValue < 0 || specialIndividualValue > 15) {
    		specialIndividualValue = 0;
    	}

    	/* 努力レベルの妥当性確認 */
    	if(HPEffortLevel < 0 || HPEffortLevel > 63) {
    		HPEffortLevel = 0;
    	}
	    if(attackEffortLevel < 0 || attackEffortLevel > 63) {
    		attackEffortLevel = 0;
    	}
    	if(blockEffortLevel < 0 || blockEffortLevel > 63) {
    		blockEffortLevel = 0;
    	}
    	if(speedEffortLevel < 0 || speedEffortLevel > 63) {
    		speedEffortLevel = 0;
    	}
    	if(specialEffortLevel < 0 || specialEffortLevel > 63) {
    		specialEffortLevel = 0;
    	}

    	/* HP個体値の算出 */
    	HPIndividualValue = (attackIndividualValue % 2 * 8) +
    			            (blockIndividualValue % 2 * 4) +
    			            (speedIndividualValue % 2 * 2) +
    			            (specialIndividualValue % 2 * 1);

    	/* 各種ステータスの算出 */
    	MAXHP = (int)
    			(Math.floor((((HPSpeciesValue+HPIndividualValue)*2+HPEffortLevel)*level)/100)+level+10);
    	attack = (int)
    			(Math.floor((((attackSpeciesValue+attackIndividualValue)*2+attackEffortLevel)*level)/100)+5);
    	block = (int)
    			(Math.floor((((blockSpeciesValue+blockIndividualValue)*2+blockEffortLevel)*level)/100)+5);
    	contact = (int)
    			(Math.floor((((contactSpeciesValue+specialIndividualValue)*2+specialEffortLevel)*level)/100)+5);
    	diffence = (int)
    			(Math.floor((((diffenceSpeciesValue+specialIndividualValue)*2+specialEffortLevel)*level)/100)+5);
    	speed = (int)
    			(Math.floor((((speedSpeciesValue+speedIndividualValue)*2+speedEffortLevel)*level)/100)+5);

    }

	/* 性別を判定するメソッド
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：無し（メスの割合・攻撃個体値に不正があっても例外を出さずに初期値採用で先に進む）
	 */
    public void sexSet() {
    	/* メスの割合・攻撃個体値の妥当性確認 */
    	if(femaleRatio < 0 || femaleRatio > 16) {
    		femaleRatio = 8;
    	}
    	if(attackIndividualValue < 0 || attackIndividualValue > 15) {
    		attackIndividualValue = 0;
    	}

    	/* 性別の判定 */
    	if (sexExistenceFlag == true) {
    		if(attackIndividualValue >= femaleRatio) {
    			sex = "male";
    		} else {
    			sex = "female";
    		}
    	} else {
    		sex = "none";
    	}
    }

	/* めざめるパワーのタイプ・威力を判定するメソッド
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：無し（個体値に不正があっても例外を出さずに初期値採用で先に進む）
	 */
    public void hiddenPowerSet() {
    	/* 個体値の妥当性確認 */
    	if(attackIndividualValue < 0 || attackIndividualValue > 15) {
    		attackIndividualValue = 0;
    	}
    	if(blockIndividualValue < 0 || blockIndividualValue > 15) {
    		blockIndividualValue = 0;
    	}
    	if(speedIndividualValue < 0 || speedIndividualValue > 15) {
    		speedIndividualValue = 0;
    	}
    	if(specialIndividualValue < 0 || specialIndividualValue > 15) {
    		specialIndividualValue = 0;
    	}

    	/* めざめるパワーのタイプ判定 */
    	int attackIndividualMod = attackIndividualValue % 4;
    	int blockIndividualMod = blockIndividualValue % 4;
    	if        (attackIndividualMod == 3 && blockIndividualMod == 3) {
    		hiddenPowerType = "dark";
    	} else if (attackIndividualMod == 3 && blockIndividualMod == 2) {
    		hiddenPowerType = "dragon";
    	} else if (attackIndividualMod == 3 && blockIndividualMod == 1) {
    		hiddenPowerType = "ice";
    	} else if (attackIndividualMod == 3 && blockIndividualMod == 0) {
    		hiddenPowerType = "psychic";
    	} else if (attackIndividualMod == 2 && blockIndividualMod == 3) {
    		hiddenPowerType = "electric";
    	} else if (attackIndividualMod == 2 && blockIndividualMod == 2) {
    		hiddenPowerType = "grass";
    	} else if (attackIndividualMod == 2 && blockIndividualMod == 1) {
    		hiddenPowerType = "water";
    	} else if (attackIndividualMod == 2 && blockIndividualMod == 0) {
    		hiddenPowerType = "fire";
    	} else if (attackIndividualMod == 1 && blockIndividualMod == 3) {
    		hiddenPowerType = "steel";
    	} else if (attackIndividualMod == 1 && blockIndividualMod == 2) {
    		hiddenPowerType = "ghost";
    	} else if (attackIndividualMod == 1 && blockIndividualMod == 1) {
    		hiddenPowerType = "bug";
    	} else if (attackIndividualMod == 1 && blockIndividualMod == 0) {
    		hiddenPowerType = "rock";
    	} else if (attackIndividualMod == 0 && blockIndividualMod == 3) {
    		hiddenPowerType = "ground";
    	} else if (attackIndividualMod == 0 && blockIndividualMod == 2) {
    		hiddenPowerType = "poison";
    	} else if (attackIndividualMod == 0 && blockIndividualMod == 1) {
    		hiddenPowerType = "flying";
    	} else if (attackIndividualMod == 0 && blockIndividualMod == 0) {
    		hiddenPowerType = "fighting";
    	}

    	/* めざめるパワーの威力計算 */
    	int attackIndividualDivision = attackIndividualValue / 8;
    	int blockIndividualDivision = blockIndividualValue / 8;
    	int speedIndividualDivision = speedIndividualValue / 8;
    	int specialIndividualDivision = specialIndividualValue / 8;
    	int specialIndividualMod = specialIndividualValue % 4;
    	hiddenPowerPower = (int)(Math.floor(((((attackIndividualDivision * 8)
    			+ (blockIndividualDivision * 4)
    			+ (speedIndividualDivision * 2)
    			+ (specialIndividualDivision * 1))
    			* 5) + specialIndividualMod) / 2));
    	hiddenPowerPower = hiddenPowerPower + 31;
    }

	/* 技情報を取得するメソッド
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：無し（技名やポイントアップ使用回数に不正があっても例外を出さずに初期値採用で先に進む）
	 */
    public void moveSet() {
    	/* 技の各種情報の定義 */
    	HashMap<String, String> moveMap = new HashMap<String, String>();
    	moveMap.put("はたく", "normal|40|100|35|0|00a");
    	moveMap.put("メガトンパンチ", "normal|80|85|20|0|00a");
    	moveMap.put("ひっかく", "normal|40|100|35|0|00a");
    	moveMap.put("はさむ", "normal|55|100|30|0|00a");
    	moveMap.put("いあいぎり", "normal|50|95|30|0|00a");
    	moveMap.put("つばさでうつ", "flying|60|100|35|0|00a");
    	moveMap.put("たたきつける", "normal|80|75|20|0|00a");
    	moveMap.put("つるのムチ", "grass|35|100|10|0|00a");
    	moveMap.put("メガトンキック", "normal|120|75|5|0|00a");
    	moveMap.put("つのでつく", "normal|65|100|25|0|00a");
    	moveMap.put("たいあたり", "normal|35|95|35|0|00a");
    	moveMap.put("みずでっぽう", "water|40|100|25|0|00a");
    	moveMap.put("ハイドロポンプ", "water|120|80|5|0|00a");
    	moveMap.put("なみのり", "water|95|100|15|0|00a");
    	moveMap.put("つつく", "flying|35|100|35|0|00a");
    	moveMap.put("ドリルくちばし", "flying|80|100|20|0|00a");
    	moveMap.put("かいりき", "normal|80|100|15|0|00a");
    	moveMap.put("いわおとし", "rock|50|90|15|0|00a");
    	moveMap.put("タマゴばくだん", "normal|100|75|10|0|00a");
    	moveMap.put("たきのぼり", "water|80|100|15|0|00a");
    	moveMap.put("メガホーン", "bug|120|85|5|0|00a");
    	moveMap.put("からてチョップ", "fighting|50|100|25|0|00b");
    	moveMap.put("はっぱカッター", "grass|55|95|25|0|00b");
    	moveMap.put("クラブハンマー", "water|90|85|10|0|00b");
    	moveMap.put("きりさく", "normal|70|100|20|0|00b");
    	moveMap.put("エアロブラスト", "flying|100|95|5|0|00b");
    	moveMap.put("クロスチョップ", "fighting|100|80|5|0|00b");
    	moveMap.put("うたう", "normal|0|55|15|0|01");
    	moveMap.put("ねむりごな", "grass|0|75|15|0|01");
    	moveMap.put("さいみんじゅつ", "psychic|0|60|20|0|01");
    	moveMap.put("あくまのキッス", "normal|0|75|10|0|01");
    	moveMap.put("キノコのほうし", "grass|0|100|15|0|01");
    	moveMap.put("どくばり", "poison|15|100|35|30|02");
    	moveMap.put("スモッグ", "poison|20|70|20|40|02");
    	moveMap.put("ヘドロこうげき", "poison|65|100|20|30|02");
    	moveMap.put("ヘドロばくだん", "poison|90|100|10|30|02");
    	moveMap.put("すいとる", "grass|20|100|20|0|03");
    	moveMap.put("メガドレイン", "grass|40|100|10|0|03");
    	moveMap.put("きゅうけつ", "bug|20|100|15|0|03");
    	moveMap.put("ギガドレイン", "grass|60|100|5|0|03");
    	moveMap.put("ほのおのパンチ", "fire|75|100|15|10|04");
    	moveMap.put("ひのこ", "fire|40|100|25|10|04");
    	moveMap.put("かえんほうしゃ", "fire|95|100|15|10|04");
    	moveMap.put("だいもんじ", "fire|120|85|5|10|04");
    	moveMap.put("れいとうパンチ", "ice|75|100|15|10|05");
    	moveMap.put("れいとうビーム", "ice|95|100|10|10|05");
    	moveMap.put("ふぶき", "ice|120|70|5|10|05");
    	moveMap.put("こなゆき", "ice|40|100|25|10|05");
    	moveMap.put("かみなりパンチ", "electric|75|100|15|10|06");
    	moveMap.put("のしかかり", "normal|85|100|15|30|06");
    	moveMap.put("でんきショック", "electric|40|100|30|10|06");
    	moveMap.put("１０まんボルト", "electric|95|100|15|10|06");
    	moveMap.put("したでなめる", "ghost|20|100|30|30|06");
    	moveMap.put("でんじほう", "electric|100|50|5|100|06");
    	moveMap.put("スパーク", "electric|65|100|20|30|06");
    	moveMap.put("りゅうのいぶき", "dragon|60|100|20|30|06");
    	moveMap.put("じばく", "normal|200|100|5|0|07");
    	moveMap.put("だいばくはつ", "normal|250|100|5|0|07");
    	moveMap.put("ゆめくい", "psychic|100|100|15|0|08");
    	moveMap.put("オウムがえし", "flying|0|100|20|0|09");
    	moveMap.put("ヨガのポーズ", "psychic|0|100|40|0|0A");
    	moveMap.put("かくばる", "normal|0|100|30|0|0A");
    	moveMap.put("かたくなる", "normal|0|100|30|0|0B");
    	moveMap.put("からにこもる", "water|0|100|40|0|0B");
    	moveMap.put("せいちょう", "normal|0|100|40|0|0D");
    	moveMap.put("かげぶんしん", "normal|0|100|15|0|10a");
    	moveMap.put("ちいさくなる", "normal|0|100|20|0|10b");
    	moveMap.put("スピードスター", "normal|60|100|20|0|11a");
    	moveMap.put("だましうち", "dark|60|100|20|0|11a");
    	moveMap.put("あてみなげ", "fighting|70|100|10|0|11b");
    	moveMap.put("なきごえ", "normal|0|100|40|0|12");
    	moveMap.put("しっぽをふる", "normal|0|100|30|0|13");
    	moveMap.put("にらみつける", "normal|0|100|30|0|13");
    	moveMap.put("いとをはく", "bug|0|95|40|0|14");
    	moveMap.put("すなかけ", "ground|0|100|15|0|17");
    	moveMap.put("えんまく", "normal|0|100|20|0|17");
    	moveMap.put("スプーンまげ", "psychic|0|80|15|0|17");
    	moveMap.put("フラッシュ", "normal|0|70|20|0|17");
    	moveMap.put("あまいかおり", "normal|0|100|20|0|18");
    	moveMap.put("くろいきり", "ice|0|100|30|0|19");
    	moveMap.put("がまん", "normal|0|100|10|0|1A");
    	moveMap.put("あばれる", "normal|90|100|20|0|1B");
    	moveMap.put("はなびらのまい", "grass|70|100|20|0|1B");
    	moveMap.put("げきりん", "dragon|90|100|15|0|1B");
    	moveMap.put("ふきとばし", "normal|0|100|20|0|1Ca");
    	moveMap.put("ほえる", "normal|0|100|20|0|1Cb");
    	moveMap.put("おうふくビンタ", "normal|15|85|10|0|1D");
    	moveMap.put("れんぞくパンチ", "normal|18|85|15|0|1D");
    	moveMap.put("みだれづき", "normal|15|85|20|0|1D");
    	moveMap.put("ミサイルばり", "bug|14|85|20|0|1D");
    	moveMap.put("とげキャノン", "normal|20|100|15|0|1D");
    	moveMap.put("たまなげ", "normal|15|85|20|0|1D");
    	moveMap.put("みだれひっかき", "normal|18|80|15|0|1D");
    	moveMap.put("ボーンラッシュ", "ground|25|80|10|0|1D");
    	moveMap.put("テクスチャー", "normal|0|100|30|0|1E");
    	moveMap.put("まわしげり", "fighting|60|85|15|30|1F");
    	moveMap.put("ずつき", "normal|70|100|15|30|1F");
    	moveMap.put("かみつく", "dark|60|100|25|30|1F");
    	moveMap.put("けたぐり", "fighting|50|90|20|30|1F");
    	moveMap.put("ホネこんぼう", "ground|65|85|20|10|1F");
    	moveMap.put("いわなだれ", "rock|75|90|10|30|1F");
    	moveMap.put("ひっさつまえば", "normal|80|90|15|10|1F");
    	moveMap.put("じこさいせい", "normal|0|100|20|0|20a");
    	moveMap.put("タマゴうみ", "normal|0|100|10|0|20a");
    	moveMap.put("ミルクのみ", "normal|0|100|10|0|20a");
    	moveMap.put("ねむる", "psychic|0|100|10|0|20b");
    	moveMap.put("どくどく", "poison|0|85|10|0|21");
    	moveMap.put("ネコにこばん", "normal|40|100|20|0|22");
    	moveMap.put("ひかりのかべ", "psychic|0|100|30|0|23");
    	moveMap.put("トライアタック", "normal|80|100|10|20|24");
    	moveMap.put("ハサミギロチン", "normal|0|30|5|0|26");
    	moveMap.put("つのドリル", "normal|1|30|5|0|26");
    	moveMap.put("じわれ", "ground|1|30|5|0|26");
    	moveMap.put("かまいたち", "normal|80|75|10|0|27");
    	moveMap.put("いかりのまえば", "normal|1|90|10|0|28");
    	moveMap.put("ソニックブーム", "normal|20|90|20|0|29");
    	moveMap.put("りゅうのいかり", "dragon|40|100|10|0|29");
    	moveMap.put("しめつける", "normal|15|75|20|0|2A");
    	moveMap.put("まきつく", "normal|15|85|20|0|2A");
    	moveMap.put("ほのおのうず", "fire|15|70|15|0|2A");
    	moveMap.put("からではさむ", "water|35|75|10|0|2A");
    	moveMap.put("うずしお", "water|15|70|15|0|2A");
    	moveMap.put("にどげり", "fighting|30|100|30|0|2C");
    	moveMap.put("ホネブーメラン", "ground|50|90|10|0|2C");
    	moveMap.put("とびげり", "fighting|70|95|25|0|2D");
    	moveMap.put("とびひざげり", "fighting|85|90|20|0|2D");
    	moveMap.put("しろいきり", "ice|0|100|30|0|2E");
    	moveMap.put("きあいだめ", "normal|0|100|30|0|2F");
    	moveMap.put("とっしん", "normal|90|85|20|0|30a");
    	moveMap.put("すてみタックル", "normal|120|100|15|0|30a");
    	moveMap.put("じごくぐるま", "fighting|80|80|25|0|30a");
    	moveMap.put("わるあがき", "physic|50|100|10|0|30b");
    	moveMap.put("ちょうおんぱ", "normal|0|55|20|0|31");
    	moveMap.put("あやしいひかり", "ghost|0|100|10|0|31");
    	moveMap.put("てんしのキッス", "normal|0|75|10|0|31");
    	moveMap.put("つるぎのまい", "normal|0|100|30|0|32");
    	moveMap.put("バリアー", "psychic|0|100|30|0|33");
    	moveMap.put("とける", "poison|0|100|40|0|33");
    	moveMap.put("こうそくいどう", "psychic|0|100|30|0|34");
    	moveMap.put("ドわすれ", "psychic|0|100|20|0|36");
    	moveMap.put("へんしん", "normal|0|100|10|0|39");
    	moveMap.put("あまえる", "normal|0|100|20|0|3A");
    	moveMap.put("いやなおと", "normal|0|85|40|0|3B");
    	moveMap.put("わたほうし", "grass|0|85|40|0|3C");
    	moveMap.put("こわいかお", "normal|0|90|10|0|3C");
    	moveMap.put("リフレクター", "psychic|0|100|20|0|41");
    	moveMap.put("どくのこな", "poison|0|75|35|0|42");
    	moveMap.put("どくガス", "poison|0|55|40|0|42");
    	moveMap.put("しびれごな", "grass|0|75|30|0|43");
    	moveMap.put("でんじは", "electric|0|100|20|0|43");
    	moveMap.put("へびにらみ", "normal|0|75|30|0|43");
    	moveMap.put("オーロラビーム", "ice|65|100|20|10|44");
    	moveMap.put("ようかいえき", "poison|40|100|30|10|45");
    	moveMap.put("アイアンテール", "steel|100|75|15|30|45");
    	moveMap.put("いわくだき", "fighting|20|100|15|50|45");
    	moveMap.put("バブルこうせん", "water|65|100|20|10|46");
    	moveMap.put("からみつく", "normal|10|100|35|10|46");
    	moveMap.put("あわ", "water|20|100|30|10|46");
    	moveMap.put("こごえるかぜ", "ice|55|95|15|100|46");
    	moveMap.put("サイコキネシス", "psychic|90|100|10|10|48");
    	moveMap.put("かみくだく", "dark|80|100|15|20|48");
    	moveMap.put("シャドーボール", "ghost|80|100|15|20|48");
    	moveMap.put("どろかけ", "ground|20|100|10|100|49");
    	moveMap.put("オクタンほう", "water|65|85|10|50|49");
    	moveMap.put("ゴッドバード", "flying|140|90|5|0|4B");
    	moveMap.put("サイケこうせん", "psychic|65|100|20|10|4C");
    	moveMap.put("ねんりき", "psychic|50|100|25|10|4C");
    	moveMap.put("ピヨピヨパンチ", "normal|70|100|10|20|4C");
    	moveMap.put("ばくれつパンチ", "fighting|100|50|5|100|4C");
    	moveMap.put("ダブルニードル", "bug|25|100|20|20|4D");
    	moveMap.put("みがわり", "normal|0|100|10|0|4F");
    	moveMap.put("はかいこうせん", "normal|150|90|5|0|50");
    	moveMap.put("いかり", "normal|20|100|20|0|51");
    	moveMap.put("ものまね", "normal|0|100|10|0|52");
    	moveMap.put("ゆびをふる", "normal|0|100|10|0|53");
    	moveMap.put("やどりぎのタネ", "grass|0|90|10|0|54");
    	moveMap.put("はねる", "normal|0|100|40|0|55");
    	moveMap.put("かなしばり", "normal|0|55|20|0|56");
    	moveMap.put("ちきゅうなげ", "fighting|1|100|20|0|57");
    	moveMap.put("ナイトヘッド", "ghost|1|100|15|0|57");
    	moveMap.put("サイコウェーブ", "psychic|1|80|15|0|58");
    	moveMap.put("カウンター", "fighting|1|100|20|0|59");
    	moveMap.put("アンコール", "normal|0|100|5|0|5A");
    	moveMap.put("いたみわけ", "normal|0|100|20|0|5B");
    	moveMap.put("いびき", "normal|40|100|15|30|5C");
    	moveMap.put("テクスチャー２", "normal|0|100|30|0|5D");
    	moveMap.put("こころのめ", "normal|0|100|5|0|5E");
    	moveMap.put("ロックオン", "normal|0|100|5|0|5E");
    	moveMap.put("スケッチ", "normal|0|100|1|0|5F");
    	moveMap.put("ねごと", "normal|0|100|10|0|61");
    	moveMap.put("みちづれ", "ghost|0|100|5|0|62");
    	moveMap.put("じたばた", "normal|1|100|15|0|63");
    	moveMap.put("きしかいせい", "fighting|1|100|15|0|63");
    	moveMap.put("うらみ", "ghost|0|100|10|0|64");
    	moveMap.put("みねうち", "normal|40|100|40|0|65");
    	moveMap.put("いやしのすず", "normal|0|100|5|0|66");
    	moveMap.put("でんこうせっか", "normal|40|100|30|0|67");
    	moveMap.put("マッハパンチ", "fighting|40|100|30|0|67");
    	moveMap.put("しんそく", "normal|80|100|5|0|67");
    	moveMap.put("トリプルキック", "fighting|10|90|10|0|68");
    	moveMap.put("どろぼう", "dark|40|100|10|100|69");
    	moveMap.put("クモのす", "bug|0|100|10|0|6A");
    	moveMap.put("くろいまなざし", "normal|0|100|5|0|6A");
    	moveMap.put("あくむ", "ghost|0|100|15|0|6B");
    	moveMap.put("かえんぐるま", "fire|60|100|25|10|6C");
    	moveMap.put("のろい", "question|0|100|10|0|6D");
    	moveMap.put("まもる", "normal|0|100|10|0|6F");
    	moveMap.put("みきり", "fighting|0|100|5|0|6F");
    	moveMap.put("まきびし", "ground|0|100|20|0|70");
    	moveMap.put("みやぶる", "normal|0|100|40|0|71");
    	moveMap.put("ほろびのうた", "normal|0|100|5|0|72");
    	moveMap.put("すなあらし", "rock|0|100|10|0|73");
    	moveMap.put("こらえる", "normal|0|100|10|0|74");
    	moveMap.put("ころがる", "rock|30|90|20|0|75");
    	moveMap.put("いばる", "normal|0|90|15|0|76");
    	moveMap.put("れんぞくぎり", "bug|10|95|20|0|77");
    	moveMap.put("メロメロ", "normal|0|100|15|0|78");
    	moveMap.put("おんがえし", "normal|102|100|20|0|79");
    	moveMap.put("プレゼント", "normal|1|90|15|0|7A");
    	moveMap.put("やつあたり", "normal|102|100|20|0|7B");
    	moveMap.put("しんぴのまもり", "normal|0|100|25|0|7C");
    	moveMap.put("せいなるほのお", "fire|100|95|5|50|7D");
    	moveMap.put("マグニチュード", "ground|1|100|30|0|7E");
    	moveMap.put("バトンタッチ", "normal|0|100|40|0|7F");
    	moveMap.put("おいうち", "dark|40|100|20|0|80");
    	moveMap.put("こうそくスピン", "normal|20|100|40|0|81");
    	moveMap.put("あさのひざし", "normal|0|100|5|0|84");
    	moveMap.put("こうごうせい", "grass|0|100|5|0|85");
    	moveMap.put("つきのひかり", "normal|0|100|5|0|86");
    	moveMap.put("めざめるパワー", "normal|1|100|15|0|87");
    	moveMap.put("あまごい", "water|0|100|5|0|88");
    	moveMap.put("にほんばれ", "fire|0|100|5|0|89");
    	moveMap.put("はがねのつばさ", "steel|70|90|25|10|8A");
    	moveMap.put("メタルクロー", "steel|50|95|35|10|8B");
    	moveMap.put("げんしのちから", "rock|60|100|5|10|8C");
    	moveMap.put("はらだいこ", "normal|0|100|10|0|8E");
    	moveMap.put("じこあんじ", "normal|0|100|10|0|8F");
    	moveMap.put("ミラーコート", "psychic|1|100|20|0|90");
    	moveMap.put("ロケットずつき", "normal|100|100|15|0|91");
    	moveMap.put("たつまき", "dragon|40|100|20|20|92");
    	moveMap.put("じしん", "ground|100|100|10|0|93");
    	moveMap.put("みらいよち", "psychic|80|90|15|0|94");
    	moveMap.put("かぜおこし", "flying|40|100|35|0|95");
    	moveMap.put("ふみつけ", "normal|65|100|20|30|96");
    	moveMap.put("ソーラービーム", "grass|120|100|10|0|97");
    	moveMap.put("かみなり", "electric|120|70|10|30|98");
    	moveMap.put("テレポート", "psychic|0|100|20|0|99");
    	moveMap.put("ふくろだたき", "dark|10|100|10|0|9A");
    	moveMap.put("そらをとぶ", "flying|70|95|15|0|9Ba");
    	moveMap.put("あなをほる", "ground|60|100|10|0|9Bb");
    	moveMap.put("まるくなる", "normal|0|100|40|0|9C");
    	moveMap.put("交代", "question|0|100|0|0|XXa");
    	moveMap.put("混乱自滅", "physic|40|100|0|0|XXb");
    	moveMap.put("技無し", "question|0|100|0|0|XXc");

    	/* ４つの技枠について処理 */
    	for (int i = 0; i < 4; i++) {

    		/* ポイントアップ使用回数の妥当性確認 */
    	    if(movePointUpTimes[i] < 0 || movePointUpTimes[i] > 3) {
    	    	movePointUpTimes[i] = 0;
        	}

    	    /* 各種情報のセット */
        	String moveInfo = moveMap.get(moveName[i]);
        	if (moveInfo == null) {
        		moveName[i]="技無し";
        		moveInfo="question|0|100|0|0|XXc";
        	}
        	String[] moveInfoSplit = moveInfo.split("\\|", 0);
        	moveType[i] = moveInfoSplit[0];
        	movePower[i] = Integer.parseInt(moveInfoSplit[1]);
        	moveHitProbability[i] = Double.parseDouble(moveInfoSplit[2]);
        	movePP[i] = Integer.parseInt(moveInfoSplit[3]);
        	movePPPrimitive[i] = movePP[i];
        	movePP[i] = (int)Math.floor(movePP[i] * (1 + (0.2 * movePointUpTimes[i])));
        	additionalEffectProbability[i] = Double.parseDouble(moveInfoSplit[4]);
        	moveEffectCode[i] = moveInfoSplit[5];
    	}

    	/* PPの最大値を退避 */
    	movePPMAX1 = movePP[0];
    	movePPMAX2 = movePP[1];
    	movePPMAX3 = movePP[2];
    	movePPMAX4 = movePP[3];

    }

	/* 持ち物名をチェックするメソッド
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：無し（持ち物名に不正があっても例外を出さずに初期値を採用する）
	 */
    public void itemCheck() {
    	switch (itemName) {
    	case "はかいのいでんし":break;
    	case "きのみ":break;
    	case "きのみジュース":break;
    	case "おうごんのみ":break;
    	case "はっかのみ":break;
    	case "どくけしのみ":break;
    	case "こおったきのみ":break;
    	case "やけたきのみ":break;
    	case "まひなおしのみ":break;
    	case "きせきのみ":break;
    	case "ふしぎなきのみ":break;
    	case "たべのこし":break;
    	case "きあいのハチマキ":break;
    	case "せんせいのツメ":break;
    	case "ピントレンズ":break;
    	case "ひかりのこな":break;
    	case "ピンクのリボン":break;
    	case "みずたまリボン":break;
    	case "くろおび":break;
    	case "するどいくちばし":break;
    	case "どくバリ":break;
    	case "やわらかいすな":break;
    	case "かたいいし":break;
    	case "ぎんのこな":break;
    	case "のろいのおふだ":break;
    	case "メタルコート":break;
    	case "もくたん":break;
    	case "しんぴのしずく":break;
    	case "きせきのタネ":break;
    	case "じしゃく":break;
    	case "まがったスプーン":break;
    	case "とけないこおり":break;
    	case "りゅうのウロコ":break;
    	case "くろいめがね":break;
    	case "おうじゃのしるし":break;
    	case "ふといホネ":break;
    	case "でんきだま":break;
    	case "メタルパウダー":break;
    	case "ながねぎ":break;
    	case "ラッキーパンチ":break;
    	case "にがいきのみ":break;
    	case "りゅうのキバ":break;
    	case "モンスターボール":break;
    	case "スーパーボール":break;
    	case "ハイパーボール":break;
    	case "マスターボール":break;
    	case "ヘビーボール":break;
    	case "レベルボール":break;
    	case "ルアーボール":break;
    	case "スピードボール":break;
    	case "フレンドボール":break;
    	case "ムーンボール":break;
    	case "ラブラブボール":break;
    	case "パークボール":break;
    	case "どくけし":break;
    	case "まひなおし":break;
    	case "ねむけざまし":break;
    	case "やけどなおし":break;
    	case "こおりなおし":break;
    	case "なんでもなおし":break;
    	case "キズぐすり":break;
    	case "いいキズぐすり":break;
    	case "すごいキズぐすり":break;
    	case "まんたんのくすり":break;
    	case "かいふくのくすり":break;
    	case "げんきのかけら":break;
    	case "げんきのかたまり":break;
    	case "おいしいみず":break;
    	case "サイコソーダ":break;
    	case "ミックスオレ":break;
    	case "モーモーミルク":break;
    	case "いかりまんじゅう":break;
    	case "ピーピーエイド":break;
    	case "ピーピーリカバー":break;
    	case "ピーピーエイダー":break;
    	case "ピーピーマックス":break;
    	case "ちからのこな":break;
    	case "ちからねっこ":break;
    	case "ばんのうごな":break;
    	case "ふっかつそう":break;
    	case "せいなるはい":break;
    	case "あなぬけのヒモ":break;
    	case "むしよけスプレー":break;
    	case "シルバースプレー":break;
    	case "ゴールドスプレー":break;
    	case "プラスパワー":break;
    	case "ディフェンダー":break;
    	case "スピーダー":break;
    	case "スペシャルアップ":break;
    	case "ヨクアタール":break;
    	case "エフェクトガード":break;
    	case "クリティカッター":break;
    	case "ピッピにんぎょう":break;
    	case "マックスアップ":break;
    	case "タウリン":break;
    	case "ブロムヘキシン":break;
    	case "インドメタシン":break;
    	case "リゾチウム":break;
    	case "ふしぎなアメ":break;
    	case "ポイントアップ":break;
    	case "ほのおのいし":break;
    	case "みずのいし":break;
    	case "かみなりのいし":break;
    	case "リーフのいし":break;
    	case "つきのいし":break;
    	case "たいようのいし":break;
    	case "かわらずのいし":break;
    	case "アップグレード":break;
    	case "しろぼんぐり":break;
    	case "くろぼんぐり":break;
    	case "あかぼんぐり":break;
    	case "あおぼんぐり":break;
    	case "きぼんぐり":break;
    	case "みどぼんぐり":break;
    	case "ももぼんぐり":break;
    	case "がくしゅうそうち":break;
    	case "ふしぎなタマゴ":break;
    	case "おまもりこばん":break;
    	case "きよめのおふだ":break;
    	case "けむりだま":break;
    	case "はながらボール":break;
    	case "なみのりメール":break;
    	case "みずいろメール":break;
    	case "にがおえメール":break;
    	case "ラブリーメール":break;
    	case "ブイブイメール":break;
    	case "へんしんメール":break;
    	case "あおぞらメール":break;
    	case "おんぷメール":break;
    	case "まぼろしメール":break;
    	case "きんのたま":break;
    	case "しんじゅ":break;
    	case "おおきなしんじゅ":break;
    	case "ほしのすな":break;
    	case "ほしのかけら":break;
    	case "じてんしゃ":break;
    	case "コインケース":break;
    	case "ゼニガメじょうろ":break;
    	case "ダウジングマシン":break;
    	case "おいしいシッポ":break;
    	case "あかいウロコ":break;
    	case "ひでんのくすり":break;
    	case "ちかのカギ":break;
    	case "カードキー":break;
    	case "ふねのチケット":break;
    	case "ぼろのつりざお":break;
    	case "いいつりざお":break;
    	case "すごいつりざお":break;
    	case "にじいろのはね":break;
    	case "ぎんいろのはね":break;
    	case "きかいぶひん":break;
    	case "おとしもの":break;
    	case "ていきけん":break;
    	case "わざましん01":break;
    	case "わざましん02":break;
    	case "わざましん03":break;
    	case "わざましん04":break;
    	case "わざましん05":break;
    	case "わざましん06":break;
    	case "わざましん07":break;
    	case "わざましん08":break;
    	case "わざましん09":break;
    	case "わざましん10":break;
    	case "わざましん11":break;
    	case "わざましん12":break;
    	case "わざましん13":break;
    	case "わざましん14":break;
    	case "わざましん15":break;
    	case "わざましん16":break;
    	case "わざましん17":break;
    	case "わざましん18":break;
    	case "わざましん19":break;
    	case "わざましん20":break;
    	case "わざましん21":break;
    	case "わざましん22":break;
    	case "わざましん23":break;
    	case "わざましん24":break;
    	case "わざましん25":break;
    	case "わざましん26":break;
    	case "わざましん27":break;
    	case "わざましん28":break;
    	case "わざましん29":break;
    	case "わざましん30":break;
    	case "わざましん31":break;
    	case "わざましん48":break;
    	case "わざましん49":break;
    	case "わざましん50":break;
    	case "ひでんましん01":break;
    	case "ひでんましん02":break;
    	case "ひでんましん03":break;
    	case "ひでんましん04":break;
    	case "ひでんましん05":break;
    	case "ひでんましん06":break;
    	case "ひでんましん07":break;
    	case "ひでんましん08":break;
    	case "ひでんましん09":break;
    	case "ひでんましん10":break;
    	case "ひでんましん11":break;
    	case "ひでんましん12":break;
    	case "きんのはっぱ":break;
    	case "ぎんのはっぱ":break;
    	case "かわらのかけら":break;
    	case "きのはこ":break;
    	case "きりのはこ":break;
    	case "カビチュウ":break;
    	case "おおきなキノコ":break;
    	case "しあわせタマゴ":break;
    	case "ちいさなキノコ":break;
    	default:itemName = "持ち物無し";
    	}
    }

}
