package party;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import mold.PokeMold;
import state.PokeStateUnit;

/*
*  PartyInfoクラス
*
*  以下の情報を一括管理するクラスです。
*  ・パーティテキスト（PBSパーティ構築ツールに相当）
*  ・PokeMoldオブジェクト
*  ・PokeStateUnitオブジェクト
*
*  変更履歴
*  2019/04/06 仮作成
*  2019/04/17 完成
*  2019/05/05 PokeStateUnitオブジェクトについて、現在どのポケモンが場に出ているのかを示す変数を追加
*  2019/07/19 ６匹パーティの時に配列の添字が確保した範囲を超えて異常終了するバグを修正
*  2020/09/06 pokeStateUnitArrayの初期化メソッドを追加
*  2020/11/21 パーティ選択やり直しの考慮を追加
*
*/

public class PartyInfo {

	/* パーティテキスト読み取り結果 */
	public String[] partyTextArray = {null,null,null,null,null,null}; //パーティテキスト生データ

	/* PokeMoldオブジェクト管理 */
	public PokeMold[] pokeMoldArray = {null,null,null,null,null,null}; //ポケモンの型の管理

	/* PokeStateUnitオブジェクト管理 */
	public PokeStateUnit[] pokeStateUnitArray = {null,null,null,null,null,null}; //ポケモンの状態の管理

	/* 場に出ているPokeStateUnitオブジェクトを指定 */
	public int pokeStateUnitArrayIndex = -1; // pokeStateUnitArrayの添字

	/* パーティテキストファイル読み込みメソッド
	 *   引数　：パーティテキストファイル名
	 *   戻り値：無し
	 *   例外　：IOException（パーティテキストファイルが開けない場合）
	 *   　　　：NullPointerException（パーティテキストファイルが空の場合）
	 */
    public void partyTextFileRead(String partyTextFileName) throws IOException,NullPointerException {

    	/* パーティテキストファイル読み込みやり直しの考慮 */
    	for(int i = 0; i < 6; i++){
    		partyTextArray[i] = null;
    	}

    	/* パーティテキストファイルパス情報 */
        String partyTextFileForder="conf";
        String partyTextFilePath=partyTextFileForder+"\\"+partyTextFileName;

        /* BufferedReaderオブジェクト生成 */
        File file = new File(partyTextFilePath);
        BufferedReader br = new BufferedReader
                           (new InputStreamReader
                           (new FileInputStream(file),"SJIS"));

        /* パーティテキストファイル読み込み */
        String str = br.readLine();
        int i = 0;
        if(str == null) {
        	br.close();
        	throw new NullPointerException();
        } else {
            while(str != null && i < 6){
            	partyTextArray[i] = str;
            	i++;
                str = br.readLine();
            }
            br.close();
        }

        return;
    }

	/* 型（PokeMoldオブジェクト）セットメソッド
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public void partyPokeMoldSet() {

    	/* 型セットやり直しの考慮 */
    	for(int i = 0; i < 6; i++){
    		pokeMoldArray[i] = null;
    	}

    	/* パーティテキストファイルのレコードからPokeMoldオブジェクトを生成し配列へ格納 */
    	int i = 0;
    	while(i < 6 && partyTextArray[i] != null){
    		pokeMoldArray[i] = new PokeMold(partyTextArray[i]);
    		i++;
    	}
    }

	/* 各々のポケモンの状態（PokeStateUnitオブジェクト）をセットするメソッド
	 *   引数　：PokeMoldオブジェクト、プレイヤー名(String)
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public void pokeStateUnitSet(PokeMold pokeMold,String playerNameDisp) {
    	/* PokeStateUnitオブジェクト配列の一番左側にセット */
    	for(int i = 0; i < 6; i++){
    		if(pokeStateUnitArray[i] == null) {
    			pokeStateUnitArray[i] = new PokeStateUnit(pokeMold,playerNameDisp);
    			i = 6;
    		}
    	}
    }

	/* どのポケモン（PokeStateUnitオブジェクト）が場に出ているのかを変更するメソッド（交代の度に実行）
	 *   引数　：交代先のポケモンの添字
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public void pokeStateUnitChange(int pokeStateUnitArrayIndex) {
    	this.pokeStateUnitArrayIndex = pokeStateUnitArrayIndex;
    }

	/* PokeStateUnitオブジェクトの初期化（選出やり直し）
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public void pokeStateUnitInitialize() {
    	for(int i = 0; i < 6; i++){
    		pokeStateUnitArray[i] = null;
    	}
    	this.pokeStateUnitArrayIndex = -1;
    }

}
