package state;

import java.io.IOException;
import java.util.Random;

import util.DialogUtil;

/*
*  PokeStatePlaceクラス
*
*  交代で初期化されバトンタッチでも引き継がれないポケモンの状態を管理するクラスです。
*  各々のポケモン間で同一オブジェクトを共有してください。
*
*  変更履歴
*  2019/04/06 仮作成
*  2019/05/03 完成
*  2019/05/04 技ではないものが選択された場合に最後に使用した技の情報が更新されないように修正
*  2019/05/04 物真似不可・変身不可の条件を追加
*  2019/05/06 締め付ける・金縛り・アンコールのターンが更新されないように条件を追加
*  2019/05/11 技指定メソッド、PP増減メソッド（物真似・変身用）、メロメロ判定メソッド、金縛り技繰り出し判定メソッド追加
*  2019/05/12 技命中時共通メソッドについて、カウンター・ミラーコートできる条件を修正
*  2019/05/15 最後に使用した技の名前を格納するメンバ変数を追加（物真似の既に覚えていない技かどうか判定で必要）
*  2019/05/18 最後に使用した技の威力・命中率・追加効果発動率を追加（オウム返しで必要だった）
*  2019/05/25 転がるの連続命中回数をカウントする変数を追加
*  2019/05/25 金縛りに失敗した時のメッセージを追加
*  2019/05/26 技コードの初期値を変更
*  2019/05/26 怒りについて、後手で繰り出した時にそのターンの先手の攻撃でボルテージが上がるバグを修正
*  2019/05/26 道連れ解除の実装のため、技使用フラグ（このターンに技を使用したかどうか）を追加
*  2019/05/26 未来予知のダメージを受けた時のメソッドを追加
*  2019/05/26 金縛り・アンコールにならない、金縛り・アンコールが解けないバグを修正
*  2019/05/26 痛み分けがカウンター・我慢対象になるバグを修正
*  2019/06/01 コンボターンカウントダウンについて、コンボ残り継続ターン数が－になるのを防ぐ修正（ソラビ選択後晴れ対応）
*  2019/06/05 寝言で出た技がカウンター・ミラーコートの対象にならないように修正
*  2019/06/08 指を振る・オウムがえしでコンボ技が出た時の管理用変数を追加
*  2019/06/08 変身メソッドについて引数の指定方法を変更（結局地道に指定した方が楽だった）
*  2019/06/08 直近に使用した技について、元々のPPも保存するように変更
*  2019/06/09 締め付ける関連のメソッドについて引数・戻り値・メッセージ出力仕様を見直し使いやすくした
*  2019/06/09 怯みフラグを立てるメソッドが無かったので追加
*  2019/06/22 暴れる等終了時の混乱判定をフラグ管理するように変更（コンボ中断時に混乱にならないように制御が必要なため）
*  2019/06/22 締め付けるが先攻か後攻かでダメージ発生ターンが変わるバグを訂正
*  2019/07/15 配列の初期化方法を誤っていたので修正
*  2019/07/29 暴れる等で混乱が発生しないバグを修正
*  2019/07/31 転がる等について、１発目を食らわないとカウンター・ミラコで返せないPBSバグを再現できていなかったのを修正
*  2019/07/31 我慢のダメージが常に0になるバグを修正
*  2020/08/27 「カウンター」「がまん」で一撃技命中を返した時に相手を倒せないことがあるバグを修正
*  2020/08/28 一撃技を「カウンター」「がまん」で返した時のダメージを元に戻した（再検証結果の反映）
*             気合のハチマキ発動フラグやみねうち発動フラグを追加（「カウンター」「がまん」のダメージ+1反映のため）
*  2020/09/12 奇跡の実で眠りを回復したことを示すフラグを追加（悪夢が回復しない仕様の実装のため）
*  2020/10/25 転がるの１発目を食らってなくてもカウンターが成功するバグを修正
*  2020/11/03 バトンタッチ成功フラグ、吠える・吹き飛ばし被成功フラグを追加
*  2020/11/21 交代先指定状態初期化メソッドを追加（使用しなくても良いと思うが念のため）
*  2021/01/01 各種コンボや金縛りやアンコールや締め付けるのターンカウント変数を追加（ルーチン作成用）
*  2021/01/23 ルーチンでの参照禁止変数をprivate化
*
*/

public class PokeStatePlace {

	/* 表示用項目（対話式で使用） */
	public String playerNameDisp = "Player"; //プレイヤー名
	public String speciesNameDisp = "けつばん"; //種族名

	/* コマンド関連（技繰り出し時に両方-1なら悪あがきと判断） */
	public int changeTarget = -1; //交代先（PokeStateUnit配列の添字を指定、交代先未定の場合は-1を指定）
	public int moveIndex = -1; //選択した技（move系配列の添字を指定、ターンの終わりに-1に戻す）

	/* テクスチャー・テクスチャー２関連 */
	public boolean pastedFlag = false; //テクスチャー・テクスチャー２でtrueにし変数「pasted～」使用
	public String pastedType1 = "normal"; //タイプ１
	public String pastedType2 = "none"; //タイプ２

	/* 物真似関連 */
	public int mimicMoveIndex = -1; //物真似した技が位置する技配列の添字を入れ変数「mimic～」使用
	public String mimicMoveName = "技無し"; //技名
	public String mimicMoveType = "question"; //技のタイプ
	public int mimicMovePower = 1; //技の威力
	public double mimicMoveHitProbability = 100.0; //技の命中率(百分率の概算値)
	public int mimicMovePP = 0; //PP
	public int mimicMovePPPrimitive = 0; //ポイントアップを使用しない場合の最大PP
	public double mimicAdditionalEffectProbability = 0.0; //追加効果発動率(百分率の概算値)
	public String mimicMoveEffectCode = "XXc"; //技の効果コード

	/* 変身関連 */
	public boolean transformedFlag = false; //変身でtrueにし変数「transformed～」使用
	public String transformedType1 = "normal"; //タイプ１
	public String transformedType2 = "none"; //タイプ２
	public int transformedAttack = 1; //攻撃実数値
	public int transformedBlock = 1; //防御実数値
	public int transformedContact = 1; //特攻実数値
	public int transformedDiffence = 1; //特防実数値
	public int transformedSpeed = 1; //素早さ実数値
	public String transformedHiddenPowerType = "dark"; //めざめるパワーのタイプ
	public int transformedHiddenPowerPower = 31; //めざめるパワーの威力
	public String[] transformedMoveName = {"技無し","技無し","技無し","技無し"}; //技名
	public String[] transformedMoveType = {"question","question","question","question"}; //技のタイプ
	public int[] transformedMovePower = {1,1,1,1}; //技の威力
	public double[] transformedMoveHitProbability = {100.0,100.0,100.0,100.0}; //技の命中率(百分率の概算値)
	public int[] transformedMovePP = {0,0,0,0}; //PP
	public int[] transformedMovePPPrimitive = {0,0,0,0}; //ポイントアップを使用しない場合の最大PP
	public double[] transformedAdditionalEffectProbability = {0.0,0.0,0.0,0.0}; //追加効果発動率(百分率の概算値)
	public String[] transformedMoveEffectCode = {"XXc","XXc","XXc","XXc"}; //技の効果コード

	/* 複数ターン関連管理領域 */
	public boolean thisTurnUseMoveFlag = false; //技使用フラグ…今のターンで技を使用したか（道連れ解除用に使用）
	public int latestUsedMoveIndex = -1; //最後に使用した技の配列位置（技を使用するまで上書きしない）
	                                       //相手が最後に何の技を使ったのかを参照する必要がある時に使用
	                                       //複数ターン技の残り継続ターンが残っている場合にどの技を繰り出すのか判定用
	public String latestUsedMoveName = "技無し"; //最後に使用した技の名前（技を使用するまで上書きしない）
                                                  //物真似用（覚えていない技かどうかの判定で必要）
	public String latestUsedMoveType = "question"; //最後に使用した技のタイプ（技を使用するまで上書きしない）
                                                    //カウンター・ミラーコート用（PokeStatePlaceクラス内部で使用）
	public int latestUsedMovePower = 1; //最後に使用した技の威力（技を使用するまで上書きしない）
                                         //オウムがえし用
	public double latestUsedMoveHitProbability = 100.0; //最後に使用した技の命中率（技を使用するまで上書きしない）
                                                          //オウムがえし用
	public int latestUsedMovePPPrimitive = 0; //最後に使用した技のポイントアップを使用しない場合の最大PP
	public double latestUsedAdditionalEffectProbability = 0.0; //最後に使用した技の追加効果発動率（技を使用ｒｙ）
                                                                 //オウムがえし用
	public String latestUsedMoveEffectCode = "XXc"; //最後に使用した技の効果コード（技を使用するまで上書きしない）
                                                     //複数ターン関連のフラグ・ターン数制御や各種フラグ制御で使う
	                                                 //（PokeStatePlaceクラス内部で使用）
	public boolean moveCopyMoveFlag = false; //ゆびをふる・オウムがえしで技を出したかどうかのフラグ
	public String latestUsedMoveNameMoveCopy = "技無し"; //最後に使用した技の名前（ゆびをふる等でコンボ技出した時用）
	public String latestUsedMoveTypeMoveCopy = "question"; //最後に使用した技のタイプ
	                                                         //（ゆびをふる等でコンボ技出した時用）
	public int latestUsedMovePowerMoveCopy = 1; //最後に使用した技の威力（ゆびをふる等でコンボ技出した時用）
	public double latestUsedMoveHitProbabilityMoveCopy = 100.0; //最後に使用した技の命中率
	                                                               //（ゆびをふる等でコンボ技出した時用）
	public int latestUsedMovePPPrimitiveMoveCopy = 0; //最後に使用した技のポイントアップを使用しない場合の最大PP
	                                                    //（ゆびをふる等でコンボ技出した時用）
	public double latestUsedAdditionalEffectProbabilityMoveCopy = 0.0; //最後に使用した技の追加効果発動率
	                                                                      //（ゆびをふる等でコンボ技出した時用）
	public String latestUsedMoveEffectCodeMoveCopy = "XXc"; //最後に使用した技の効果コード
	                                                          //（ゆびをふる等でコンボ技出した時用）
	private int comboRestTurn = 0; //複数ターン技の残り継続ターン数（発生で1～5代入、動くと-1、動けない・外れで0代入）…ルーチンでの参照禁止
	                               //暴れる・転がる・花びらの舞・逆鱗・
	                               //穴を掘る・空を飛ぶ・かまいたち・ゴッドバード・ソーラービーム・
	                               //我慢用
	public int comboCountTrun = -1; //複数ターン技ターンカウント（初期値-1、発生で0を入れ、1回動く毎に+1）…ルーチン作成用
	public boolean confusionReservationFlag = false; //暴れる・花びらの舞・逆鱗で残りターンカウント0時混乱予約フラグ
	                                                    //通常のカウントダウンで残り継続ターン数0になったらtrue
	                                                    //実際に混乱になったらfalse
	public int thisTrunGetPhysicDamage = 0; //そのターンに相手の技で最初に受けた物理ダメージ（ターン終了時に初期化）
	                                           //カウンター・怒り用
	public int thisTrunGetSpecialDamage = 0; //そのターンに相手の技で最初に受けた特殊ダメージ（ターン終了時に初期化）
                                              //ミラーコート・怒り用
	public boolean comboFirstFlag = false; //カウンター・ミラーコートのPBSバグ再現用フラグ
	                                          //暴れる・転がる・花びらの舞・逆鱗の１発目の時だけtrue
	                                          //ターン終了時にfalseに戻す
	public boolean comboFirstDamageFlag = false; //カウンター・ミラーコートのPBSバグ再現用フラグ
	                                                //暴れる・転がる・花びらの舞・逆鱗の１発目を食らった時だけtrue
	                                                //他の技を受けた時にfalseに戻す
	                                                //暴れる・転がる・花びらの舞・逆鱗はこれがtrueでないと返せない
	public int accumulationGetDamage = 0; //相手から受けたダメージの累積（残り継続ターン数0で初期化）
	                                       //我慢用

	/* 各種フラグ類 */
	public boolean winceFlag = false; //怯みフラグ（発生でtrueを入れ、ターンの終わりにfalse）
	public boolean deprivingMindFlag = false; //メロメロフラグ（被メロメロフラグと同時にtrue時のみ効果を発揮）
	public boolean deprivedMindFlag = false; //被メロメロフラグ（メロメロフラグと同時にtrue時のみ効果を発揮）
	public int moveParalysisIndex = -1; //金縛りされている技（技配列の添字）
	private int moveParalysisRestTurn = 0; //残り金縛りターン数（発生で1～7を入れ、技の繰り出し判定で-1）…ルーチンでの参照禁止
	public int moveParalysisCountTrun = -1; //金縛りターンカウント（初期値-1、金縛り発生で0を入れ、1判定毎に+1）…ルーチン作成用
	public int encoredMoveIndex = -1; //アンコールされている技（技配列の添字）
	private int encoredRestTurn = 0; //残りアンコールターン数（発生で2～7を入れ、ターンの終わりに-1）…ルーチンでの参照禁止
	public int encoredCountTrun = -1; //アンコールターンカウント（初期値-1、アンコール発生で0を入れ、1ターン毎に+1）…ルーチン作成用
	public boolean lockonFlag = false; //被ロックオンフラグ（技の命中判定で命中になってからfalseに戻す点に注意）
	public boolean seeingThroughFlag = false; //見破るフラグ（被見破るフラグと同時にtrue時のみ効果を発揮）
	public boolean seenThroughFlag = false; //被見破るフラグ（見破るフラグと同時にtrue時のみ効果を発揮）
	private int twiningRestTrun = 0; //締め付ける残りターン数（発生で2～5、ターン終了-1、被締め付けると同時のみ効果有）…ルーチンでの参照禁止
	private int twindRestTrun = 0; //被締め付ける残りターン数（発生で2～5、ターン終了-1、締め付けると同時のみ効果有）…ルーチンでの参照禁止
	public int twiningCountTrun = -1; //締め付けるターンカウント（初期値-1、締め付ける発生で0を入れ、1ターン毎に+1）…ルーチン作成用
	public boolean roundShapeFlag = false; //丸くなるフラグ
	public boolean smallShapeFlag = false; //小さくなるフラグ
	public boolean intoUndergroundFlag = false; //地中フラグ
	public boolean intoSkyFlag = false; //空中フラグ
	public int protectContinuousTrun = 1; //守る継続ターン（守る・見切り・堪える成功で+1、別の技使用or失敗で1に戻す）
	public boolean protectSuccessFlag = false; //守る成功フラグ（守る・見切り・堪える成功でture、ターン終了時false）
	public boolean downTogetherFlag = false; //道連れフラグ（相手or自分の技使用でfalseに戻す）
	public int angerContinuousTrun = 1; //怒り継続ターン（最後の使用技が怒り以外または相手からのダメージ0で1に戻す）
	public int slashContinuousTrun = 1; //連続切り継続ターン（最後の使用技が連続切り以外または外れで1に戻す）
	public boolean slashContinuousHitFlag = false; //連続切り命中フラグ（連続切り命中でture、ターン終了時にfalse）
	public int rollContinuousTrun = 1; //転がる継続ターン（最後の使用技が転がる以外または外れまたはコンボ終了で1に）
	public boolean rollContinuousHitFlag = false; //転がる命中フラグ（連続切り命中ture、ターン終了orコンボ終了false）
	public int destructionBeemStackRestTrun = 0; //破壊光線反動残りターン（破壊光線命中で2代入、ターンの終わりに-1）
	public boolean nightmareFlag = false; //悪夢フラグ（眠る回復でfalseに戻す、PBSではバトン対象外）
	public boolean spiritFlag = false; //気合のハチマキ発動フラグ（ターンの終わりにfalse）
	public boolean mineuchiFlag = false; //みねうち発動フラグ（ターンの終わりにfalse）
	public boolean wakeupByMiracleBerryFlag = false; //眠りを奇跡の実で回復したかどうかのフラグ（ターン終了時にfalse）
	public boolean batonSuccessFlag = false; //バトンタッチが成功したかどうかのフラグ（ターン終了時にfalse）
	public boolean forceChangedFlag = false; //吠える・吹き飛ばしを成功されたかどうかのフラグ（ターン終了時にfalse）

	/* デフォルトコンストラクタ：何もしない
	 * 　　　　　　　　　　　　　（直接プレイヤー名を代入する場合はこちらのコンストラクタを使用）
	 */
    public PokeStatePlace() {
    }

	/* コンストラクタ：表示用項目のプレイヤー名をセット
	 * 　　　　　　　　（表示用項目のプレイヤー名を使用したい場合はこちらのコンストラクタを使用）
	 *   引数　：プレイヤー名（String）
	 */
    public PokeStatePlace(String playerNameDisp) {
    	this.playerNameDisp = playerNameDisp;
    }

	/* 交代時・バトンタッチ時・死に出し時初期化共通メソッド
	 *   タイミング：交代orバトンタッチor死に出しを行う度
	 *   引数　　　：種族名（String）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeBatonInitCommon(String speciesNameDisp) {
    	this.speciesNameDisp = speciesNameDisp;
    	changeTarget = -1;
    	moveIndex = -1;
    	pastedFlag = false;
    	pastedType1 = "normal";
    	pastedType2 = "none";
    	mimicMoveIndex = -1;
    	mimicMoveName = "技無し";
    	mimicMoveType = "question";
    	mimicMovePower = 1;
    	mimicMoveHitProbability = 100.0;
    	mimicMovePP = 0;
    	mimicMovePPPrimitive = 0;
    	mimicAdditionalEffectProbability = 0.0;
    	mimicMoveEffectCode = "XXc";
    	transformedFlag = false;
    	transformedType1 = "normal";
    	transformedType2 = "none";
    	transformedAttack = 1;
    	transformedBlock = 1;
    	transformedContact = 1;
    	transformedDiffence = 1;
    	transformedSpeed = 1;
    	transformedHiddenPowerType = "dark";
    	transformedHiddenPowerPower = 31;
    	transformedMoveName = null;
    	transformedMoveName = new String[4];
    	transformedMoveType = null;
    	transformedMoveType = new String[4];
    	transformedMovePower = null;
    	transformedMovePower = new int[4];
    	transformedMoveHitProbability = null;
    	transformedMoveHitProbability = new double[4];
    	transformedMovePP = null;
    	transformedMovePP = new int[4];
    	transformedMovePPPrimitive = null;
    	transformedMovePPPrimitive = new int[4];
    	transformedAdditionalEffectProbability = null;
    	transformedAdditionalEffectProbability = new double[4];
    	transformedMoveEffectCode = null;
    	transformedMoveEffectCode = new String[4];
    	for(int i = 0; i < 4; i++){
        	transformedMoveName[i] = "技無し";
        	transformedMoveType[i] = "question";
        	transformedMovePower[i] = 1;
        	transformedMoveHitProbability[i] = 100.0;
        	transformedMovePP[i] = 0;
        	transformedMovePPPrimitive[i] = 0;
        	transformedAdditionalEffectProbability[i] = 0.0;
        	transformedMoveEffectCode[i] = "XXc";
    	}
    	thisTurnUseMoveFlag = false;
    	latestUsedMoveIndex = -1;
    	latestUsedMoveName = "技無し";
    	latestUsedMoveType = "question";
    	latestUsedMovePower = 1;
    	latestUsedMoveHitProbability = 100.0;
    	latestUsedMovePPPrimitive = 0;
    	latestUsedAdditionalEffectProbability = 0.0;
    	latestUsedMoveEffectCode = "XXc";
    	moveCopyMoveFlag = false;
    	latestUsedMoveNameMoveCopy = "技無し";
    	latestUsedMoveTypeMoveCopy = "question";
    	latestUsedMovePowerMoveCopy = 1;
    	latestUsedMoveHitProbabilityMoveCopy = 100.0;
    	latestUsedMovePPPrimitiveMoveCopy = 0;
    	latestUsedAdditionalEffectProbabilityMoveCopy = 0.0;
    	latestUsedMoveEffectCodeMoveCopy = "XXc";
    	comboRestTurn = 0;
    	comboCountTrun = -1;
    	confusionReservationFlag = false;
    	thisTrunGetPhysicDamage = 0;
    	thisTrunGetSpecialDamage = 0;
    	comboFirstDamageFlag = false;
    	accumulationGetDamage = 0;
    	winceFlag = false;
    	deprivingMindFlag = false;
    	deprivedMindFlag = false;
    	moveParalysisIndex = -1;
    	moveParalysisRestTurn = 0;
    	moveParalysisCountTrun = -1;
    	encoredMoveIndex = -1;
    	encoredRestTurn = 0;
    	encoredCountTrun = -1;
    	lockonFlag = false;
    	seeingThroughFlag = false;
    	seenThroughFlag = false;
    	twiningRestTrun = 0;
    	twindRestTrun = 0;
    	twiningCountTrun = -1;
    	roundShapeFlag = false;
    	smallShapeFlag = false;
    	intoUndergroundFlag = false;
    	intoSkyFlag = false;
    	protectContinuousTrun = 1;
    	protectSuccessFlag = false;
    	downTogetherFlag = false;
    	angerContinuousTrun = 1;
    	slashContinuousTrun = 1;
    	slashContinuousHitFlag = false;
    	rollContinuousTrun = 1;
    	rollContinuousHitFlag = false;
    	destructionBeemStackRestTrun = 0;
    	nightmareFlag = false;
    	spiritFlag = false;
    	mineuchiFlag = false;
    	wakeupByMiracleBerryFlag = false;
    	batonSuccessFlag = false;
    	forceChangedFlag = false;
    }

	/* 技繰り出し時共通メソッド
	 *   タイミング：技の繰り出し判定・PPの判定が終了した後
	 *   引数　　　：技の配列位置（int）、技の名前（String）、
	 *   　　　　　：技のタイプ（String）、技の威力（int）、
	 *   　　　　　：技の命中率（double）、技の元々のPP（int）、
	 *   　　　　　：技の追加効果発動率（double）、技の効果コード（String）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void whenMovingCommon(int latestUsedMoveIndex,String latestUsedMoveName,
                                   String latestUsedMoveType,int latestUsedMovePower,
                                   double latestUsedMoveHitProbability,int latestUsedMovePPPrimitive,
                                   double latestUsedAdditionalEffectProbability,String latestUsedMoveEffectCode) {
    	// 技ではないもの（交代、混乱自滅、技無し）が選択された場合は処理を行わない
    	if (!latestUsedMoveEffectCode.equals("XXa")&&
    	    !latestUsedMoveEffectCode.equals("XXb")&&
    	    !latestUsedMoveEffectCode.equals("XXc")) {
        	thisTurnUseMoveFlag = true;
        	this.latestUsedMoveIndex = latestUsedMoveIndex;
        	this.latestUsedMoveName = latestUsedMoveName;
        	this.latestUsedMoveType = latestUsedMoveType;
        	this.latestUsedMovePower = latestUsedMovePower;
        	this.latestUsedMoveHitProbability = latestUsedMoveHitProbability;
        	this.latestUsedMovePPPrimitive = latestUsedMovePPPrimitive;
        	this.latestUsedAdditionalEffectProbability = latestUsedAdditionalEffectProbability;
        	this.latestUsedMoveEffectCode = latestUsedMoveEffectCode;
        	if (!this.latestUsedMoveEffectCode.equals("62")) {
        		downTogetherFlag = false;
        	}
    	}
    }

	/* 指を振る・オウムがえしで出した技管理メソッド
	 *   タイミング：指を出すで繰り出す技が判明した後
	 *   引数　　　：技の名前（String）、技のタイプ（String）、
	 *   　　　　　：技の威力（int）、技の命中率（double）、
	 *   　　　　　：技の元々のPP（int）、
	 *   　　　　　：技の追加効果発動率（double）、
	 *   　　　　　：技の効果コード（String）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void whenMoveCopyCommon(String latestUsedMoveNameMoveCopy,String latestUsedMoveTypeMoveCopy,
    	                              int latestUsedMovePowerMoveCopy,double latestUsedMoveHitProbabilityMoveCopy,
    	                              int latestUsedMovePPPrimitiveMoveCopy,
    		                          double latestUsedAdditionalEffectProbabilityMoveCopy,
    		                          String latestUsedMoveEffectCodeMoveCopy) {
    	moveCopyMoveFlag = true;
        this.latestUsedMoveNameMoveCopy = latestUsedMoveNameMoveCopy;
        this.latestUsedMoveTypeMoveCopy = latestUsedMoveTypeMoveCopy;
        this.latestUsedMovePowerMoveCopy = latestUsedMovePowerMoveCopy;
        this.latestUsedMoveHitProbabilityMoveCopy = latestUsedMoveHitProbabilityMoveCopy;
        this.latestUsedAdditionalEffectProbabilityMoveCopy = latestUsedAdditionalEffectProbabilityMoveCopy;
        this.latestUsedAdditionalEffectProbabilityMoveCopy = latestUsedAdditionalEffectProbabilityMoveCopy;
        this.latestUsedMoveEffectCodeMoveCopy = latestUsedMoveEffectCodeMoveCopy;
    }

	/* 技命中時共通メソッド
	 *   タイミング：相手のポケモンから攻撃が命中した時
	 *   引数　　　：本体HP変化量（int）、最後の攻撃による本体HP変化量（int）、相手のPokeStatePlaceオブジェクト
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void whenGetDamageCommon(int HPVariation,int LastHPVariation,PokeStatePlace pokeStatePlaceOpponent)
    		throws IOException {

    	//気合のハチマキ発動時やみねうち発動時は返すダメージを1増やす
    	if (spiritFlag || mineuchiFlag) {
    		LastHPVariation = LastHPVariation + 1;
    		accumulationGetDamage = accumulationGetDamage + 1;
    	}

    	//プレゼント回復はダメージ扱いしない
    	if (HPVariation > 0){
    		HPVariation = 0;
    		LastHPVariation = 0;
    	}

    	//痛み分けはダメージ扱いしない
    	if (pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("5B")){
    		HPVariation = 0;
    		LastHPVariation = 0;
    	}

    	//一撃技を返すと一撃で倒せる→実ダメージを返すが正しい
    	// if (pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("26")){
    	// 	LastHPVariation = 999;
    	// 	accumulationGetDamage = accumulationGetDamage + 999;
    	// }

    	//カウンター・ミラーコート用
    	//連続技の場合は最後の攻撃のダメージのみ反映
    	//「ゆびをふる」「オウムがえし」「ねごと」は対象外
    	if (!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("09") &&
    	    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("53") &&
    	    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("61")) {
    		switch (pokeStatePlaceOpponent.latestUsedMoveType) {
    		case "normal":
    		case "fighting":
    		case "flying":
    		case "poison":
    		case "ground":
    		case "rock":
    		case "bug":
    		case "ghost":
    		case "steel":
    		case "physic":
    			thisTrunGetPhysicDamage = LastHPVariation * (-1);
    			break;
    		case "fire":
    		case "water":
    		case "grass":
    		case "electric":
    		case "psychic":
    		case "ice":
    		case "dragon":
    		case "dark":
    			thisTrunGetSpecialDamage = LastHPVariation * (-1);
    			break;
    		default:break;
    		}

    		//暴れる、転がる、花びらの舞、逆鱗を返す時のPBSバグ再現（１発目を食らわないと返せないバグ）
    		if (pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("1B") ||
    			pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("75")){
    			if(pokeStatePlaceOpponent.comboFirstFlag == true) {
    				comboFirstDamageFlag = true;
        		} else {
        			// ２発目を食らってもカウンター可になったりカウンター不可になったりしないので何もしない
        		}
    			if (!comboFirstDamageFlag) {
    				thisTrunGetPhysicDamage = 0;
    				thisTrunGetSpecialDamage = 0;
    			}
    		} else {
    			comboFirstDamageFlag = false;
    		}
    	}

    	//我慢用
    	if (latestUsedMoveEffectCode.equals("1A") && comboRestTurn > 0) {
    		accumulationGetDamage = accumulationGetDamage + (HPVariation * (-1));
    	} else {
    		accumulationGetDamage = 0;
    	}

    	//怒りターンカウント
    	if (latestUsedMoveEffectCode.equals("51") && HPVariation < 0) {
        	angerContinuousTrun++;
        	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"の怒りのボルテージが上がった");
        }

    	//被ロックオン解除
    	lockonFlag = false;

    }

	/* 技命中時共通メソッド（未来予知用）
	 *   タイミング：ターン終了時（優先順１）
	 *   引数　　　：本体HP変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void whenGetPredictionCommon(int HPVariation) throws IOException {

    	//怒りターンカウント
    	if (latestUsedMoveEffectCode.equals("51") && HPVariation < 0) {
        	angerContinuousTrun++;
        	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"の怒りのボルテージが上がった");
        }

    	//被ロックオン解除
    	lockonFlag = false;

    }

	/* ターン終了時共通メソッド
	 *   タイミング：ターン終了時の処理が終わった後
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void turnEndCommon(){

    	// 技使用フラグをリセット
    	thisTurnUseMoveFlag = false;

    	// 怯み解除
    	winceFlag = false;

    	// 守る等ターンカウント
    	if ((latestUsedMoveEffectCode.equals("6F") ||
    		latestUsedMoveEffectCode.equals("74"))&&
    		protectSuccessFlag == true) {
    			protectContinuousTrun++;
    	} else {
    		protectContinuousTrun = 1;
    	}
    	protectSuccessFlag = false;

    	// 怒りターンカウント（未選択時リセット）
    	if (!latestUsedMoveEffectCode.equals("51")){
    		angerContinuousTrun = 1;
    	}

    	// 連続切りターンカウント
    	if (latestUsedMoveEffectCode.equals("77") && slashContinuousHitFlag == true) {
    		slashContinuousTrun++;
    	} else {
    		slashContinuousTrun = 1;
    	}
    	slashContinuousHitFlag = false;

    	// 転がるターンカウント
    	if (latestUsedMoveEffectCode.equals("75") && rollContinuousHitFlag == true && comboRestTurn > 0) {
    		rollContinuousTrun++;
    	} else {
    		rollContinuousTrun = 1;
    	}
    	rollContinuousHitFlag = false;

    	// 破壊光線反動ターンカウント
    	if (destructionBeemStackRestTrun > 0) {
    		destructionBeemStackRestTrun--;
    	}

    	// カウンター・ミラーコート・怒り用項目初期化
    	thisTrunGetPhysicDamage = 0;
    	thisTrunGetSpecialDamage = 0;
    	comboFirstFlag = false;

    	// 指を振る・オウム返しで出した技管理初期化
    	if (comboRestTurn <= 0) {
        	moveCopyMoveFlag = false;
        	latestUsedMoveNameMoveCopy = "技無し";
        	latestUsedMoveTypeMoveCopy = "question";
        	latestUsedMovePowerMoveCopy = 1;
        	latestUsedMoveHitProbabilityMoveCopy = 100.0;
        	latestUsedAdditionalEffectProbabilityMoveCopy = 0.0;
        	latestUsedMoveEffectCodeMoveCopy = "XXc";
    	}

    	// 気合のハチマキやみねうちを未発動に戻す
    	spiritFlag = false;
    	mineuchiFlag = false;

    	// バトンタッチ・吠える・吹き飛ばしの成功状態を元に戻す
    	batonSuccessFlag = false;
    	forceChangedFlag = false;

    	// コマンド関連添字初期化
    	changeTarget = -1;
    	moveIndex = -1;

    }

	/* 交代先指定メソッド
	 *   タイミング：交代orバトンor吠える吹き飛ばしor死に出しで交代先を指定する度
	 *   引数　　　：PokeStateUnit配列の添字（int）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeTargetSet(int changeTarget){
    	this.changeTarget = changeTarget;
    }

	/* 交代先指定状態初期化メソッド
	 *   タイミング：交代orバトンor吠える吹き飛ばしor死に出しによる交代が終了した後
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeTargetInit(){
    	this.changeTarget = -1;
    }

	/* 技指定メソッド
	 *   タイミング：技を選択する度
	 *   引数　　　：技配列の添字（int）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void moveIndexSet(int moveIndex){
    	this.moveIndex = moveIndex;
    }

	/* テクスチャーメソッド
	 *   タイミング：テクスチャー・テクスチャー２を行う度
	 *   引数　　　：タイプ名（String）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void textureType(String pastedType) throws IOException{
    	pastedFlag = true;
    	pastedType1 = pastedType;
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"に"+pastedType1+"を貼りつけた");
    }

	/* 物真似メソッド
	 *   タイミング：物真似を行う度
	 *   引数　　　：物真似した技が位置する技配列の添字（int）、技名（String）、技のタイプ（String）、技の威力（int）、
	 *   　　　　　　技の命中率（double）、PP（int）、追加効果発動率（double）、技の効果コード（String）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 *   　　　　　：技配列の指定が不正かどうかの判定は呼び出し側で行うこと
	 *   　　　　　：既に覚えている技かどうかの判定は呼び出し側で行うこと
	 */
    public void mimicMove(int mimicMoveIndex,String mimicMoveName,String mimicMoveType,
    		int mimicMovePower,double mimicMoveHitProbability,int mimicMovePP,
    		double mimicAdditionalEffectProbability,String mimicMoveEffectCode) throws IOException{
    	// ものまね、わるあがき、ねごと、オウムがえし、ゆびをふる、スケッチは物真似不可
    	if (!mimicMoveEffectCode.equals("52")&&
    	    !mimicMoveEffectCode.equals("30b")&&
    	    !mimicMoveEffectCode.equals("61")&&
    	    !mimicMoveEffectCode.equals("09")&&
    	    !mimicMoveEffectCode.equals("53")&&
    	    !mimicMoveEffectCode.equals("5F")) {
        	this.mimicMoveIndex = mimicMoveIndex;
        	this.mimicMoveName = mimicMoveName;
        	this.mimicMoveType = mimicMoveType;
        	this.mimicMovePower = mimicMovePower;
        	this.mimicMoveHitProbability = mimicMoveHitProbability;
        	this.mimicMovePP = mimicMovePP;
        	mimicMovePPPrimitive = mimicMovePP;
        	this.mimicAdditionalEffectProbability = mimicAdditionalEffectProbability;
        	this.mimicMoveEffectCode = mimicMoveEffectCode;
        	DialogUtil.dialogMessageOutput
        		(playerNameDisp+"の"+speciesNameDisp+"は"+this.mimicMoveName+"を物真似した");
    	} else {
        	DialogUtil.dialogMessageOutput
    			(playerNameDisp+"の"+speciesNameDisp+"は物真似に失敗した");
    	}
    }

	/* 物真似した技のPP増減メソッド
	 *   タイミング：物真似した技のPPが増減する度
	 *   引数　　　：PP変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void mimicPPUpDown(int PPVariation) throws IOException {
    	mimicMovePP = mimicMovePP + PPVariation;
    	if (mimicMovePP < 0) {
    		mimicMovePP = 0;
    	}
    	if (PPVariation != -1) {
    		DialogUtil.dialogMessageOutput
        	(playerNameDisp+"の"+speciesNameDisp+"の"+mimicMoveName+"が"+
    		String.format("%01d",PPVariation)+"増えた");
    	}
    }

	/* 変身メソッド
	 *   タイミング：変身を行う度
	 *   引数　　　：タイプ１(String)、タイプ２(String)、
	 *   　　　　　：ABCDSの実数値（全てint）、
	 *   　　　　　：めざめるパワーのタイプ（String）、めざめるパワーの威力（int）、
	 *   　　　　　：技名・技タイプ・技威力・命中率・PP・元のPP・追加効果発動率・技コード（全て所定の型の配列）、
	 *   　　　　　：相手が変身済みかどうか（boolean）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void transformPoke(String transformedType1,String transformedType2,
    		                    int transformedAttack,int transformedBlock,int transformedContact,
    		                    int transformedDiffence, int transformedSpeed,
    		                    String transformedHiddenPowerType, int transformedHiddenPowerPower,
    		                    String[] transformedMoveName,String[] transformedMoveType,
    		                    int[] transformedMovePower,double[] transformedMoveHitProbability,
    		                    int[] transformedMovePP,int[] transformedMovePPPrimitive,
    		                    double[] transformedAdditionalEffectProbability,
    		                    String[] transformedMoveEffectCode,
    		                    boolean oppenentTransformedFlag) throws IOException{
    	if (!oppenentTransformedFlag) {
        	transformedFlag = true;
        	this.transformedType1 = transformedType1;
        	this.transformedType2 = transformedType2;
        	this.transformedAttack = transformedAttack;
        	this.transformedBlock = transformedBlock;
        	this.transformedContact = transformedContact;
        	this.transformedDiffence = transformedDiffence;
        	this.transformedSpeed = transformedSpeed;
        	this.transformedHiddenPowerType = transformedHiddenPowerType;
        	this.transformedHiddenPowerPower = transformedHiddenPowerPower;
        	for(int i = 0; i < 4; i++){
        		this.transformedMoveName[i] = transformedMoveName[i];
        		this.transformedMoveType[i] = transformedMoveType[i];
        		this.transformedMovePower[i] = transformedMovePower[i];
        		this.transformedMoveHitProbability[i] = transformedMoveHitProbability[i];
        		this.transformedMovePP[i] = 5;
        		this.transformedMovePPPrimitive[i] = transformedMovePPPrimitive[i];
        		this.transformedAdditionalEffectProbability[i] = transformedAdditionalEffectProbability[i];
        		this.transformedMoveEffectCode[i] = transformedMoveEffectCode[i];
        	}
        	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は変身した");
    	} else {
        	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は変身に失敗した");
    	}
    }

	/* 変身した技のPP増減メソッド
	 *   タイミング：変身した技のPPが増減する度
	 *   引数　　　：技配列番号（int）、PP変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 *   　　　　　：存在しない技が指定された場合は何もせずに終了
	 */
    public void transformedPPUpDown(int moveIndex,int PPVariation) throws IOException {
    	if(moveIndex >= 0 && moveIndex <= 3) {
    		transformedMovePP[moveIndex] = transformedMovePP[moveIndex] + PPVariation;
    		if (transformedMovePP[moveIndex] < 0) {
    			transformedMovePP[moveIndex] = 0;
    		}
    		if (PPVariation != -1) {
    			DialogUtil.dialogMessageOutput
        		(playerNameDisp+"の"+speciesNameDisp+"の"+transformedMoveName[moveIndex]+"が"+
    			String.format("%01d",PPVariation)+"増えた");
    		}
    	}
    }

	/* 複数ターン技残り継続ターン数セットメソッド
	 *   タイミング：複数ターン技１ターン目時、技の繰り出し判定・PPの判定が終了した後
	 *   引数　　　：残り継続ターン数初期値（int）、技の効果コード（String）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void comboStart(int comboTurn,String moveCode) {
    	comboCountTrun = 0;
    	comboRestTurn = comboTurn;
    	if (moveCode.equals("1B")||moveCode.equals("75")) {
    		comboFirstFlag = true;
    	}
    }

	/* 複数ターン技残り継続ターン数カウントダウンメソッド
	 *   タイミング：複数ターン技２ターン目以降時、技の繰り出し判定が成功した後
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void comboCountDown() {
    	comboCountTrun++;
    	comboRestTurn--;
    	if (comboRestTurn <= 0) {
    		comboCountTrun = -1;
    		comboRestTurn = 0;
    		if (latestUsedMoveEffectCode.equals("1B")) {
    			confusionReservationFlag = true;
    		}
    	}
    }

	/* コンボ中かどうかを返すメソッド
	 *   タイミング：コンボ中かどうかの判定時
	 *   引数　　　：無し
	 *   戻り値　　：boolean(コンボ中ならtrue、コンボしてないならfalse)
	 *   例外　　　：無し
	 */
    public boolean isBeingCombo() {
    	return comboRestTurn > 0;
    }

	/* コンボ中ではないかどうかを返すメソッド
	 *   タイミング：コンボ中ではないかどうかの判定時
	 *   引数　　　：無し
	 *   戻り値　　：boolean(コンボ中ではないならtrue、コンボ中ならfalse)
	 *   例外　　　：無し
	 */
    public boolean isNotBeingCombo() {
    	return comboRestTurn == 0;
    }

	/* コンボ中の最終ターンかどうかを返すメソッド
	 *   タイミング：コンボ中の最終ターンかどうかの判定時（コンボの最終ターンでのみ効果が違う技がある、我慢とか）
	 *   引数　　　：無し
	 *   戻り値　　：boolean(コンボ中の最終ターンならtrue、それ以外ならfalse)
	 *   例外　　　：無し
	 */
    public boolean isBeingComboLastTrun() {
    	return comboRestTurn == 1;
    }

	/* 暴れる等終了時混乱判定メソッド
	 *   タイミング：ターン終了時（優先順３）
	 *   引数　　　：無し
	 *   戻り値　　：boolean(混乱するならtrue,しないならfalse)
	 *   例外　　　：無し
	 */
    public boolean comboEndConfusionJudge() {
    	if (confusionReservationFlag) {
    		confusionReservationFlag = false;
    		return true;
    	} else {
    		return false;
    	}
    }

	/* 複数ターン技残り継続ターン数リセットメソッド
	 *   タイミング：複数ターン技２ターン目以降時、技の繰り出し判定で失敗した後
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void comboCountReset() {
    	comboCountTrun = -1;
    	comboRestTurn = 0;
    }

	/* 我慢ダメージリセットメソッド
	 *   タイミング：我慢が解き放たれた時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void bideEnd() {
    	accumulationGetDamage = 0;
    }

	/* 怯み発生時用メソッド
	 *   タイミング：怯みが発生した時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void winceFlagSet() {
    	winceFlag = true;
    }

	/* メロメロにした時用メソッド
	 *   タイミング：メロメロにした時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void deprivingMind() throws IOException{
    	deprivingMindFlag = true;
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"はメロメロにした");
    }

	/* メロメロにされた時用メソッド
	 *   タイミング：メロメロにされた時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void deprivedMind() throws IOException{
    	deprivedMindFlag = true;
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"はメロメロにされた");
    }

	/* メロメロ判定メソッド
	 *   タイミング：技の繰り出し判定時（優先順５）　※メロメロ状態の時のみ呼び出し
	 *   引数　　　：無し
	 *   戻り値　　：boolean（trueで動けない、falseで動ける）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public boolean deprivingMindJudge() throws IOException{
    	Random random = new Random();
        int deprivingMindRandom = random.nextInt(2);
        int deprivingMindBoarder = 1;
        if (deprivingMindRandom < deprivingMindBoarder) {
        	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"はメロメロで技を繰り出せない");
        	return true;
        } else {
        	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"はメロメロだが技を繰り出せる");
        	return false;
        }
    }

	/* 金縛りされた時用メソッド
	 *   タイミング：金縛りされた時
	 *   引数　　　：対象の技の配列番号（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 *   　　　　　：存在しない技が指定された場合は何もせずに終了
	 */
    public void moveParalysisSet(int moveParalysisIndex) throws IOException{
    	if(moveParalysisIndex >= 0 && moveParalysisIndex <= 3 && this.moveParalysisIndex == -1) {
        	this.moveParalysisIndex = moveParalysisIndex;
        	Random random = new Random();
        	moveParalysisRestTurn = random.nextInt(7) + 1;
        	moveParalysisCountTrun = 0;
        	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は金縛りになった");
    	} else {
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は金縛りになる技がなかった");
    	}
    }

	/* 金縛りカウントダウンメソッド
	 *   タイミング：技を繰り出す直前（優先順６（金縛り判定前））
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void moveParalysisCountDown() throws IOException{
    	if (moveParalysisRestTurn > 0) {
    		moveParalysisCountTrun++;
    		moveParalysisRestTurn--;
    		if(moveParalysisRestTurn == 0) {
    			moveParalysisCountTrun = -1;
    			moveParalysisIndex = -1;
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"のかなしばりが解けた");
    		}
    	}
    }

	/* 金縛り技繰り出し判定メソッド
	 *   タイミング：技を繰り出す直前（優先順６）
	 *   引数　　　：無し
	 *   戻り値　　：boolean（trueで動けない、falseで動ける）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public boolean moveParalysisJudge() throws IOException{
    	if (moveIndex == moveParalysisIndex) {
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"はかなしばりで動けない");
    		return true;
    	} else {
    		return false;
    	}
    }

	/* 金縛り中かどうかを返すメソッド
	 *   タイミング：金縛り中かどうかの判定時
	 *   引数　　　：無し
	 *   戻り値　　：boolean(金縛り中ならtrue、金縛り中でないならfalse)
	 *   例外　　　：無し
	 */
    public boolean isMoveParalysis() {
    	return moveParalysisRestTurn > 0;
    }

	/* アンコール用メソッド
	 *   タイミング：アンコールされた時
	 *   引数　　　：対象の技の配列番号（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 *   　　　　　：存在しない技が指定された場合は何もせずに終了
	 */
    public void encoredSet(int encoredMoveIndex) throws IOException{
    	if(encoredMoveIndex >= 0 && encoredMoveIndex <= 3 && this.encoredMoveIndex == -1) {
        	this.encoredMoveIndex = encoredMoveIndex;
        	encoredCountTrun = 0;
        	Random random = new Random();
        	encoredRestTurn = random.nextInt(6) + 2;
        	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"はアンコールされた");
    	} else {
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"へのアンコールは失敗した");
    	}
    }

	/* アンコールカウントダウンメソッド
	 *   タイミング：ターン終了時（優先順１０（アンコール及びそれ以降の内部的な処理））
	 *   引数　　　：アンコール対象の技のPP（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void encoredCountDown(int movePP) throws IOException{
    	if(encoredRestTurn > 0) {
    		encoredCountTrun++;
    		encoredRestTurn--;
    		if (movePP == 0) {
    			encoredRestTurn = 0;
    		}
    		if(encoredRestTurn == 0) {
    			encoredCountTrun = -1;
    			encoredMoveIndex = -1;
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"のアンコールが解けた");
    		}
    	}
    }

	/* アンコール中かどうかを返すメソッド
	 *   タイミング：アンコール中かどうかの判定時
	 *   引数　　　：無し
	 *   戻り値　　：boolean(アンコール中ならtrue、アンコール中でないならfalse)
	 *   例外　　　：無し
	 */
    public boolean isEncored() {
    	return encoredRestTurn > 0;
    }

	/* ロックオン用メソッド
	 *   タイミング：ロックオンされた時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void beenLockon() throws IOException{
    	lockonFlag = true;
        DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"はロックオンされた");
    }

	/* 見破った時用メソッド
	 *   タイミング：見破った時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void seeingThrough() throws IOException{
    	seeingThroughFlag = true;
        DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は見破った");
    }

	/* 見破られた時用メソッド
	 *   タイミング：見破られた時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void seenThrough() throws IOException{
    	seenThroughFlag = true;
        DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は見破られた");
    }

	/* 締め付けた時用メソッド
	 *   タイミング：締め付けた時・スピンされた時
	 *   引数　　　：締め付けターン（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void twiningSet(int twiningRestTrun) throws IOException{
    	if ((this.twiningRestTrun == 0 && twiningRestTrun > 0)||(this.twiningRestTrun > 0 && twiningRestTrun == 0)) {
    		this.twiningRestTrun = twiningRestTrun;
    		if (this.twiningRestTrun > 0) {
    			twiningCountTrun = 0;
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は締め付けた");
    		} else {
    			twiningCountTrun = -1;
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"の締め付けは解除された");
    		}
    	}
    }

	/* 締め付けられた時用メソッド
	 *   タイミング：締め付けられた時・スピンした時
	 *   引数　　　：締め付けターン（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void twindSet(int twindRestTrun) throws IOException{
    	if ((this.twindRestTrun == 0 && twindRestTrun > 0)||(this.twindRestTrun > 0 && twindRestTrun == 0)) {
    		this.twindRestTrun = twindRestTrun;
    		if (this.twindRestTrun > 0) {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は締め付けられた");
    		} else {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は締め付けから解放された");
    		}
    	}
    }

	/* 締め付けターン生成メソッド
	 *   タイミング：「しめつける」系の技を使用した時
	 *   引数　　　：無し
	 *   戻り値　　：締め付けターン（int）
	 *   例外　　　：無し
	 */
    public int twiningTwindTrunGenerate(){
    	Random random = new Random();
    	return random.nextInt(4) + 2;
    }

	/* 締め付けカウントダウンメソッド（補正用）
	 *   タイミング：ターン開始時（技・交代先選択前）
	 *   　　　　　：先攻・後攻でダメージ発生ターンの違いを生み出さずに、締め付け解放と同時に交代できるようにするため
	 *   引数　　　：相手の締め付けられカウント(int)
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void twiningCountDown(int twindRestTrun) {
    	if(twindRestTrun == 0) {
    		twiningCountTrun = -1;
    		twiningRestTrun = 0;
    	}
    	if(twiningRestTrun > 0) {
    		twiningCountTrun++;
    		twiningRestTrun--;
    		if(twiningRestTrun == 0) {
    			twiningCountTrun = -1;
    			return;
    		}
    	}
    	return;
    }

	/* ターン終了時締め付けられカウントダウンメソッド
	 *   タイミング：ターン終了時（優先順５（ダメージ発生後））
	 *   引数　　　：相手の締め付けカウント（int）
	 *   戻り値　　：締め付けダメージの発生
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public boolean twindCountDown(int twiningRestTrun) throws IOException{
    	if(twiningRestTrun == 0) {
    		twindRestTrun = 0;
    	}
    	if(twindRestTrun > 0) {
    		twindRestTrun--;
    		if(twindRestTrun == 0) {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"はこのターンのダメージを最後に締め付けから解放");
    		}
			return true;
    	}
    	return false;
    }

	/* 締め付け残りターンを返すメソッド
	 *   タイミング：カウントダウン時（相手のPokeStatePlaceオブジェクトのメンバ変数を参照する必要がある）
	 *   引数　　　：無し
	 *   戻り値　　：int(締め付け残りターン)
	 *   例外　　　：無し
	 */
    public int getTwiningRestTrun() {
    	return twiningRestTrun;
    }

	/* 締め付けられ残りターンを返すメソッド
	 *   タイミング：カウントダウン時（相手のPokeStatePlaceオブジェクトのメンバ変数を参照する必要がある）
	 *   引数　　　：無し
	 *   戻り値　　：int(締め付けられ残りターン)
	 *   例外　　　：無し
	 */
    public int getTwindRestTrun() {
    	return twindRestTrun;
    }


	/* 締め付け中かどうかを返すメソッド
	 *   タイミング：締め付け中かどうかの判定時
	 *   引数　　　：無し
	 *   戻り値　　：boolean(締め付け中ならtrue、締め付け中でないならfalse)
	 *   例外　　　：無し
	 */
    public boolean isTwining() {
    	return twiningRestTrun > 0;
    }

	/* 締め付けられ中かどうかを返すメソッド
	 *   タイミング：締め付けられ中かどうかの判定時
	 *   引数　　　：無し
	 *   戻り値　　：boolean(締め付けられ中ならtrue、締め付けられ中でないならfalse)
	 *   例外　　　：無し
	 */
    public boolean isTwind() {
    	return twindRestTrun > 0;
    }

	/* 丸くなった時用メソッド
	 *   タイミング：丸くなった時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void roundShape() throws IOException{
    	roundShapeFlag = true;
        DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は丸くなった");
    }

	/* 小さくなった時用メソッド
	 *   タイミング：小さくなった時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void smallShape() throws IOException{
    	smallShapeFlag = true;
        DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は小さくなった");
    }

	/* 地中制御用メソッド
	 *   タイミング：「あなをほる」１ターン目（地中に潜った）時・２ターン目（攻撃）時
	 *   引数　　　：boolean（１ターン目はtrue、２ターン目はfalse）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void goIntoUnderground(boolean intoUndergroundFlag) throws IOException{
    	this.intoUndergroundFlag = intoUndergroundFlag;
    	if (this.intoUndergroundFlag) {
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は地中に潜った");
    	} else {
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は地中から出てきた");
    	}
    }

	/* 空中制御用メソッド
	 *   タイミング：「そらをとぶ」１ターン目（空高く飛び上がった）時・２ターン目（攻撃）時
	 *   引数　　　：boolean（１ターン目はtrue、２ターン目はfalse）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void goIntoSky(boolean intoSkyFlag) throws IOException{
    	this.intoSkyFlag = intoSkyFlag;
    	if (this.intoSkyFlag) {
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は空中に飛びあがった");
    	} else {
    		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は空中から降りてきた");
    	}
    }

	/* 守る成功制御メソッド
	 *   タイミング：「まもる」「みきり」「こらえる」成功時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void protect() throws IOException{
    	protectSuccessFlag = true;
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は守っている");
    }

	/* 道連れ制御用メソッド
	 *   タイミング：「みちづれ」使用時（ケース１）、相手の技使用後（技使用後の判定が開始する前）（ケース２）
	 *   引数　　　：boolean（ケース１はtrue、ケース２はfalse）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void downTogetherWaiting(boolean downTogetherFlag) throws IOException{
    	this.downTogetherFlag = downTogetherFlag;
    	if (this.downTogetherFlag) {
        	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は道連れしようとしている");
    	}
    }

	/* 連続切り命中制御メソッド
	 *   タイミング：「れんぞくぎり」命中時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void slashContinuousHit(){
    	slashContinuousHitFlag = true;
    }

	/* 転がる命中制御メソッド
	 *   タイミング：「ころがる」命中時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void rollContinuousHit(){
    	rollContinuousHitFlag = true;
    }

	/* 破壊光線命中制御メソッド
	 *   タイミング：「はかいこうせん」命中時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void destructionBeemHit(){
    	destructionBeemStackRestTrun = 2;
    }

	/* 悪夢制御用メソッド
	 *   タイミング：悪夢になった時・眠りが治った時（技を打った後優先順３（ダメージ発生前））
	 *   引数　　　：boolean（悪夢になった時はtrue、眠りが治った時はfalse）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void beenIntoNightmare(boolean nightmareFlag) throws IOException{
    	// 奇跡の実で眠りから回復した場合は悪夢状態のまま
    	if (!(wakeupByMiracleBerryFlag && !nightmareFlag)) {
    		this.nightmareFlag = nightmareFlag;
    		if (this.nightmareFlag) {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は悪夢になった");
    		}
    	}
    }

	/* 気合のハチマキ発動メソッド
	 *   タイミング：「きあいのハチマキ」発動時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void spiritTrigger() {
    	spiritFlag = true;
    }

	/* みねうち発動メソッド
	 *   タイミング：「みねうち」発動時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void mineuchiTrigger() {
    	mineuchiFlag = true;
    }

	/* 奇跡の実による眠り管理メソッド
	 *   タイミング：眠りから回復した時
	 *   引数　　　：boolean（奇跡で回復した時はtrue、薄荷や自然治癒で回復した時はfalse）
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void wakeupByMiracleBerry(boolean wakeupByMiracleBerryFlag) {
    	this.wakeupByMiracleBerryFlag = wakeupByMiracleBerryFlag;
    }

	/* バトンタッチ成功メソッド
	 *   タイミング：バトンタッチに成功した時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void batonSuccess() {
    	this.batonSuccessFlag = true;
    }

	/* 吠える・吹き飛ばし被成功メソッド
	 *   タイミング：相手の吠える・吹き飛ばしが成功した時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void forceChanged() {
    	this.forceChangedFlag = true;
    }
}
