package state;

import java.io.IOException;
import java.util.Random;

import mold.PokeMold;
import util.DialogUtil;

/*
*  PokeStateUnitクラス
*
*  交代後も保持する必要がある各々のポケモンの状態を管理するクラスです。
*  ポケモン１匹１匹についてオブジェクトを生成してください。
*
*  変更履歴
*  2019/04/06 仮作成
*  2019/04/29 完成
*  2019/05/06 状態異常にするメソッドについて、状態異常上書きの制御の誤りを修正
*  2019/05/11 持ち物変更メソッドについてログを出力するように修正、眠り自然回復判定メソッドにメッセージ追加
*  2019/05/11 眠り自然回復判定メソッドのメソッド名変更、凍り自然回復判定メソッドは判定のみ行うように修正
*  2019/05/11 麻痺痺れ判定メソッドを追加
*  2019/05/26 技コードの初期値を変更
*  2019/07/15 初期化メソッドを追加
*  2019/07/28 コメントにスペル誤りがあったので修正
*  2019/08/02 HPが0になったら状態異常を回復するように修正（２匹同時眠り判定、２匹同時凍り判定のため）
*  2020/09/12 スケッチを実装
*  2020/09/12 悪あがきフラグ廃止（少なくともPBSでは使える技があるのに悪あがきを打ち続けるということはない）
*  2020/09/13 不思議な木の実でスケッチのPPを回復すると5回復してしまうバグを修正
*  2021/01/01 眠るターンカウント変数を追加（ルーチン作成用）
*  2021/01/23 対戦の中で明かされる情報に当たる変数がどれかをコメントに定義（AI作成時に考慮が必要）
*             （Wiki記載（https://seesaawiki.jp/pbs-thread/d/%a5%b8%a5%e0%be%eb%c2%e7%b2%f1%b0%ec%b7%e2%cc%b5%a4%b72000%a4%ce%a5%eb%a1%bc%a5%eb）の「例外」にあたるケースは省略）
*  2021/01/23 ルーチンでの参照禁止変数をprivate化
*
*/

public class PokeStateUnit {

	/* 表示用項目（対話式で使用） */
	public String playerNameDisp = "Player"; //プレイヤー名
	public String speciesNameDisp = "けつばん"; //種族名

	/* 型関連 */
	public String speciesName = "けつばん"; //種族名
	public String sex = "none"; //性別
	public String type1 = "normal"; //タイプ１
	public String type2 = "none"; //タイプ２
	public int level = 50; //レベル
	public int MAXHP = 1; //最大HP　※ポケモンを場に出した時点で相手に明らかになる情報
	public int attack = 1; //攻撃実数値　※相手に明らかにならない情報（推測は可能）
	public int block = 1; //防御実数値　※相手に明らかにならない情報（推測は可能）
	public int contact = 1; //特攻実数値　※相手に明らかにならない情報（推測は可能）
	public int diffence = 1; //特防実数値　※相手に明らかにならない情報（推測は可能）
	public int speed = 1; //素早さ実数値　※相手に明らかにならない情報（推測は可能）
	public int attackSpeciesValue = 1; //攻撃種族値（袋叩きで使用）　※相手に明らかにならない情報（推測は可能）
	public int blockSpeciesValue = 1; //防御種族値（袋叩きで使用）　※相手に明らかにならない情報（推測は可能）
	public String hiddenPowerType = "dark"; //めざめるパワーのタイプ　※相手に明らかにならない情報（推測は可能）
	public int hiddenPowerPower = 31; //めざめるパワーの威力　※相手に明らかにならない情報（推測は可能）
	public String[] moveName = {"技無し","技無し","技無し","技無し"}; //技名　※技を繰り出した時点で相手に明らかになる情報
	public String[] moveType = {"question","question","question","question"}; //技のタイプ　※技を繰り出した時点で相手に明らかになる情報
	public int[] movePower = {1,1,1,1}; //技の威力　※技を繰り出した時点で相手に明らかになる情報
	public double[] moveHitProbability = {100.0,100.0,100.0,100.0}; //技の命中率(百分率の概算値)　※技を繰り出した時点で相手に明らかになる情報
	public int[] movePP = {0,0,0,0}; //PP　※「わるあがき」や「ふしぎなきのみ」が発動した時点で相手に明らかになる情報
	public int[] movePPPrimitive = {0,0,0,0}; //ポイントアップを使用しない場合の最大PP　※技を繰り出した時点で相手に明らかになる情報
	public double[] additionalEffectProbability = {0.0,0.0,0.0,0.0}; //追加効果発動率(百分率の概算値)　※技を繰り出した時点で相手に明らかになる情報
	public String[] moveEffectCode = {"XXc","XXc","XXc","XXc"}; //技の効果コード　※技を繰り出した時点で相手に明らかになる情報
	public String itemName = "持ち物無し"; //持ち物　※選出時、及び持ち物の発動時に相手に明らかになる情報

	/* スケッチ関連 */
	public int sketchMoveIndex = -1; //スケッチした技が位置する技配列の添字を入れ変数「sketch～」使用
	public String sketchMoveName = "技無し"; //技名
	public String sketchMoveType = "question"; //技のタイプ
	public int sketchMovePower = 1; //技の威力
	public double sketchMoveHitProbability = 100.0; //技の命中率(百分率の概算値)
	public int sketchMovePP = 0; //PP
	public int sketchMovePPPrimitive = 0; //ポイントアップを使用しない場合の最大PP
	public double sketchAdditionalEffectProbability = 0.0; //追加効果発動率(百分率の概算値)
	public String sketchMoveEffectCode = "XXc"; //技の効果コード

	/* 現在HP管理 */
	public int actualHP = 1; //現在HP　※ポケモンを場に出した時点で相手に明らかになる情報

	/* 状態異常管理 */
	public String badHealth = "none"; //状態異常("none","burn","freeze","paralysis","poison","sleep")
	private int sleepRestTrun = 0; //残り眠りターン数（2～5をセットし、技を使用する直前に-1、1→0で状態異常を"none"に）…ルーチンでの参照禁止
	                                //起きた時の即行動を再現するため、真の値よりも1多い値をセットする。
	public int sleepCountTrun = -1; //眠るターンカウント（初期値-1、sleep発生で0を入れ、1判定毎に+1）…ルーチン作成用
	public boolean sleepByMyselfFlag = false; //自分で眠ったかフラグ（自分から眠ったらtrue）…ルーチン作成用

	/* 悪あがきモード */
	// public boolean struggleFlag = false; //一度でも悪あがきを選択するとtrue。一生悪あがきしかできなくなる。

	/* デフォルトコンストラクタ：何もしない
	 * 　　　　　　　　　　　　　（直接各メンバ変数に値を代入する場合はこちらのコンストラクタを使用）
	 */
    public PokeStateUnit() {
    }

	/* コンストラクタ：PokeMoldオブジェクトから初期状態をセット
	 * 　　　　　　　　（パーティテキストを入力としたい場合はこちらのコンストラクタを使用）
	 *   引数　：PokeMoldオブジェクト、プレイヤー名（String）
	 */
	public PokeStateUnit(PokeMold pokeMold,String playerNameDisp) {

		/* 表示用項目セット */
		this.playerNameDisp = playerNameDisp;

		/* 初期化 */
		initialize(pokeMold);
	}

	/* 初期化メソッド
	 *   タイミング：対戦を開始する時
	 *   引数　　　：PokeMoldオブジェクト
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
	public void initialize(PokeMold pokeMold) {
		/* 各型関連項目をセット */
		speciesName = pokeMold.speciesName;
		sex = pokeMold.sex;
		type1 = pokeMold.type1;
		type2 = pokeMold.type2;
		level = pokeMold.level;
		MAXHP = pokeMold.MAXHP;
		attack = pokeMold.attack;
		block = pokeMold.block;
		contact = pokeMold.contact;
		diffence = pokeMold.diffence;
		speed = pokeMold.speed;
		attackSpeciesValue = pokeMold.attackSpeciesValue;
		blockSpeciesValue = pokeMold.blockSpeciesValue;
		hiddenPowerType = pokeMold.hiddenPowerType;
		hiddenPowerPower = pokeMold.hiddenPowerPower;
		moveName = pokeMold.moveName;
		moveType = pokeMold.moveType;
		movePower = pokeMold.movePower;
		moveHitProbability = pokeMold.moveHitProbability;
		movePP[0] = pokeMold.movePPMAX1;
		movePP[1] = pokeMold.movePPMAX2;
		movePP[2] = pokeMold.movePPMAX3;
		movePP[3] = pokeMold.movePPMAX4;
		movePPPrimitive = pokeMold.movePPPrimitive;
		additionalEffectProbability = pokeMold.additionalEffectProbability;
		moveEffectCode = pokeMold.moveEffectCode;
		itemName = pokeMold.itemName;
		sketchMoveIndex = -1;
		sketchMoveName = "技無し";
		sketchMoveType = "question";
		sketchMovePower = 1;
		sketchMoveHitProbability = 100.0;
		sketchMovePP = 0;
		sketchMovePPPrimitive = 0;
		sketchAdditionalEffectProbability = 0.0;
		sketchMoveEffectCode = "XXc";
		badHealth = "none";
		sleepRestTrun = 0;
		sleepCountTrun = -1;
		sleepByMyselfFlag = false;
		// struggleFlag = false;

		/* 初期HPに最大HPをセット */
		actualHP = MAXHP;

		/* 表示用項目をセット */
		speciesNameDisp = speciesName;
	}

	/* 持ち物変更メソッド
	 *   タイミング：持ち物消費時、泥棒時
	 *   引数　　　：持ち物名（消費時や泥棒された時は「持ち物無し」、泥棒した時は相手の持ち物）（String）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void changeItem(String itemName) throws IOException {
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"の変更前持ち物："+this.itemName);
    	this.itemName = itemName;
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"の変更後持ち物："+this.itemName);
    }

	/* HP増減メソッド
	 *   タイミング：HPが増減する度
	 *   引数　　　：HP変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void HPUpDown(int HPVariation) throws IOException {
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化前HP："+String.format("%03d",actualHP));
    	actualHP = actualHP + HPVariation;
    	if (actualHP <= 0) {
    		actualHP = 0;
    		badHealth = "none";
    	} else if (actualHP > MAXHP) {
    		actualHP = MAXHP;
    	}
    	DialogUtil.dialogMessageOutput
    		(playerNameDisp+"の"+speciesNameDisp+"の変化後HP："+String.format("%03d",actualHP));
    }

	/* PP増減メソッド
	 *   タイミング：PPが増減する度
	 *   引数　　　：技配列番号（int）、PP変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 *   　　　　　：存在しない技が指定された場合は何もせずに終了
	 */
    public void PPUpDown(int moveIndex,int PPVariation) throws IOException {
    	if(moveIndex >= 0 && moveIndex <= 3) {
    		movePP[moveIndex] = movePP[moveIndex] + PPVariation;
    		if (movePP[moveIndex] < 0) {
    			movePP[moveIndex] = 0;
    		}
    		if (movePPPrimitive[moveIndex] == 1 && PPVariation == 5) { // スケッチを不思議な木の実で回復を考慮
    			movePP[moveIndex] = 1;
    			DialogUtil.dialogMessageOutput
        		(playerNameDisp+"の"+speciesNameDisp+"の"+moveName[moveIndex]+"が"+
    			String.format("%01d",1)+"増えた");
    		} else if (PPVariation != -1) {
    			DialogUtil.dialogMessageOutput
        		(playerNameDisp+"の"+speciesNameDisp+"の"+moveName[moveIndex]+"が"+
    			String.format("%01d",PPVariation)+"増えた");
    		}
    	}
    }

	/* 状態異常にするメソッド
	 *   タイミング：状態異常にかかるor治す度（技「ねむる」使用時は別メソッド使用）
	 *   引数　　　：状態異常名("none"or"poison"or"paralysis"or"sleep"or"burn"or"freeze")
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void changeBadHealth(String badHealthName) throws IOException {

    	/* 状態異常上書きはしない */
    	if (!badHealth.equals("none") && !badHealthName.equals("none")) {
    		return;
    	}

    	/* 状態異常にするor回復する */
    	badHealth = badHealthName;
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は"+badHealth+"になった");

    	/* 眠るのターン数セット */
    	if (badHealthName.equals("sleep")) {
    		sleepCountTrun = 0;
        	Random random = new Random();
        	sleepRestTrun = random.nextInt(4) + 2;
    	}
    }

	/* 技「ねむる」使用時の状態異常上書きメソッド
	 *   タイミング：技「ねむる」使用時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void goIntoSleepMyself() throws IOException {

    	/* 眠り状態にする */
    	badHealth = "sleep";
    	DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は"+badHealth+"になった");

    	/* 眠るのターン数セット(2ターン眠る) */
    	sleepCountTrun = 0;
    	sleepRestTrun = 3;
    	sleepByMyselfFlag = true;
    }

	/* 眠り自然回復判定メソッド（カウントダウン）
	 *   タイミング：技使用時（優先順１（眠りで動けないの前））
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void sleepRecoverCountDown() throws IOException {

    	/* 眠り状態の回復判定 */
    	if (badHealth.equals("sleep")) {
    		sleepCountTrun++;
    		sleepRestTrun--;
    		if (sleepRestTrun == 0) {
    			badHealth = "none";
    			sleepCountTrun = -1;
    			sleepByMyselfFlag = false;
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は目を覚ました");
    		} else {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"＜ぐうぐう");
    		}
    	}
    }

	/* このターンぐうぐうするかどうかを返すメソッド
	 *   タイミング：このターンぐうぐうするかの判定時
	 *   引数　　　：無し
	 *   戻り値　　：boolean(ぐうぐうするならtrue、しないならfalse)
	 *   例外　　　：無し
	 */
    public boolean isSleepThisTrun() {
    	return sleepRestTrun > 1;
    }

	/* このターンぐうぐうしないかどうかを返すメソッド
	 *   タイミング：このターンぐうぐうしないかどうかかの判定時
	 *   引数　　　：無し
	 *   戻り値　　：boolean(ぐうぐうしないならtrue、するならfalse)
	 *   例外　　　：無し
	 */
    public boolean isNotSleepThisTrun() {
    	return sleepRestTrun <= 1;
    }

	/* 凍り自然回復判定メソッド
	 *   タイミング：ターン終了時（優先順７）
	 *   引数　　　：無し
	 *   戻り値　　：boolean（trueで解ける、falseで解けない）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public boolean freezeRecoverJudgement() throws IOException {

    	/* 凍り状態の回復判定 */
    	if (badHealth.equals("freeze")) {
        	Random random = new Random();
        	int result = random.nextInt(10);
    		if (result == 0) {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"の氷は解けた");
    			return true;
    		} else {
    			DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"の氷は解けない");
    			return false;
    		}
    	} else {
    		return false;
    	}
    }

	/* 麻痺痺れ判定メソッド
	 *   タイミング：技の繰り出し判定時（優先順７）
	 *   引数　　　：無し
	 *   戻り値　　：boolean（trueで痺れ、falseで痺れない）
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public boolean confusionSelfDestructionJudge() throws IOException{
    	if (badHealth.equals("paralysis")) {
    		Random random = new Random();
        	int paralysisRandom = random.nextInt(4);
        	int paralysisBoarder = 1;
        	if (paralysisRandom < paralysisBoarder) {
        		DialogUtil.dialogMessageOutput(playerNameDisp+"の"+speciesNameDisp+"は痺れて動けない");
        		return true;
        	} else {
        		return false;
        	}
    	} else {
    		return false;
    	}
    }

	/* スケッチメソッド
	 *   タイミング：スケッチを行う度
	 *   引数　　　：スケッチした技が位置する技配列の添字（int）、技名（String）、技のタイプ（String）、技の威力（int）、
	 *   　　　　　　技の命中率（double）、PP（int）、追加効果発動率（double）、技の効果コード（String）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 *   　　　　　：技配列の指定が不正かどうかの判定は呼び出し側で行うこと
	 *   　　　　　：既に覚えている技かどうかの判定は呼び出し側で行うこと
	 */
    public void sketchMove(int sketchMoveIndex,String sketchMoveName,String sketchMoveType,
    		int sketchMovePower,double sketchMoveHitProbability,int sketchMovePP,
    		double sketchAdditionalEffectProbability,String sketchMoveEffectCode) throws IOException{
    	// ものまね、わるあがき、ねごと、オウムがえし、ゆびをふる、スケッチ、へんしん、じばく、だいばくはつはスケッチ不可
    	if (!sketchMoveEffectCode.equals("52")&&
    	    !sketchMoveEffectCode.equals("30b")&&
    	    !sketchMoveEffectCode.equals("61")&&
    	    !sketchMoveEffectCode.equals("09")&&
    	    !sketchMoveEffectCode.equals("53")&&
    	    !sketchMoveEffectCode.equals("5F")&&
    	    !sketchMoveEffectCode.equals("39")&&
    	    !sketchMoveEffectCode.equals("07")) {
        	this.sketchMoveIndex = sketchMoveIndex;
        	this.sketchMoveName = sketchMoveName;
        	this.sketchMoveType = sketchMoveType;
        	this.sketchMovePower = sketchMovePower;
        	this.sketchMoveHitProbability = sketchMoveHitProbability;
        	this.sketchMovePP = sketchMovePP;
        	sketchMovePPPrimitive = sketchMovePP;
        	this.sketchAdditionalEffectProbability = sketchAdditionalEffectProbability;
        	this.sketchMoveEffectCode = sketchMoveEffectCode;
        	DialogUtil.dialogMessageOutput
        		(playerNameDisp+"の"+speciesNameDisp+"は"+this.sketchMoveName+"をスケッチした");
    	} else {
        	DialogUtil.dialogMessageOutput
    			(playerNameDisp+"の"+speciesNameDisp+"はスケッチに失敗した");
    	}
    }

	/* スケッチした技のPP増減メソッド
	 *   タイミング：スケッチした技のPPが増減する度
	 *   引数　　　：PP変化量（int）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void sketchPPUpDown(int PPVariation) throws IOException {
    	sketchMovePP = sketchMovePP + PPVariation;
    	if (sketchMovePP < 0) {
    		sketchMovePP = 0;
    	}
    	if (PPVariation != -1) {
    		DialogUtil.dialogMessageOutput
        	(playerNameDisp+"の"+speciesNameDisp+"の"+sketchMoveName+"が"+
    		String.format("%01d",PPVariation)+"増えた");
    	}
    }

	/* 悪あがきモード移行メソッド
	 *   タイミング：悪あがき使用時
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    // public void goIntoStruggle() throws IOException {
    	// struggleFlag = true;
    // }
}
