package util;

import java.util.Scanner;

/*
*  CommandScannerクラス
*
*  標準入力でのコマンド入力を求めるクラスです。
*  対話式（都度コマンド入力するツール）でのみ使用します。
*
*  このクラスのメンバ変数を直接参照・編集したくなることはないと思うのでカプセル化しておきます。
*
*  変更履歴
*  2020/08/22 新規作成
*
*/

public class CommandScanner {

	/* スキャナー本体 */
	private static Scanner scanner = new Scanner(System.in);

	/* 先手後手の保持 */
	private static int firstHandPlayer = 0; // 0はデフォルト、1ならP1、2ならP2

	/* コマンド入力を求めるダイアログを表示する処理
	 *   引数　：String（入力を求めるメッセージ）
	 *   戻り値：String（入力されたコマンド）
	 *   例外　：無し
	 */
    public static String commandScan(String message) {
    	System.out.print(message);
    	return scanner.nextLine();
    }

	/* コマンド入力を求めるダイアログを表示する処理（P1用、形だけ用意）
	 *   引数　：String（入力を求めるメッセージ）
	 *   戻り値：String（入力されたコマンド）
	 *   例外　：無し
	 */
    public static String commandScanP1(String message) {
    	return commandScan(message);
    }

	/* コマンド入力を求めるダイアログを表示する処理（P2用、形だけ用意）
	 *   引数　：String（入力を求めるメッセージ）
	 *   戻り値：String（入力されたコマンド）
	 *   例外　：無し
	 */
    public static String commandScanP2(String message) {
    	return commandScan(message);
    }

	/* 先手のプレイヤーの登録
	 *   引数　：int（先手のプレイヤー）
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public static void firstHandRegist(int firstHandPlayerArg) {
    	firstHandPlayer = firstHandPlayerArg;
    }

	/* コマンド入力を求めるダイアログを表示する処理（先手用、形だけ用意）
	 *   引数　：String（入力を求めるメッセージ）
	 *   戻り値：String（入力されたコマンド）
	 *   例外　：無し
	 */
    public static String commandScanFirstHand(String message) {
    	if (firstHandPlayer==1) {
    		return commandScanP1(message);
    	} else if (firstHandPlayer==2) {
    		return commandScanP2(message);
    	} else {
    		return "";
    	}
    }

	/* コマンド入力を求めるダイアログを表示する処理（後手用、形だけ用意）
	 *   引数　：String（入力を求めるメッセージ）
	 *   戻り値：String（入力されたコマンド）
	 *   例外　：無し
	 */
    public static String commandScanSecondHand(String message) {
    	if (firstHandPlayer==1) {
    		return commandScanP2(message);
    	} else if (firstHandPlayer==2) {
    		return commandScanP1(message);
    	} else {
    		return "";
    	}
    }

	/* 終了処理
	 *   引数　：無し
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public static void scanEnd() {
    	scanner.close();
    }

}
