package calc;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Random;

import config.Precondition;
import party.PartyInfo;
import state.PlaceState;
import state.PokeStateBaton;
import state.PokeStatePlace;
import state.PokeStateWhole;
import util.DialogUtil;

/*
*  MoveCalcクラス
*
*  技関連の計算を行うクラスです。
*  便宜上、混乱自滅のダメージ計算も行います。
*  また、交代要否を返すメソッドも用意します。
*
*  変更履歴
*  2019/06/09 新規作成
*  2019/06/11 変身でランクがコピーされないバグ、変身が地中・空中の相手に当たるバグを修正
*  2019/06/16 標準メソッドの名前を間違えていたので修正
*  2019/06/29 恨みについて、相手が物真似や変身をしている場合にそれを考慮するように修正
*  2019/06/30 吹き飛ばし・吠えるの交代先選択のロジックに誤りがあったため修正
*  2019/07/08 交代できるポケモンがいない場合にバトンタッチが失敗するように修正
*  2019/07/10 覚えている技が指を振るで出てくるバグを修正
*  2019/07/15 配列の初期化方法を誤っていたので修正
*  2019/07/20 プレゼントについて、威力240や威力360は発生しないものと仮定した実装に変更
*  2019/07/22 回復技の回復量が相手のポケモンの最大HPに依存していたバグを修正
*  2019/07/25 のろいの効果が自分のタイプによって変わるべき所、相手のタイプによって変わっていたバグを修正
*  2019/07/25 吹き飛ばし・吠えるで交代先がランダムで決まっていなかったバグを修正
*  2019/07/27 「技無し」の技コードを「XXc」に統一
*  2019/07/27 いびきが眠る状態の時のみ失敗する挙動になっていたバグを修正
*  2019/07/27 毒々や追加効果毒が毒・鋼以外のポケモンにも無効になっていたバグを修正
*  2019/07/27 身代わりへの攻撃で反動発生のバグ、反動付きの技で相手のポケモンを倒した時の反動ダメージが多すぎるバグを修正
*  2019/07/28 反動ダメージが1多かったバグを修正
*  2019/07/28 HP満タンで眠るが成功するバグを修正
*  2019/07/28 道連れが成功しないバグを修正
*  2019/07/28 毒状態のポケモンに毒々を打つと猛毒になるバグを修正
*  2019/07/29 転がるの継続ターン数が５ターン固定になっていなかった＆外れても継続するバグを修正
*  2019/08/02 指を振る等をアンコールできてしまうバグを修正
*  2019/08/06 麻痺技で毒が入ってしまうバグを修正
*  2019/08/13 トリプルキックの命中判定が最初の１回にしか行われないバグを修正
*  2019/08/13 トリプルキック倍率を後手の通常の攻撃にも反映するように修正（PBSトリプルキックバグ再現）
*  2019/08/20 選んだ技のタイプに対応しているタイプ強化アイテムで混乱自滅ダメージが1.1倍になるGB実機仕様を仮実装
*             （PBSが同じ仕様かは未検証なのであくまで仮）
*  2019/08/31 晴れている時にも凍りになってしまうバグを修正
*  2019/09/23 身代わり中の守る・見切り・堪えるが成功してしまうバグを修正
*  2019/09/30 「どくばり」の威力が誤っていたバグを修正
*  2019/09/30 「どくどく」「ダブルニードル」以外の技で毒を入れる際、毒ポケモンにしか毒が入らないバグを修正
*  2019/11/06 「バトンタッチ」の成功判定で相手のポケモンの残り数を参照していたバグを修正
*  2019/11/26 「じこあんじ」がランク変動の無い相手に成功するバグを修正
*  2019/12/31 プレゼントのダメージ計算式を最新のダメージ計算式（v0.3版）に更新
*  2020/01/05 「じたばた」「きしかいせい」の威力20-40の境界が実機と異なるバグを実装
*  2020/04/11 「カウンター」「ミラーコート」「がまん」が空を飛ぶ中・穴を掘る中の相手に対して命中しないバグを修正
*  2020/05/03 残り１体で滅びの歌が成功するバグを修正
*  2020/07/18 「ころがる」のPP誤り、「ころがる」が外れた後のコンボ継続判定、コンボ中に眠らされた場合の処理を修正
*  2020/08/11 一撃技の命中率計算に「命中率」「回避率」のランク変動が適用されないPBSのバグを実装
*  2020/08/11 PBSの仕様に合わせ、メールも泥棒できるように修正
*  2020/08/15 バトンタッチ発動失敗時（バトン先が居ない時）にも追い打ちダメージが２倍になるバグを修正
*  2020/08/23 身代わり状態や呪い状態のポケモンに対する呪いが成功するバグを修正
*  2020/08/23 検証完了したためタイプ強化アイテムで混乱自滅ダメージが1.1倍になる対応を削除
*  2020/08/25 あくむ・きあいだめ・せいちょう・バトンタッチのタイプの誤りを修正
*             いたみわけ・こなゆき・たたきつける・テクスチャーのPPの誤りを修正
*             うずしお・ソニックブームの命中率の誤りを修正
*             スパークの威力の誤りを修正
*  2020/08/27 「カウンター」「ミラーコート」のダメージがランダム化されるバグを修正
*  2020/08/28 気合のハチマキやみねうちが発動したかどうかのフラグを保持されるように修正
*  2020/09/12 スケッチを実装
*  2020/09/12 ゴーストタイプに対して悪あがきの反動が発生しないPBSバグを実装
*  2020/09/12 覚えている技でもPPが切れている場合は指を振るで繰り出せるように修正
*  2020/09/12 悪あがきフラグ廃止（実機にはあるらしいが、少なくともPBSにはない）
*  2020/09/18 バッチ実行時のコンパイルエラーを解消
*  2020/10/18 物真似した技の位置が誤っているバグを修正、物真似等による技上書きが永続するバグを修正
*  2020/10/24 みねうちの処理と穴を掘るの処理が入れ替わっていたバグを修正
*             スケッチした技の位置が誤っているバグを修正
*             聖なる炎・火炎車で凍りが直らないバグを修正
*             固定ダメージ技でランダム化されていたバグを修正、サイコウェーブのダメージが誤っていたバグを修正
*  2020/10/25 テクスチャーでタイプが変化しないバグを修正
*             袋叩きで相手側の防御の参照箇所が誤っているバグを修正
*             性別不明がメロメロになるバグを修正
*  2020/11/01 捨て身反動で引き分けの有無の設定を追加
*  2020/11/03 バトンタッチ成功フラグ、吠える・吹き飛ばし被成功フラグを追加
*  2020/11/08 めざめるパワーに王者の印が適用されないバグを修正
*  2020/12/05 腹太鼓に必要な現在HPに1足りなくても腹太鼓が成功するPBS仕様を反映
*  2020/12/31 寝言に対する恨みが失敗するPBS仕様を反映
*  2021/01/23 ルーチンでの参照禁止変数をprivate化したことに伴う修正
*
*/

public class MoveCalc extends DamageCalc {

	/* 参照・反映用Stateオブジェクト */
	public PartyInfo partyInfoMyself = null; //PokeStateUnit配列
	public PartyInfo partyInfoOpponent = null; //PokeStateUnit配列
	public PokeStatePlace pokeStatePlaceMyself = null;
	public PokeStatePlace pokeStatePlaceOpponent = null;
	public PokeStateBaton pokeStateBatonMyself = null;
	public PokeStateBaton pokeStateBatonOpponent = null;
	public PokeStateWhole pokeStateWholeMyself = null;
	public PokeStateWhole pokeStateWholeOpponent = null;
	public PlaceState placeState = null;
	public boolean firstHandFlag = true; //先手かどうかのフラグ（吠える・吹き飛ばしで使用）

	/* 計算用一時領域（自分・上書きされる可能性のあるもの） */
	public String type1Myself = "normal"; //タイプ１
	public String type2Myself = "none"; //タイプ２
	public int attackMyself = 1; //攻撃実数値
	public int blockMyself = 1; //防御実数値
	public int contactMyself = 1; //特攻実数値
	public int diffenceMyself = 1; //特防実数値
	public String[] moveNameMyself = {"技無し","技無し","技無し","技無し"}; //技名
	public int[] movePPMyself = {0,0,0,0}; //技の現在のPP（スケッチ・物真似・変身考慮、指を振るの成否判定でのみ使用）
	public String[] moveTypeMyself = {"question","question","question","question"}; //技のタイプ
	public int[] movePowerMyself = {1,1,1,1}; //技の威力
	public double[] moveHitProbabilityMyself = {100.0,100.0,100.0,100.0}; //技の命中率(百分率の概算値)
	public int[] movePPPrimitiveMyself = {0,0,0,0}; //技の元々のPP（スケッチ・物真似・変身考慮）
	public double[] additionalEffectProbabilityMyself = {0.0,0.0,0.0,0.0}; //追加効果発動率(百分率の概算値)
	public String[] moveEffectCodeMyself = {"XXc","XXc","XXc","XXc"}; //技の効果コード
	public String moveNameChoseMyself = "技無し"; //技情報上書き後に選択された技名
	public String moveTypeChoseMyself = "question"; //技情報上書き後に選択された技のタイプ
	public int movePowerChoseMyself = 1; //技情報上書き後に選択された技の威力
	public double moveHitProbabilityChoseMyself = 100.0; //技情報上書き後に選択された技の命中率(百分率の概算値)
	public int movePPPrimitiveChoseMyself = 0; //技情報上書き後に選択された技の元々のPP
	public double additionalEffectProbabilityChoseMyself = 0.0; //技情報上書き後に選択された追加効果発動率(百分率概算)
	public String moveEffectCodeChoseMyself = "00a"; //技情報上書き後に選択された技の効果コード
	// public String moveTypeChoseMyselfBeforeSelfDestruction = "question"; //混乱自滅時に選択していた技のタイプ

	/* 初期化用領域（自分） */
	public String[] moveNameMyselfInitializer = {"技無し","技無し","技無し","技無し"}; //技名
	public String[] moveTypeMyselfInitializer = {"question","question","question","question"}; //技のタイプ
	public int[] movePowerMyselfInitializer = {1,1,1,1}; //技の威力
	public double[] moveHitProbabilityMyselfInitializer = {100.0,100.0,100.0,100.0}; //技の命中率(百分率の概算値)
	public int[] movePPPrimitiveMyselfInitializer = {0,0,0,0}; //技の元々のPP（スケッチ・物真似・変身考慮）
	public double[] additionalEffectProbabilityMyselfInitializer = {0.0,0.0,0.0,0.0}; //追加効果発動率(百分率の概算値)
	public String[] moveEffectCodeMyselfInitializer = {"XXc","XXc","XXc","XXc"}; //技の効果コード

	/* 計算用一時領域（相手・上書きされる可能性のあるもの） */
	/* 技関連の変数に関しては、物真似で別の技に置き換わったポケモンに変身した時に使用する */
	public String type1Opponent = "normal"; //タイプ１
	public String type2Opponent = "none"; //タイプ２
	public int attackOpponent = 1; //攻撃実数値
	public int blockOpponent = 1; //防御実数値
	public int contactOpponent = 1; //特攻実数値
	public int diffenceOpponent = 1; //特防実数値
	public String[] moveNameOpponent = {"技無し","技無し","技無し","技無し"}; //技名
	public String[] moveTypeOpponent = {"question","question","question","question"}; //技のタイプ
	public int[] movePowerOpponent = {1,1,1,1}; //技の威力
	public double[] moveHitProbabilityOpponent = {100.0,100.0,100.0,100.0}; //技の命中率(百分率の概算値)
	public int[] movePPPrimitiveOpponent = {0,0,0,0}; //技の元々のPP（物真似・変身考慮）
	public double[] additionalEffectProbabilityOpponent = {0.0,0.0,0.0,0.0}; //追加効果発動率(百分率の概算値)
	public String[] moveEffectCodeOpponent = {"XXc","XXc","XXc","XXc"}; //技の効果コード

	/* 反映用一時領域 */
	public int HPUpDownMyself = 0; //自分のHPの増減
	public int scapegoatHPUpDownMyself = 0; //自分の身代わりHPの増減
	public int HPUpDownOpponent = 0; //相手のHPの増減
	public int LastHPUpDownOpponent = 0; //相手のHPの増減（連続技の最後の攻撃技によるもの：カウンター・ミラコ用）
	public int scapegoatHPUpDownOpponent = 0; //相手の身代わりHPの増減
	public boolean moveHitFlag = false; //攻撃が相手に当たったかどうか
	public boolean changeWeatherFlag = false; //天候を変更するかどうか
	public boolean sleepMyselfFlag = false; //自ら眠りになるかどうか
	public boolean recoveryFreezeMyselfFlag = false; //自分の凍りを治すかどうか
	public boolean everyoneGoodHealthFlag = false; //癒しの鈴を使ったかどうか
	public String badHealthMyselfTmp = null; //自分を状態異常にする場合にその状態異常名を格納（聖なる炎・火炎車用）
	public String badHealthOpponentTmp = null; //相手を状態異常にする場合にその状態異常名を格納
	public boolean morePoisonFlag = false; //相手を猛毒にしたかどうか
	public boolean confusionFlag = false; //相手を混乱にしたかどうか
	public boolean stealFlag = false; //泥棒が決まったかどうか
	public int comboStartTurnSet = 0; //コンボ開始時のターンセット（転がる等が1発目かどうかの判定にも使用）
	public boolean comboCountDownFlag = false; //コンボターンをカウントダウンするかどうか
	public boolean comboCountResetFlag = false; //コンボターンをリセットするかどうか
	public boolean comboCountResetFlagWhenSleep = false; //催眠でコンボターンをリセットするかどうか
	public boolean goIntoSkyFlag = false; //空中に行ったかどうか
	public boolean goIntoUndergroundFlag = false; //地中に行ったかどうか
	public boolean returnOnGroundFlag = false; //地上に戻ったかどうか
	public boolean foulFlagMyself = false; //自爆・大爆発の反則フラグを立てるかどうか
	public boolean foulFlagOpponent = false; //道連れの反則フラグを立てるかどうか
	// public boolean struggleFlag = false; //悪あがきフラグを立てるかどうか
	public int attackRankUpDownMyself = 0; //攻撃ランク変動値
	public int blockRankUpDownMyself = 0; //防御ランク変動値
	public int contactRankUpDownMyself = 0; //特攻ランク変動値
	public int diffenceRankUpDownMyself = 0; //特防ランク変動値
	public int speedRankUpDownMyself = 0; //素早さランク変動値
	public int hitProbabilityRankUpDownMyself = 0; //命中率ランク変動値
	public int avoidProbabilityRankUpDownMyself = 0; //回避率ランク変動値
	public int criticalHitProbabilityRankUpDownMyself = 0; //急所率ランク変動値
	public int attackRankUpDownOpponent = 0; //攻撃ランク変動値
	public int blockRankUpDownOpponent = 0; //防御ランク変動値
	public int contactRankUpDownOpponent = 0; //特攻ランク変動値
	public int diffenceRankUpDownOpponent = 0; //特防ランク変動値
	public int speedRankUpDownOpponent = 0; //素早さランク変動値
	public int hitProbabilityRankUpDownOpponent = 0; //命中率ランク変動値
	public int avoidProbabilityRankUpDownOpponent = 0; //回避率ランク変動値
	public int criticalHitProbabilityRankUpDownOpponent = 0; //急所率ランク変動値
	public boolean blackFogFlag = false; //黒い霧を使ったかどうか
	public boolean SelfSuggestionFlag = false; //自己暗示を使ったかどうか
	public boolean typePasteFlag = false; //テクスチャー・テクスチャー２を使ったかどうか
	public String type1MyselfPaste = "normal"; //タイプ１
	public String type2MyselfPaste = "none"; //タイプ２
	public boolean sketchFlag = false; //物真似を使ったかどうか
	public boolean mimicFlag = false; //物真似を使ったかどうか
	public boolean transformFlag = false; //変身を使ったかどうか
	public int PPDownOpponent = 0; //相手のPPの減少量（恨み用）
	public boolean bideReleaseFlag = false; //我慢が解き放たれたかどうか
	public int pokeStateUnitArrayOpponentIndexChanged = -1; //相手の変更後の吠える・吹き飛ばし成功時にセット
	public boolean reflectorFlag = false; //リフレクターしたかどうか
	public boolean lightWallFlag = false; //光の壁をしたかどうか
	public boolean mysteryProtectFlag = false; //神秘の守りをしたかどうか
	public boolean twiningFlag = false; //締め付けるをしたかどうか
	public boolean whiteFogFlag = false; //白い霧をしたかどうか
	public boolean winceFlag = false; //相手を怯みにしたかどうか
	public boolean destructionBeemStackSetFlag = false; //破壊光線反動が発生したかどうか
	public boolean parasiteFlag = false; //相手を宿り木にしたかどうか
	public boolean moveParalysisFlag = false; //相手を金縛りにしたかどうか
	public boolean encoreFlag = false; //相手をアンコールにしたかどうか
	public boolean lockOnFlag = false; //相手をロックオンしたかどうか
	public boolean downTogetherWaitFlag = false; //道連れを選択したかどうか
	public boolean trappingFlag = false; //黒い眼差しにしたかどうか
	public boolean nightmareFlag = false; //悪夢にしたかどうか
	public boolean curceFlag = false; //呪いにしたかどうか
	public boolean protectSuccessFlag = false; //守る・見切り・堪えるが成功したかどうか
	public boolean spikesFlag = false; //撒きびしを撒いたかどうか
	public boolean seeingThroughFlag = false; //見破ったかどうか
	public boolean startRuinFlag = false; //滅びの歌をしたかどうか
	public boolean slashContinuousHitFlag = false; //連続切りが命中したかどうか
	public boolean rollContinuousHitFlag = false; //転がるが命中したかどうか
	public boolean deprivingMindFlag = false; //メロメロにしたかどうか
	public boolean spinFlag = false; //高速スピンしたかどうか
	public int predictionDamage = 0; //未来予知の予約ダメージ
	public boolean smallShapeFlag = false; //小さくなるしたかどうか
	public boolean roundShapeFlag = false; //丸くなるしたかどうか
	public boolean moveCopyFlag = false; //指を振る・オウム返しをしたかどうか
	public boolean batonFailFlag = false; //バトンタッチに失敗したかどうか
	public boolean spiritFlag = false; //気合のハチマキが発動したかどうか
	public boolean mineuchiFlag = false; //みねうちが発動したかどうか
	public boolean changeMyselfFlag = false; //自分が交代するかどうか（バトンタッチ時、交代時にtrue）

	/* 出力領域 */
	public int changeNecessityNumber = 0; //0なら交代不要、1なら自分のみ交代、2なら相手のみ交代、3なら両方交代

	/* 技繰り出し標準メソッド
	 *   引数　　　：自分のPokeStateUnit配列、相手のPokeStateUnit配列、
	 *   　　　　　：自分のPokeStatePlaceオブジェクト、相手のPokeStatePlaceオブジェクト、
	 *   　　　　　：自分のPokeStateBatonオブジェクト、相手のPokeStateBatonオブジェクト、
	 *   　　　　　：自分のPokeStateWholeオブジェクト、相手のPokeStateWholeオブジェクト、
	 *   　　　　　：PlaceStateオブジェクト、先手かどうかのフラグ（boolean）
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void moveCalcCommon(PartyInfo partyInfoMyself,PartyInfo partyInfoOpponent,
                                 PokeStatePlace pokeStatePlaceMyself,PokeStatePlace pokeStatePlaceOpponent,
                                 PokeStateBaton pokeStateBatonMyself,PokeStateBaton pokeStateBatonOpponent,
                                 PokeStateWhole pokeStateWholeMyself,PokeStateWhole pokeStateWholeOpponent,
                                 PlaceState placeState,boolean firstHandFlag) throws IOException {

		// 初期値セット（各種Stateクラス（PokeStateUnitクラスは配列）、先手フラグ）
    	this.partyInfoMyself = partyInfoMyself;
    	this.partyInfoOpponent = partyInfoOpponent;
    	this.pokeStatePlaceMyself = pokeStatePlaceMyself;
    	this.pokeStatePlaceOpponent = pokeStatePlaceOpponent;
    	this.pokeStateBatonMyself = pokeStateBatonMyself;
    	this.pokeStateBatonOpponent = pokeStateBatonOpponent;
    	this.pokeStateWholeMyself = pokeStateWholeMyself;
    	this.pokeStateWholeOpponent = pokeStateWholeOpponent;
    	this.placeState = placeState;
    	this.firstHandFlag = firstHandFlag;

		// 計算用一時領域・反映用一時領域・出力用領域初期化メソッド呼び出し
    	calcTmpOutputAreaInit();

		// 変身・テクスチャー・物真似によるステータス上書きメソッド呼び出し
    	calcAreaStatusSet();

		// 技情報上書きメソッド呼び出し
    	calcAreaMoveSet();

		// 各種技の処理メソッド呼び出し
    	moveExecution();

		// 相手の道連れにより道連れになるか否かの判定メソッド呼び出し
    	downTogetherJudge();

		// 技使用結果の反映メソッド呼び出し
    	calcResultApply();

		// 出力（交代要否）の判定メソッド呼び出し
    	changeNecessitySet();
    }

	/* 計算用一時領域・反映用一時領域・出力用領域初期化メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void calcTmpOutputAreaInit() {
    	type1Myself = "normal";
    	type2Myself = "none";
    	attackMyself = 1;
    	blockMyself = 1;
    	contactMyself = 1;
    	diffenceMyself = 1;
    	moveNameMyself = null;
    	moveNameMyself = new String[4];
    	movePPMyself = null;
    	movePPMyself = new int[4];
    	moveTypeMyself = null;
    	moveTypeMyself = new String[4];
    	movePowerMyself = null;
    	movePowerMyself = new int[4];
    	moveHitProbabilityMyself = null;
    	moveHitProbabilityMyself = new double[4];
    	movePPPrimitiveMyself = null;
    	movePPPrimitiveMyself = new int[4];
    	additionalEffectProbabilityMyself = null;
    	additionalEffectProbabilityMyself = new double[4];
    	moveEffectCodeMyself = null;
    	moveEffectCodeMyself = new String[4];
    	for(int i = 0; i < 4; i++){
    		moveNameMyself[i] = "技無し";
    		movePPMyself[i] = 0;
    		moveTypeMyself[i] = "question";
    		movePowerMyself[i] = 1;
    		moveHitProbabilityMyself[i] = 100.0;
    		movePPPrimitiveMyself[i] = 0;
    		additionalEffectProbabilityMyself[i] = 0.0;
    		moveEffectCodeMyself[i] = "XXc";
    	}
    	moveNameChoseMyself = "技無し";
    	moveTypeChoseMyself = "question";
    	movePowerChoseMyself = 1;
    	moveHitProbabilityChoseMyself = 100.0;
		movePPPrimitiveChoseMyself = 0;
    	additionalEffectProbabilityChoseMyself = 0.0;
    	moveEffectCodeChoseMyself = "XXc";
    	// moveTypeChoseMyselfBeforeSelfDestruction = "question";
    	type1Opponent = "normal";
    	type2Opponent = "none";
    	attackOpponent = 1;
    	blockOpponent = 1;
    	contactOpponent = 1;
    	diffenceOpponent = 1;
    	moveNameOpponent = null;
    	moveNameOpponent = new String[4];
    	moveTypeOpponent = null;
    	moveTypeOpponent = new String[4];
    	movePowerOpponent = null;
    	movePowerOpponent = new int[4];
    	moveHitProbabilityOpponent = null;
    	moveHitProbabilityOpponent = new double[4];
    	movePPPrimitiveOpponent = null;
    	movePPPrimitiveOpponent = new int[4];
    	additionalEffectProbabilityOpponent = null;
    	additionalEffectProbabilityOpponent = new double[4];
    	moveEffectCodeOpponent = null;
    	moveEffectCodeOpponent = new String[4];
    	for(int i = 0; i < 4; i++){
    		moveNameOpponent[i] = "技無し";
    		moveTypeOpponent[i] = "question";
    		movePowerOpponent[i] = 1;
    		moveHitProbabilityOpponent[i] = 100.0;
    		movePPPrimitiveOpponent[i] = 0;
    		additionalEffectProbabilityOpponent[i] = 0.0;
    		moveEffectCodeOpponent[i] = "XXc";
    	}
    	HPUpDownMyself = 0;
    	scapegoatHPUpDownMyself = 0;
    	HPUpDownOpponent = 0;
    	LastHPUpDownOpponent = 0;
    	scapegoatHPUpDownOpponent = 0;
    	moveHitFlag = false;
    	changeWeatherFlag = false;
    	sleepMyselfFlag = false;
    	recoveryFreezeMyselfFlag = false;
    	everyoneGoodHealthFlag = false;
    	badHealthMyselfTmp = null;
    	badHealthOpponentTmp = null;
    	morePoisonFlag = false;
    	confusionFlag = false;
    	stealFlag = false;
    	comboStartTurnSet = 0;
    	comboCountDownFlag = false;
    	comboCountResetFlag = false;
    	comboCountResetFlagWhenSleep = false;
    	goIntoSkyFlag = false;
    	goIntoUndergroundFlag = false;
    	returnOnGroundFlag = false;
    	foulFlagMyself = false;
    	foulFlagOpponent = false;
    	// struggleFlag = false;
    	attackRankUpDownMyself = 0;
    	blockRankUpDownMyself = 0;
    	contactRankUpDownMyself = 0;
    	diffenceRankUpDownMyself = 0;
    	speedRankUpDownMyself = 0;
    	hitProbabilityRankUpDownMyself = 0;
    	avoidProbabilityRankUpDownMyself = 0;
    	criticalHitProbabilityRankUpDownMyself = 0;
    	attackRankUpDownOpponent = 0;
    	blockRankUpDownOpponent = 0;
    	contactRankUpDownOpponent = 0;
    	diffenceRankUpDownOpponent = 0;
    	speedRankUpDownOpponent = 0;
    	hitProbabilityRankUpDownOpponent = 0;
    	avoidProbabilityRankUpDownOpponent = 0;
    	criticalHitProbabilityRankUpDownOpponent = 0;
    	blackFogFlag = false;
    	SelfSuggestionFlag = false;
    	typePasteFlag = false;
    	type1MyselfPaste = "normal";
    	type2MyselfPaste = "none";
    	sketchFlag = false;
    	mimicFlag = false;
    	transformFlag = false;
    	PPDownOpponent = 0;
    	bideReleaseFlag = false;
    	pokeStateUnitArrayOpponentIndexChanged = -1;
    	reflectorFlag = false;
    	lightWallFlag = false;
    	mysteryProtectFlag = false;
    	twiningFlag = false;
    	whiteFogFlag = false;
    	winceFlag = false;
    	destructionBeemStackSetFlag = false;
    	parasiteFlag = false;
    	moveParalysisFlag = false;
    	encoreFlag = false;
    	lockOnFlag = false;
    	downTogetherWaitFlag = false;
    	trappingFlag = false;
    	nightmareFlag = false;
    	curceFlag = false;
    	protectSuccessFlag = false;
    	spikesFlag = false;
    	seeingThroughFlag = false;
    	startRuinFlag = false;
    	slashContinuousHitFlag = false;
    	deprivingMindFlag = false;
    	spinFlag = false;
    	predictionDamage = 0;
    	smallShapeFlag = false;
    	roundShapeFlag = false;
    	moveCopyFlag = false;
    	batonFailFlag = false;
    	spiritFlag = false;
    	mineuchiFlag = false;
    	changeMyselfFlag = false;
    	changeNecessityNumber = 0;
    }

	/* 変身・テクスチャー・物真似によるステータス上書きメソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void calcAreaStatusSet() {

    	// スケッチ・物真似・変身を考慮しない場合のステータスをセット
    	type1Myself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].type1;
    	type2Myself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].type2;
    	attackMyself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].attack;
    	blockMyself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].block;
    	contactMyself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].contact;
    	diffenceMyself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].diffence;
    	for(int i = 0; i < 4; i++){
    		moveNameMyself[i] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].moveName[i];
    		movePPMyself[i] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].movePP[i];
    		moveTypeMyself[i] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].moveType[i];
    		movePowerMyself[i] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].movePower[i];
    		moveHitProbabilityMyself[i] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].moveHitProbability[i];
    		movePPPrimitiveMyself[i] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].movePPPrimitive[i];
    		additionalEffectProbabilityMyself[i] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].additionalEffectProbability[i];
    		moveEffectCodeMyself[i] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].moveEffectCode[i];
    	}
    	type1Opponent = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].type1;
    	type2Opponent = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].type2;
    	attackOpponent = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].attack;
    	blockOpponent = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].block;
    	contactOpponent = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].contact;
    	diffenceOpponent = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].diffence;
    	for(int i = 0; i < 4; i++){
    		moveNameOpponent[i] = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].moveName[i];
    		moveTypeOpponent[i] = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].moveType[i];
    		movePowerOpponent[i] = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].movePower[i];
    		moveHitProbabilityOpponent[i] = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].moveHitProbability[i];
    		movePPPrimitiveOpponent[i] = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].movePPPrimitive[i];
    		additionalEffectProbabilityOpponent[i] = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].additionalEffectProbability[i];
    		moveEffectCodeOpponent[i] = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].moveEffectCode[i];
    	}

    	// 変身を考慮した場合のステータスをセット
    	if (pokeStatePlaceMyself.transformedFlag) {
    		type1Myself = pokeStatePlaceMyself.transformedType1;
    		type2Myself = pokeStatePlaceMyself.transformedType2;
    		attackMyself = pokeStatePlaceMyself.transformedAttack;
    		blockMyself = pokeStatePlaceMyself.transformedBlock;
    		contactMyself = pokeStatePlaceMyself.transformedContact;
    		diffenceMyself = pokeStatePlaceMyself.transformedDiffence;
        	for(int i = 0; i < 4; i++){
        		moveNameMyself[i] = pokeStatePlaceMyself.transformedMoveName[i];
        		movePPMyself[i] = pokeStatePlaceMyself.transformedMovePP[i];
        		moveTypeMyself[i] = pokeStatePlaceMyself.transformedMoveType[i];
        		movePowerMyself[i] = pokeStatePlaceMyself.transformedMovePower[i];
        		moveHitProbabilityMyself[i] = pokeStatePlaceMyself.transformedMoveHitProbability[i];
        		movePPPrimitiveMyself[i] = pokeStatePlaceMyself.transformedMovePPPrimitive[i];
        		additionalEffectProbabilityMyself[i] = pokeStatePlaceMyself.transformedAdditionalEffectProbability[i];
        		moveEffectCodeMyself[i] = pokeStatePlaceMyself.transformedMoveEffectCode[i];
        	}
    	}
    	if (pokeStatePlaceOpponent.transformedFlag) {
    		type1Opponent = pokeStatePlaceOpponent.transformedType1;
    		type2Opponent = pokeStatePlaceOpponent.transformedType2;
    		attackOpponent = pokeStatePlaceOpponent.transformedAttack;
    		blockOpponent = pokeStatePlaceOpponent.transformedBlock;
    		contactOpponent = pokeStatePlaceOpponent.transformedContact;
    		diffenceOpponent = pokeStatePlaceOpponent.transformedDiffence;
        	for(int i = 0; i < 4; i++){
        		moveNameOpponent[i] = pokeStatePlaceOpponent.transformedMoveName[i];
        		moveTypeOpponent[i] = pokeStatePlaceOpponent.transformedMoveType[i];
        		movePowerOpponent[i] = pokeStatePlaceOpponent.transformedMovePower[i];
        		moveHitProbabilityOpponent[i] = pokeStatePlaceOpponent.transformedMoveHitProbability[i];
        		movePPPrimitiveOpponent[i] = pokeStatePlaceOpponent.transformedMovePPPrimitive[i];
        		additionalEffectProbabilityOpponent[i] = pokeStatePlaceOpponent.transformedAdditionalEffectProbability[i];
        		moveEffectCodeOpponent[i] = pokeStatePlaceOpponent.transformedMoveEffectCode[i];
        	}
    	}

    	// テクスチャーを考慮した場合のステータスをセット
    	if (pokeStatePlaceMyself.pastedFlag) {
    		type1Myself = pokeStatePlaceMyself.pastedType1;
    		type2Myself = pokeStatePlaceMyself.pastedType2;
    	}
    	if (pokeStatePlaceOpponent.pastedFlag) {
    		type1Opponent = pokeStatePlaceOpponent.pastedType1;
    		type2Opponent = pokeStatePlaceOpponent.pastedType2;
    	}

    	// スケッチを考慮した場合のステータスをセット
    	if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex >= 0 && partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex <= 3) {
    		moveNameMyself[partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveName;
    		movePPMyself[partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex] =
            	partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMovePP;
    		moveTypeMyself[partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveType;
    		movePowerMyself[partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMovePower;
    		moveHitProbabilityMyself[partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex] =
    			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveHitProbability;
    		movePPPrimitiveMyself[partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex] =
        		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMovePPPrimitive;
    		additionalEffectProbabilityMyself[partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex] =
    			partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchAdditionalEffectProbability;
    		moveEffectCodeMyself[partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMoveEffectCode;
    	}
    	if (partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveIndex >= 0 && partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveIndex <= 3) {
    		moveNameOpponent[partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveName;
    		moveTypeOpponent[partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveType;
    		movePowerOpponent[partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMovePower;
    		moveHitProbabilityOpponent[partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveIndex] =
    			partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveHitProbability;
    		movePPPrimitiveOpponent[partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveIndex] =
            	partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMovePPPrimitive;
    		additionalEffectProbabilityOpponent[partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveIndex] =
    			partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchAdditionalEffectProbability;
    		moveEffectCodeOpponent[partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveIndex] = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveEffectCode;
    	}

    	// 物真似を考慮した場合のステータスをセット
    	if (pokeStatePlaceMyself.mimicMoveIndex >= 0 && pokeStatePlaceMyself.mimicMoveIndex <= 3) {
    		moveNameMyself[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMoveName;
    		movePPMyself[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMovePP;
    		moveTypeMyself[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMoveType;
    		movePowerMyself[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMovePower;
    		moveHitProbabilityMyself[pokeStatePlaceMyself.mimicMoveIndex] =
    			pokeStatePlaceMyself.mimicMoveHitProbability;
    		movePPPrimitiveMyself[pokeStatePlaceMyself.mimicMoveIndex] =
        		pokeStatePlaceMyself.mimicMovePPPrimitive;
    		additionalEffectProbabilityMyself[pokeStatePlaceMyself.mimicMoveIndex] =
    			pokeStatePlaceMyself.mimicAdditionalEffectProbability;
    		moveEffectCodeMyself[pokeStatePlaceMyself.mimicMoveIndex] = pokeStatePlaceMyself.mimicMoveEffectCode;
    	}
    	if (pokeStatePlaceOpponent.mimicMoveIndex >= 0 && pokeStatePlaceOpponent.mimicMoveIndex <= 3) {
    		moveNameOpponent[pokeStatePlaceOpponent.mimicMoveIndex] = pokeStatePlaceOpponent.mimicMoveName;
    		moveTypeOpponent[pokeStatePlaceOpponent.mimicMoveIndex] = pokeStatePlaceOpponent.mimicMoveType;
    		movePowerOpponent[pokeStatePlaceOpponent.mimicMoveIndex] = pokeStatePlaceOpponent.mimicMovePower;
    		moveHitProbabilityOpponent[pokeStatePlaceOpponent.mimicMoveIndex] =
    			pokeStatePlaceOpponent.mimicMoveHitProbability;
    		movePPPrimitiveOpponent[pokeStatePlaceOpponent.mimicMoveIndex] =
            	pokeStatePlaceOpponent.mimicMovePPPrimitive;
    		additionalEffectProbabilityOpponent[pokeStatePlaceOpponent.mimicMoveIndex] =
    			pokeStatePlaceOpponent.mimicAdditionalEffectProbability;
    		moveEffectCodeOpponent[pokeStatePlaceOpponent.mimicMoveIndex] = pokeStatePlaceOpponent.mimicMoveEffectCode;
    	}

    	// 指を振る・オウム返しを考慮した場合のステータスをセット
    	if (pokeStatePlaceMyself.moveCopyMoveFlag) {
    		moveNameMyself[pokeStatePlaceMyself.moveIndex] = pokeStatePlaceMyself.latestUsedMoveNameMoveCopy;
    		moveTypeMyself[pokeStatePlaceMyself.moveIndex] = pokeStatePlaceMyself.latestUsedMoveTypeMoveCopy;
    		movePowerMyself[pokeStatePlaceMyself.moveIndex] = pokeStatePlaceMyself.latestUsedMovePowerMoveCopy;
    		moveHitProbabilityMyself[pokeStatePlaceMyself.moveIndex] =
    			pokeStatePlaceMyself.latestUsedMoveHitProbabilityMoveCopy;
    		movePPPrimitiveMyself[pokeStatePlaceMyself.moveIndex] =
    			pokeStatePlaceMyself.latestUsedMovePPPrimitiveMoveCopy;
    		additionalEffectProbabilityMyself[pokeStatePlaceMyself.moveIndex] =
    			pokeStatePlaceMyself.latestUsedAdditionalEffectProbabilityMoveCopy;
    		moveEffectCodeMyself[pokeStatePlaceMyself.moveIndex] =
    			pokeStatePlaceMyself.latestUsedMoveEffectCodeMoveCopy;
    	}

    }

	/* 交代・混乱自滅・悪あがき・オウム返し・指を振る・トライアタック（３分割）による技情報上書きメソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void calcAreaMoveSet() throws IOException {

    	// 上書き前の技をセット
    	if (pokeStatePlaceMyself.moveIndex >= 0 && pokeStatePlaceMyself.moveIndex <= 3) {
        	moveNameChoseMyself = moveNameMyself[pokeStatePlaceMyself.moveIndex];
        	moveTypeChoseMyself = moveTypeMyself[pokeStatePlaceMyself.moveIndex];
        	movePowerChoseMyself = movePowerMyself[pokeStatePlaceMyself.moveIndex];
        	moveHitProbabilityChoseMyself = moveHitProbabilityMyself[pokeStatePlaceMyself.moveIndex];
        	movePPPrimitiveChoseMyself = movePPPrimitiveMyself[pokeStatePlaceMyself.moveIndex];
        	additionalEffectProbabilityChoseMyself = additionalEffectProbabilityMyself[pokeStatePlaceMyself.moveIndex];
        	moveEffectCodeChoseMyself = moveEffectCodeMyself[pokeStatePlaceMyself.moveIndex];
    	}

    	// 交代
    	if (pokeStatePlaceMyself.changeTarget != -1 && pokeStatePlaceMyself.moveIndex == -1) {
    		moveNameChoseMyself = "交代";
    		moveTypeChoseMyself = "question";
    		movePowerChoseMyself = 0;
    		moveHitProbabilityChoseMyself = 100.0;
    		movePPPrimitiveChoseMyself = 0;
    		additionalEffectProbabilityChoseMyself = 0.0;
    		moveEffectCodeChoseMyself = "XXa";
    	}

    	// 混乱自滅
    	if (pokeStateBatonMyself.confusionSelfDestructionFlag) {
    		// moveTypeChoseMyselfBeforeSelfDestruction = moveTypeChoseMyself;
    		moveNameChoseMyself = "混乱自滅";
    		moveTypeChoseMyself = "physic";
    		movePowerChoseMyself = 40;
    		moveHitProbabilityChoseMyself = 100.0;
    		movePPPrimitiveChoseMyself = 0;
    		additionalEffectProbabilityChoseMyself = 0.0;
    		moveEffectCodeChoseMyself = "XXb";
    	}

    	// 悪あがき
    	if ((pokeStatePlaceMyself.moveIndex == -1 && pokeStatePlaceMyself.changeTarget == -1) /*||
    	     partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].struggleFlag*/) {
    		moveNameChoseMyself = "わるあがき";
    		moveTypeChoseMyself = "physic";
    		movePowerChoseMyself = 50;
    		moveHitProbabilityChoseMyself = 100.0;
    		movePPPrimitiveChoseMyself = 10;
    		additionalEffectProbabilityChoseMyself = 0.0;
    		moveEffectCodeChoseMyself = "30b";
    	}

    	// 指を振る・オウム返しによるコンボ技
    	if (pokeStatePlaceMyself.moveCopyMoveFlag) {
    		moveNameChoseMyself = pokeStatePlaceMyself.latestUsedMoveNameMoveCopy;
    		moveTypeChoseMyself = pokeStatePlaceMyself.latestUsedMoveTypeMoveCopy;
    		movePowerChoseMyself = pokeStatePlaceMyself.latestUsedMovePowerMoveCopy;
    		moveHitProbabilityChoseMyself = pokeStatePlaceMyself.latestUsedMoveHitProbabilityMoveCopy;
    		movePPPrimitiveChoseMyself = pokeStatePlaceMyself.latestUsedMovePPPrimitiveMoveCopy;
    		additionalEffectProbabilityChoseMyself = pokeStatePlaceMyself.latestUsedAdditionalEffectProbabilityMoveCopy;
    		moveEffectCodeChoseMyself = pokeStatePlaceMyself.latestUsedMoveEffectCodeMoveCopy;
    	}

		// 61(寝言)
    	// 61(寝言)、1A(我慢)、27(かまいたち)、4B(ゴッドバード)、91(ロケットずつき)、97(ソーラービーム)、
    	// 9Ba(空を飛ぶ)、9Bb(穴を掘る)、XXa(交代)、XXb(混乱自滅)、XXc(技無し)は選ばれない
    	// 1B(暴れる・花びらの舞・逆鱗)、75(転がる)は通常の単発攻撃扱いとなるので一時的に技コード00aを設定
    	// 技を置き換える前にメッセージを出力
    	if (moveEffectCodeChoseMyself.equals("61") &&
    	    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("sleep")) {
			ArrayList<Integer> moveArray = new ArrayList<Integer>();
		    for (int i = 0 ; i < 4 ; i++){
		    	if (!moveEffectCodeMyself[i].equals("61") &&
			        !moveEffectCodeMyself[i].equals("1A") &&
			        !moveEffectCodeMyself[i].equals("27") &&
			        !moveEffectCodeMyself[i].equals("4B") &&
			        !moveEffectCodeMyself[i].equals("91") &&
			        !moveEffectCodeMyself[i].equals("97") &&
			        !moveEffectCodeMyself[i].equals("9Ba") &&
			        !moveEffectCodeMyself[i].equals("0Bb") &&
			        !moveEffectCodeMyself[i].equals("XXa") &&
			        !moveEffectCodeMyself[i].equals("XXb") &&
			        !moveEffectCodeMyself[i].equals("XXc")) {
		    		moveArray.add(new Integer(i));
			    	}
		    }
		    if (moveArray.size() > 0) {
	    		DialogUtil.dialogMessageOutput(
	    				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].playerNameDisp+
	    				"の"+
	    				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesNameDisp+
	    				"の"+
	    				moveNameChoseMyself);
    	    	Random random = new Random();
    	    	int moveRandom = random.nextInt(moveArray.size());
    		    for (int i = 0 ; i < moveArray.size() ; i++){
    				if (i==moveRandom) {
    		        	moveNameChoseMyself = moveNameMyself[moveArray.get(i).intValue()];
    		        	moveTypeChoseMyself = moveTypeMyself[moveArray.get(i).intValue()];
    		        	movePowerChoseMyself = movePowerMyself[moveArray.get(i).intValue()];
    		        	moveHitProbabilityChoseMyself = moveHitProbabilityMyself[moveArray.get(i).intValue()];
    		        	movePPPrimitiveChoseMyself = movePPPrimitiveMyself[moveArray.get(i).intValue()];
    		        	additionalEffectProbabilityChoseMyself = additionalEffectProbabilityMyself[moveArray.get(i).intValue()];
    		        	moveEffectCodeChoseMyself = moveEffectCodeMyself[moveArray.get(i).intValue()];
    				}
    		    }
    		    if (moveEffectCodeChoseMyself.equals("1B") || moveEffectCodeChoseMyself.equals("75")) {
    		    	moveEffectCodeChoseMyself = "00a";
    		    }
		    }
    	}

		// 09(オウムがえし)
    	// 　※59(カウンター)、90(ミラーコート)、72(滅びの歌)、39(変身)、5D(テクスチャー２)、8F(自己暗示)、
    	// 　　09(オウムがえし)、53(指を振る)、61(寝言)、94(未来予知)、6D(のろい)はオウム返し不可。
    	// 技を置き換える前にメッセージ出力
    	if (moveEffectCodeChoseMyself.equals("09")) {
    		DialogUtil.dialogMessageOutput(
    				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].playerNameDisp+
    				"の"+
    				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesNameDisp+
    				"の"+
    				moveNameChoseMyself);
    		if(!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("59")&&
    		   !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("90")&&
    		   !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("72")&&
    		   !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("39")&&
    		   !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("5D")&&
    		   !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("8F")&&
    		   !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("09")&&
    		   !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("53")&&
    		   !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("61")&&
    		   !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("94")&&
    		   !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("6D")) {
    			moveCopyFlag = true;
        		moveNameChoseMyself = pokeStatePlaceOpponent.latestUsedMoveName;
        		moveTypeChoseMyself = pokeStatePlaceOpponent.latestUsedMoveType;
        		movePowerChoseMyself = pokeStatePlaceOpponent.latestUsedMovePower;
        		moveHitProbabilityChoseMyself = pokeStatePlaceOpponent.latestUsedMoveHitProbability;
        		movePPPrimitiveChoseMyself = pokeStatePlaceOpponent.latestUsedMovePPPrimitive;
        		additionalEffectProbabilityChoseMyself = pokeStatePlaceOpponent.latestUsedAdditionalEffectProbability;
        		moveEffectCodeChoseMyself = pokeStatePlaceOpponent.latestUsedMoveEffectCode;
    		}
    	}

    	// 53(指を振る)
    	// 出てくる可能性のある技の情報を配列に格納→ランダムで添字を生成し情報取得
    	//（覚えている技は指を振るで発動しないので再抽選とする）
    	// 技を置き換える前にメッセージ出力
    	if (moveEffectCodeChoseMyself.equals("53")) {
    		DialogUtil.dialogMessageOutput(
    				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].playerNameDisp+
    				"の"+
    				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesNameDisp+
    				"の"+
    				moveNameChoseMyself);
    		String moveArray[] = new String[242];
    		moveArray[0] = "はたく|normal|40|100|35|0|00a";
    		moveArray[1] = "メガトンパンチ|normal|80|85|20|0|00a";
    		moveArray[2] = "ひっかく|normal|40|100|35|0|00a";
    		moveArray[3] = "はさむ|normal|55|100|30|0|00a";
    		moveArray[4] = "いあいぎり|normal|50|95|30|0|00a";
    		moveArray[5] = "つばさでうつ|flying|60|100|35|0|00a";
    		moveArray[6] = "たたきつける|normal|80|75|20|0|00a";
    		moveArray[7] = "つるのムチ|grass|35|100|10|0|00a";
    		moveArray[8] = "メガトンキック|normal|120|75|5|0|00a";
    		moveArray[9] = "つのでつく|normal|65|100|25|0|00a";
    		moveArray[10] = "たいあたり|normal|35|95|35|0|00a";
    		moveArray[11] = "みずでっぽう|water|40|100|25|0|00a";
    		moveArray[12] = "ハイドロポンプ|water|120|80|5|0|00a";
    		moveArray[13] = "なみのり|water|95|100|15|0|00a";
    		moveArray[14] = "つつく|flying|35|100|35|0|00a";
    		moveArray[15] = "ドリルくちばし|flying|80|100|20|0|00a";
    		moveArray[16] = "かいりき|normal|80|100|15|0|00a";
    		moveArray[17] = "いわおとし|rock|50|90|15|0|00a";
    		moveArray[18] = "タマゴばくだん|normal|100|75|10|0|00a";
    		moveArray[19] = "たきのぼり|water|80|100|15|0|00a";
    		moveArray[20] = "メガホーン|bug|120|85|5|0|00a";
    		moveArray[21] = "からてチョップ|fighting|50|100|25|0|00b";
    		moveArray[22] = "はっぱカッター|grass|55|95|25|0|00b";
    		moveArray[23] = "クラブハンマー|water|90|85|10|0|00b";
    		moveArray[24] = "きりさく|normal|70|100|20|0|00b";
    		moveArray[25] = "エアロブラスト|flying|100|95|5|0|00b";
    		moveArray[26] = "クロスチョップ|fighting|100|80|5|0|00b";
    		moveArray[27] = "うたう|normal|0|55|15|0|01";
    		moveArray[28] = "ねむりごな|grass|0|75|15|0|01";
    		moveArray[29] = "さいみんじゅつ|psychic|0|60|20|0|01";
    		moveArray[30] = "あくまのキッス|normal|0|75|10|0|01";
    		moveArray[31] = "キノコのほうし|grass|0|100|15|0|01";
    		moveArray[32] = "どくばり|poison|15|100|35|30|02";
    		moveArray[33] = "スモッグ|poison|20|70|20|40|02";
    		moveArray[34] = "ヘドロこうげき|poison|65|100|20|30|02";
    		moveArray[35] = "ヘドロばくだん|poison|90|100|10|30|02";
    		moveArray[36] = "すいとる|grass|20|100|20|0|03";
    		moveArray[37] = "メガドレイン|grass|40|100|10|0|03";
    		moveArray[38] = "きゅうけつ|bug|20|100|15|0|03";
    		moveArray[39] = "ギガドレイン|grass|60|100|5|0|03";
    		moveArray[40] = "ほのおのパンチ|fire|75|100|15|10|04";
    		moveArray[41] = "ひのこ|fire|40|100|25|10|04";
    		moveArray[42] = "かえんほうしゃ|fire|95|100|15|10|04";
    		moveArray[43] = "だいもんじ|fire|120|85|5|10|04";
    		moveArray[44] = "れいとうパンチ|ice|75|100|15|10|05";
    		moveArray[45] = "れいとうビーム|ice|95|100|10|10|05";
    		moveArray[46] = "ふぶき|ice|120|70|5|10|05";
    		moveArray[47] = "こなゆき|ice|40|100|25|10|05";
    		moveArray[48] = "かみなりパンチ|electric|75|100|15|10|06";
    		moveArray[49] = "のしかかり|normal|85|100|15|30|06";
    		moveArray[50] = "でんきショック|electric|40|100|30|10|06";
    		moveArray[51] = "１０まんボルト|electric|95|100|15|10|06";
    		moveArray[52] = "したでなめる|ghost|20|100|30|30|06";
    		moveArray[53] = "でんじほう|electric|100|50|5|100|06";
    		moveArray[54] = "スパーク|electric|65|100|20|30|06";
    		moveArray[55] = "りゅうのいぶき|dragon|60|100|20|30|06";
    		moveArray[56] = "じばく|normal|200|100|5|0|07";
    		moveArray[57] = "だいばくはつ|normal|250|100|5|0|07";
    		moveArray[58] = "ゆめくい|psychic|100|100|15|0|08";
    		moveArray[59] = "ヨガのポーズ|psychic|0|100|40|0|0A";
    		moveArray[60] = "かくばる|normal|0|100|30|0|0A";
    		moveArray[61] = "かたくなる|normal|0|100|30|0|0B";
    		moveArray[62] = "からにこもる|water|0|100|40|0|0B";
    		moveArray[63] = "せいちょう|normal|0|100|40|0|0D";
    		moveArray[64] = "かげぶんしん|normal|0|100|15|0|10a";
    		moveArray[65] = "ちいさくなる|normal|0|100|20|0|10b";
    		moveArray[66] = "スピードスター|normal|60|100|20|0|11a";
    		moveArray[67] = "だましうち|dark|60|100|20|0|11a";
    		moveArray[68] = "あてみなげ|fighting|70|100|10|0|11b";
    		moveArray[69] = "なきごえ|normal|0|100|40|0|12";
    		moveArray[70] = "しっぽをふる|normal|0|100|30|0|13";
    		moveArray[71] = "にらみつける|normal|0|100|30|0|13";
    		moveArray[72] = "いとをはく|bug|0|95|40|0|14";
    		moveArray[73] = "すなかけ|ground|0|100|15|0|17";
    		moveArray[74] = "えんまく|normal|0|100|20|0|17";
    		moveArray[75] = "スプーンまげ|psychic|0|80|15|0|17";
    		moveArray[76] = "フラッシュ|normal|0|70|20|0|17";
    		moveArray[77] = "あまいかおり|normal|0|100|20|0|18";
    		moveArray[78] = "くろいきり|ice|0|100|30|0|19";
    		moveArray[79] = "がまん|normal|0|100|10|0|1A";
    		moveArray[80] = "あばれる|normal|90|100|20|0|1B";
    		moveArray[81] = "はなびらのまい|grass|70|100|20|0|1B";
    		moveArray[82] = "げきりん|dragon|90|100|15|0|1B";
    		moveArray[83] = "ふきとばし|normal|0|100|20|0|1Ca";
    		moveArray[84] = "ほえる|normal|0|100|20|0|1Cb";
    		moveArray[85] = "おうふくビンタ|normal|15|85|10|0|1D";
    		moveArray[86] = "れんぞくパンチ|normal|18|85|15|0|1D";
    		moveArray[87] = "みだれづき|normal|15|85|20|0|1D";
    		moveArray[88] = "ミサイルばり|bug|14|85|20|0|1D";
    		moveArray[89] = "とげキャノン|normal|20|100|15|0|1D";
    		moveArray[90] = "たまなげ|normal|15|85|20|0|1D";
    		moveArray[91] = "みだれひっかき|normal|18|80|15|0|1D";
    		moveArray[92] = "ボーンラッシュ|ground|25|80|10|0|1D";
    		moveArray[93] = "テクスチャー|normal|0|100|30|0|1E";
    		moveArray[94] = "まわしげり|fighting|60|85|15|30|1F";
    		moveArray[95] = "ずつき|normal|70|100|15|30|1F";
    		moveArray[96] = "かみつく|dark|60|100|25|30|1F";
    		moveArray[97] = "けたぐり|fighting|50|90|20|30|1F";
    		moveArray[98] = "ホネこんぼう|ground|65|85|20|10|1F";
    		moveArray[99] = "いわなだれ|rock|75|90|10|30|1F";
    		moveArray[100] = "ひっさつまえば|normal|80|90|15|10|1F";
    		moveArray[101] = "じこさいせい|normal|0|100|20|0|20a";
    		moveArray[102] = "タマゴうみ|normal|0|100|10|0|20a";
    		moveArray[103] = "ミルクのみ|normal|0|100|10|0|20a";
    		moveArray[104] = "ねむる|psychic|0|100|10|0|20b";
    		moveArray[105] = "どくどく|poison|0|85|10|0|21";
    		moveArray[106] = "ネコにこばん|normal|40|100|20|0|22";
    		moveArray[107] = "ひかりのかべ|psychic|0|100|30|0|23";
    		moveArray[108] = "トライアタック|normal|80|100|10|20|24";
    		moveArray[109] = "ハサミギロチン|normal|0|30|5|0|26";
    		moveArray[110] = "つのドリル|normal|1|30|5|0|26";
    		moveArray[111] = "じわれ|ground|1|30|5|0|26";
    		moveArray[112] = "かまいたち|normal|80|75|10|0|27";
    		moveArray[113] = "いかりのまえば|normal|1|90|10|0|28";
    		moveArray[114] = "ソニックブーム|normal|20|90|20|0|29";
    		moveArray[115] = "りゅうのいかり|dragon|40|100|10|0|29";
    		moveArray[116] = "しめつける|normal|15|75|20|0|2A";
    		moveArray[117] = "まきつく|normal|15|85|20|0|2A";
    		moveArray[118] = "ほのおのうず|fire|15|70|15|0|2A";
    		moveArray[119] = "からではさむ|water|35|75|10|0|2A";
    		moveArray[120] = "うずしお|water|15|70|15|0|2A";
    		moveArray[121] = "にどげり|fighting|30|100|30|0|2C";
    		moveArray[122] = "ホネブーメラン|ground|50|90|10|0|2C";
    		moveArray[123] = "とびげり|fighting|70|95|25|0|2D";
    		moveArray[124] = "とびひざげり|fighting|85|90|20|0|2D";
    		moveArray[125] = "しろいきり|ice|0|100|30|0|2E";
    		moveArray[126] = "きあいだめ|normal|0|100|30|0|2F";
    		moveArray[127] = "とっしん|normal|90|85|20|0|30a";
    		moveArray[128] = "すてみタックル|normal|120|100|15|0|30a";
    		moveArray[129] = "じごくぐるま|fighting|80|80|25|0|30a";
    		moveArray[130] = "ちょうおんぱ|normal|0|55|20|0|31";
    		moveArray[131] = "あやしいひかり|ghost|0|100|10|0|31";
    		moveArray[132] = "てんしのキッス|normal|0|75|10|0|31";
    		moveArray[133] = "つるぎのまい|normal|0|100|30|0|32";
    		moveArray[134] = "バリアー|psychic|0|100|30|0|33";
    		moveArray[135] = "とける|poison|0|100|40|0|33";
    		moveArray[136] = "こうそくいどう|psychic|0|100|30|0|34";
    		moveArray[137] = "ドわすれ|psychic|0|100|20|0|36";
    		moveArray[138] = "へんしん|normal|0|100|10|0|39";
    		moveArray[139] = "あまえる|normal|0|100|20|0|3A";
    		moveArray[140] = "いやなおと|normal|0|85|40|0|3B";
    		moveArray[141] = "わたほうし|grass|0|85|40|0|3C";
    		moveArray[142] = "こわいかお|normal|0|90|10|0|3C";
    		moveArray[143] = "リフレクター|psychic|0|100|20|0|41";
    		moveArray[144] = "どくのこな|poison|0|75|35|0|42";
    		moveArray[145] = "どくガス|poison|0|55|40|0|42";
    		moveArray[146] = "しびれごな|grass|0|75|30|0|43";
    		moveArray[147] = "でんじは|electric|0|100|20|0|43";
    		moveArray[148] = "へびにらみ|normal|0|75|30|0|43";
    		moveArray[149] = "オーロラビーム|ice|65|100|20|10|44";
    		moveArray[150] = "ようかいえき|poison|40|100|30|10|45";
    		moveArray[151] = "アイアンテール|steel|100|75|15|30|45";
    		moveArray[152] = "いわくだき|fighting|20|100|15|50|45";
    		moveArray[153] = "バブルこうせん|water|65|100|20|10|46";
    		moveArray[154] = "からみつく|normal|10|100|35|10|46";
    		moveArray[155] = "あわ|water|20|100|30|10|46";
    		moveArray[156] = "こごえるかぜ|ice|55|95|15|100|46";
    		moveArray[157] = "サイコキネシス|psychic|90|100|10|10|48";
    		moveArray[158] = "かみくだく|dark|80|100|15|20|48";
    		moveArray[159] = "シャドーボール|ghost|80|100|15|20|48";
    		moveArray[160] = "どろかけ|ground|20|100|10|100|49";
    		moveArray[161] = "オクタンほう|water|65|85|10|50|49";
    		moveArray[162] = "ゴッドバード|flying|140|90|5|0|4B";
    		moveArray[163] = "サイケこうせん|psychic|65|100|20|10|4C";
    		moveArray[164] = "ねんりき|psychic|50|100|25|10|4C";
    		moveArray[165] = "ピヨピヨパンチ|normal|70|100|10|20|4C";
    		moveArray[166] = "ばくれつパンチ|fighting|100|50|5|100|4C";
    		moveArray[167] = "ダブルニードル|bug|25|100|20|20|4D";
    		moveArray[168] = "みがわり|normal|0|100|10|0|4F";
    		moveArray[169] = "はかいこうせん|normal|150|90|5|0|50";
    		moveArray[170] = "いかり|normal|20|100|20|0|51";
    		moveArray[171] = "ゆびをふる|normal|0|100|10|0|53";
    		moveArray[172] = "やどりぎのタネ|grass|0|90|10|0|54";
    		moveArray[173] = "はねる|normal|0|100|40|0|55";
    		moveArray[174] = "かなしばり|normal|0|55|20|0|56";
    		moveArray[175] = "ちきゅうなげ|fighting|1|100|20|0|57";
    		moveArray[176] = "ナイトヘッド|ghost|1|100|15|0|57";
    		moveArray[177] = "サイコウェーブ|psychic|1|80|15|0|58";
    		moveArray[178] = "アンコール|normal|0|100|5|0|5A";
    		moveArray[179] = "いたみわけ|normal|0|100|20|0|5B";
    		moveArray[180] = "いびき|normal|40|100|15|30|5C";
    		moveArray[181] = "テクスチャー２|normal|0|100|30|0|5D";
    		moveArray[182] = "こころのめ|normal|0|100|5|0|5E";
    		moveArray[183] = "ロックオン|normal|0|100|5|0|5E";
    		moveArray[184] = "ねごと|normal|0|100|10|0|61";
    		moveArray[185] = "みちづれ|ghost|0|100|5|0|62";
    		moveArray[186] = "じたばた|normal|1|100|15|0|63";
    		moveArray[187] = "きしかいせい|fighting|1|100|15|0|63";
    		moveArray[188] = "うらみ|ghost|0|100|10|0|64";
    		moveArray[189] = "みねうち|normal|40|100|40|0|65";
    		moveArray[190] = "いやしのすず|normal|0|100|5|0|66";
    		moveArray[191] = "でんこうせっか|normal|40|100|30|0|67";
    		moveArray[192] = "マッハパンチ|fighting|40|100|30|0|67";
    		moveArray[193] = "しんそく|normal|80|100|5|0|67";
    		moveArray[194] = "トリプルキック|fighting|10|90|10|0|68";
    		moveArray[195] = "どろぼう|dark|40|100|10|100|69";
    		moveArray[196] = "クモのす|bug|0|100|10|0|6A";
    		moveArray[197] = "くろいまなざし|normal|0|100|5|0|6A";
    		moveArray[198] = "あくむ|ghost|0|100|15|0|6B";
    		moveArray[199] = "かえんぐるま|fire|60|100|25|10|6C";
    		moveArray[200] = "のろい|question|0|100|10|0|6D";
    		moveArray[201] = "まきびし|ground|0|100|20|0|70";
    		moveArray[202] = "みやぶる|normal|0|100|40|0|71";
    		moveArray[203] = "ほろびのうた|normal|0|100|5|0|72";
    		moveArray[204] = "すなあらし|rock|0|100|10|0|73";
    		moveArray[205] = "ころがる|rock|30|90|20|0|75";
    		moveArray[206] = "いばる|normal|0|90|15|0|76";
    		moveArray[207] = "れんぞくぎり|bug|10|95|20|0|77";
    		moveArray[208] = "メロメロ|normal|0|100|15|0|78";
    		moveArray[209] = "おんがえし|normal|102|100|20|0|79";
    		moveArray[210] = "プレゼント|normal|1|90|15|0|7A";
    		moveArray[211] = "やつあたり|normal|102|100|20|0|7B";
    		moveArray[212] = "しんぴのまもり|normal|0|100|25|0|7C";
    		moveArray[213] = "せいなるほのお|fire|100|95|5|50|7D";
    		moveArray[214] = "マグニチュード|ground|1|100|30|0|7E";
    		moveArray[215] = "バトンタッチ|normal|0|100|40|0|7F";
    		moveArray[216] = "おいうち|dark|40|100|20|0|80";
    		moveArray[217] = "こうそくスピン|normal|20|100|40|0|81";
    		moveArray[218] = "あさのひざし|normal|0|100|5|0|84";
    		moveArray[219] = "こうごうせい|grass|0|100|5|0|85";
    		moveArray[220] = "つきのひかり|normal|0|100|5|0|86";
    		moveArray[221] = "めざめるパワー|normal|1|100|15|0|87";
    		moveArray[222] = "あまごい|water|0|100|5|0|88";
    		moveArray[223] = "にほんばれ|fire|0|100|5|0|89";
    		moveArray[224] = "はがねのつばさ|steel|70|90|25|10|8A";
    		moveArray[225] = "メタルクロー|steel|50|95|35|10|8B";
    		moveArray[226] = "げんしのちから|rock|60|100|5|10|8C";
    		moveArray[227] = "はらだいこ|normal|0|100|10|0|8E";
    		moveArray[228] = "じこあんじ|normal|0|100|10|0|8F";
    		moveArray[229] = "ロケットずつき|normal|100|100|15|0|91";
    		moveArray[230] = "たつまき|dragon|40|100|20|20|92";
    		moveArray[231] = "じしん|ground|100|100|10|0|93";
    		moveArray[232] = "みらいよち|psychic|80|90|15|0|94";
    		moveArray[233] = "かぜおこし|flying|40|100|35|0|95";
    		moveArray[234] = "ふみつけ|normal|65|100|20|30|96";
    		moveArray[235] = "ソーラービーム|grass|120|100|10|0|97";
    		moveArray[236] = "かみなり|electric|120|70|10|30|98";
    		moveArray[237] = "テレポート|psychic|0|100|20|0|99";
    		moveArray[238] = "ふくろだたき|dark|10|100|10|0|9A";
    		moveArray[239] = "そらをとぶ|flying|70|95|15|0|9Ba";
    		moveArray[240] = "あなをほる|ground|60|100|10|0|9Bb";
    		moveArray[241] = "まるくなる|normal|0|100|40|0|9C";
    		boolean confirmFlag = false;
    		while(!confirmFlag) {
    			Random random = new Random();
    			int moveRandom = random.nextInt(242);
    			String[] moveArraySplit = moveArray[moveRandom].split("\\|", 0);
    			moveCopyFlag = true;
    			moveNameChoseMyself = moveArraySplit[0];
    			moveTypeChoseMyself = moveArraySplit[1];
    			movePowerChoseMyself = Integer.parseInt(moveArraySplit[2]);
    			moveHitProbabilityChoseMyself = Double.parseDouble(moveArraySplit[3]);
    			movePPPrimitiveChoseMyself = Integer.parseInt(moveArraySplit[4]);
    			additionalEffectProbabilityChoseMyself = Double.parseDouble(moveArraySplit[5]);
    			moveEffectCodeChoseMyself = moveArraySplit[6];
    			// 覚えている技（残りPPが0ではない）は指を振るで出せない
    			if (moveNameChoseMyself.equals(moveNameMyself[0]) && movePPMyself[0] > 0) {
    				confirmFlag = false;
    			} else if (moveNameChoseMyself.equals(moveNameMyself[1]) && movePPMyself[1] > 0) {
    				confirmFlag = false;
    			} else if (moveNameChoseMyself.equals(moveNameMyself[2]) && movePPMyself[2] > 0) {
    				confirmFlag = false;
    			} else if (moveNameChoseMyself.equals(moveNameMyself[3]) && movePPMyself[3] > 0) {
    				confirmFlag = false;
    			} else {
    				confirmFlag = true;
    			}
    		}
    	}

		// 24(トライアタック)
    	if (moveEffectCodeChoseMyself.equals("24")) {
        	Random random = new Random();
        	int triRandom = random.nextInt(3);
        	switch (triRandom) {
        	case 0:
        		moveEffectCodeChoseMyself = "24a";
        		break;
        	case 1:
        		moveEffectCodeChoseMyself = "24b";
        		break;
        	case 2:
        		moveEffectCodeChoseMyself = "24c";
        		break;
        	default: //ここに来ることはないが念のため
        		moveEffectCodeChoseMyself = "24a";
        		break;
        	}
    	}

    	// 技を繰り出すメッセージの出力
		DialogUtil.dialogMessageOutput(
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].playerNameDisp+
				"の"+
				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesNameDisp+
				"の"+
				moveNameChoseMyself);

    }

	/* 各種技の処理メソッド（王者の印の判定も含む、身代わりの有無や相手のタイプによる挙動の違いに注意）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void moveExecution() throws IOException {

    	// 汎用の中間結果
    	int damage = 0; // ダメージ
    	int validity = 100; // 補助技が有効かどうか(0だと無効)
    	double hitProbability = 100.0; // 命中率
    	double criticalHitProbability = 6.6; //急所率
    	double subjectValue = 1.0; // 攻撃力
    	double objectValue = 1.0; //防御力
    	double levelValue = 50.0; //レベル

    	// 本処理
    	switch (moveEffectCodeChoseMyself) {

		// 00a(通常の攻撃技)
		// 00b(急所技)
		// 22(猫に小判)
		// 67(先制技)
		// 79(恩返し)
		// 7B(八つ当たり)
    	case "00a":
    	case "00b":
    	case "22":
    	case "67":
    	case "79":
    	case "7B":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
    		                                    pokeStateBatonMyself.hitProbabilityRank,
    		                                    pokeStateBatonOpponent.avoidProbabilityRank,
    		                                    partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
    		                                    pokeStatePlaceMyself.seeingThroughFlag,
    		                                    pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
			if (hitJudge(hitProbability,
			             pokeStatePlaceOpponent.protectSuccessFlag,
			             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
				subjectValue = subjectValueCalc(moveTypeChoseMyself,
						                        attackMyself,
						                        contactMyself,
						                        pokeStateBatonMyself.attackRank,
						                        pokeStateBatonMyself.contactRank,
						                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
						                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
						                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
				objectValue = objectValueCalc(moveTypeChoseMyself,
				                              blockOpponent,
				                              diffenceOpponent,
				                              pokeStateBatonOpponent.blockRank,
				                              pokeStateBatonOpponent.diffenceRank,
				                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
				                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
				                              pokeStateWholeOpponent.blockHalfFlag,
				                              pokeStateWholeOpponent.reflectorRestTurn,
				                              pokeStateWholeOpponent.lightWallRestTurn);
				levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
				damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
				criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
				                                                    moveEffectCodeChoseMyself,
				    						                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
				    						                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
				if(criticalHitJudge(criticalHitProbability)){
					damage = criticalHitDamageCalc(damage,
					                               moveTypeChoseMyself,
					                               pokeStateBatonMyself.attackRank,
					                               pokeStateBatonMyself.contactRank,
					                               pokeStateBatonOpponent.blockRank,
					                               pokeStateBatonOpponent.diffenceRank,
					                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
					                               pokeStateWholeOpponent.reflectorRestTurn,
					                               pokeStateWholeOpponent.lightWallRestTurn);
				}
				damage = typeReinforceItemCalc(damage,
				                               moveTypeChoseMyself,
				                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
				damage = constPlus2Calc(damage);
				damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
				damage = typeMatchCalc(damage,
				                       moveTypeChoseMyself,
				                       type1Myself,
				                       type2Myself);
				damage = typeCompatibilityCalc(damage,
				                               moveTypeChoseMyself,
				                               type1Opponent,
				                               type2Opponent,
		                                       pokeStatePlaceMyself.seeingThroughFlag,
		                                       pokeStatePlaceOpponent.seenThroughFlag);
				damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
				if (damage > 0) {
					moveHitFlag = true;
					damage = damageRandomCalc(damage);
					if (pokeStateBatonOpponent.scapegoatHP > 0) {
						scapegoatHPUpDownOpponent = damage * (-1);
					} else {
				    	spiritFlag = spiritTriggerCalc(damage,
				    			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
				    			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
						damage = finalDamageCalc(damage,
					                             partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
					                             pokeStatePlaceOpponent.protectSuccessFlag,
					                             pokeStatePlaceOpponent.latestUsedMoveEffectCode,
					                             moveEffectCodeChoseMyself,
					                             spiritFlag);
						HPUpDownOpponent = damage * (-1);
						LastHPUpDownOpponent = HPUpDownOpponent;
						if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
							if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
								winceFlag = true;
							}
						}
					}
				}
			}
    		break;

		// 01(催眠技)
    	case "01":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
    		             pokeStatePlaceOpponent.protectSuccessFlag,
    		             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			validity = typeCompatibilityCalc(100,
                                                 moveTypeChoseMyself,
                                                 type1Opponent,
                                                 type2Opponent,
                                                 pokeStatePlaceMyself.seeingThroughFlag,
                                                 pokeStatePlaceOpponent.seenThroughFlag);
    			if (validity > 0) {
        			moveHitFlag = true;
					if (pokeStateBatonOpponent.scapegoatHP == 0) {
						for (int i = 0; i < 6; i++) {
							if (partyInfoOpponent.pokeStateUnitArray[i] != null) {
								if (partyInfoOpponent.pokeStateUnitArray[i].badHealth.equals("sleep")) {
									validity = 0;
								}
							}
						}
						if (validity > 0) {
							if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
								badHealthOpponentTmp = "sleep";
								comboCountResetFlagWhenSleep = true;
							} else {
								DialogUtil.dialogMessageOutput("神秘の守りで無効化");
							}
						} else {
							DialogUtil.dialogMessageOutput("２匹以上同時に眠りにならない");
						}
					} else {
						DialogUtil.dialogMessageOutput("身代わりで無効化");
					}
    			}
    		}
    		break;

		// 02(追加効果毒(毒ポケモンは毒にならない))
    	case "02":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
    		             pokeStatePlaceOpponent.protectSuccessFlag,
    		             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
				if (damage > 0) {
	    			moveHitFlag = true;
					damage = damageRandomCalc(damage);
					if (pokeStateBatonOpponent.scapegoatHP > 0) {
						scapegoatHPUpDownOpponent = damage * (-1);
					} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
						damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
						HPUpDownOpponent = damage * (-1);
						LastHPUpDownOpponent = HPUpDownOpponent;
						if (!type1Opponent.equals("poison") &&
						    !type2Opponent.equals("poison")) {
							if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
								if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
									badHealthOpponentTmp = "poison";
								} else {
									DialogUtil.dialogMessageOutput("神秘の守りで無効化");
								}
							}
						} else {
							DialogUtil.dialogMessageOutput("毒ポケモンは毒にはならない");
						}
					}
				}
    		}
    		break;

		// 03(ギガドレイン等吸収技)
    	case "03":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					DialogUtil.dialogMessageOutput("身代わりで無効化");
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
						LastHPUpDownOpponent = HPUpDownOpponent;
    					HPUpDownMyself = ratioCalc(damage,1,2);
    				}
    			}
    		}
    		break;

		// 04(追加効果火傷(炎ポケモンは火傷にならない)、相手の凍り解凍)
    	case "04":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
						LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].badHealth.equals("freeze")) {
    						badHealthOpponentTmp = "none";
    					} else if (!type1Opponent.equals("fire") &&
    					             !type2Opponent.equals("fire")) {
    						if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    							if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
    								badHealthOpponentTmp = "burn";
    							} else {
    								DialogUtil.dialogMessageOutput("神秘の守りで無効化");
    							}
    						}
    					} else {
    						DialogUtil.dialogMessageOutput("炎ポケモンは火傷にはならない");
    					}
    				}
    			}
    		}
    		break;

		// 05(追加効果凍り(氷ポケモンは凍りにならない、２匹以上同時に凍りにならない、晴れてる時は凍らない))
    	case "05":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
						LastHPUpDownOpponent = HPUpDownOpponent;
    					if (!type1Opponent.equals("ice") &&
					         !type2Opponent.equals("ice")) {
    						for (int i = 0; i < 6; i++) {
    							if (partyInfoOpponent.pokeStateUnitArray[i] != null) {
    								if (partyInfoOpponent.pokeStateUnitArray[i].badHealth.equals("freeze")) {
    									validity = 0;
    								}
    							}
    						}
    						if (validity > 0) {
    							if (!placeState.weather.equals("sunny")) {
    								if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    									if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
    										badHealthOpponentTmp = "freeze";
    									} else {
    										DialogUtil.dialogMessageOutput("神秘の守りで無効化");
    									}
    								}
    							} else {
    								DialogUtil.dialogMessageOutput("晴れている時は凍りにならない");
    							}
    						} else {
    							DialogUtil.dialogMessageOutput("２匹以上同時に凍りにならない");
    						}
    					} else {
    						DialogUtil.dialogMessageOutput("氷ポケモンは凍りにはならない");
    					}
    				}
    			}
    		}
    		break;

		// 06(追加効果麻痺)
		// 24c(トライアタック（麻痺）)
    	case "06":
    	case "24c":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
							if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
								badHealthOpponentTmp = "paralysis";
							} else {
								DialogUtil.dialogMessageOutput("神秘の守りで無効化");
							}
    					}
    				}
    			}
    		}
    		break;

		// 07(爆破)
    	case "07":
    		foulFlagMyself = true;
    		HPUpDownMyself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP * (-1);
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 08(夢食い)
    	case "08":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (!partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].badHealth.equals("sleep")) {
    			hitProbability = 0.0;
            }
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					DialogUtil.dialogMessageOutput("身代わりで無効化");
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					HPUpDownMyself = ratioCalc(damage,1,2);
    				}
    			}
    		}
    		break;

		// 09(オウムがえし（失敗）)
		// 53(指を振る（通常ここには来ない）)
		// 55(はねる)
        // 61(寝言)
		// 99(テレポート)
    	// XXc(技無し（通常選ばれない）)
    	case "09":
    	case "53":
    	case "55":
    	case "61":
    	case "99":
    	case "XXc":
    		DialogUtil.dialogMessageOutput("しかし何も起こらなかった！");
    		break;

		// 0A(自分の攻撃ランク+1)
    	case "0A":
    		attackRankUpDownMyself = 1;
    		break;

		// 0B(自分の防御ランク+1)
    	case "0B":
    		blockRankUpDownMyself = 1;
    		break;

		// 0D(自分の特攻ランク+1)
    	case "0D":
    		contactRankUpDownMyself = 1;
    		break;

		// 10a(自分の回避率ランク+1)
    	case "10a":
    		avoidProbabilityRankUpDownMyself = 1;
    		break;

		// 10b(小さくなる)
    	case "10b":
    		avoidProbabilityRankUpDownMyself = 1;
    		smallShapeFlag = true;
    		break;

		// 11a(必中技)
		// 11b(当て身投げ)
    	case "11a":
    	case "11b":
    		hitProbability = 100.0;
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
    		    pokeStatePlaceOpponent.protectSuccessFlag,
    		    pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
    			                                attackMyself,
    			                                contactMyself,
    			                                pokeStateBatonMyself.attackRank,
    			                                pokeStateBatonMyself.contactRank,
    			                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
    			                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
    			                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 12(相手の攻撃ランク-1)
    	case "12":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
    		             pokeStatePlaceOpponent.protectSuccessFlag,
    		             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
					if (pokeStateBatonOpponent.whiteFogFlag == false) {
						attackRankUpDownOpponent = -1;
					} else {
						DialogUtil.dialogMessageOutput("白い霧で無効化");
					}
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

		// 13(相手の防御ランク-1)
    	case "13":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
					if (pokeStateBatonOpponent.whiteFogFlag == false) {
						blockRankUpDownOpponent = -1;
					} else {
						DialogUtil.dialogMessageOutput("白い霧で無効化");
					}
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

		// 14(相手の素早さランク-1)
    	case "14":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
    	                 pokeStatePlaceOpponent.protectSuccessFlag,
    	                 pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
					if (pokeStateBatonOpponent.whiteFogFlag == false) {
						speedRankUpDownOpponent = -1;
					} else {
						DialogUtil.dialogMessageOutput("白い霧で無効化");
					}
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

		// 17(相手の命中率ランク-1)
    	case "17":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
    		             pokeStatePlaceOpponent.protectSuccessFlag,
    		             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
					if (pokeStateBatonOpponent.whiteFogFlag == false) {
						hitProbabilityRankUpDownOpponent = -1;
					} else {
						DialogUtil.dialogMessageOutput("白い霧で無効化");
					}
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

		// 18(相手の回避率ランク-1)
    	case "18":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
					if (pokeStateBatonOpponent.whiteFogFlag == false) {
						avoidProbabilityRankUpDownOpponent = -1;
					} else {
						DialogUtil.dialogMessageOutput("白い霧で無効化");
					}
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

		// 19(黒い霧)
    	case "19":
    		blackFogFlag = true;
    		break;

		// 1A(我慢)
    	case "1A":
			comboCountDownFlag = true;
    		if (pokeStatePlaceMyself.isNotBeingCombo()) {
    	    	Random random = new Random();
    	    	int comboTrunRandom = random.nextInt(2) + 2;
    	    	comboStartTurnSet = comboTrunRandom;
    		} else {
        		if (pokeStatePlaceMyself.isBeingComboLastTrun()) {
        			bideReleaseFlag = true;
            		hitProbability = 100.0; // ポケモンWikiでは命中率100%となっているが、POKeMON Battle HiSTORiAの通り必中が正しい
            		if (hitJudge(hitProbability,
            		    pokeStatePlaceOpponent.protectSuccessFlag,
            		    pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
            			damage = pokeStatePlaceMyself.accumulationGetDamage * 2;
            			validity = typeCompatibilityCalc(100,
                                                         moveTypeChoseMyself,
                                                         type1Opponent,
                                                         type2Opponent,
                                                         pokeStatePlaceMyself.seeingThroughFlag,
                                                         pokeStatePlaceOpponent.seenThroughFlag);
            			if (validity == 0) {
            				damage = 0;
            			}
            			if (damage > 0) {
                			moveHitFlag = true;
            				if (pokeStateBatonOpponent.scapegoatHP > 0) {
            					scapegoatHPUpDownOpponent = damage * (-1);
            				} else {
        				    	spiritFlag = spiritTriggerCalc(damage,
	    			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
	    			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
            					damage = finalDamageCalc(damage,
                                                         partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                         pokeStatePlaceOpponent.protectSuccessFlag,
                                                         pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                         moveEffectCodeChoseMyself,
                                                         spiritFlag);
            					HPUpDownOpponent = damage * (-1);
            					LastHPUpDownOpponent = HPUpDownOpponent;
            					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
            						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
            							winceFlag = true;
            						}
            					}
            				}
            			}
            		}
        		}
    		}
    		break;

		// 1B(暴れる・花びらの舞・逆鱗)
    	case "1B":
			comboCountDownFlag = true;
    		if (pokeStatePlaceMyself.isNotBeingCombo()) {
    	    	Random random = new Random();
    	    	int comboTrunRandom = random.nextInt(2) + 2;
    	    	comboStartTurnSet = comboTrunRandom;
    		}
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 1Ca(吹き飛ばし)
    	case "1Ca":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (firstHandFlag) {
    			hitProbability = 0.0;
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			int pokeCountOpponent = 0;
    	    	for(int i = 0; i < 6; i++){
    	    		if (partyInfoOpponent.pokeStateUnitArray[i] != null) {
    	    			if (partyInfoOpponent.pokeStateUnitArray[i].actualHP > 0) {
    	    				if (i != partyInfoOpponent.pokeStateUnitArrayIndex) {
    	    					pokeCountOpponent++;
    	    				}
    	    			}
    	    		}
    	    	}
    	    	if (pokeCountOpponent > 0) {
        			moveHitFlag = true;
        	    	Random random = new Random();
        	    	int forceChangeRandom = random.nextInt(pokeCountOpponent);
        	    	int forceChangeBorder = 0;
        	    	for(int i = 0; i < 6; i++){
        	    		if (partyInfoOpponent.pokeStateUnitArray[i] != null) {
        	    			if (partyInfoOpponent.pokeStateUnitArray[i].actualHP > 0) {
        	    				if (i != partyInfoOpponent.pokeStateUnitArrayIndex) {
        	    					if (forceChangeBorder == forceChangeRandom) {
        	    						pokeStateUnitArrayOpponentIndexChanged = i;
        	    					}
        	    					forceChangeBorder++;
        	    				}
        	    			}
        	    		}
        	    	}
					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
							winceFlag = true;
						}
					}
    	    	} else {
    	    		DialogUtil.dialogMessageOutput("相手は他に交代できるポケモンがいない");
    	    	}
    		}
    		break;

		// 1Cb(吠える)
    	case "1Cb":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (firstHandFlag) {
    			hitProbability = 0.0;
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			int pokeCountOpponent = 0;
    	    	for(int i = 0; i < 6; i++){
    	    		if (partyInfoOpponent.pokeStateUnitArray[i] != null) {
    	    			if (partyInfoOpponent.pokeStateUnitArray[i].actualHP > 0) {
    	    				if (i != partyInfoOpponent.pokeStateUnitArrayIndex) {
    	    					pokeCountOpponent++;
    	    				}
    	    			}
    	    		}
    	    	}
    	    	if (pokeCountOpponent > 0) {
        			moveHitFlag = true;
        	    	Random random = new Random();
        	    	int forceChangeRandom = random.nextInt(pokeCountOpponent);
        	    	int forceChangeBorder = 0;
        	    	for(int i = 0; i < 6; i++){
        	    		if (partyInfoOpponent.pokeStateUnitArray[i] != null) {
        	    			if (partyInfoOpponent.pokeStateUnitArray[i].actualHP > 0) {
        	    				if (i != partyInfoOpponent.pokeStateUnitArrayIndex) {
        	    					if (forceChangeBorder == forceChangeRandom) {
        	    						pokeStateUnitArrayOpponentIndexChanged = i;
        	    					}
        	    					forceChangeBorder++;
        	    				}
        	    			}
        	    		}
        	    	}
    				if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    					if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    						winceFlag = true;
    					}
    				}
    			} else {
    				DialogUtil.dialogMessageOutput("相手は他に交代できるポケモンがいない");
    			}
    		}
    		break;

		// 1D(連続攻撃技)
    	case "1D":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
    		             pokeStatePlaceOpponent.protectSuccessFlag,
    		             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			int damageBase = damage;
    			int attackTimes = 0;
				Random random = new Random();
				int attackTimesRandom = random.nextInt(8);
				if (attackTimesRandom >= 0 && attackTimesRandom <= 2) {
					attackTimes = 2;
				} else if (attackTimesRandom >= 3 && attackTimesRandom <= 5) {
					attackTimes = 3;
				} else if (attackTimesRandom == 6) {
					attackTimes = 4;
				} else {
					attackTimes = 5;
				}
				DialogUtil.dialogMessageOutput(attackTimes+"回当たった");
    			for(int i = 0; i < attackTimes; i++){
    				damage = damageBase;
    				criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                        moveEffectCodeChoseMyself,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    				if(criticalHitJudge(criticalHitProbability)){
    					damage = criticalHitDamageCalc(damage,
                                                       moveTypeChoseMyself,
                                                       pokeStateBatonMyself.attackRank,
                                                       pokeStateBatonMyself.contactRank,
                                                       pokeStateBatonOpponent.blockRank,
                                                       pokeStateBatonOpponent.diffenceRank,
                                                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                       pokeStateWholeOpponent.reflectorRestTurn,
                                                       pokeStateWholeOpponent.lightWallRestTurn);
    				}
    				damage = typeReinforceItemCalc(damage,
                                                   moveTypeChoseMyself,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    				damage = constPlus2Calc(damage);
    				damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    				damage = typeMatchCalc(damage,
                                           moveTypeChoseMyself,
                                           type1Myself,
                                           type2Myself);
    				damage = typeCompatibilityCalc(damage,
                                                   moveTypeChoseMyself,
                                                   type1Opponent,
                                                   type2Opponent,
                                                   pokeStatePlaceMyself.seeingThroughFlag,
                                                   pokeStatePlaceOpponent.seenThroughFlag);
    				damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    				if (damage > 0) {
    	    			moveHitFlag = true;
    					damage = damageRandomCalc(damage);
    					if (pokeStateBatonOpponent.scapegoatHP > (scapegoatHPUpDownOpponent * (-1))) {
    						scapegoatHPUpDownOpponent = scapegoatHPUpDownOpponent + (damage * (-1));
    					} else {
    						if (!spiritFlag) {
    							spiritFlag = spiritTriggerCalc(damage,
 			                                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP + HPUpDownOpponent,
 			                                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    						}
    						damage = finalDamageCalc(damage,
                                                     partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP + HPUpDownOpponent,
                                                     pokeStatePlaceOpponent.protectSuccessFlag,
                                                     pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                     moveEffectCodeChoseMyself,
                                                     spiritFlag);
    						HPUpDownOpponent = HPUpDownOpponent + (damage * (-1));
    	    				if (i == (attackTimes - 1)) {
    	    	    			LastHPUpDownOpponent = damage;
    	    	    			if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    	    	    				if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    	    	    					winceFlag = true;
    	    	    				}
    	    	    			}
    	    				}
    					}
    				}
    			}
    		}
    		break;

		// 1E(テクスチャー)
    	case "1E":
    		if (true) {
    			ArrayList<String> typeArray = new ArrayList<String>();
    			String typeCandidate = "normal";
    			for (int i = 0 ; i < 4 ; i++){
    				typeCandidate = moveTypeMyself[i];
    				// 自分のタイプと同じ場合や、のろいはテクスチャーできない
    				// (当然、技無しの場合もテクスチャー不可。「技無し」がタイプ？？？として扱われるとみなして実装。)
    				if (typeCandidate.equals(type1Myself) ||
    				    typeCandidate.equals(type2Myself) ||
    				    typeCandidate.equals("question")) {
    					continue;
    				}
    				// テクスチャー、スケッチもテクスチャーできない
    				if (moveEffectCodeMyself[i].equals("1E") ||
		    	        moveEffectCodeMyself[i].equals("5F")) {
    					continue;
    				}
    				if (typeArray.size() == 0) {
    					typeArray.add(typeCandidate);
    				} else {
    					for (int j = 0 ; j < typeArray.size() ; j++){
    						boolean duplicationFlag = false;
    						if (typeCandidate.equals(typeArray.get(j))) {
    							duplicationFlag = true;
    						}
    						if (j == typeArray.size()-1 && !duplicationFlag) {
    							typeArray.add(typeCandidate);
    						}
    					}
    				}
    			}
    			if (typeArray.size() > 0) {
    				typePasteFlag = true;
    				Random random = new Random();
    				int typeRandom = random.nextInt(typeArray.size());
    				for (int i = 0 ; i < typeArray.size() ; i++){
    					if (i==typeRandom) {
    						type1MyselfPaste = typeArray.get(i);
    						type2MyselfPaste = "none";
    					}
    				}
    			} else {
    				DialogUtil.dialogMessageOutput("テクスチャーに失敗した");
    			}
    		}
    		break;

		// 1F(追加効果怯み)
    	case "1F":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						winceFlag = true;
    					}
    				}
    			}
    		}
    		break;

		// 20a(再生系)
    	case "20a":
    		// 本当は標準メソッドのratioCalcを使いたいが、実績があるのでこのまま
    		HPUpDownMyself = (int) Math.floor(
    				partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP / 2);
    		break;

		// 20b(眠る)
    	case "20b":
    		if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP >
    		    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP) {
    			HPUpDownMyself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP;
    			sleepMyselfFlag = true;
    		} else {
    			DialogUtil.dialogMessageOutput("HP満タンでの眠るは成功しない");
    		}
    		break;

		// 21(毒々(毒ポケモンは毒にならない))
    	case "21":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			validity = typeCompatibilityCalc(100,
                                                 moveTypeChoseMyself,
                                                 type1Opponent,
                                                 type2Opponent,
                                                 pokeStatePlaceMyself.seeingThroughFlag,
                                                 pokeStatePlaceOpponent.seenThroughFlag);
				if (type1Opponent.equals("poison") ||
		            type2Opponent.equals("poison")) {
					validity = 0;
					DialogUtil.dialogMessageOutput("毒ポケモンは毒にはならない");
				}
    			if (validity > 0) {
        			moveHitFlag = true;
    				if (pokeStateBatonOpponent.scapegoatHP == 0) {
						if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
							badHealthOpponentTmp = "poison";
							if (partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].badHealth.equals("poison")||
							    partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].badHealth.equals("burn")) {
								DialogUtil.dialogMessageOutput("既に毒・火傷になっているので猛毒にはならない");
							} else {
								morePoisonFlag = true;
							}
						} else {
							DialogUtil.dialogMessageOutput("神秘の守りで無効化");
						}
    				} else {
    					DialogUtil.dialogMessageOutput("身代わりで無効化");
    				}
    			}
    		}
    		break;

		// 23(光の壁)
    	case "23":
    		lightWallFlag = true;
    		break;

   		// 24a(トライアタック（火傷、炎ポケモンも火傷になる、相手の凍り解凍）)
        	case "24a":
        		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                    pokeStateBatonMyself.hitProbabilityRank,
                                                    pokeStateBatonOpponent.avoidProbabilityRank,
                                                    partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                    pokeStatePlaceMyself.seeingThroughFlag,
                                                    pokeStatePlaceOpponent.seenThroughFlag,
        		                                    pokeStatePlaceOpponent.lockonFlag);
        		if (!pokeStatePlaceOpponent.lockonFlag) {
            		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                		pokeStatePlaceOpponent.intoSkyFlag) {
                		hitProbability = 0.0;
                	}
        		}
        		if (hitJudge(hitProbability,
                             pokeStatePlaceOpponent.protectSuccessFlag,
                             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
        			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                    attackMyself,
                                                    contactMyself,
                                                    pokeStateBatonMyself.attackRank,
                                                    pokeStateBatonMyself.contactRank,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
        			objectValue = objectValueCalc(moveTypeChoseMyself,
                                                  blockOpponent,
                                                  diffenceOpponent,
                                                  pokeStateBatonOpponent.blockRank,
                                                  pokeStateBatonOpponent.diffenceRank,
                                                  partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                                  partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                  pokeStateWholeOpponent.blockHalfFlag,
                                                  pokeStateWholeOpponent.reflectorRestTurn,
                                                  pokeStateWholeOpponent.lightWallRestTurn);
        			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
        			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
        			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                        moveEffectCodeChoseMyself,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
        			if(criticalHitJudge(criticalHitProbability)){
        				damage = criticalHitDamageCalc(damage,
                                                       moveTypeChoseMyself,
                                                       pokeStateBatonMyself.attackRank,
                                                       pokeStateBatonMyself.contactRank,
                                                       pokeStateBatonOpponent.blockRank,
                                                       pokeStateBatonOpponent.diffenceRank,
                                                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                       pokeStateWholeOpponent.reflectorRestTurn,
                                                       pokeStateWholeOpponent.lightWallRestTurn);
        			}
        			damage = typeReinforceItemCalc(damage,
                                                   moveTypeChoseMyself,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
        			damage = constPlus2Calc(damage);
        			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
        			damage = typeMatchCalc(damage,
                                           moveTypeChoseMyself,
                                           type1Myself,
                                           type2Myself);
        			damage = typeCompatibilityCalc(damage,
                                                   moveTypeChoseMyself,
                                                   type1Opponent,
                                                   type2Opponent,
                                                   pokeStatePlaceMyself.seeingThroughFlag,
                                                   pokeStatePlaceOpponent.seenThroughFlag);
        			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
        			if (damage > 0) {
            			moveHitFlag = true;
        				damage = damageRandomCalc(damage);
        				if (pokeStateBatonOpponent.scapegoatHP > 0) {
        					scapegoatHPUpDownOpponent = damage * (-1);
        				} else {
    				    	spiritFlag = spiritTriggerCalc(damage,
 			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
 			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
        					damage = finalDamageCalc(damage,
                                                     partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                     pokeStatePlaceOpponent.protectSuccessFlag,
                                                     pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                     moveEffectCodeChoseMyself,
                                                     spiritFlag);
        					HPUpDownOpponent = damage * (-1);
    						LastHPUpDownOpponent = HPUpDownOpponent;
        					if (partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].badHealth.equals("freeze")) {
        						badHealthOpponentTmp = "none";
        					} else if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
        						if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
        							badHealthOpponentTmp = "burn";
        						} else {
        							DialogUtil.dialogMessageOutput("神秘の守りで無効化");
        						}
        					}
        				}
        			}
        		}
        		break;

    		// 24b(トライアタック（凍り）(氷ポケモンも凍りになる、２匹以上同時に凍りになる、晴れてる時は凍らない（多分）))
        	case "24b":
        		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                    pokeStateBatonMyself.hitProbabilityRank,
                                                    pokeStateBatonOpponent.avoidProbabilityRank,
                                                    partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                    pokeStatePlaceMyself.seeingThroughFlag,
                                                    pokeStatePlaceOpponent.seenThroughFlag,
        		                                    pokeStatePlaceOpponent.lockonFlag);
        		if (!pokeStatePlaceOpponent.lockonFlag) {
            		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                		pokeStatePlaceOpponent.intoSkyFlag) {
                		hitProbability = 0.0;
                	}
        		}
        		if (hitJudge(hitProbability,
                             pokeStatePlaceOpponent.protectSuccessFlag,
                             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
        			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                    attackMyself,
                                                    contactMyself,
                                                    pokeStateBatonMyself.attackRank,
                                                    pokeStateBatonMyself.contactRank,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
        			objectValue = objectValueCalc(moveTypeChoseMyself,
                                                  blockOpponent,
                                                  diffenceOpponent,
                                                  pokeStateBatonOpponent.blockRank,
                                                  pokeStateBatonOpponent.diffenceRank,
                                                  partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                                  partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                  pokeStateWholeOpponent.blockHalfFlag,
                                                  pokeStateWholeOpponent.reflectorRestTurn,
                                                  pokeStateWholeOpponent.lightWallRestTurn);
        			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
        			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
        			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                        moveEffectCodeChoseMyself,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
        			if(criticalHitJudge(criticalHitProbability)){
        				damage = criticalHitDamageCalc(damage,
                                                       moveTypeChoseMyself,
                                                       pokeStateBatonMyself.attackRank,
                                                       pokeStateBatonMyself.contactRank,
                                                       pokeStateBatonOpponent.blockRank,
                                                       pokeStateBatonOpponent.diffenceRank,
                                                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                       pokeStateWholeOpponent.reflectorRestTurn,
                                                       pokeStateWholeOpponent.lightWallRestTurn);
        			}
        			damage = typeReinforceItemCalc(damage,
                                                   moveTypeChoseMyself,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
        			damage = constPlus2Calc(damage);
        			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
        			damage = typeMatchCalc(damage,
                                           moveTypeChoseMyself,
                                           type1Myself,
                                           type2Myself);
        			damage = typeCompatibilityCalc(damage,
                                                   moveTypeChoseMyself,
                                                   type1Opponent,
                                                   type2Opponent,
                                                   pokeStatePlaceMyself.seeingThroughFlag,
                                                   pokeStatePlaceOpponent.seenThroughFlag);
        			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
        			if (damage > 0) {
            			moveHitFlag = true;
        				damage = damageRandomCalc(damage);
        				if (pokeStateBatonOpponent.scapegoatHP > 0) {
        					scapegoatHPUpDownOpponent = damage * (-1);
        				} else {
    				    	spiritFlag = spiritTriggerCalc(damage,
 			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
 			                      partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
        					damage = finalDamageCalc(damage,
                                                     partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                     pokeStatePlaceOpponent.protectSuccessFlag,
                                                     pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                     moveEffectCodeChoseMyself,
                                                     spiritFlag);
        					HPUpDownOpponent = damage * (-1);
    						LastHPUpDownOpponent = HPUpDownOpponent;
    						if (!placeState.weather.equals("sunny")) {
    							if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    								if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
    									badHealthOpponentTmp = "freeze";
    								} else {
    									DialogUtil.dialogMessageOutput("神秘の守りで無効化");
    								}
    							}
							} else {
								DialogUtil.dialogMessageOutput("晴れている時は凍りにならない");
							}
        				}
        			}
        		}
        		break;

		// 26(一撃技)
    	case "26":
    		int levelDiff = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level -
    		                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].level;
    		moveHitProbabilityChoseMyself = moveHitProbabilityChoseMyself
    		                              + ( levelDiff * 2.0 / 256.0);
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                6,
                                                6,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (levelDiff < 0) {
    			hitProbability = 0.0;
    		}
    		if (hitJudge(hitProbability,
    		             pokeStatePlaceOpponent.protectSuccessFlag,
    		             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			damage = 65535;
    			validity = typeCompatibilityCalc(100,
                                                 moveTypeChoseMyself,
                                                 type1Opponent,
                                                 type2Opponent,
                                                 pokeStatePlaceMyself.seeingThroughFlag,
                                                 pokeStatePlaceOpponent.seenThroughFlag);
    			if (validity == 0) {
    				damage = 0;
    			}
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    				}
    			}
    		}
    		break;

		// 27(かまいたち)
		// 4B(ゴッドバード)
    	case "27":
    	case "4B":
			comboCountDownFlag = true;
    		if (pokeStatePlaceMyself.isNotBeingCombo()) {
    	    	comboStartTurnSet = 2;
    		} else {
        		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                    pokeStateBatonMyself.hitProbabilityRank,
                                                    pokeStateBatonOpponent.avoidProbabilityRank,
                                                    partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                    pokeStatePlaceMyself.seeingThroughFlag,
                                                    pokeStatePlaceOpponent.seenThroughFlag,
        		                                    pokeStatePlaceOpponent.lockonFlag);
        		if (!pokeStatePlaceOpponent.lockonFlag) {
            		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                		pokeStatePlaceOpponent.intoSkyFlag) {
                		hitProbability = 0.0;
                	}
        		}
        		if (hitJudge(hitProbability,
                             pokeStatePlaceOpponent.protectSuccessFlag,
                             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
        			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                    attackMyself,
                                                    contactMyself,
                                                    pokeStateBatonMyself.attackRank,
                                                    pokeStateBatonMyself.contactRank,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
        			objectValue = objectValueCalc(moveTypeChoseMyself,
                                                  blockOpponent,
                                                  diffenceOpponent,
                                                  pokeStateBatonOpponent.blockRank,
                                                  pokeStateBatonOpponent.diffenceRank,
                                                  partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                                  partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                  pokeStateWholeOpponent.blockHalfFlag,
                                                  pokeStateWholeOpponent.reflectorRestTurn,
                                                  pokeStateWholeOpponent.lightWallRestTurn);
        			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
        			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
        			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                        moveEffectCodeChoseMyself,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
        			if(criticalHitJudge(criticalHitProbability)){
        				damage = criticalHitDamageCalc(damage,
                                                       moveTypeChoseMyself,
                                                       pokeStateBatonMyself.attackRank,
                                                       pokeStateBatonMyself.contactRank,
                                                       pokeStateBatonOpponent.blockRank,
                                                       pokeStateBatonOpponent.diffenceRank,
                                                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                       pokeStateWholeOpponent.reflectorRestTurn,
                                                       pokeStateWholeOpponent.lightWallRestTurn);
        			}
        			damage = typeReinforceItemCalc(damage,
                                                   moveTypeChoseMyself,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
        			damage = constPlus2Calc(damage);
        			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
        			damage = typeMatchCalc(damage,
                                           moveTypeChoseMyself,
                                           type1Myself,
                                           type2Myself);
        			damage = typeCompatibilityCalc(damage,
                                                   moveTypeChoseMyself,
                                                   type1Opponent,
                                                   type2Opponent,
                                                   pokeStatePlaceMyself.seeingThroughFlag,
                                                   pokeStatePlaceOpponent.seenThroughFlag);
        			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
        			if (damage > 0) {
            			moveHitFlag = true;
        				damage = damageRandomCalc(damage);
        				if (pokeStateBatonOpponent.scapegoatHP > 0) {
        					scapegoatHPUpDownOpponent = damage * (-1);
        				} else {
    				    	spiritFlag = spiritTriggerCalc(damage,
 			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
 			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
        					damage = finalDamageCalc(damage,
                                                     partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                     pokeStatePlaceOpponent.protectSuccessFlag,
                                                     pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                     moveEffectCodeChoseMyself,
                                                     spiritFlag);
        					HPUpDownOpponent = damage * (-1);
        					LastHPUpDownOpponent = HPUpDownOpponent;
        					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
        						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
        							winceFlag = true;
        						}
        					}
        				}
        			}
        		}
    		}
    		break;

		// 28(いかりのまえば)
    	case "28":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
		                 pokeStatePlaceOpponent.protectSuccessFlag,
		                 pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			if (partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP == 1) {
    				damage = 1;
    			} else {
    				damage = (int) Math.floor
    						(partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP / 2.0);
    			}
    			validity = typeCompatibilityCalc(100,
    			                                 moveTypeChoseMyself,
    			                                 type1Opponent,
    			                                 type2Opponent,
    			                                 pokeStatePlaceMyself.seeingThroughFlag,
    			                                 pokeStatePlaceOpponent.seenThroughFlag);
    			if (validity == 0) {
    				damage = 0;
    			}
    			if (damage > 0) {
        			moveHitFlag = true;
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 29(威力値の固定ダメージ)
    	case "29":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			damage = movePowerChoseMyself;
    			validity = typeCompatibilityCalc(100,
                                                 moveTypeChoseMyself,
                                                 type1Opponent,
                                                 type2Opponent,
                                                 pokeStatePlaceMyself.seeingThroughFlag,
                                                 pokeStatePlaceOpponent.seenThroughFlag);
    			if (validity == 0) {
    				damage = 0;
    			}
    			if (damage > 0) {
        			moveHitFlag = true;
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 2A(締め付ける系)
    	case "2A":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
            if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
            	subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
            	objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
            	levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
            	damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
            	criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
            	if(criticalHitJudge(criticalHitProbability)){
            		damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
            	}
            	damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
            	damage = constPlus2Calc(damage);
            	damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
            	damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
            	damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
            	damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
            	if (damage > 0) {
                	moveHitFlag = true;
                	twiningFlag = true;
            		damage = damageRandomCalc(damage);
            		if (pokeStateBatonOpponent.scapegoatHP > 0) {
            			scapegoatHPUpDownOpponent = damage * (-1);
            		} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
            			damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
            			HPUpDownOpponent = damage * (-1);
            			LastHPUpDownOpponent = HPUpDownOpponent;
            		}
            	}
            }
    		break;

		// 2C(２回連続攻撃)
    	case "2C":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
    			                                attackMyself,
    			                                contactMyself,
    			                                pokeStateBatonMyself.attackRank,
    			                                pokeStateBatonMyself.contactRank,
    			                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
    			                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
    			                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			int damageBase = damage;
    			int attackTimes = 2;
				DialogUtil.dialogMessageOutput(attackTimes+"回当たった");
    			for(int i = 0; i < attackTimes; i++){
    				damage = damageBase;
    				criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                        moveEffectCodeChoseMyself,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    				if(criticalHitJudge(criticalHitProbability)){
    					damage = criticalHitDamageCalc(damage,
                                                       moveTypeChoseMyself,
                                                       pokeStateBatonMyself.attackRank,
                                                       pokeStateBatonMyself.contactRank,
                                                       pokeStateBatonOpponent.blockRank,
                                                       pokeStateBatonOpponent.diffenceRank,
                                                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                       pokeStateWholeOpponent.reflectorRestTurn,
                                                       pokeStateWholeOpponent.lightWallRestTurn);
    				}
    				damage = typeReinforceItemCalc(damage,
                                                   moveTypeChoseMyself,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    				damage = constPlus2Calc(damage);
    				damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    				damage = typeMatchCalc(damage,
                                           moveTypeChoseMyself,
                                           type1Myself,
                                           type2Myself);
    				damage = typeCompatibilityCalc(damage,
                                                   moveTypeChoseMyself,
                                                   type1Opponent,
                                                   type2Opponent,
                                                   pokeStatePlaceMyself.seeingThroughFlag,
                                                   pokeStatePlaceOpponent.seenThroughFlag);
    				damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    				if (damage > 0) {
    	    			moveHitFlag = true;
    					damage = damageRandomCalc(damage);
    					if (pokeStateBatonOpponent.scapegoatHP > (scapegoatHPUpDownOpponent * (-1))) {
    						scapegoatHPUpDownOpponent = scapegoatHPUpDownOpponent + (damage * (-1));
    					} else {
    						if (!spiritFlag) {
    							spiritFlag = spiritTriggerCalc(damage,
 			                                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP + HPUpDownOpponent,
 			                                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    						}
    						damage = finalDamageCalc(damage,
                                                     partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP + HPUpDownOpponent,
                                                     pokeStatePlaceOpponent.protectSuccessFlag,
                                                     pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                     moveEffectCodeChoseMyself,
                                                     spiritFlag);
    						HPUpDownOpponent = HPUpDownOpponent + (damage * (-1));
    	    				if (i == (attackTimes - 1)) {
    	    	    			LastHPUpDownOpponent = damage;
    	    	    			if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    	    	    				if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    	    	    					winceFlag = true;
    	    	    				}
    	    	    			}
    	    				}
    					}
    				}
    			}
    		}
    		break;

		// 2D(外れで自分に本来与えるはずのダメージの1/4のダメージ)
    	case "2D":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		} else {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			damage = damageRandomCalc(damage);
				if (pokeStateBatonOpponent.scapegoatHP > 0) {
					damage = 0;
				}
				HPUpDownMyself = ratioCalc(damage,1,4) * (-1);
				DialogUtil.dialogMessageOutput("勢いあまって膝をぶつけた");
    		}
    		break;

		// 2E(白い霧)
    	case "2E":
    		whiteFogFlag = true;
    		break;

		// 2F(急所ランク+1)
    	case "2F":
    		criticalHitProbabilityRankUpDownMyself = 1;
    		break;

		// 30a(命中で本体に与えたダメージの1/4の反動)
    	case "30a":
    		moveExecution_30a();
    		break;

		// 30b(悪あがき)
    	case "30b":
    		moveExecution_30b();
    		break;

		// 31(混乱技)
    	case "31":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
    				if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
    					confusionFlag = true;
    				} else {
    					DialogUtil.dialogMessageOutput("神秘の守りで無効化");
    				}
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

		// 32(自分の攻撃ランク+2)
    	case "32":
    		attackRankUpDownMyself = 2;
    		break;

		// 33(自分の防御ランク+2)
    	case "33":
    		blockRankUpDownMyself = 2;
    		break;

		// 34(自分の素早さランク+2)
    	case "34":
    		speedRankUpDownMyself = 2;
    		break;

		// 36(自分の特防ランク+2)
    	case "36":
    		diffenceRankUpDownMyself = 2;
    		break;

		// 39(変身)
    	case "39":
    		if (pokeStatePlaceOpponent.lockonFlag ||
    		    (!pokeStatePlaceOpponent.intoUndergroundFlag &&
                 !pokeStatePlaceOpponent.intoSkyFlag)) {
        		if (!pokeStatePlaceOpponent.transformedFlag) {
        			transformFlag = true;
        		} else {
        			DialogUtil.dialogMessageOutput("変身相手に変身できない");
        		}
    		} else {
    			DialogUtil.dialogMessageOutput("地中や空中にいる相手に変身できない");
    		}
    		break;

		// 3A(相手の攻撃ランク-2)
    	case "3A":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
    				if (pokeStateBatonOpponent.whiteFogFlag == false) {
    					attackRankUpDownOpponent = -2;
    				} else {
    					DialogUtil.dialogMessageOutput("白い霧で無効化");
    				}
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

		// 3B(相手の防御ランク-2)
    	case "3B":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
    				if (pokeStateBatonOpponent.whiteFogFlag == false) {
    					blockRankUpDownOpponent = -2;
    				} else {
    					DialogUtil.dialogMessageOutput("白い霧で無効化");
    				}
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

		// 3C(相手の素早さランク-2)
    	case "3C":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
    				if (pokeStateBatonOpponent.whiteFogFlag == false) {
    					speedRankUpDownOpponent = -2;
    				} else {
    					DialogUtil.dialogMessageOutput("白い霧で無効化");
    				}
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

		// 41(リフレクター)
    	case "41":
    		reflectorFlag = true;
    		break;

		// 42(毒技(毒ポケモンは毒にならない))
    	case "42":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			validity = typeCompatibilityCalc(100,
                                                 moveTypeChoseMyself,
                                                 type1Opponent,
                                                 type2Opponent,
                                                 pokeStatePlaceMyself.seeingThroughFlag,
                                                 pokeStatePlaceOpponent.seenThroughFlag);
				if (type1Opponent.equals("poison") ||
			        type2Opponent.equals("poison")) {
					validity = 0;
					DialogUtil.dialogMessageOutput("毒ポケモンは毒にはならない");
				}
    			if (validity > 0) {
        			moveHitFlag = true;
    				if (pokeStateBatonOpponent.scapegoatHP == 0) {
    					if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
    						badHealthOpponentTmp = "poison";
    					} else {
    						DialogUtil.dialogMessageOutput("神秘の守りで無効化");
    					}
    				} else {
    					DialogUtil.dialogMessageOutput("身代わりで無効化");
    				}
    			}
    		}
    		break;

		// 43(麻痺技)
    	case "43":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
    				pokeStatePlaceOpponent.protectSuccessFlag,
    				pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			validity = typeCompatibilityCalc(100,
                                                 moveTypeChoseMyself,
                                                 type1Opponent,
                                                 type2Opponent,
                                                 pokeStatePlaceMyself.seeingThroughFlag,
                                                 pokeStatePlaceOpponent.seenThroughFlag);
    			if (validity > 0) {
        			moveHitFlag = true;
    				if (pokeStateBatonOpponent.scapegoatHP == 0) {
   						if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
    						badHealthOpponentTmp = "paralysis";
    					} else {
    						DialogUtil.dialogMessageOutput("神秘の守りで無効化");
    					}
    				} else {
    					DialogUtil.dialogMessageOutput("身代わりで無効化");
    				}
    			}
    		}
    		break;

		// 44(追加効果で相手の攻撃ランク-1)
    	case "44":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    	    				if (pokeStateBatonOpponent.whiteFogFlag == false) {
    	    					attackRankUpDownOpponent = -1;
    	    				} else {
    	    					DialogUtil.dialogMessageOutput("白い霧で無効化");
    	    				}
    					}
    				}
    			}
    		}
    		break;

		// 45(追加効果で相手の防御ランク-1)
    	case "45":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						if (pokeStateBatonOpponent.whiteFogFlag == false) {
    							blockRankUpDownOpponent = -1;
    						} else {
    							DialogUtil.dialogMessageOutput("白い霧で無効化");
    						}
    					}
    				}
    			}
    		}
    		break;

		// 46(追加効果で相手の素早さランク-1)
    	case "46":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						if (pokeStateBatonOpponent.whiteFogFlag == false) {
    							speedRankUpDownOpponent = -1;
    						} else {
    							DialogUtil.dialogMessageOutput("白い霧で無効化");
    						}
    					}
    				}
    			}
    		}
    		break;

		// 48(追加効果で相手の特防ランク-1)
    	case "48":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						if (pokeStateBatonOpponent.whiteFogFlag == false) {
    							diffenceRankUpDownOpponent = -1;
    						} else {
    							DialogUtil.dialogMessageOutput("白い霧で無効化");
    						}
    					}
    				}
    			}
    		}
    		break;

		// 49(追加効果で相手の命中率ランク-1)
    	case "49":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						if (pokeStateBatonOpponent.whiteFogFlag == false) {
    							hitProbabilityRankUpDownOpponent = -1;
    						} else {
    							DialogUtil.dialogMessageOutput("白い霧で無効化");
    						}
    					}
    				}
    			}
    		}
    		break;

		// 4C(追加効果で混乱)
    	case "4C":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    	    				if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
    	    					confusionFlag = true;
    	    				} else {
    	    					DialogUtil.dialogMessageOutput("神秘の守りで無効化");
    	    				}
    					}
    				}
    			}
    		}
    		break;

		// 4D(ダブルニードル(毒ポケモンは毒にならない))
    	case "4D":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			int damageBase = damage;
    			int attackTimes = 2;
				DialogUtil.dialogMessageOutput(attackTimes+"回当たった");
    			for(int i = 0; i < attackTimes; i++){
    				damage = damageBase;
    				criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                        moveEffectCodeChoseMyself,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    				if(criticalHitJudge(criticalHitProbability)){
    					damage = criticalHitDamageCalc(damage,
                                                       moveTypeChoseMyself,
                                                       pokeStateBatonMyself.attackRank,
                                                       pokeStateBatonMyself.contactRank,
                                                       pokeStateBatonOpponent.blockRank,
                                                       pokeStateBatonOpponent.diffenceRank,
                                                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                       pokeStateWholeOpponent.reflectorRestTurn,
                                                       pokeStateWholeOpponent.lightWallRestTurn);
    				}
    				damage = typeReinforceItemCalc(damage,
                                                   moveTypeChoseMyself,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    				damage = constPlus2Calc(damage);
    				damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    				damage = typeMatchCalc(damage,
                                           moveTypeChoseMyself,
                                           type1Myself,
                                           type2Myself);
    				damage = typeCompatibilityCalc(damage,
                                                   moveTypeChoseMyself,
                                                   type1Opponent,
                                                   type2Opponent,
                                                   pokeStatePlaceMyself.seeingThroughFlag,
                                                   pokeStatePlaceOpponent.seenThroughFlag);
    				damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    				if (damage > 0) {
    	    			moveHitFlag = true;
    					damage = damageRandomCalc(damage);
    					if (pokeStateBatonOpponent.scapegoatHP > (scapegoatHPUpDownOpponent * (-1))) {
    						scapegoatHPUpDownOpponent = scapegoatHPUpDownOpponent + (damage * (-1));
    					} else {
    						if (!spiritFlag) {
    							spiritFlag = spiritTriggerCalc(damage,
 			                                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP + HPUpDownOpponent,
 			                                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    						}
    						damage = finalDamageCalc(damage,
                                                     partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP + HPUpDownOpponent,
                                                     pokeStatePlaceOpponent.protectSuccessFlag,
                                                     pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                     moveEffectCodeChoseMyself,
                                                     spiritFlag);
    						HPUpDownOpponent = HPUpDownOpponent + (damage * (-1));
    	    				if (i == (attackTimes - 1)) {
    	    	    			LastHPUpDownOpponent = damage;
    	    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    	    						if (!type1Opponent.equals("poison") &&
    	    	                        !type2Opponent.equals("poison")) {
    	    	    					if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
    	    	    						badHealthOpponentTmp = "poison";
    	    	    					} else {
    	    	    						DialogUtil.dialogMessageOutput("神秘の守りで無効化");
    	    	    					}
    	    	    				} else {
    	    	    					DialogUtil.dialogMessageOutput("毒ポケモンは毒にはならない");
    	    	    				}
    	    					}
    	    	    			if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    	    	    				if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    	    	    					winceFlag = true;
    	    	    				}
    	    	    			}
    	    				}
    					}
    				}
    			}
    		}
    		break;

		// 4F(身代わり)
    	case "4F":
    		if (pokeStateBatonMyself.scapegoatHP == 0) {
    			int HPquarter = (int) Math.floor(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP / 4.0);
    			if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP > HPquarter) {
    				HPUpDownMyself = HPquarter * (-1);
    				scapegoatHPUpDownMyself = HPquarter;
    			} else {
    				DialogUtil.dialogMessageOutput("HP不足で身代わり作れず");
    			}
    		} else {
    			DialogUtil.dialogMessageOutput("既に身代わりはある");
    		}
    		break;

		// 50(破壊光線)
    	case "50":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				destructionBeemStackSetFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 51(怒り)
    	case "51":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageMultipleCalc(damage,
    			                            pokeStatePlaceMyself.angerContinuousTrun - 1);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 52(物真似)
    	case "52":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStatePlaceOpponent.latestUsedMoveIndex >= 0 && pokeStatePlaceOpponent.latestUsedMoveIndex <= 3) {
    				boolean duplicateFlag = false;
    				for (int i = 0; i < 4; i++) {
    					if (pokeStatePlaceOpponent.latestUsedMoveName.equals(moveNameMyself[i])) {
    						duplicateFlag = true;
    					}
    				}
    				if (!duplicateFlag) {
    					// 52(物真似)、30b(悪あがき)、53(指を振る)、5F(スケッチ)、61(ねごと)、09(オウムがえし)に対しては失敗
    					if (!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("52")&&
    					    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("30b")&&
    					    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("53")&&
    					    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("5F")&&
    					    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("61")&&
    					    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("09")) {
        					mimicFlag = true;
    					} else {
    						DialogUtil.dialogMessageOutput("物真似できなかった");
    					}
    				} else {
    					DialogUtil.dialogMessageOutput("物真似できなかった");
    				}
    			} else {
    				DialogUtil.dialogMessageOutput("物真似できなかった");
    			}
    		}
    		break;

		// 54(宿り木の種)
    	case "54":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			validity = 100;
				if (type1Opponent.equals("grass") ||
				    type2Opponent.equals("grass")) {
					validity = 0;
					DialogUtil.dialogMessageOutput("草ポケモンは宿り木にはならない");
				}
				if (validity > 0) {
					moveHitFlag = true;
					if (pokeStateBatonOpponent.scapegoatHP == 0) {
						parasiteFlag = true;
					} else {
						DialogUtil.dialogMessageOutput("身代わりで無効化");
					}
				}
    		}
    		break;

		// 56(金縛り)
    	case "56":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			moveParalysisFlag = true;
    		}
    		break;

		// 57(レベル固定ダメージ)
    	case "57":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			damage = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			validity = typeCompatibilityCalc(100,
                                                 moveTypeChoseMyself,
                                                 type1Opponent,
                                                 type2Opponent,
                                                 pokeStatePlaceMyself.seeingThroughFlag,
                                                 pokeStatePlaceOpponent.seenThroughFlag);
    			if (validity == 0) {
    				damage = 0;
    			}
    			if (damage > 0) {
    				moveHitFlag = true;
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
                        scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 58(サイコウェーブ)
    	case "58":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
    			    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			int maxDamage = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level,
    					                    3,
    					                    2);
    			Random random = new Random();
    			int levelRandom = random.nextInt(maxDamage) + 1;
    			damage = levelRandom;
    			validity = typeCompatibilityCalc(100,
                                                 moveTypeChoseMyself,
                                                 type1Opponent,
                                                 type2Opponent,
                                                 pokeStatePlaceMyself.seeingThroughFlag,
                                                 pokeStatePlaceOpponent.seenThroughFlag);
    			if (validity == 0) {
    				damage = 0;
    			}
    			if (damage > 0) {
    				moveHitFlag = true;
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 59(カウンター)
    	case "59":
    		hitProbability = 100.0;
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			damage = pokeStatePlaceMyself.thisTrunGetPhysicDamage * 2;
    			validity = typeCompatibilityCalc(100,
                                                 moveTypeChoseMyself,
                                                 type1Opponent,
                                                 type2Opponent,
                                                 pokeStatePlaceMyself.seeingThroughFlag,
                                                 pokeStatePlaceOpponent.seenThroughFlag);
    			if (validity == 0) {
    				damage = 0;
    			}
    			if (damage > 0) {
    				moveHitFlag = true;
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 5A(アンコール)
    	case "5A":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			// 39(変身)、52(物真似)、5F(スケッチ)、09(オウムがえし)、53(指を振る)、5A(アンコール)、30b(悪あがき)はアンコールできない
    			if (!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("39")&&
					!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("52")&&
					!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("5F")&&
					!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("09")&&
					!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("53")&&
					!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("5A")&&
					!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("30b")) {
    				encoreFlag = true;
    			} else {
    				DialogUtil.dialogMessageOutput("選択された技はアンコールできない");
    			}
    		}
    		break;

		// 5B(痛み分け)
    	case "5B":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
				if (pokeStateBatonOpponent.scapegoatHP == 0) {
	    			int HPaverage = 0;
	    			HPaverage = (int) Math.floor(
	    					(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP +
	    					 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP)
	    					/ 2.0
	    					);
	    			HPUpDownMyself = HPaverage - partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP;
	    			HPUpDownOpponent = HPaverage - partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP;
				} else {
					DialogUtil.dialogMessageOutput("身代わりで無効化");
				}
    		}
    		break;

		// 5C(いびき)
    	case "5C":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
    					pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (!partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("sleep")) {
    			hitProbability = 0.0;
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						winceFlag = true;
    					}
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 5D(テクスチャー２)
    	case "5D":
    		// のろい、へんしん、オウムがえし、ねごと、ゆびをふる、スケッチ、ものまねはテクスチャー２できない
    		if (pokeStatePlaceOpponent.latestUsedMoveIndex != -1 &&
    		    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("6D") &&
    		    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("39") &&
    		    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("09") &&
    		    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("61") &&
    		    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("5F") &&
    		    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("52")) {
    			ArrayList<String> typeArray = new ArrayList<String>();
    			if (pokeStatePlaceOpponent.latestUsedMoveType.equals("normal")){
    				typeArray.add("rock");
    				typeArray.add("steel");
    				typeArray.add("ghost");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("fighting")){
    				typeArray.add("flying");
    				typeArray.add("poison");
    				typeArray.add("bug");
    				typeArray.add("psychic");
    				typeArray.add("ghost");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("flying")){
    				typeArray.add("rock");
    				typeArray.add("steel");
    				typeArray.add("electric");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("poison")){
    				typeArray.add("poison");
    				typeArray.add("ground");
    				typeArray.add("rock");
    				typeArray.add("ghost");
    				typeArray.add("steel");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("ground")){
    				typeArray.add("bug");
    				typeArray.add("grass");
    				typeArray.add("flying");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("rock")){
    				typeArray.add("fighting");
    				typeArray.add("ground");
    				typeArray.add("steel");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("bug")){
    				typeArray.add("fighting");
    				typeArray.add("flying");
    				typeArray.add("poison");
    				typeArray.add("ghost");
    				typeArray.add("steel");
    				typeArray.add("fire");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("ghost")){
    				typeArray.add("steel");
    				typeArray.add("dark");
    				typeArray.add("normal");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("steel")){
    				typeArray.add("steel");
    				typeArray.add("fire");
    				typeArray.add("water");
    				typeArray.add("electric");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("fire")){
    				typeArray.add("rock");
    				typeArray.add("fire");
    				typeArray.add("water");
    				typeArray.add("dragon");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("water")){
    				typeArray.add("water");
    				typeArray.add("grass");
    				typeArray.add("dragon");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("grass")){
    				typeArray.add("flying");
    				typeArray.add("poison");
    				typeArray.add("bug");
    				typeArray.add("steel");
    				typeArray.add("fire");
    				typeArray.add("grass");
    				typeArray.add("dragon");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("electric")){
    				typeArray.add("grass");
    				typeArray.add("electric");
    				typeArray.add("dragon");
    				typeArray.add("ground");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("psychic")){
    				typeArray.add("steel");
    				typeArray.add("psychic");
    				typeArray.add("dark");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("ice")){
    				typeArray.add("steel");
    				typeArray.add("fire");
    				typeArray.add("water");
    				typeArray.add("ice");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("dragon")){
    				typeArray.add("steel");
    			} else if (pokeStatePlaceOpponent.latestUsedMoveType.equals("dark")){
    				typeArray.add("fighting");
    				typeArray.add("steel");
    				typeArray.add("dark");
    			}
    			if (typeArray.size() > 0) {
    				typePasteFlag = true;
	    	    	Random random = new Random();
	    	    	int typeRandom = random.nextInt(typeArray.size());
	    	    	for (int i = 0 ; i < typeArray.size() ; i++){
	    	    		if (i==typeRandom) {
	    	    			type1MyselfPaste = typeArray.get(i);
	    	    			type2MyselfPaste = "none";
	    	    		}
	    	    	}
    			} else {
    				DialogUtil.dialogMessageOutput("テクスチャー２に失敗した");
    			}
    		} else {
    			DialogUtil.dialogMessageOutput("テクスチャー２に失敗した");
    		}
    		break;

		// 5E(ロックオン系)
    	case "5E":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
    			    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
    				lockOnFlag = true;
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

    	// 5F(スケッチ)
    	case "5F":
    		moveExecution_5F();
    		break;

		// 62(道連れ)
    	case "62":
    		downTogetherWaitFlag = true;
    		break;

		// 63(じたばた・起死回生)
    	case "63":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
    		                                    pokeStateBatonMyself.hitProbabilityRank,
    		                                    pokeStateBatonOpponent.avoidProbabilityRank,
    		                                    partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
    		                                    pokeStatePlaceMyself.seeingThroughFlag,
    		                                    pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			double MAXHPDouble = (double) partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP;
    			double actualHPDouble = (double) partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP;
    			double HPRatio = actualHPDouble / MAXHPDouble;
    			double border1 = 0.042;
    			double border2 = 0.105;
    			double border3 = 0.209;
    			double border4 = 0.355;
    			double border5 = 0.775;
    			if (HPRatio < border1) {
    				movePowerChoseMyself = 200;
    			} else if (HPRatio >= border1 && HPRatio < border2) {
    				movePowerChoseMyself = 150;
    			} else if (HPRatio >= border2 && HPRatio < border3) {
    				movePowerChoseMyself = 100;
    			} else if (HPRatio >= border3 && HPRatio < border4) {
    				movePowerChoseMyself = 80;
    			} else if (HPRatio >= border4 && HPRatio < border5) {
    				movePowerChoseMyself = 40;
    			} else {
    				movePowerChoseMyself = 20;
    			}
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 64(恨み)
    	case "64":
    		moveExecution_64();
    		break;

    	// 65(みねうち)
        case "65":
        	moveExecution_65();
        	break;

		// 66(癒しの鈴)
    	case "66":
    		everyoneGoodHealthFlag = true;
    		break;

		// 68(トリプルキック)
    	case "68":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (true) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			int damageBase = damage;
    			int attackTimes = 0;
    			Random random = new Random();
    			int attackTimesRandom = random.nextInt(3);
    			if (attackTimesRandom == 0) {
    				attackTimes = 1;
    			} else if (attackTimesRandom == 1) {
    				attackTimes = 2;
    			} else {
    				attackTimes = 3;
    			}
    			for(int i = 0; i < attackTimes; i++){
    				damage = damageBase;
    				criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                        moveEffectCodeChoseMyself,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    				if(criticalHitJudge(criticalHitProbability)){
    					damage = criticalHitDamageCalc(damage,
                                                       moveTypeChoseMyself,
                                                       pokeStateBatonMyself.attackRank,
                                                       pokeStateBatonMyself.contactRank,
                                                       pokeStateBatonOpponent.blockRank,
                                                       pokeStateBatonOpponent.diffenceRank,
                                                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                       pokeStateWholeOpponent.reflectorRestTurn,
                                                       pokeStateWholeOpponent.lightWallRestTurn);
    				}
    				damage = typeReinforceItemCalc(damage,
                                                   moveTypeChoseMyself,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    				damage = constPlus2Calc(damage);
    				damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    				damage = typeMatchCalc(damage,
                                           moveTypeChoseMyself,
                                           type1Myself,
                                           type2Myself);
    				damage = typeCompatibilityCalc(damage,
                                                   moveTypeChoseMyself,
                                                   type1Opponent,
                                                   type2Opponent,
                                                   pokeStatePlaceMyself.seeingThroughFlag,
                                                   pokeStatePlaceOpponent.seenThroughFlag);
    				placeState.tripleKickRatioSet(i + 1);
    				damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    				if (hitJudge(hitProbability,
                            pokeStatePlaceOpponent.protectSuccessFlag,
                            pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    					DialogUtil.dialogMessageOutput((i+1)+"回当たった");
    				} else {
    					damage = 0;
    					i = attackTimes; // １回でも外れればそこで攻撃終了
    				}
    				if (damage > 0) {
    					moveHitFlag = true;
    					damage = damageRandomCalc(damage);
    					if (pokeStateBatonOpponent.scapegoatHP > (scapegoatHPUpDownOpponent * (-1))) {
    						scapegoatHPUpDownOpponent = scapegoatHPUpDownOpponent + (damage * (-1));
    					} else {
    						if (!spiritFlag) {
    							spiritFlag = spiritTriggerCalc(damage,
 			                                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP + HPUpDownOpponent,
 			                                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    						}
    						damage = finalDamageCalc(damage,
                                                     partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP + HPUpDownOpponent,
                                                     pokeStatePlaceOpponent.protectSuccessFlag,
                                                     pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                     moveEffectCodeChoseMyself,
                                                     spiritFlag);
    						HPUpDownOpponent = HPUpDownOpponent + (damage * (-1));
    						if (i == (attackTimes - 1)) {
    							LastHPUpDownOpponent = damage;
    							if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    								if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    									winceFlag = true;
    								}
    							}
    						}
    					}
    				}
    			}
    		}
    		break;

		// 69(泥棒)
    	case "69":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("持ち物無し") &&
    							!partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName.equals("持ち物無し")) {
    							// PBSはメールへの泥棒は失敗しない
    							//if (!partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName.equals("はながらボール") &&
    							//    !partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName.equals("なみのりメール") &&
    							//    !partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName.equals("みずいろメール") &&
    							//    !partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName.equals("にがおえメール") &&
    							//    !partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName.equals("ラブリーメール") &&
    							//    !partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName.equals("ブイブイメール") &&
    							//    !partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName.equals("へんしんメール") &&
    							//    !partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName.equals("あおぞらメール") &&
    							//    !partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName.equals("おんぷメール") &&
    							//    !partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName.equals("まぼろしメール") ) {
    								stealFlag = true;
    							//}
    						}
    					}
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 6A(黒い眼差し系)
    	case "6A":
    		trappingFlag = true;
    		break;

		// 6B(悪夢)
    	case "6B":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (!partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].badHealth.equals("sleep")) {
    			hitProbability = 0.0;
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
    				nightmareFlag = true;
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

		// 6C(火炎車)
		// 7D(聖なる炎)
    	case "6C":
    	case "7D":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("freeze")) {
    					recoveryFreezeMyselfFlag = true;
    				}
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].badHealth.equals("freeze")) {
    						badHealthOpponentTmp = "none";
    					} else if (!type1Opponent.equals("fire") &&
                                    !type2Opponent.equals("fire")) {
    						if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    							if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
    								badHealthOpponentTmp = "burn";
    							} else {
    								DialogUtil.dialogMessageOutput("神秘の守りで無効化");
    							}
    						}
    					} else {
    						DialogUtil.dialogMessageOutput("炎ポケモンは火傷にはならない");
    					}
    				}
    				if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth.equals("freeze")) {
    					badHealthMyselfTmp = "none";
    				}
    			}
    		}
    		break;

		// 6D(のろい)
    	case "6D":
    		if (!type1Myself.equals("ghost") &&
                 !type2Myself.equals("ghost")){
    			attackRankUpDownMyself = 1;
    			blockRankUpDownMyself = 1;
    			speedRankUpDownMyself = -1;
    		} else {
    			if (!pokeStateBatonOpponent.curcedFlag) {
        			if (pokeStateBatonOpponent.scapegoatHP == 0) {
        				HPUpDownMyself = (int) Math.floor(
            					partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP / 2.0) * (-1);
            				curceFlag = true;
        			} else {
        				DialogUtil.dialogMessageOutput("身代わりで無効化");
        			}
    			} else {
    				DialogUtil.dialogMessageOutput("呪いがかかっているポケモンへの呪いは成功しない");
    			}
    		}
    		break;

		// 6F(守る系)
    	// 74(堪える)
    	case "6F":
    	case "74":
    		if (true) {
    			Random random = new Random();
    			int protectRandom = random.nextInt(256);
    			int protectBoarder = 0;
    			if (pokeStatePlaceMyself.protectContinuousTrun == 1) {
    				protectBoarder = 255;
    			} else if (pokeStatePlaceMyself.protectContinuousTrun == 2) {
    				protectBoarder = 127;
    			} else if (pokeStatePlaceMyself.protectContinuousTrun == 3) {
    				protectBoarder = 63;
    			} else if (pokeStatePlaceMyself.protectContinuousTrun == 4) {
    				protectBoarder = 31;
    			} else if (pokeStatePlaceMyself.protectContinuousTrun == 5) {
    				protectBoarder = 15;
    			} else if (pokeStatePlaceMyself.protectContinuousTrun == 6) {
    				protectBoarder = 7;
    			} else if (pokeStatePlaceMyself.protectContinuousTrun == 7) {
    				protectBoarder = 3;
    			} else if (pokeStatePlaceMyself.protectContinuousTrun == 8) {
    				protectBoarder = 1;
    			} else {
    				protectBoarder = 0;
    			}
    			if (pokeStateBatonMyself.scapegoatHP > 0) {
    				protectBoarder = 0;
    				DialogUtil.dialogMessageOutput("身代わり中は必ず失敗");
    			}
    			if (protectRandom < protectBoarder) {
    				protectSuccessFlag = true;
    			} else {
    				DialogUtil.dialogMessageOutput(moveNameChoseMyself+"失敗");
    			}
    		}
    		break;

		// 70(撒きびし)
    	case "70":
    		spikesFlag = true;
    		break;

		// 71(見破る)
    	case "71":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                          pokeStatePlaceOpponent.protectSuccessFlag,
                          pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			seeingThroughFlag = true;
    		}
    		break;

		// 72(滅びの歌)
    	case "72":
    		int remainPokeCount = 0;
			for (int i = 0 ; i < 6 ; i++){
				if (partyInfoMyself.pokeStateUnitArray[i] != null) {
					if (partyInfoMyself.pokeStateUnitArray[i].actualHP > 0) {
						remainPokeCount++;
					}
				}
			}
			if (remainPokeCount > 1) {
				startRuinFlag = true;
			} else {
				DialogUtil.dialogMessageOutput("残り１体での滅びの歌は失敗");
			}
    		break;

		// 73(砂嵐)
		// 88(雨乞い)
		// 89(日本晴れ)
    	case "73":
    	case "88":
    	case "89":
    		changeWeatherFlag = true;
    		break;

		// 75(転がる)
    	case "75":
			comboCountDownFlag = true;
    		if (pokeStatePlaceMyself.isNotBeingCombo()) {
    	    	comboStartTurnSet = 5;
    		}
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			int rollRatio = pokeStatePlaceMyself.rollContinuousTrun - 1;
    			if (pokeStatePlaceMyself.roundShapeFlag) {
    				rollRatio = rollRatio + 1;
    			}
    			damage = damageMultipleCalc(damage,rollRatio);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
        			rollContinuousHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			} else {
    				comboCountDownFlag = false;
    				comboCountResetFlag = true;
    			}
    		} else {
    			comboCountDownFlag = false;
    			comboCountResetFlag = true;
    		}
    		break;

		// 76(威張る)
    	case "76":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (pokeStateBatonOpponent.attackRank == 12) {
    			hitProbability = 0.0;
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
				attackRankUpDownOpponent = 2;
    			if (pokeStateBatonOpponent.scapegoatHP == 0) {
    				if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
    					confusionFlag = true;
    				} else {
    					DialogUtil.dialogMessageOutput("神秘の守りで無効化");
    				}
    			} else {
    				DialogUtil.dialogMessageOutput("身代わりで無効化");
    			}
    		}
    		break;

		// 77(連続切り)
    	case "77":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageMultipleCalc(damage,pokeStatePlaceMyself.slashContinuousTrun - 1);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				slashContinuousHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 78(メロメロ)
    	case "78":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if ((partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sex.equals("male") &&
    			 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sex.equals("male")) ||
    			(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sex.equals("female") &&
    		     partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sex.equals("female")) ||
    			 partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sex.equals("none") ||
    			 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sex.equals("none")) {
    			hitProbability = 0.0;
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			moveHitFlag = true;
    			deprivingMindFlag = true;
    		}
    		break;

		// 7A(プレゼント（仕様がよくわからないがとりあえず対戦考察wikiの考察結果通りに実装）)
    	case "7A":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			// 威力240、威力360は出ないものとして実装
    			Random random = new Random();
    			int movePowerRandom = random.nextInt(10000);
    			if (pokeStateBatonOpponent.scapegoatHP > 0) {
    				movePowerRandom = 2000;
    			}
    			if (movePowerRandom >= 0 && movePowerRandom < 2000) {
    				movePowerChoseMyself = 0;
    			} else if (movePowerRandom >= 2000 && movePowerRandom < 6000) {
    				movePowerChoseMyself = 40;
    			} else if (movePowerRandom >= 6000 && movePowerRandom < 9000) {
    				movePowerChoseMyself = 80;
    			//} else if (movePowerRandom >= 9000 && movePowerRandom < 9960) {
    			} else {
    				movePowerChoseMyself = 120;
    			//} else if (movePowerRandom >= 9960 && movePowerRandom < 9980) {
    			//	movePowerChoseMyself = 240;
    			//} else {
    			//	movePowerChoseMyself = 360;
    			}
    			if (movePowerChoseMyself > 0) {
    				subjectValue = presentSubjectValueCalc_v_0_3(type1Opponent,
    						                                     type2Opponent,
    						                                     pokeStateBatonMyself.attackRank,
    						                                     partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
    						                                     partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
    				                                             partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
    			    	                                         pokeStatePlaceMyself.seeingThroughFlag,
    			    	                                         pokeStatePlaceOpponent.seenThroughFlag);
    				objectValue = presentObjectValueCalc_v_0_3(type1Myself,
                                                               type2Myself,
                                                               pokeStateBatonOpponent.blockRank,
                                                               partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                                               partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                               pokeStateWholeOpponent.blockHalfFlag,
                                                               pokeStateWholeOpponent.reflectorRestTurn);
    				levelValue = presentLevelValueCalc_v_0_3(type1Opponent,
                                                             type2Opponent);
    				damage = presentDamageBodyCalc_v_0_3(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    				criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
    	                                                                moveEffectCodeChoseMyself,
    	                                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
    	                                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
        			if(criticalHitJudge(criticalHitProbability)){
        				damage = criticalHitDamageCalc(damage,
                                                       moveTypeChoseMyself,
                                                       pokeStateBatonMyself.attackRank,
                                                       pokeStateBatonMyself.contactRank,
                                                       pokeStateBatonOpponent.blockRank,
                                                       pokeStateBatonOpponent.diffenceRank,
                                                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                       pokeStateWholeOpponent.reflectorRestTurn,
                                                       pokeStateWholeOpponent.lightWallRestTurn);
        			}
    				damage = typeReinforceItemCalc(damage,
    	                                           moveTypeChoseMyself,
    	                                           partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    				damage = constPlus2Calc(damage);
    				damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    				damage = typeMatchCalc(damage,
    	                                   moveTypeChoseMyself,
    	                                   type1Myself,
    	                                   type2Myself);
    				damage = typeCompatibilityCalc(damage,
    	                                           moveTypeChoseMyself,
    	                                           type1Opponent,
    	                                           type2Opponent,
    	                                           pokeStatePlaceMyself.seeingThroughFlag,
    	                                           pokeStatePlaceOpponent.seenThroughFlag);
    				damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    				if (damage > 0) {
    					moveHitFlag = true;
    					damage = damageRandomCalc(damage);
    					damage = presentDamageReRandomCalc(damage,objectValue);
    					if (pokeStateBatonOpponent.scapegoatHP > 0) {
    						scapegoatHPUpDownOpponent = damage * (-1);
    					} else {
    				    	spiritFlag = spiritTriggerCalc(damage,
 			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
 			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    						damage = finalDamageCalc(damage,
    	                                             partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
    	                                             pokeStatePlaceOpponent.protectSuccessFlag,
    	                                             pokeStatePlaceOpponent.latestUsedMoveEffectCode,
    	                                             moveEffectCodeChoseMyself,
    	                                             spiritFlag);
    						HPUpDownOpponent = damage * (-1);
    						LastHPUpDownOpponent = HPUpDownOpponent;
    						if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    							if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    								winceFlag = true;
    							}
    						}
    					}
    				}
    			} else {
        			validity = typeCompatibilityCalc(100,
                                                     moveTypeChoseMyself,
                                                     type1Opponent,
                                                     type2Opponent,
                                                     pokeStatePlaceMyself.seeingThroughFlag,
                                                     pokeStatePlaceOpponent.seenThroughFlag);
        			if (validity > 0) {
        				moveHitFlag = true;
						HPUpDownOpponent = 80;
						LastHPUpDownOpponent = 80;
						// ダメージが発生していないので、王者の印の追加効果判定は行われないと推測
        			}
    			}
    		}
    		break;

		// 7C(神秘の守り)
    	case "7C":
    		mysteryProtectFlag = true;
    		break;

		// 7E(マグニチュード)
    	case "7E":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);

    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
    				pokeStatePlaceOpponent.protectSuccessFlag,
    				pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			Random random = new Random();
    			int movePowerRandom = random.nextInt(20);
    			if (movePowerRandom == 0) {
    				movePowerChoseMyself = 10;
    				DialogUtil.dialogMessageOutput("マグニチュード４");
    			} else if (movePowerRandom >= 1 && movePowerRandom < 3) {
    				movePowerChoseMyself = 30;
    				DialogUtil.dialogMessageOutput("マグニチュード５");
    			} else if (movePowerRandom >= 3 && movePowerRandom < 7) {
    				movePowerChoseMyself = 50;
    				DialogUtil.dialogMessageOutput("マグニチュード６");
    			} else if (movePowerRandom >= 7 && movePowerRandom < 13) {
    				movePowerChoseMyself = 70;
    				DialogUtil.dialogMessageOutput("マグニチュード７");
    			} else if (movePowerRandom >= 13 && movePowerRandom < 17) {
    				movePowerChoseMyself = 90;
    				DialogUtil.dialogMessageOutput("マグニチュード８");
    			} else if (movePowerRandom >= 17 && movePowerRandom < 19) {
    				movePowerChoseMyself = 110;
    				DialogUtil.dialogMessageOutput("マグニチュード９");
    			} else {
    				movePowerChoseMyself = 150;
    				DialogUtil.dialogMessageOutput("マグニチュード１０");
    			}
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			if (pokeStatePlaceOpponent.intoUndergroundFlag) {
    				damage = damageMultipleCalc(damage,1);
    			}
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    						scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 7F(バトンタッチ)
    	case "7F":
			int pokeCountMyself = 0;
	    	for(int i = 0; i < 6; i++){
	    		if (partyInfoMyself.pokeStateUnitArray[i] != null) {
	    			if (partyInfoMyself.pokeStateUnitArray[i].actualHP > 0) {
	    				if (i != partyInfoMyself.pokeStateUnitArrayIndex) {
	    					pokeCountMyself++;
	    				}
	    			}
	    		}
	    	}
	    	if (pokeCountMyself > 0) {
	    		changeMyselfFlag = true;
	    	} else {
	    		batonFailFlag = true;
	    		DialogUtil.dialogMessageOutput("交代できるポケモンがいない");
	    	}
    		break;

		// 80(追い打ち)
    	case "80":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
    		             pokeStatePlaceOpponent.protectSuccessFlag,
    		             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			if (pokeStateWholeOpponent.waitingChangeFlag) {
    				damage = damageMultipleCalc(damage,1);
    			}
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 81(高速スピン)
    	case "81":
    		spinFlag = true; // PBSではダメージ0でも高速スピンの効果発動
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

    	// 84(朝の日差し)
    	// 85(光合成)
    	// 86(月の光)
        case "84":
        case "85":
        case "86":
        	if (placeState.weather.equals("rainy") || placeState.weather.equals("dusty")) {
        		HPUpDownMyself = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP,
        		                           1,
        		                           4);
        	} else if (placeState.weather.equals("sunny")) {
        		HPUpDownMyself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP;
            } else {
        		HPUpDownMyself = ratioCalc(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP,
                                           1,
                                           2);
            }
        	break;

		// 87(めざめるパワー)
    	case "87":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			movePowerChoseMyself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].hiddenPowerPower;
    			moveTypeChoseMyself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].hiddenPowerType;
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 8A(追加効果で自分の防御ランク+1)
    	case "8A":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						blockRankUpDownMyself = 1;
    					}
    				}
    			}
    		}
    		break;

		// 8B(追加効果で自分の攻撃ランク+1)
    	case "8B":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						attackRankUpDownMyself = 1;
    					}
    				}
    			}
    		}
    		break;

		// 8C(原始の力)
    	case "8C":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
    			    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						attackRankUpDownMyself = 1;
    						blockRankUpDownMyself = 1;
    						contactRankUpDownMyself = 1;
    						diffenceRankUpDownMyself = 1;
    						speedRankUpDownMyself = 1;
    					}
    				}
    			}
    		}
    		break;

		// 8E(腹太鼓)
    	case "8E":
    		moveExecution_8E();
    		break;

		// 8F(自己暗示)
    	case "8F":
    		if (pokeStateBatonOpponent.attackRank == 6 &&
                pokeStateBatonOpponent.blockRank == 6 &&
                pokeStateBatonOpponent.contactRank == 6 &&
                pokeStateBatonOpponent.diffenceRank == 6 &&
                pokeStateBatonOpponent.speedRank == 6 &&
                pokeStateBatonOpponent.hitProbabilityRank == 6 &&
                pokeStateBatonOpponent.avoidProbabilityRank == 6) {
    			DialogUtil.dialogMessageOutput("ランク変動の無い相手に自己暗示は成功しない");
    		} else {
    		    SelfSuggestionFlag = true;
    		}
    		break;

		// 90(ミラーコート)
    	case "90":
    		hitProbability = 100.0;
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			damage = pokeStatePlaceMyself.thisTrunGetSpecialDamage * 2;
    			validity = typeCompatibilityCalc(100,
                                                 moveTypeChoseMyself,
                                                 type1Opponent,
                                                 type2Opponent,
                                                 pokeStatePlaceMyself.seeingThroughFlag,
                                                 pokeStatePlaceOpponent.seenThroughFlag);
    			if (validity == 0) {
    				damage = 0;
    			}
    			if (damage > 0) {
    				moveHitFlag = true;
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 91(ロケットずつき)
    	case "91":
			comboCountDownFlag = true;
    		if (pokeStatePlaceMyself.isNotBeingCombo()) {
    	    	comboStartTurnSet = 2;
    	    	blockRankUpDownMyself = 1;
    		} else {
        		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                    pokeStateBatonMyself.hitProbabilityRank,
                                                    pokeStateBatonOpponent.avoidProbabilityRank,
                                                    partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                    pokeStatePlaceMyself.seeingThroughFlag,
                                                    pokeStatePlaceOpponent.seenThroughFlag,
        		                                    pokeStatePlaceOpponent.lockonFlag);
        		if (!pokeStatePlaceOpponent.lockonFlag) {
            		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                		pokeStatePlaceOpponent.intoSkyFlag) {
                		hitProbability = 0.0;
                	}
        		}
        		if (hitJudge(hitProbability,
                             pokeStatePlaceOpponent.protectSuccessFlag,
                             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
        			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                    attackMyself,
                                                    contactMyself,
                                                    pokeStateBatonMyself.attackRank,
                                                    pokeStateBatonMyself.contactRank,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
        			objectValue = objectValueCalc(moveTypeChoseMyself,
                                                  blockOpponent,
                                                  diffenceOpponent,
                                                  pokeStateBatonOpponent.blockRank,
                                                  pokeStateBatonOpponent.diffenceRank,
                                                  partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                                  partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                  pokeStateWholeOpponent.blockHalfFlag,
                                                  pokeStateWholeOpponent.reflectorRestTurn,
                                                  pokeStateWholeOpponent.lightWallRestTurn);
        			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
        			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
        			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                        moveEffectCodeChoseMyself,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
        			if(criticalHitJudge(criticalHitProbability)){
        				damage = criticalHitDamageCalc(damage,
                                                       moveTypeChoseMyself,
                                                       pokeStateBatonMyself.attackRank,
                                                       pokeStateBatonMyself.contactRank,
                                                       pokeStateBatonOpponent.blockRank,
                                                       pokeStateBatonOpponent.diffenceRank,
                                                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                       pokeStateWholeOpponent.reflectorRestTurn,
                                                       pokeStateWholeOpponent.lightWallRestTurn);
        			}
        			damage = typeReinforceItemCalc(damage,
                                                   moveTypeChoseMyself,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
        			damage = constPlus2Calc(damage);
        			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
        			damage = typeMatchCalc(damage,
                                           moveTypeChoseMyself,
                                           type1Myself,
                                           type2Myself);
        			damage = typeCompatibilityCalc(damage,
                                                   moveTypeChoseMyself,
                                                   type1Opponent,
                                                   type2Opponent,
                                                   pokeStatePlaceMyself.seeingThroughFlag,
                                                   pokeStatePlaceOpponent.seenThroughFlag);
        			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
        			if (damage > 0) {
            			moveHitFlag = true;
        				damage = damageRandomCalc(damage);
        				if (pokeStateBatonOpponent.scapegoatHP > 0) {
        					scapegoatHPUpDownOpponent = damage * (-1);
        				} else {
    				    	spiritFlag = spiritTriggerCalc(damage,
 			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
 			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
        					damage = finalDamageCalc(damage,
                                                     partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                     pokeStatePlaceOpponent.protectSuccessFlag,
                                                     pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                     moveEffectCodeChoseMyself,
                                                     spiritFlag);
        					HPUpDownOpponent = damage * (-1);
        					LastHPUpDownOpponent = HPUpDownOpponent;
        					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
        						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
        							winceFlag = true;
        						}
        					}
        				}
        			}
        		}
    		}
    		break;

		// 92(竜巻)
    	case "92":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			if (pokeStatePlaceOpponent.intoSkyFlag) {
    				damage = damageMultipleCalc(damage,1);
    			}
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						winceFlag = true;
    					}
    				}
    			}
    		}
    		break;

		// 93(地震)
    	case "93":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			if (pokeStatePlaceOpponent.intoUndergroundFlag) {
    				damage = damageMultipleCalc(damage,1);
    			}
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 94(未来予知)
    	case "94":
    		damage = 0;
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                    pokeStatePlaceOpponent.protectSuccessFlag,
                    pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			damage = constPlus2Calc(damage);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			damage = damageRandomCalc(damage);
    		}
    		predictionDamage = damage;
    		break;

		// 95(風起こし)
    	case "95":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			if (pokeStatePlaceOpponent.intoSkyFlag) {
    				damage = damageMultipleCalc(damage,1);
    			}
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
    		break;

		// 96(踏みつけ)
    	case "96":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			if (pokeStatePlaceOpponent.smallShapeFlag) {
    				damage = damageMultipleCalc(damage,1);
    			}
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						winceFlag = true;
    					}
    				}
    			}
    		}
    		break;

		// 97(ソーラービーム)
    	case "97":
			comboCountDownFlag = true;
    		if (pokeStatePlaceMyself.isNotBeingCombo() &&
    			!placeState.weather.equals("sunny")) {
    	    	comboStartTurnSet = 2;
    		} else {
        		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                    pokeStateBatonMyself.hitProbabilityRank,
                                                    pokeStateBatonOpponent.avoidProbabilityRank,
                                                    partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                    pokeStatePlaceMyself.seeingThroughFlag,
                                                    pokeStatePlaceOpponent.seenThroughFlag,
        		                                    pokeStatePlaceOpponent.lockonFlag);
        		if (!pokeStatePlaceOpponent.lockonFlag) {
            		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                		pokeStatePlaceOpponent.intoSkyFlag) {
                		hitProbability = 0.0;
                	}
        		}
        		if (hitJudge(hitProbability,
                             pokeStatePlaceOpponent.protectSuccessFlag,
                             pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
        			if (placeState.weather.equals("rainy")) {
        				movePowerChoseMyself = movePowerChoseMyself / 2;
        			}
        			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                    attackMyself,
                                                    contactMyself,
                                                    pokeStateBatonMyself.attackRank,
                                                    pokeStateBatonMyself.contactRank,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
        			objectValue = objectValueCalc(moveTypeChoseMyself,
                                                  blockOpponent,
                                                  diffenceOpponent,
                                                  pokeStateBatonOpponent.blockRank,
                                                  pokeStateBatonOpponent.diffenceRank,
                                                  partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                                  partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                  pokeStateWholeOpponent.blockHalfFlag,
                                                  pokeStateWholeOpponent.reflectorRestTurn,
                                                  pokeStateWholeOpponent.lightWallRestTurn);
        			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
        			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
        			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                        moveEffectCodeChoseMyself,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
        			if(criticalHitJudge(criticalHitProbability)){
        				damage = criticalHitDamageCalc(damage,
                                                       moveTypeChoseMyself,
                                                       pokeStateBatonMyself.attackRank,
                                                       pokeStateBatonMyself.contactRank,
                                                       pokeStateBatonOpponent.blockRank,
                                                       pokeStateBatonOpponent.diffenceRank,
                                                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                       pokeStateWholeOpponent.reflectorRestTurn,
                                                       pokeStateWholeOpponent.lightWallRestTurn);
        			}
        			damage = typeReinforceItemCalc(damage,
                                                   moveTypeChoseMyself,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
        			damage = constPlus2Calc(damage);
        			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
        			damage = typeMatchCalc(damage,
                                           moveTypeChoseMyself,
                                           type1Myself,
                                           type2Myself);
        			damage = typeCompatibilityCalc(damage,
                                                   moveTypeChoseMyself,
                                                   type1Opponent,
                                                   type2Opponent,
                                                   pokeStatePlaceMyself.seeingThroughFlag,
                                                   pokeStatePlaceOpponent.seenThroughFlag);
        			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
        			if (damage > 0) {
            			moveHitFlag = true;
        				damage = damageRandomCalc(damage);
        				if (pokeStateBatonOpponent.scapegoatHP > 0) {
        					scapegoatHPUpDownOpponent = damage * (-1);
        				} else {
    				    	spiritFlag = spiritTriggerCalc(damage,
 			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
 			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
        					damage = finalDamageCalc(damage,
                                                     partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                     pokeStatePlaceOpponent.protectSuccessFlag,
                                                     pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                     moveEffectCodeChoseMyself,
                                                     spiritFlag);
        					HPUpDownOpponent = damage * (-1);
        					LastHPUpDownOpponent = HPUpDownOpponent;
        					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
        						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
        							winceFlag = true;
        						}
        					}
        				}
        			}
        		}
    		}
    		break;

		// 98(雷)
    	case "98":
    		if (placeState.weather.equals("sunny")) {
    			moveHitProbabilityChoseMyself = 50.0;
    		}
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (placeState.weather.equals("rainy")) {
    			moveHitProbabilityChoseMyself = 100.0;
    		}
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
    				moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (additionalEffectJudge(additionalEffectProbabilityChoseMyself,moveNameChoseMyself)) {
    						if (pokeStateWholeOpponent.mysteryProtectRestTurn == 0) {
    							badHealthOpponentTmp = "paralysis";
    						} else {
    							DialogUtil.dialogMessageOutput("神秘の守りで無効化");
    						}
    					}
    				}
    			}
    		}
    		break;

		// 9A(袋叩き)
    	case "9A":
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
                                                pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
    			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
                    pokeStatePlaceOpponent.intoSkyFlag) {
    				hitProbability = 0.0;
    			}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			ArrayList<Integer> pokeArray = new ArrayList<Integer>();
    			for (int i = 0 ; i < 6 ; i++){
    				if (partyInfoMyself.pokeStateUnitArray[i] != null) {
    					if (partyInfoMyself.pokeStateUnitArray[i].actualHP > 0 &&
    					    (partyInfoMyself.pokeStateUnitArray[i].badHealth.equals("none") ||
    					     i == partyInfoMyself.pokeStateUnitArrayIndex)) {
    						pokeArray.add(i);
    					}
    				}
    			}
    		    for(int i = 0; i < pokeArray.size(); i++){
    		    	DialogUtil.dialogMessageOutput(partyInfoMyself.pokeStateUnitArray[pokeArray.get(i)].speciesNameDisp+"の攻撃");
    		    	subjectValue = partyInfoMyself.pokeStateUnitArray[pokeArray.get(i)].attackSpeciesValue;
    		    	objectValue = partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].blockSpeciesValue;
    		    	levelValue = partyInfoMyself.pokeStateUnitArray[pokeArray.get(i)].level;
    		    	damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    		    	criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                        moveEffectCodeChoseMyself,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    		    	if(criticalHitJudge(criticalHitProbability)){
    		    		// プレゼントと同じように急所で各種貫通が発生すると仮定
        				damage = criticalHitDamageCalc(damage,
                                                       moveTypeChoseMyself,
                                                       pokeStateBatonMyself.attackRank,
                                                       pokeStateBatonMyself.contactRank,
                                                       pokeStateBatonOpponent.blockRank,
                                                       pokeStateBatonOpponent.diffenceRank,
                                                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                       pokeStateWholeOpponent.reflectorRestTurn,
                                                       pokeStateWholeOpponent.lightWallRestTurn);
    		    	}
    		    	damage = constPlus2Calc(damage);
    		    	damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    		    	damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    		    	if (damage > 0) {
    		    		moveHitFlag = true;
    		    		damage = damageRandomCalc(damage);
    		    		if (pokeStateBatonOpponent.scapegoatHP > (scapegoatHPUpDownOpponent * (-1))) {
    		    			scapegoatHPUpDownOpponent = scapegoatHPUpDownOpponent + (damage * (-1));
    		    		} else {
    						if (!spiritFlag) {
    							spiritFlag = spiritTriggerCalc(damage,
 			                                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP + HPUpDownOpponent,
 			                                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    						}
    		    			damage = finalDamageCalc(damage,
    		    		                             partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP + HPUpDownOpponent,
    		    		                             pokeStatePlaceOpponent.protectSuccessFlag,
    		    		                             pokeStatePlaceOpponent.latestUsedMoveEffectCode,
    		    		                             moveEffectCodeChoseMyself,
    		    		                             spiritFlag);
    		    			HPUpDownOpponent = HPUpDownOpponent + (damage * (-1));
    		    			if (i == (pokeArray.size() - 1)) {
    		    				LastHPUpDownOpponent = damage;
    		    				if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    		    					if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    		    						winceFlag = true;
    		    					}
    		    				}
    		    			}
    		    		}
    		    	}
    		    }
    		}
    		break;

		// 9Ba(空を飛ぶ)
    	case "9Ba":
    		moveExecution_9Ba();
    		break;

		// 9Bb(穴を掘る)
    	case "9Bb":
    		moveExecution_9Bb();
    		break;

		// 9C(丸くなる)
    	case "9C":
    		blockRankUpDownMyself = 1;
    		roundShapeFlag = true;
    		break;

        // XXa(交代)
        case "XXa":
        	changeMyselfFlag = true;
        	break;

		// XXb(混乱自滅)
    	case "XXb":
    		if (true) {
    			subjectValue = confuseSubjectValueCalc(attackMyself,
    					                               pokeStateBatonMyself.attackRank,
    					                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = confuseSubjectValueCalc(blockMyself,
    				                              	  pokeStateBatonMyself.blockRank,
    				                              	  pokeStateWholeMyself.blockHalfFlag,
    				                              	  pokeStateWholeMyself.reflectorRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			// damage = typeReinforceItemCalc(damage,
                //                                moveTypeChoseMyselfBeforeSelfDestruction,
                //                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = damageRandomCalc(damage);
		    	spiritFlag = spiritTriggerCalc(damage,
	                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
	                       partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
				damage = finalDamageCalc(damage,
						                 partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP,
						                 false,
						                 "XXb",
						                 "XXb",
						                 spiritFlag);
				HPUpDownMyself = damage * (-1);
    		}
    		break;

    	}
    }

	/* 命中で本体に与えたダメージの1/4の反動技用の処理（メソッドのサイズが65535バイトを超えてエラーになるのを抑止、順次切り出し）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void moveExecution_30a() throws IOException {

    	// 汎用の中間結果
    	int damage = 0; // ダメージ
    	double hitProbability = 100.0; // 命中率
    	double criticalHitProbability = 6.6; //急所率
    	double subjectValue = 1.0; // 攻撃力
    	double objectValue = 1.0; //防御力
    	double levelValue = 50.0; //レベル

		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                pokeStateBatonMyself.hitProbabilityRank,
                pokeStateBatonOpponent.avoidProbabilityRank,
                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                pokeStatePlaceMyself.seeingThroughFlag,
                pokeStatePlaceOpponent.seenThroughFlag,
                pokeStatePlaceOpponent.lockonFlag);
		if (!pokeStatePlaceOpponent.lockonFlag) {
			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
					pokeStatePlaceOpponent.intoSkyFlag) {
				hitProbability = 0.0;
			}
		}
		if (hitJudge(hitProbability,
				pokeStatePlaceOpponent.protectSuccessFlag,
				pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                attackMyself,
                contactMyself,
                pokeStateBatonMyself.attackRank,
                pokeStateBatonMyself.contactRank,
                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
			objectValue = objectValueCalc(moveTypeChoseMyself,
              blockOpponent,
              diffenceOpponent,
              pokeStateBatonOpponent.blockRank,
              pokeStateBatonOpponent.diffenceRank,
              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
              pokeStateWholeOpponent.blockHalfFlag,
              pokeStateWholeOpponent.reflectorRestTurn,
              pokeStateWholeOpponent.lightWallRestTurn);
			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                    moveEffectCodeChoseMyself,
                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
			if(criticalHitJudge(criticalHitProbability)){
				damage = criticalHitDamageCalc(damage,
                   moveTypeChoseMyself,
                   pokeStateBatonMyself.attackRank,
                   pokeStateBatonMyself.contactRank,
                   pokeStateBatonOpponent.blockRank,
                   pokeStateBatonOpponent.diffenceRank,
                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                   pokeStateWholeOpponent.reflectorRestTurn,
                   pokeStateWholeOpponent.lightWallRestTurn);
			}
			damage = typeReinforceItemCalc(damage,
               moveTypeChoseMyself,
               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
			damage = constPlus2Calc(damage);
			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
			damage = typeMatchCalc(damage,
					moveTypeChoseMyself,
					type1Myself,
					type2Myself);
			damage = typeCompatibilityCalc(damage,
               moveTypeChoseMyself,
               type1Opponent,
               type2Opponent,
               pokeStatePlaceMyself.seeingThroughFlag,
               pokeStatePlaceOpponent.seenThroughFlag);
			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
			if (damage > 0) {
				moveHitFlag = true;
				damage = damageRandomCalc(damage);
				if (pokeStateBatonOpponent.scapegoatHP > 0) {
					scapegoatHPUpDownOpponent = damage * (-1);
				} else {
					spiritFlag = spiritTriggerCalc(damage,
							partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
							partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
					damage = finalDamageCalc(damage,
							partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
							pokeStatePlaceOpponent.protectSuccessFlag,
							pokeStatePlaceOpponent.latestUsedMoveEffectCode,
							moveEffectCodeChoseMyself,
							spiritFlag);
					HPUpDownOpponent = damage * (-1);
					LastHPUpDownOpponent = HPUpDownOpponent;
					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
							winceFlag = true;
						}
					}
					// 本当は標準メソッドのratioCalcを使うように直したいが、稼働実績があるのでこのまま
					if (partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP > damage) {
						HPUpDownMyself = ((int) Math.floor(damage * (1.0/4.0))) * (-1);
					} else {
						HPUpDownMyself = ((int) Math.floor(partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP * (1.0/4.0))) * (-1);
						// 決着がついた後の反動発生無しの設定が入っている場合、反動ダメージをキャンセル
						if (!Precondition.recoilWhenEndTurnFlag) {
			    			int pokeCountOpponent = 0;
			    	    	for(int i = 0; i < 6; i++){
			    	    		if (partyInfoOpponent.pokeStateUnitArray[i] != null) {
			    	    			if (partyInfoOpponent.pokeStateUnitArray[i].actualHP > 0) {
			    	    				pokeCountOpponent++;
			    	    			}
			    	    		}
			    	    	}
			    	    	if (pokeCountOpponent == 1) {
			    	    		HPUpDownMyself = 0;
			    	    	}
						}
					}
				}
			}
		}
    }

	/* 悪あがき用の処理（メソッドのサイズが65535バイトを超えてエラーになるのを抑止、順次切り出し）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void moveExecution_30b() throws IOException {

    	// 汎用の中間結果
    	int damage = 0; // ダメージ
    	double hitProbability = 100.0; // 命中率
    	double criticalHitProbability = 6.6; //急所率
    	double subjectValue = 1.0; // 攻撃力
    	double objectValue = 1.0; //防御力
    	double levelValue = 50.0; //レベル

		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                pokeStateBatonMyself.hitProbabilityRank,
                pokeStateBatonOpponent.avoidProbabilityRank,
                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                pokeStatePlaceMyself.seeingThroughFlag,
                pokeStatePlaceOpponent.seenThroughFlag,
                pokeStatePlaceOpponent.lockonFlag);
		if (!pokeStatePlaceOpponent.lockonFlag) {
			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
					pokeStatePlaceOpponent.intoSkyFlag) {
				hitProbability = 0.0;
			}
		}
		if (hitJudge(hitProbability,
				pokeStatePlaceOpponent.protectSuccessFlag,
				pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
			// struggleFlag = true;
			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                attackMyself,
                contactMyself,
                pokeStateBatonMyself.attackRank,
                pokeStateBatonMyself.contactRank,
                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
			objectValue = objectValueCalc(moveTypeChoseMyself,
              blockOpponent,
              diffenceOpponent,
              pokeStateBatonOpponent.blockRank,
              pokeStateBatonOpponent.diffenceRank,
              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
              pokeStateWholeOpponent.blockHalfFlag,
              pokeStateWholeOpponent.reflectorRestTurn,
              pokeStateWholeOpponent.lightWallRestTurn);
			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                    moveEffectCodeChoseMyself,
                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
			if(criticalHitJudge(criticalHitProbability)){
				damage = criticalHitDamageCalc(damage,
                   moveTypeChoseMyself,
                   pokeStateBatonMyself.attackRank,
                   pokeStateBatonMyself.contactRank,
                   pokeStateBatonOpponent.blockRank,
                   pokeStateBatonOpponent.diffenceRank,
                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                   pokeStateWholeOpponent.reflectorRestTurn,
                   pokeStateWholeOpponent.lightWallRestTurn);
			}
			damage = typeReinforceItemCalc(damage,
               moveTypeChoseMyself,
               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
			damage = constPlus2Calc(damage);
			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
			damage = typeMatchCalc(damage,
					moveTypeChoseMyself,
					type1Myself,
					type2Myself);
			damage = typeCompatibilityCalc(damage,
               moveTypeChoseMyself,
               type1Opponent,
               type2Opponent,
               pokeStatePlaceMyself.seeingThroughFlag,
               pokeStatePlaceOpponent.seenThroughFlag);
			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
			if (damage > 0) {
				moveHitFlag = true;
				damage = damageRandomCalc(damage);
				if (pokeStateBatonOpponent.scapegoatHP > 0) {
					scapegoatHPUpDownOpponent = damage * (-1);
				} else {
					spiritFlag = spiritTriggerCalc(damage,
							partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
							partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
					damage = finalDamageCalc(damage,
							partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
							pokeStatePlaceOpponent.protectSuccessFlag,
							pokeStatePlaceOpponent.latestUsedMoveEffectCode,
							moveEffectCodeChoseMyself,
							spiritFlag);
					HPUpDownOpponent = damage * (-1);
					LastHPUpDownOpponent = HPUpDownOpponent;
					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
							winceFlag = true;
						}
					}
					// 相手のポケモンのタイプがゴースト以外なら反動発生（ゴーストに対してはPBSバグにより反動発生しない）
					// 本当は標準メソッドのratioCalcを使うように直したいが、稼働実績があるのでこのまま
					if (!type1Opponent.equals("ghost") && !type2Opponent.equals("ghost")) {
						if (partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP > damage) {
							HPUpDownMyself = ((int) Math.floor(damage * (1.0/4.0))) * (-1);
						} else {
							HPUpDownMyself = ((int) Math.floor(partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP * (1.0/4.0))) * (-1);
							// 決着がついた後の反動発生無しの設定が入っている場合、反動ダメージをキャンセル
							if (!Precondition.recoilWhenEndTurnFlag) {
				    			int pokeCountOpponent = 0;
				    	    	for(int i = 0; i < 6; i++){
				    	    		if (partyInfoOpponent.pokeStateUnitArray[i] != null) {
				    	    			if (partyInfoOpponent.pokeStateUnitArray[i].actualHP > 0) {
				    	    				pokeCountOpponent++;
				    	    			}
				    	    		}
				    	    	}
				    	    	if (pokeCountOpponent == 1) {
				    	    		HPUpDownMyself = 0;
				    	    	}
							}
						}
					}
				}
			}
		}
    }

	/* スケッチ用の処理（メソッドのサイズが65535バイトを超えてエラーになるのを抑止、順次切り出し）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void moveExecution_5F() throws IOException {

    	// 汎用の中間結果
    	double hitProbability = 100.0; // 命中率

		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                pokeStateBatonMyself.hitProbabilityRank,
                pokeStateBatonOpponent.avoidProbabilityRank,
                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                pokeStatePlaceMyself.seeingThroughFlag,
                pokeStatePlaceOpponent.seenThroughFlag,
                pokeStatePlaceOpponent.lockonFlag);
		if (!pokeStatePlaceOpponent.lockonFlag) {
			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
					pokeStatePlaceOpponent.intoSkyFlag) {
				hitProbability = 0.0;
			}
		}
		if (hitJudge(hitProbability,
				pokeStatePlaceOpponent.protectSuccessFlag,
				pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
			moveHitFlag = true;
			if (pokeStatePlaceOpponent.latestUsedMoveIndex >= 0 && pokeStatePlaceOpponent.latestUsedMoveIndex <= 3) {
				boolean duplicateFlag = false;
				for (int i = 0; i < 4; i++) {
					if (pokeStatePlaceOpponent.latestUsedMoveName.equals(moveNameMyself[i])) {
						duplicateFlag = true;
					}
				}
				if (!duplicateFlag) {
					// 52(物真似)、30b(悪あがき)、53(指を振る)、5F(スケッチ)、61(ねごと)、09(オウムがえし)、39(へんしん)、07(爆破)に対しては失敗
					if (!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("52")&&
							!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("30b")&&
							!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("53")&&
							!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("5F")&&
							!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("61")&&
							!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("09")&&
							!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("39")&&
							!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("07")) {
						sketchFlag = true;
					} else {
						DialogUtil.dialogMessageOutput("スケッチできなかった");
					}
				} else {
					DialogUtil.dialogMessageOutput("スケッチできなかった");
				}
			} else {
				DialogUtil.dialogMessageOutput("スケッチできなかった");
			}
		}
    }

	/* うらみ用の処理（メソッドのサイズが65535バイトを超えてエラーになるのを抑止、順次切り出し）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void moveExecution_64() throws IOException {

    	// 汎用の中間結果
    	double hitProbability = 100.0; // 命中率

		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                pokeStateBatonMyself.hitProbabilityRank,
                pokeStateBatonOpponent.avoidProbabilityRank,
                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                pokeStatePlaceMyself.seeingThroughFlag,
                pokeStatePlaceOpponent.seenThroughFlag,
                pokeStatePlaceOpponent.lockonFlag);
		if (!pokeStatePlaceOpponent.lockonFlag) {
			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
					pokeStatePlaceOpponent.intoSkyFlag) {
				hitProbability = 0.0;
			}
		}
		if (hitJudge(hitProbability,
				pokeStatePlaceOpponent.protectSuccessFlag,
				pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
			moveHitFlag = true;
			if (pokeStatePlaceOpponent.latestUsedMoveIndex >= 0 && pokeStatePlaceOpponent.latestUsedMoveIndex <= 3) {
				// 30b(悪あがき)、61(ねごと)に対しては失敗
				// 61(ねごと)はPBS独自挙動
				if (!pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("30b")&&
				    !pokeStatePlaceOpponent.latestUsedMoveEffectCode.equals("61")) {
					Random random = new Random();
					int PPRandom = random.nextInt(4) + 2;
					PPDownOpponent = PPRandom * (-1);
				} else {
					DialogUtil.dialogMessageOutput("相手が繰り出した技は恨みが成功する技ではない");
				}
			} else {
				DialogUtil.dialogMessageOutput("恨む技がなかった");
			}
		}
    }

	/* みねうち用の処理（メソッドのサイズが65535バイトを超えてエラーになるのを抑止、順次切り出し）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void moveExecution_65() throws IOException {

    	// 汎用の中間結果
    	int damage = 0; // ダメージ
    	double hitProbability = 100.0; // 命中率
    	double criticalHitProbability = 6.6; //急所率
    	double subjectValue = 1.0; // 攻撃力
    	double objectValue = 1.0; //防御力
    	double levelValue = 50.0; //レベル

		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                pokeStateBatonMyself.hitProbabilityRank,
                pokeStateBatonOpponent.avoidProbabilityRank,
                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                pokeStatePlaceMyself.seeingThroughFlag,
                pokeStatePlaceOpponent.seenThroughFlag,
                pokeStatePlaceOpponent.lockonFlag);
		if (!pokeStatePlaceOpponent.lockonFlag) {
			if (pokeStatePlaceOpponent.intoUndergroundFlag ||
					pokeStatePlaceOpponent.intoSkyFlag) {
				hitProbability = 0.0;
			}
		}
		if (hitJudge(hitProbability,
				pokeStatePlaceOpponent.protectSuccessFlag,
				pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
			subjectValue = subjectValueCalc(moveTypeChoseMyself,
				attackMyself,
                contactMyself,
                pokeStateBatonMyself.attackRank,
                pokeStateBatonMyself.contactRank,
                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
			objectValue = objectValueCalc(moveTypeChoseMyself,
              blockOpponent,
              diffenceOpponent,
              pokeStateBatonOpponent.blockRank,
              pokeStateBatonOpponent.diffenceRank,
              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
              pokeStateWholeOpponent.blockHalfFlag,
              pokeStateWholeOpponent.reflectorRestTurn,
              pokeStateWholeOpponent.lightWallRestTurn);
			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                    moveEffectCodeChoseMyself,
			                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
			                        partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
			if(criticalHitJudge(criticalHitProbability)){
				damage = criticalHitDamageCalc(damage,
                   moveTypeChoseMyself,
                   pokeStateBatonMyself.attackRank,
                   pokeStateBatonMyself.contactRank,
                   pokeStateBatonOpponent.blockRank,
                   pokeStateBatonOpponent.diffenceRank,
                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                   pokeStateWholeOpponent.reflectorRestTurn,
                   pokeStateWholeOpponent.lightWallRestTurn);
			}
			damage = typeReinforceItemCalc(damage,
               moveTypeChoseMyself,
               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
			damage = constPlus2Calc(damage);
			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
			damage = typeMatchCalc(damage,
                                   moveTypeChoseMyself,
                                   type1Myself,
                                   type2Myself);
			damage = typeCompatibilityCalc(damage,
               moveTypeChoseMyself,
               type1Opponent,
               type2Opponent,
               pokeStatePlaceMyself.seeingThroughFlag,
               pokeStatePlaceOpponent.seenThroughFlag);
			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
			if (damage > 0) {
				moveHitFlag = true;
				damage = damageRandomCalc(damage);
				if (pokeStateBatonOpponent.scapegoatHP > 0) {
					scapegoatHPUpDownOpponent = damage * (-1);
				} else {
					spiritFlag = spiritTriggerCalc(damage,
                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
					if (damage > partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP) {
						mineuchiFlag = true;
					}
					damage = finalDamageCalc(damage,
                                             partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                             pokeStatePlaceOpponent.protectSuccessFlag,
                                             pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                             moveEffectCodeChoseMyself,
                                             spiritFlag);
					HPUpDownOpponent = damage * (-1);
					LastHPUpDownOpponent = HPUpDownOpponent;
					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
							winceFlag = true;
						}
					}
				}
			}
		}
    }

	/* 腹太鼓用の処理（メソッドのサイズが65535バイトを超えてエラーになるのを抑止、順次切り出し）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void moveExecution_8E() throws IOException {
		int HPHalf = (int) Math.floor(partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].MAXHP / 2.0);
		if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP > HPHalf &&
			pokeStateBatonMyself.attackRank < 12) {
			HPUpDownMyself = HPHalf * (-1);
			attackRankUpDownMyself = 12;
		} else if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP == HPHalf && // PBS仕様
    			    pokeStateBatonMyself.attackRank < 12) {
			HPUpDownMyself = (HPHalf * (-1)) + 1;
			attackRankUpDownMyself = 12;
		} else {
			DialogUtil.dialogMessageOutput("腹太鼓に失敗した");
		}
    }

	/* 空を飛ぶ用の処理（メソッドのサイズが65535バイトを超えてエラーになるのを抑止、順次切り出し）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void moveExecution_9Ba() throws IOException {

    	// 汎用の中間結果
    	int damage = 0; // ダメージ
    	double hitProbability = 100.0; // 命中率
    	double criticalHitProbability = 6.6; //急所率
    	double subjectValue = 1.0; // 攻撃力
    	double objectValue = 1.0; //防御力
    	double levelValue = 50.0; //レベル

		comboCountDownFlag = true;
		if (pokeStatePlaceMyself.isNotBeingCombo()) {
	    	comboStartTurnSet = 2;
	    	goIntoSkyFlag = true;
		} else {
			returnOnGroundFlag = true;
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
		}
    }

	/* 穴を掘る用の処理（メソッドのサイズが65535バイトを超えてエラーになるのを抑止、順次切り出し）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void moveExecution_9Bb() throws IOException {

    	// 汎用の中間結果
    	int damage = 0; // ダメージ
    	double hitProbability = 100.0; // 命中率
    	double criticalHitProbability = 6.6; //急所率
    	double subjectValue = 1.0; // 攻撃力
    	double objectValue = 1.0; //防御力
    	double levelValue = 50.0; //レベル

		comboCountDownFlag = true;
		if (pokeStatePlaceMyself.isNotBeingCombo()) {
	    	comboStartTurnSet = 2;
	    	goIntoUndergroundFlag = true;
		} else {
			returnOnGroundFlag = true;
    		hitProbability = hitProbabilityCalc(moveHitProbabilityChoseMyself,
                                                pokeStateBatonMyself.hitProbabilityRank,
                                                pokeStateBatonOpponent.avoidProbabilityRank,
                                                partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                                pokeStatePlaceMyself.seeingThroughFlag,
                                                pokeStatePlaceOpponent.seenThroughFlag,
    		                                    pokeStatePlaceOpponent.lockonFlag);
    		if (!pokeStatePlaceOpponent.lockonFlag) {
        		if (pokeStatePlaceOpponent.intoUndergroundFlag ||
            		pokeStatePlaceOpponent.intoSkyFlag) {
            		hitProbability = 0.0;
            	}
    		}
    		if (hitJudge(hitProbability,
                         pokeStatePlaceOpponent.protectSuccessFlag,
                         pokeStatePlaceOpponent.latestUsedMoveEffectCode)) {
    			subjectValue = subjectValueCalc(moveTypeChoseMyself,
                                                attackMyself,
                                                contactMyself,
                                                pokeStateBatonMyself.attackRank,
                                                pokeStateBatonMyself.contactRank,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName,
                                                partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth);
    			objectValue = objectValueCalc(moveTypeChoseMyself,
                                              blockOpponent,
                                              diffenceOpponent,
                                              pokeStateBatonOpponent.blockRank,
                                              pokeStateBatonOpponent.diffenceRank,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speciesName,
                                              partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName,
                                              pokeStateWholeOpponent.blockHalfFlag,
                                              pokeStateWholeOpponent.reflectorRestTurn,
                                              pokeStateWholeOpponent.lightWallRestTurn);
    			levelValue = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].level;
    			damage = damageBodyCalc(movePowerChoseMyself,subjectValue,objectValue,levelValue);
    			criticalHitProbability = criticalHitProbabilityCalc(pokeStateBatonMyself.criticalHitProbabilityRank,
                                                                    moveEffectCodeChoseMyself,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].speciesName,
                                                                    partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			if(criticalHitJudge(criticalHitProbability)){
    				damage = criticalHitDamageCalc(damage,
                                                   moveTypeChoseMyself,
                                                   pokeStateBatonMyself.attackRank,
                                                   pokeStateBatonMyself.contactRank,
                                                   pokeStateBatonOpponent.blockRank,
                                                   pokeStateBatonOpponent.diffenceRank,
                                                   partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].badHealth,
                                                   pokeStateWholeOpponent.reflectorRestTurn,
                                                   pokeStateWholeOpponent.lightWallRestTurn);
    			}
    			damage = typeReinforceItemCalc(damage,
                                               moveTypeChoseMyself,
                                               partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName);
    			damage = constPlus2Calc(damage);
    			damage = weatherDamageCalc(damage,placeState.weather,moveTypeChoseMyself,moveEffectCodeChoseMyself);
    			damage = typeMatchCalc(damage,
                                       moveTypeChoseMyself,
                                       type1Myself,
                                       type2Myself);
    			damage = typeCompatibilityCalc(damage,
                                               moveTypeChoseMyself,
                                               type1Opponent,
                                               type2Opponent,
                                               pokeStatePlaceMyself.seeingThroughFlag,
                                               pokeStatePlaceOpponent.seenThroughFlag);
    			damage = damageTripleKickBugCalc(damage,placeState.tripleKickRatio);
    			if (damage > 0) {
        			moveHitFlag = true;
    				damage = damageRandomCalc(damage);
    				if (pokeStateBatonOpponent.scapegoatHP > 0) {
    					scapegoatHPUpDownOpponent = damage * (-1);
    				} else {
				    	spiritFlag = spiritTriggerCalc(damage,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
			                       partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    					damage = finalDamageCalc(damage,
                                                 partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP,
                                                 pokeStatePlaceOpponent.protectSuccessFlag,
                                                 pokeStatePlaceOpponent.latestUsedMoveEffectCode,
                                                 moveEffectCodeChoseMyself,
                                                 spiritFlag);
    					HPUpDownOpponent = damage * (-1);
    					LastHPUpDownOpponent = HPUpDownOpponent;
    					if (partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].itemName.equals("おうじゃのしるし")) {
    						if (additionalEffectJudge(11.7,"おうじゃのしるし")) {
    							winceFlag = true;
    						}
    					}
    				}
    			}
    		}
		}
    }

	/* 相手の道連れにより道連れになるか否かの判定メソッド（成功時は反則フラグも立てる）
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void downTogetherJudge() throws IOException {
    	if ((HPUpDownOpponent * (-1)) >= partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP) {
    		if (pokeStatePlaceOpponent.downTogetherFlag) {
    			HPUpDownMyself = partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP * (-1);
    			foulFlagOpponent = true;
    			DialogUtil.dialogMessageOutput("道連れ発動");
    		}
    	}
    }

	/* 技使用結果の反映メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：IOException（対話式でログファイルを出力できない場合）
	 */
    public void calcResultApply() throws IOException {

		// 自分の本体HPの反映
    	if (HPUpDownMyself != 0) {
    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].HPUpDown(HPUpDownMyself);
    	}

		// 自分の身代わりHPの反映
    	if (scapegoatHPUpDownMyself != 0) {
    		pokeStateBatonMyself.scapegoatHPUpDown(scapegoatHPUpDownMyself);
    	}

		// 相手の本体HPの反映
    	if (HPUpDownOpponent != 0) {
    		partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].HPUpDown(HPUpDownOpponent);
    	}

		// 相手の身代わりHPの反映
    	if (scapegoatHPUpDownOpponent != 0) {
    		pokeStateBatonOpponent.scapegoatHPUpDown(scapegoatHPUpDownOpponent);
    	}

    	// 指を振る・オウム返しを使用した時のコンボ技管理
    	if (moveCopyFlag) {
    		pokeStatePlaceMyself.whenMoveCopyCommon(moveNameChoseMyself,
    				                                moveTypeChoseMyself,
    				                                movePowerChoseMyself,
    				                                moveHitProbabilityChoseMyself,
    				                                movePPPrimitiveChoseMyself,
    				                                additionalEffectProbabilityChoseMyself,
    				                                moveEffectCodeChoseMyself);
    	}

    	// コンボターンのセットの反映（開始時）
    	if (comboStartTurnSet != 0) {
    		pokeStatePlaceMyself.comboStart(comboStartTurnSet, moveEffectCodeChoseMyself);
    	}

 		// コンボターンのカウントダウンの反映（継続時）
    	if (comboCountDownFlag) {
    		pokeStatePlaceMyself.comboCountDown();
    	}

    	// コンボターンのリセットの反映（転がる外れ時）
    	if (comboCountResetFlag) {
    		pokeStatePlaceMyself.comboCountReset();
    	}

    	// 気合のハチマキ発動の反映（カウンター・ミラーコート・我慢のダメージに影響）
    	if (spiritFlag) {
    		pokeStatePlaceOpponent.spiritTrigger();
    	}

    	// みねうち発動の反映（カウンター・我慢のダメージに影響）
    	if (mineuchiFlag) {
    		pokeStatePlaceOpponent.mineuchiTrigger();
    	}

 		// 技が命中したかどうかの反映（PokeStatePlaceクラス共通処理の呼び出し）
    	if (moveHitFlag) {
    		pokeStatePlaceOpponent.whenGetDamageCommon(HPUpDownOpponent, LastHPUpDownOpponent, pokeStatePlaceMyself);
    	}

		// 天候の反映
    	if (changeWeatherFlag) {
    		placeState.weatherChange(moveEffectCodeChoseMyself);
    	}

		// 自ら眠り状態になる反映
    	if (sleepMyselfFlag) {
    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].goIntoSleepMyself();
    	}

		// 自分の全てのポケモンの状態異常回復反映（癒しの鈴）
    	if (everyoneGoodHealthFlag) {
			for (int i = 0 ; i < 6 ; i++){
				if (partyInfoMyself.pokeStateUnitArray[i] != null) {
					if (partyInfoMyself.pokeStateUnitArray[i].actualHP > 0) {
						partyInfoMyself.pokeStateUnitArray[i].changeBadHealth("none");
					}
				}
			}
    	}

		// 自分に状態異常をかける反映
    	if (badHealthMyselfTmp != null) {
    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeBadHealth(badHealthMyselfTmp);
    	}

		// 相手に状態異常をかける反映
    	if (badHealthOpponentTmp != null) {
    		partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].changeBadHealth(badHealthOpponentTmp);
    	}

    	// 催眠でコンボターンをリセットする反映
    	if (comboCountResetFlagWhenSleep) {
    		pokeStatePlaceOpponent.comboCountReset();
    	}

		// 相手を猛毒にする反映
    	if (morePoisonFlag) {
    		pokeStateBatonOpponent.morePoisonStart();
    	}

		// 相手を混乱にする反映
    	if (confusionFlag) {
    		pokeStateBatonOpponent.confusionRestTimeUpDown(pokeStateBatonOpponent.confusionTimeGenerate());
    	}

		// 泥棒の反映
    	if (stealFlag) {
    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].changeItem(partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].itemName);
    		partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].changeItem("持ち物無し");
    	}

		// 空中・地中へ行く・戻る反映
    	// 空中・地中が被る可能性はある、かつ空中・地中は同時に解除になると仮定する
    	if (goIntoSkyFlag) {
    		pokeStatePlaceMyself.goIntoSky(true);
    	} else if (goIntoUndergroundFlag) {
    		pokeStatePlaceMyself.goIntoUnderground(true);
    	}
    	if (returnOnGroundFlag) {
    		if (pokeStatePlaceMyself.intoSkyFlag){
    			pokeStatePlaceMyself.goIntoSky(false);
    		}
    		if (pokeStatePlaceMyself.intoUndergroundFlag){
    			pokeStatePlaceMyself.goIntoUnderground(false);
    		}
    	}

		// 自爆・大爆発による反則フラグの反映
    	if (foulFlagMyself) {
    		pokeStateBatonMyself.explosionFoulFlagSet();
    	}

    	// 道連れによる反則フラグの反映
    	if (foulFlagOpponent) {
    		pokeStateBatonOpponent.explosionFoulFlagSet();
    	}

		// 悪あがきモード移行反映
    	// if (struggleFlag) {
    	//	partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].goIntoStruggle();
    	// }

		// 自分の各種ランク変動の反映
    	if (attackRankUpDownMyself != 0) {
    		pokeStateBatonMyself.attackRankUpDown(attackRankUpDownMyself);
    	}
    	if (blockRankUpDownMyself != 0) {
    		pokeStateBatonMyself.blockRankUpDown(blockRankUpDownMyself);
    	}
    	if (contactRankUpDownMyself != 0) {
    		pokeStateBatonMyself.contactRankUpDown(contactRankUpDownMyself);
    	}
    	if (diffenceRankUpDownMyself != 0) {
    		pokeStateBatonMyself.diffenceRankUpDown(diffenceRankUpDownMyself);
    	}
    	if (speedRankUpDownMyself != 0) {
    		pokeStateBatonMyself.speedRankUpDown(speedRankUpDownMyself);
    	}
    	if (hitProbabilityRankUpDownMyself != 0) {
    		pokeStateBatonMyself.hitProbabilityRankUpDown(hitProbabilityRankUpDownMyself);
    	}
    	if (avoidProbabilityRankUpDownMyself != 0) {
    		pokeStateBatonMyself.avoidProbabilityRankUpDown(avoidProbabilityRankUpDownMyself);
    	}
    	if (criticalHitProbabilityRankUpDownMyself != 0) {
    		pokeStateBatonMyself.criticalHitProbabilityRankUpDown(criticalHitProbabilityRankUpDownMyself);
    	}

		// 相手の各種ランク変動の反映
    	if (attackRankUpDownOpponent != 0) {
    		pokeStateBatonOpponent.attackRankUpDown(attackRankUpDownOpponent);
    	}
    	if (blockRankUpDownOpponent != 0) {
    		pokeStateBatonOpponent.blockRankUpDown(blockRankUpDownOpponent);
    	}
    	if (contactRankUpDownOpponent != 0) {
    		pokeStateBatonOpponent.contactRankUpDown(contactRankUpDownOpponent);
    	}
    	if (diffenceRankUpDownOpponent != 0) {
    		pokeStateBatonOpponent.diffenceRankUpDown(diffenceRankUpDownOpponent);
    	}
    	if (speedRankUpDownOpponent != 0) {
    		pokeStateBatonOpponent.speedRankUpDown(speedRankUpDownOpponent);
    	}
    	if (hitProbabilityRankUpDownOpponent != 0) {
    		pokeStateBatonOpponent.hitProbabilityRankUpDown(hitProbabilityRankUpDownOpponent);
    	}
    	if (avoidProbabilityRankUpDownOpponent != 0) {
    		pokeStateBatonOpponent.avoidProbabilityRankUpDown(avoidProbabilityRankUpDownOpponent);
    	}
    	if (criticalHitProbabilityRankUpDownOpponent != 0) {
    		pokeStateBatonOpponent.criticalHitProbabilityRankUpDown(criticalHitProbabilityRankUpDownOpponent);
    	}

		// 黒い霧の反映
    	if (blackFogFlag) {
    		pokeStateBatonMyself.attackRankUpDown(6 - pokeStateBatonMyself.attackRank);
    		pokeStateBatonMyself.blockRankUpDown(6 - pokeStateBatonMyself.blockRank);
    		pokeStateBatonMyself.contactRankUpDown(6 - pokeStateBatonMyself.contactRank);
    		pokeStateBatonMyself.diffenceRankUpDown(6 - pokeStateBatonMyself.diffenceRank);
    		pokeStateBatonMyself.speedRankUpDown(6 - pokeStateBatonMyself.speedRank);
    		pokeStateBatonMyself.hitProbabilityRankUpDown(6 - pokeStateBatonMyself.hitProbabilityRank);
    		pokeStateBatonMyself.avoidProbabilityRankUpDown(6 - pokeStateBatonMyself.avoidProbabilityRank);
    		pokeStateBatonOpponent.attackRankUpDown(6 - pokeStateBatonOpponent.attackRank);
    		pokeStateBatonOpponent.blockRankUpDown(6 - pokeStateBatonOpponent.blockRank);
    		pokeStateBatonOpponent.contactRankUpDown(6 - pokeStateBatonOpponent.contactRank);
    		pokeStateBatonOpponent.diffenceRankUpDown(6 - pokeStateBatonOpponent.diffenceRank);
    		pokeStateBatonOpponent.speedRankUpDown(6 - pokeStateBatonOpponent.speedRank);
    		pokeStateBatonOpponent.hitProbabilityRankUpDown(6 - pokeStateBatonOpponent.hitProbabilityRank);
    		pokeStateBatonOpponent.avoidProbabilityRankUpDown(6 - pokeStateBatonOpponent.avoidProbabilityRank);
    	}

		// 自己暗示の反映
    	if (SelfSuggestionFlag) {
    		pokeStateBatonMyself.rankCopy(pokeStateBatonOpponent.attackRank,
    		                              pokeStateBatonOpponent.blockRank,
    		                              pokeStateBatonOpponent.contactRank,
    		                              pokeStateBatonOpponent.diffenceRank,
    		                              pokeStateBatonOpponent.speedRank,
    		                              pokeStateBatonOpponent.hitProbabilityRank,
    		                              pokeStateBatonOpponent.avoidProbabilityRank);
    	}

		// テクスチャー・テクスチャー２の反映
    	if (typePasteFlag) {
    		pokeStatePlaceMyself.textureType(type1MyselfPaste);
    	}

		// スケッチの反映
    	if (sketchFlag) {
    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].sketchMove(pokeStatePlaceMyself.latestUsedMoveIndex,
    				                                                                               pokeStatePlaceOpponent.latestUsedMoveName,
    				                                                                               pokeStatePlaceOpponent.latestUsedMoveType,
    				                                                                               pokeStatePlaceOpponent.latestUsedMovePower,
    				                                                                               pokeStatePlaceOpponent.latestUsedMoveHitProbability,
    				                                                                               pokeStatePlaceOpponent.latestUsedMovePPPrimitive,
    				                                                                               pokeStatePlaceOpponent.latestUsedAdditionalEffectProbability,
    				                                                                               pokeStatePlaceOpponent.latestUsedMoveEffectCode);
    	}

		// 物真似の反映
    	if (mimicFlag) {
    		pokeStatePlaceMyself.mimicMove(pokeStatePlaceMyself.latestUsedMoveIndex,
    				                       pokeStatePlaceOpponent.latestUsedMoveName,
    				                       pokeStatePlaceOpponent.latestUsedMoveType,
    				                       pokeStatePlaceOpponent.latestUsedMovePower,
    				                       pokeStatePlaceOpponent.latestUsedMoveHitProbability,
    				                       pokeStatePlaceOpponent.latestUsedMovePPPrimitive,
    				                       pokeStatePlaceOpponent.latestUsedAdditionalEffectProbability,
    				                       pokeStatePlaceOpponent.latestUsedMoveEffectCode);
    	}

		// 変身の反映（スケッチや物真似で置き換わった技の方をコピーすることに注意）
    	if (transformFlag) {
    		pokeStatePlaceMyself.transformPoke(partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].type1,
    		                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].type2,
    		                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].attack,
    		                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].block,
    		                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].contact,
    		                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].diffence,
    		                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].speed,
    		                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].hiddenPowerType,
    		                                   partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].hiddenPowerPower,
    		                                   moveNameOpponent,
    		                                   moveTypeOpponent,
    		                                   movePowerOpponent,
    		                                   moveHitProbabilityOpponent,
    		                                   movePPPrimitiveOpponent,
    		                                   movePPPrimitiveOpponent,
    		                                   additionalEffectProbabilityOpponent,
    		                                   moveEffectCodeOpponent,
    		                                   pokeStatePlaceOpponent.transformedFlag);
    		pokeStateBatonMyself.rankCopy(pokeStateBatonOpponent.attackRank,
                                          pokeStateBatonOpponent.blockRank,
                                          pokeStateBatonOpponent.contactRank,
                                          pokeStateBatonOpponent.diffenceRank,
                                          pokeStateBatonOpponent.speedRank,
                                          pokeStateBatonOpponent.hitProbabilityRank,
                                          pokeStateBatonOpponent.avoidProbabilityRank);
    	}

		// 相手のPPの反映（恨み）
    	if (PPDownOpponent != 0) {
    		if (pokeStatePlaceOpponent.moveIndex == pokeStatePlaceOpponent.mimicMoveIndex) {
    			pokeStatePlaceOpponent.mimicPPUpDown(PPDownOpponent);
    		} else if (pokeStatePlaceOpponent.moveIndex == partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchMoveIndex) {
    			partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].sketchPPUpDown(PPDownOpponent);
    		} else if (pokeStatePlaceOpponent.transformedFlag){
    			pokeStatePlaceOpponent.transformedPPUpDown(pokeStatePlaceOpponent.moveIndex,PPDownOpponent);
    		} else {
    			partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].PPUpDown(pokeStatePlaceOpponent.moveIndex,PPDownOpponent);
    		}
    	}

		// 我慢累積ダメージリセットの反映
    	if (bideReleaseFlag) {
    		pokeStatePlaceMyself.bideEnd();
    	}

		// 吠える・吹き飛ばしの反映
    	if (pokeStateUnitArrayOpponentIndexChanged != -1) {
    		pokeStatePlaceOpponent.changeTargetSet(pokeStateUnitArrayOpponentIndexChanged);
    		pokeStatePlaceOpponent.forceChanged();
    	}

		// リフレクターの反映
    	if (reflectorFlag) {
    		pokeStateWholeMyself.reflectorStart();
    	}

		// 光の壁の反映
    	if (lightWallFlag) {
    		pokeStateWholeMyself.lightWallStart();
    	}

		// 神秘の守りの反映
    	if (mysteryProtectFlag) {
    		pokeStateWholeMyself.mysteryProtectStart();
    	}

		// 締め付ける反映
    	if (twiningFlag) {
    		int twiningTwindRandom = pokeStatePlaceMyself.twiningTwindTrunGenerate();
    		pokeStatePlaceMyself.twiningSet(twiningTwindRandom);
    		pokeStatePlaceOpponent.twindSet(twiningTwindRandom);
    	}

		// 白い霧反映
    	if (whiteFogFlag) {
    		pokeStateBatonMyself.beingWhiteFog();
    	}

		// 怯みの反映
    	if (winceFlag) {
    		pokeStatePlaceOpponent.winceFlagSet();
    	}

		// 破壊光線命中動けないの反映
    	if (destructionBeemStackSetFlag) {
    		pokeStatePlaceMyself.destructionBeemHit();
    	}

		// 宿り木の反映
    	if (parasiteFlag) {
    		pokeStateBatonOpponent.parasited(parasiteFlag);
    	}

		// 金縛りの反映
    	if (moveParalysisFlag) {
    		pokeStatePlaceOpponent.moveParalysisSet(pokeStatePlaceOpponent.latestUsedMoveIndex);
    	}

		// アンコールの反映
    	if (encoreFlag) {
    		pokeStatePlaceOpponent.encoredSet(pokeStatePlaceOpponent.latestUsedMoveIndex);
    	}

		// ロックオンの反映
    	if (lockOnFlag) {
    		pokeStatePlaceOpponent.beenLockon();
    	}

		// 道連れの反映
    	if (downTogetherWaitFlag) {
    		pokeStatePlaceMyself.downTogetherWaiting(downTogetherWaitFlag);
    	}

		// 黒い眼差しの反映
    	if (trappingFlag) {
    		pokeStateBatonMyself.trapping();
    		pokeStateBatonOpponent.trapped();
    	}

		// 悪夢の反映
    	if (nightmareFlag) {
    		pokeStatePlaceOpponent.beenIntoNightmare(nightmareFlag);
    	}

		// 呪いの反映
    	if (curceFlag) {
    		pokeStateBatonOpponent.curced();
    	}

		// 守る・見切り・堪えるの反映
    	if (protectSuccessFlag) {
    		pokeStatePlaceMyself.protect();
    	}

		// 撒きびしの反映
    	if (spikesFlag) {
    		pokeStateWholeOpponent.spikesSet(spikesFlag);
    	}

		// 見破るの反映
    	if (seeingThroughFlag) {
    		pokeStatePlaceMyself.seeingThrough();
    		pokeStatePlaceOpponent.seenThrough();
    	}

		// 滅びの歌の反映
    	if (startRuinFlag) {
    		pokeStateBatonMyself.beingRuinStart();
    		pokeStateBatonOpponent.beenRuinStart();
    	}

		// 連続切りの反映
    	if (slashContinuousHitFlag) {
    		pokeStatePlaceMyself.slashContinuousHit();
    	}

    	// 転がるの反映
    	if (rollContinuousHitFlag) {
    		pokeStatePlaceMyself.rollContinuousHit();
    	}

		// メロメロの反映
    	if (deprivingMindFlag) {
    		pokeStatePlaceMyself.deprivingMind();
    		pokeStatePlaceOpponent.deprivedMind();
    	}

		// 高速スピンの反映
    	if (spinFlag) {
    		pokeStateWholeMyself.spikesSet(false);
    		pokeStateBatonMyself.parasited(false);
    		pokeStatePlaceMyself.twindSet(0);
    		pokeStatePlaceOpponent.twiningSet(0);
    	}

		// 未来予知の反映
    	if (predictionDamage != 0) {
    		pokeStateWholeOpponent.setPrediction(predictionDamage,placeState.currentTrun);
    	}

		// 小さくなるの反映
    	if (smallShapeFlag) {
    		pokeStatePlaceMyself.smallShape();
    	}

		// 丸くなるの反映
    	if (roundShapeFlag) {
    		pokeStatePlaceMyself.roundShape();
    	}

    	// バトンタッチ成功の反映
    	if (moveEffectCodeChoseMyself.equals("7F") && changeMyselfFlag) {
    		pokeStatePlaceMyself.batonSuccess();
    	}

    	// バトンタッチ失敗の反映
    	if (batonFailFlag) {
    		pokeStateWholeMyself.changeFailed();
    	}
    }

	/* 出力（交代要否）の判定メソッド
	 *   引数　　　：無し
	 *   戻り値　　：無し
	 *   例外　　　：無し
	 */
    public void changeNecessitySet() {
    	changeNecessityNumber = 0;
    	if (changeMyselfFlag ||
    		partyInfoMyself.pokeStateUnitArray[partyInfoMyself.pokeStateUnitArrayIndex].actualHP <= 0){
    		changeNecessityNumber = changeNecessityNumber + 1;
    	}
    	if (pokeStateUnitArrayOpponentIndexChanged != -1 ||
    		partyInfoOpponent.pokeStateUnitArray[partyInfoOpponent.pokeStateUnitArrayIndex].actualHP <= 0) {
    		changeNecessityNumber = changeNecessityNumber + 2;
    	}
    }
}
